Package: scdhlm
Title: Estimating Hierarchical Linear Models for Single-Case Designs
Version: 0.7.4
Authors@R: c(
    person("James", "Pustejovsky", email = "jepusto@gmail.com", role = c("aut", "cre")),
    person("Man", "Chen", role = "aut"), 
    person("Bethany", "Hamilton", role = "aut")
	)
Description: Provides a set of tools for estimating hierarchical linear
    models and effect sizes based on data from single-case designs. 
    Functions are provided for calculating standardized mean difference effect sizes that 
    are directly comparable to standardized mean differences estimated from between-subjects randomized experiments,
    as described in Hedges, Pustejovsky, and Shadish (2012) <DOI:10.1002/jrsm.1052>; 
    Hedges, Pustejovsky, and Shadish (2013) <DOI:10.1002/jrsm.1086>; 
    Pustejovsky, Hedges, and Shadish (2014) <DOI:10.3102/1076998614547577>; 
    and Chen, Pustejovsky, Klingbeil, and Van Norman (2023) <DOI:10.1016/j.jsp.2023.02.002>. 
    Includes an interactive web interface.
URL: https://jepusto.github.io/scdhlm/
BugReports: https://github.com/jepusto/scdhlm/issues
License: GPL-3
VignetteBuilder: knitr
LazyData: true
Imports: stats, utils, readxl, rlang, dplyr, tidyselect, magrittr,
        lmeInfo (>= 0.3.0)
Depends: nlme, R (>= 3.0.1)
Suggests: knitr, markdown, rmarkdown, ggplot2, plyr, boot, parallel,
        shiny, shinytest, glue, janitor, rclipboard, rvest, testthat,
        snow, doSNOW, foreach, iterators, reshape, rlecuyer
RoxygenNote: 7.3.3
Encoding: UTF-8
Language: en-US
NeedsCompilation: no
Packaged: 2026-01-21 20:17:14 UTC; jamespustejovsky
Author: James Pustejovsky [aut, cre],
  Man Chen [aut],
  Bethany Hamilton [aut]
Maintainer: James Pustejovsky <jepusto@gmail.com>
Repository: CRAN
Date/Publication: 2026-01-21 20:40:02 UTC
Built: R 4.6.0; ; 2026-04-26 15:00:46 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(CI_g,g_HPS)
S3method(CI_g,g_REML)
S3method(print,g_HPS)
S3method(print,g_REML)
S3method(simulate,g_REML)
S3method(summary,g_HPS)
S3method(summary,g_REML)
export(CI_g)
export(Info_Expected_lmeAR1)
export(batch_calc_BCSMD)
export(calc_BCSMD)
export(compare_RML_HPS)
export(default_times)
export(design_matrix)
export(effect_size_ABk)
export(effect_size_MB)
export(extract_varcomp)
export(g_REML)
export(g_mlm)
export(graph_SCD)
export(phase_pairs)
export(preprocess_SCD)
export(session_by_treatment)
export(shine_scd)
export(simulate_MB2)
export(simulate_MB4)
export(varcomp_vcov)
import(nlme)
import(stats)
importFrom(dplyr,.data)
importFrom(lmeInfo,CI_g)
importFrom(lmeInfo,extract_varcomp)
importFrom(lmeInfo,g_mlm)
importFrom(lmeInfo,varcomp_vcov)
importFrom(magrittr,"%>%")
importFrom(readxl,read_excel)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")

<!-- README.md is generated from README.Rmd. Please edit that file -->
<!-- badges: start -->

[![R-CMD-check](https://github.com/jepusto/scdhlm/workflows/R-CMD-check/badge.svg)](https://github.com/jepusto/scdhlm/actions)
[![Codecov
Status](https://codecov.io/gh/jepusto/scdhlm/branch/master/graph/badge.svg)](https://codecov.io/gh/jepusto/scdhlm?branch=master)
[![](http://www.r-pkg.org/badges/version/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
[![](http://cranlogs.r-pkg.org/badges/grand-total/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
[![](http://cranlogs.r-pkg.org/badges/last-month/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
<!-- badges: end -->

# Estimating Hierarchical Linear Models for Single-Case Designs

`scdhlm` provides a set of tools for estimating hierarchical linear
models and effect sizes based on data from single-case designs. The
estimated effect sizes, as described in Pustejovsky, Hedges, and Shadish
(2014), are comparable in principle to standardized mean differences
(SMDs) estimated from between-subjects randomized experiments. The
package includes functions for estimating design-comparable SMDs based
on data from treatment reversal designs with replication across
participants (Hedges, Pustejovsky, & Shadish, 2012), across-participant
multiple baseline designs and multiple probe designs (Hedges,
Pustejovsky, & Shadish, 2013; Pustejovsky, Hedges, & Shadish, 2014), and
more complex variations of multiple baseline designs (Chen, Pustejovsky,
Klingbeil, & Van Norman, 2023). Two estimation methods are available:
moment estimation and restricted maximum likelihood estimation. The
package also includes an interactive web interface implemented using
Shiny.

# Acknowledgment

<img src="https://raw.githubusercontent.com/jepusto/scdhlm/master/images/IES_InstituteOfEducationSciences_RGB.svg" width="40%" align = "right" alt = "Institute of Education Sciences logo"/>

The development of this R package was supported in part by the Institute
of Education Sciences, U.S. Department of Education, through [Grant
R324U190002](https://ies.ed.gov/funding/grantsearch/details.asp?ID=3358)
to the University of Oregon. The contents of the package do not
necessarily represent the views of the Institute or the U.S. Department
of Education.

# Citations

Please cite this R package as follows:

> Pustejovsky, J. E., Chen, M., & Hamilton, B. J. (2023). scdhlm:
> Estimating hierarchical linear models for single-case designs (Version
> 0.7.2) \[R package\]. <https://CRAN.R-project.org/package=scdhlm>

Please cite the web application as follows:

> Pustejovsky, J. E., Chen, M., Hamilton, B., & Grekov, P. (2023).
> scdhlm: A web-based calculator for between-case standardized mean
> differences (Version 0.7.2) \[Web application\].
> <https://jepusto.shinyapps.io/scdhlm>

# Installation

You can install the released version of `scdhlm` from
[CRAN](https://CRAN.R-project.org) with:

``` r
install.packages("scdhlm")
```

You can install the development version from
[GitHub](https://github.com/) with:

``` r
# install.packages("remotes")
remotes::install_github("jepusto/scdhlm")
```

You can access a local version of the [interactive
web-app](https://jepusto.shinyapps.io/scdhlm/) by running the commands:

``` r
library(scdhlm)
shine_scd()
```

# Demonstration

Here we demonstrate how to use `scdhlm` to calculate design-comparable
SMDs based on data from different single-case designs. We will first
demonstrate the recommended approach, which uses restricted maximum
likelihood (REML) estimation. We will then demonstrate the older, moment
estimation methods. The moment estimation methods were the originally
proposed approach (described in Hedges, Pustejovsky, & Shadish, 2012,
2013). The package provides these methods for sake of completeness, but
we no longer recommend them for general use.

## Estimating SMDs using REML with `g_mlm()`

Laski, Charlop, and Schreibman (1988) used a multiple baseline across
individuals to evaluate the effect of a training program for parents on
the speech production of their autistic children, as measured using a
partial interval recording procedure. The design included eight
children. One child was measured separately with each parent; following
Hedges, Pustejovsky, and Shadish (2013), only the measurements taken
with the mother are included in our analysis.

For this study, we will estimate a design-comparable SMD using
restricted maximum likelihood (REML) methods, as described by
Pustejovsky and colleagues (2014). This is a two-step process. The first
step is to estimate a hierarchical linear model for the data, treated
the measurements as nested within cases. We fit the model using
`nlme::lme()`

``` r
library(nlme)
library(scdhlm)
data(Laski)

# Pustejovsky, Hedges, & Shadish (2014)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
Laski_RML
#> Linear mixed-effects model fit by REML
#>   Data: Laski 
#>   Log-restricted-likelihood: -519.1424
#>   Fixed: outcome ~ treatment 
#>        (Intercept) treatmenttreatment 
#>           39.07612           30.68366 
#> 
#> Random effects:
#>  Formula: ~1 | case
#>         (Intercept) Residual
#> StdDev:    15.68278  13.8842
#> 
#> Correlation Structure: AR(1)
#>  Formula: ~time | case 
#>  Parameter estimate(s):
#>      Phi 
#> 0.252769 
#> Number of Observations: 128
#> Number of Groups: 8
```

The summary of the fitted model displays estimates of the component
parameters, including the within-case and between-case standard
deviations, auto-correlation, and (unstandardized) treatment effect
estimate. These estimated components will be used to calculate the
effect size in next step.

The estimated variance components from the fitted model can be obtained
using `extract_varcomp()`:

``` r
varcomp_Laski_RML <- extract_varcomp(Laski_RML)
varcomp_Laski_RML
#> $Tau
#> $Tau$case
#> case.var((Intercept)) 
#>              245.9497 
#> 
#> 
#> $cor_params
#> [1] 0.252769
#> 
#> $var_params
#> numeric(0)
#> 
#> $sigma_sq
#> [1] 192.7711
#> 
#> attr(,"class")
#> [1] "varcomp"
```

The estimated between-case variance is 245.95, the estimated
auto-correlation is 0.253, the estimated and the estimated within-case
variance is 192.771. These estimated variance components will be used to
calculate the effect size in next step.

The second step in the process is to estimate a design-comparable SMD
using `scdhlm::g_mlm()`. The SMD parameter can be defined as the ratio
of a linear combination of the fitted model’s fixed effect parameters
over the square root of a linear combination of the model’s variance
components. `g_mlm()` takes the fitted `lme` model object as input,
followed by the vectors `p_const` and `r_const`, which specify the
components of the fixed effects and variance estimates that are to be
used in constructing the design-comparable SMD. Note that `r_const` is a
vector of 0s and 1s which specify whether to use the variance component
parameters for calculating the effect size: random effects variances,
correlation structure parameters, variance structure parameters, and
level-1 error variance. The function calculates an effect size estimate
by first substituting maximum likelihood or restricted maximum
likelihood estimates in place of the corresponding parameters, then
applying a small-sample correction. The small-sample correction and the
standard error are based on approximating the distribution of the
estimator by a t distribution, with degrees of freedom given by a
Satterthwaite approximation (Pustejovsky, Hedges, & Shadish, 2014). The
`g_mlm()` function includes an option allowing use of the expected or
average form of the Fisher information matrix in the calculations.

In this example, we use the treatment effect in the numerator of the
effect size and the sum of the between-case and within-case variance
components in the denominator of the effect size. The constants are
therefore given by `p_const = c(0, 1)` and `r_const = c(1, 0, 1)`. The
effect size estimated is calculated as:

``` r
Laski_ES_RML <- g_mlm(Laski_RML, p_const = c(0, 1), r_const = c(1, 0, 1))

Laski_ES_RML
#>                           est    se
#> unadjusted effect size  1.465 0.299
#> adjusted effect size    1.405 0.286
#> degree of freedom      18.552
```

The adjusted SMD effect size estimate is 1.405 with standard error of
0.286 and degree of freedom 18.6.

A `summary()` method is included, which returns more detail about the
model parameter estimates and effect size estimate when setting
`returnModel = TRUE` (the default) in `g_mlm()`:

``` r
summary(Laski_ES_RML)
#>                                           est      se
#> Tau.case.case.var((Intercept))        245.950 142.179
#> cor_params                              0.253   0.100
#> sigma_sq                              192.771  28.265
#> total variance                        438.721 144.047
#> (Intercept)                            39.076   5.989
#> treatmenttreatment                     30.684   2.996
#> treatment effect at a specified time   30.684   2.996
#> unadjusted effect size                  1.465   0.299
#> adjusted effect size                    1.405   0.286
#> degree of freedom                      18.552        
#> constant kappa                          0.143        
#> logLik                               -519.142
```

The `CI_g()` calculates a symmetric confidence interval using a central
t distribution (the default) or an asymmetric confidence interval using
non-central t distribution (setting `symmetric = FALSE`).

``` r
CI_g(Laski_ES_RML)
#> [1] 0.8046224 2.0051521

CI_g(Laski_ES_RML, symmetric = FALSE)
#> [1] 0.9143684 2.0046719
```

The symmetric confidence interval is \[0.805, 2.005\] and the asymmetric
confidence interval is \[0.914, 2.005\].

## Estimating SMDs using `effect_size_ABk()`

Lambert, Cartledge, Heward, and Lo (2006) tested the effect of using
response cards (compared to single-student responding) during math
lessons in two fourth-grade classrooms. The investigators collected data
on rates of disruptive behavior and academic response for nine focal
students, using an ABAB design. This example is discussed in Hedges,
Pustejovsky, and Shadish (2012), who selected it because the design was
close to balanced and used a relatively large number of cases. Their
calculations can be replicated using the `effect_size_ABk()` function.
To use this function, the user must provide the names of five variables:

- the outcome variable,
- a variable indicating the treatment condition,
- a variable listing the case on which the outcome was measured,
- a variable indicating the phase of treatment (i.e., each replication
  of a baseline and treatment condition), and
- a variable listing the session number.

In the `Lambert` dataset, these variables are called respectively
`outcome`, `treatment`, `case`, `phase`, and `time`. Given these inputs,
the design-comparable SMD is calculated as follows for the measure of
academic response:

``` r
data(Lambert)

Lambert_academic <- subset(Lambert, measure == "academic response")
Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
                              phase = phase, time = time, data = Lambert_academic)

Lambert_ES
#>                            est    se
#> unadjusted effect size   7.171 0.484
#> adjusted effect size     7.128 0.482
#> degree of freedom      126.671
```

The adjusted effect size estimate `delta_hat` is equal to 7.128; its
variance `V_delta_hat` is equal to 0.232. A standard error for
`delta_hat` can be calculated by taking the square root of
`V_delta_hat`: `sqrt(Lambert_ES$V_delta_hat)` = 0.482. The effect size
estimate is bias-corrected in a manner analogous to Hedges’ g correction
for SMDs from a between-subjects design. The degrees of freedom `nu` are
estimated based on a Satterthwaite-type approximation, which is equal to
126.7 in this example.

A summary() method is included to return more detail about the model
parameter estimates and effect size estimates:

``` r
summary(Lambert_ES)
#>                                       est    se
#> within-case variance                0.018      
#> sample variance                     0.014      
#> intra-class correlation             0.000      
#> auto-correlation                    0.091      
#> numerator of effect size estimate   0.858      
#> unadjusted effect size              7.171 0.484
#> adjusted effect size                7.128 0.482
#> degree of freedom                 126.671      
#> scalar constant                     0.166
```

## Estimating SMDs using `effect_size_MB()`

Saddler, Behforooz, and Asaro (2008) used a multiple baseline design to
investigate the effect of an instructional technique on the writing of
fourth grade students. The investigators assessed the intervention’s
effect on measures of writing quality, sentence complexity, and use of
target constructions.

Design-comparable SMDs can be estimated based on these data using the
`effect_size_MB()` function. The following code calculates a
design-comparable SMD estimate for the measure of writing quality:

``` r
data(Saddler)

Saddler_quality <- subset(Saddler, measure=="writing quality")
quality_ES <- effect_size_MB(outcome, treatment, case, time, data = Saddler_quality)

quality_ES
#>                          est    se
#> unadjusted effect size 2.149 0.634
#> adjusted effect size   1.963 0.579
#> degree of freedom      8.918
```

The adjusted effect size estimate `delta_hat` is equal to 1.963, with
sampling variance of `V_delta_hat` equal to 0.335 and a standard error
of 0.579.

`summary(quality_ES)` returns more detail about the model parameter
estimates and effect size estimates:

``` r
summary(quality_ES)
#>                                     est    se
#> within-case variance              0.349      
#> sample variance                   0.952      
#> intra-class correlation           0.633      
#> auto-correlation                  0.100      
#> numerator of effect size estimate 2.097      
#> unadjusted effect size            2.149 0.634
#> adjusted effect size              1.963 0.579
#> degree of freedom                 8.918      
#> scalar constant                   0.201
```

# References

Chen, M., Pustejovsky, J. E., Klingbeil, D. A., & Van Norman, E. R.
(2023). Between-case standardized mean differences: Flexible methods for
single-case designs. *Journal of School Psychology, 98*, 16-38.
<https://doi.org/10.1016/j.jsp.2023.02.002>

Gilmour, A. R., Thompson, R., & Cullis, B. R. (1995). Average
information REML: An efficient algorithm for variance parameter
estimation in linear mixed models. *Biometrics, 51*(4), 1440–1450.
<https://doi.org/10.2307/2533274>

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012). A
standardized mean difference effect size for single case designs.
*Research Synthesis Methods, 3*(3), 224-239.
<https://doi.org/10.1002/jrsm.1052>

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). A
standardized mean difference effect size for multiple baseline designs
across individuals. *Research Synthesis Methods, 4*(4), 324-341.
<https://doi.org/10.1002/jrsm.1086>

Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). Effects
of response cards on disruptive behavior and academic responding during
math lessons by fourth-grade urban students. *Journal of Positive
Behavior Interventions, 8*(2), 88-99.
<https://doi.org/10.1177/10983007060080020701>

Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). Training parents
to use the natural language paradigm to increase their autistic
children’s speech. *Journal of Applied Behavior Analysis, 21*(4),
391–400. <https://doi.org/10.1901/jaba.1988.21-391>

Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014).
Design-comparable effect sizes in multiple baseline designs: A general
modeling framework. *Journal of Educational and Behavioral Statistics,
39*(4), 211-227. <https://doi.org/10.3102/1076998614547577>

Saddler, B., Behforooz, B., & Asaro, K. (2008). The effects of
sentence-combining instruction on the writing of fourth-grade students
with writing difficulties. *The Journal of Special Education, 42*(2),
79–90. <https://doi.org/10.1177/0022466907310371>

---
output: github_document
---

<!-- README.md is generated from README.Rmd. Please edit that file -->

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.path = "man/figures/README-",
  out.width = "100%"
)
```

<!-- badges: start -->
[![R-CMD-check](https://github.com/jepusto/scdhlm/workflows/R-CMD-check/badge.svg)](https://github.com/jepusto/scdhlm/actions)
[![Codecov Status](https://codecov.io/gh/jepusto/scdhlm/branch/master/graph/badge.svg)](https://codecov.io/gh/jepusto/scdhlm?branch=master)
[![](http://www.r-pkg.org/badges/version/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
[![](http://cranlogs.r-pkg.org/badges/grand-total/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
[![](http://cranlogs.r-pkg.org/badges/last-month/scdhlm)](https://CRAN.R-project.org/package=scdhlm)
<!-- badges: end -->

# Estimating Hierarchical Linear Models for Single-Case Designs

`scdhlm` provides a set of tools for estimating hierarchical linear models and effect sizes based on data from single-case designs. The estimated effect sizes, as described in Pustejovsky, Hedges, and Shadish (2014), are comparable in principle to standardized mean differences (SMDs) estimated from between-subjects randomized experiments. The package includes functions for estimating design-comparable SMDs based on data from treatment reversal designs with replication across participants (Hedges, Pustejovsky, & Shadish, 2012), across-participant multiple baseline designs and multiple probe designs (Hedges, Pustejovsky, & Shadish, 2013; Pustejovsky, Hedges, & Shadish, 2014), and more complex variations of multiple baseline designs (Chen, Pustejovsky, Klingbeil, & Van Norman, 2023). Two estimation methods are available: moment estimation and restricted maximum likelihood estimation. The package also includes an interactive web interface implemented using Shiny.

# Acknowledgment

<img src="https://raw.githubusercontent.com/jepusto/scdhlm/master/images/IES_InstituteOfEducationSciences_RGB.svg" width="40%" align = "right" alt = "Institute of Education Sciences logo"/>

The development of this R package was supported in part by the Institute of Education Sciences, U.S. Department of Education, through [Grant R324U190002](https://ies.ed.gov/funding/grantsearch/details.asp?ID=3358) to the University of Oregon. The contents of the package do not necessarily represent the views of the Institute or the U.S. Department of Education.

# Citations

Please cite this R package as follows:

> Pustejovsky, J. E., Chen, M., & Hamilton, B. J. (`r substr(packageDate("scdhlm"),1,4)`). scdhlm: Estimating hierarchical linear models for single-case designs (Version `r packageVersion("scdhlm")`) [R package]. https://CRAN.R-project.org/package=scdhlm

Please cite the web application as follows:

> Pustejovsky, J. E., Chen, M., Hamilton, B., & Grekov, P. (`r substr(packageDate("scdhlm"),1,4)`). scdhlm: A web-based calculator for between-case standardized mean differences (Version `r packageVersion("scdhlm")`) [Web application]. https://jepusto.shinyapps.io/scdhlm

# Installation

You can install the released version of `scdhlm` from [CRAN](https://CRAN.R-project.org) with:

```{r, eval=FALSE}
install.packages("scdhlm")
```

You can install the development version from [GitHub](https://github.com/) with:

```{r, eval=FALSE}
# install.packages("remotes")
remotes::install_github("jepusto/scdhlm")
```

You can access a local version of the [interactive web-app](https://jepusto.shinyapps.io/scdhlm/) by running the commands:

```{r, eval=FALSE}
library(scdhlm)
shine_scd()
```

# Demonstration

Here we demonstrate how to use `scdhlm` to calculate design-comparable SMDs based on data from different single-case designs. We will first demonstrate the recommended approach, which uses restricted maximum likelihood (REML) estimation. We will then demonstrate the older, moment estimation methods. The moment estimation methods were the originally proposed approach (described in Hedges, Pustejovsky, & Shadish, 2012, 2013). The package provides these methods for sake of completeness, but we no longer recommend them for general use. 

## Estimating SMDs using REML with `g_mlm()`

Laski, Charlop, and Schreibman (1988) used a multiple baseline across individuals to evaluate the effect of a training program for parents on the speech production of their autistic children, as measured using a partial interval recording procedure. The design included eight children. One child was measured separately with each parent; following Hedges, Pustejovsky, and Shadish (2013), only the measurements taken with the mother are included in our analysis.

For this study, we will estimate a design-comparable SMD using restricted maximum likelihood (REML) methods, as described by Pustejovsky and colleagues (2014). This is a two-step process. The first step is to estimate a hierarchical linear model for the data, treated the measurements as nested within cases. We fit the model using `nlme::lme()`

```{r Laski}
library(nlme)
library(scdhlm)
data(Laski)

# Pustejovsky, Hedges, & Shadish (2014)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
Laski_RML
```

The summary of the fitted model displays estimates of the component parameters, including the within-case and between-case standard deviations, auto-correlation, and (unstandardized) treatment effect estimate. These estimated components will be used to calculate the effect size in next step.

The estimated variance components from the fitted model can be obtained using `extract_varcomp()`:
```{r Laski-varcomp}
varcomp_Laski_RML <- extract_varcomp(Laski_RML)
varcomp_Laski_RML
```
The estimated between-case variance is `r round(as.numeric(varcomp_Laski_RML$Tau$case), 3)`, the estimated auto-correlation is `r round(varcomp_Laski_RML$cor_params, 3)`, the estimated  and the estimated within-case variance is `r round(varcomp_Laski_RML$sigma_sq, 3)`. These estimated variance components will be used to calculate the effect size in next step.

The second step in the process is to estimate a design-comparable SMD using `scdhlm::g_mlm()`.
The SMD parameter can be defined as the ratio of a linear combination of the fitted model’s fixed effect parameters over the square root of a linear combination of the model’s variance components. `g_mlm()` takes the fitted `lme` model object as input, followed by the vectors `p_const` and `r_const`, which specify the components of the fixed effects and variance estimates that are to be used in constructing the design-comparable SMD. Note that `r_const` is a vector of 0s and 1s which specify whether to use the variance component parameters for calculating the effect size: random effects variances, correlation structure parameters, variance structure parameters, and level-1 error variance. The function calculates an effect size estimate by first substituting maximum likelihood or restricted maximum likelihood estimates in place of the corresponding parameters, then applying a small-sample correction. The small-sample correction and the standard error are based on approximating the distribution of the estimator by a t distribution, with degrees of freedom given by a Satterthwaite approximation (Pustejovsky, Hedges, & Shadish, 2014). The `g_mlm()` function includes an option allowing use of the expected or average form of the Fisher information matrix in the calculations.

In this example, we use the treatment effect in the numerator of the effect size and the sum of the between-case and within-case variance components in the denominator of the effect size. The constants are therefore given by `p_const = c(0, 1)` and `r_const = c(1, 0, 1)`. The effect size estimated is calculated as:

```{r Laski-bc-smd, eval=TRUE}
Laski_ES_RML <- g_mlm(Laski_RML, p_const = c(0, 1), r_const = c(1, 0, 1))

Laski_ES_RML

```

The adjusted SMD effect size estimate is `r round(Laski_ES_RML$g_AB, 3)` with standard error of `r round(Laski_ES_RML$SE_g_AB, 3)` and degree of freedom `r round(Laski_ES_RML$nu, 1)`.

A `summary()` method is included, which returns more detail about the model parameter estimates and effect size estimate when setting `returnModel = TRUE` (the default) in `g_mlm()`:

```{r Laski-summary, eval=TRUE}
summary(Laski_ES_RML)
```

The `CI_g()` calculates a symmetric confidence interval using a central t distribution (the default) or an asymmetric confidence interval using non-central t distribution (setting `symmetric = FALSE`). 

```{r Laski-CI, eval=TRUE}
CI_g(Laski_ES_RML)

CI_g(Laski_ES_RML, symmetric = FALSE)
```

The symmetric confidence interval is `r paste0("[", round(CI_g(Laski_ES_RML),3)[1], ", ",  round(CI_g(Laski_ES_RML),3)[2], "]")` and the asymmetric confidence interval is `r paste0("[", round(CI_g(Laski_ES_RML, symmetric = FALSE),3)[1], ", ",  round(CI_g(Laski_ES_RML, symmetric = FALSE),3)[2], "]")`.

## Estimating SMDs using `effect_size_ABk()`

Lambert, Cartledge, Heward, and Lo (2006) tested the effect of using response cards (compared to single-student responding) during math lessons in two fourth-grade classrooms. The investigators collected data on rates of disruptive behavior and academic response for nine focal students, using an ABAB design. This example is discussed in Hedges, Pustejovsky, and Shadish (2012), who selected it because the design was close to balanced and used a relatively large number of cases. Their calculations can be replicated using the `effect_size_ABk()` function. To use this function, the user must provide the names of five variables: 

* the outcome variable, 
* a variable indicating the treatment condition, 
* a variable listing the case on which the outcome was measured, 
* a variable indicating the phase of treatment (i.e., each replication of a baseline and treatment condition), and 
* a variable listing the session number.

In the `Lambert` dataset, these variables are called respectively `outcome`, `treatment`, `case`, `phase`, and `time`. Given these inputs, the design-comparable SMD is calculated as follows for the measure of academic response:

```{r}
data(Lambert)

Lambert_academic <- subset(Lambert, measure == "academic response")
Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
                              phase = phase, time = time, data = Lambert_academic)

Lambert_ES
```

The adjusted effect size estimate `delta_hat` is equal to `r round(Lambert_ES$delta_hat, 3)`; its variance `V_delta_hat` is equal to `r round(Lambert_ES$V_delta_hat,3)`. A standard error for `delta_hat` can be calculated by taking the square root of `V_delta_hat`: `sqrt(Lambert_ES$V_delta_hat)` = `r round(sqrt(Lambert_ES$V_delta_hat),3)`. The effect size estimate is bias-corrected in a manner analogous to Hedges' g correction for SMDs from a between-subjects design. The degrees of freedom `nu` are estimated based on a Satterthwaite-type approximation, which is equal to `r round(Lambert_ES$nu,1)` in this example.

A summary() method is included to return more detail about the model parameter estimates and effect size estimates:

```{r}
summary(Lambert_ES)
```

## Estimating SMDs using `effect_size_MB()`

Saddler, Behforooz, and Asaro (2008) used a multiple baseline design to investigate the effect of an instructional technique on the writing of fourth grade students. The investigators assessed the intervention's effect on measures of writing quality, sentence complexity, and use of target constructions. 

Design-comparable SMDs can be estimated based on these data using the `effect_size_MB()` function. The following code calculates a design-comparable SMD estimate for the measure of writing quality:

```{r}
data(Saddler)

Saddler_quality <- subset(Saddler, measure=="writing quality")
quality_ES <- effect_size_MB(outcome, treatment, case, time, data = Saddler_quality)

quality_ES
```

The adjusted effect size estimate `delta_hat` is equal to `r round(quality_ES$delta_hat, 3)`, with sampling variance of `V_delta_hat` equal to `r round(quality_ES$V_delta_hat, 3)` and a standard error of `r round(sqrt(quality_ES$V_delta_hat), 3)`.

`summary(quality_ES)` returns more detail about the model parameter estimates and effect size estimates:

```{r}
summary(quality_ES)
```


# References

Chen, M., Pustejovsky, J. E., Klingbeil, D. A., & Van Norman, E. R. (2023). Between-case standardized mean differences: Flexible methods for single-case designs. _Journal of School Psychology, 98_, 16-38. https://doi.org/10.1016/j.jsp.2023.02.002

Gilmour, A. R., Thompson, R., & Cullis, B. R. (1995). Average information REML: An efficient algorithm for variance parameter estimation in linear mixed models. _Biometrics, 51_(4), 1440–1450. https://doi.org/10.2307/2533274

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012). A standardized mean difference effect size for single case designs. _Research Synthesis Methods, 3_(3), 224-239. https://doi.org/10.1002/jrsm.1052

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). A standardized mean difference effect size for multiple baseline designs across individuals. _Research Synthesis Methods, 4_(4), 324-341. https://doi.org/10.1002/jrsm.1086

Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). Effects of response cards on disruptive behavior and academic responding during math lessons by fourth-grade urban students. _Journal of Positive Behavior Interventions, 8_(2), 88-99. https://doi.org/10.1177/10983007060080020701

Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). Training parents to use the natural language paradigm to increase their autistic children’s speech. _Journal of Applied Behavior Analysis, 21_(4), 391–400. https://doi.org/10.1901/jaba.1988.21-391

Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). Design-comparable effect sizes in multiple baseline designs: A general modeling framework. _Journal of Educational and Behavioral Statistics, 39_(4), 211-227. https://doi.org/10.3102/1076998614547577

Saddler, B., Behforooz, B., & Asaro, K. (2008). The effects of sentence-combining instruction on the writing of fourth-grade students with writing difficulties. _The Journal of Special Education, 42_(2), 79–90. https://doi.org/10.1177/0022466907310371

set.seed(1)
#------------------------------------
# Design parameters
#------------------------------------
iterations <- 20000
beta <- c(0,1,0,0)
phi <- seq(-7L, 7L, 2) / 10L
rho <- seq(0.0, 0.8, 0.2)
tau1_ratio <- c(0.1, 0.5)
tau_corr <- 0
m <- 3:6
n <- c(8, 16)

print(lengths <- c(length(phi), length(rho), length(tau1_ratio), length(tau_corr), length(m), length(n)))
print(combos <- prod(lengths))
parms <- expand.grid(phi = phi, rho = rho, tau1_ratio = tau1_ratio, tau_corr = tau_corr, m = m, n=n)[sample(combos),]
head(parms)


#--------------------------------------------------------
# run simulations in parallel on Windows via SNOW
#--------------------------------------------------------

library(plyr)
library(snow)
library(foreach)
library(iterators)
library(doSNOW)
library(rlecuyer)

cluster <- makeCluster(parallel::detectCores(), type = "SOCK")
registerDoSNOW(cluster)

# set up parallel random number generator
clusterSetupRNGstream(cluster, 20131003)

# execute simulations
system.time(MB2_array <- maply(parms, .fun = simulate_MB2, 
                                iterations = iterations, beta = beta, 
                                .drop=FALSE, .parallel=TRUE,
                                .paropts = list(.packages="scdhlm")))
stopCluster(cluster)


##------------------------------------------------
## reshape and save results
##------------------------------------------------
library(reshape)

names(dimnames(MB2_array))[7:8] <- c("stat","moment")
MB2results <- cast(melt(MB2_array), ... ~ moment)
attr(MB2results, "iterations") <- iterations
attr(MB2results, "beta") <- beta
save(MB2results, file="data/MB2results.RData", compress="xz")

# set up parallel facilities
library(plyr)
library(snow)
library(foreach)
library(iterators)
library(doSNOW)
library(rlecuyer)
parallel <- TRUE


library(nlme)
library(scdhlm)
library(reshape)
library(ggplot2)
library(boot)

##-----------------------------------------------------------------
## Read in data
##-----------------------------------------------------------------

data(Schutte)

all_weeks <- with(Schutte, expand.grid(week=unique(week),case=unique(case)))
Schutte <- merge(Schutte, all_weeks, all=TRUE)
Schutte$treatment[is.na(Schutte$treatment)] <- "treatment"

# create time-by-trt interaction
Schutte$trt.week <- with(Schutte, unlist(tapply((treatment=="treatment") * week, 
                                                list(treatment,case), 
                                                function(x) x - min(x))) + (treatment=="treatment"))

# center at week 9
Center <- 9
Schutte$week <- Schutte$week - Center

# exclude case 4
Schutte_all <- subset(Schutte, case != 4)
Schutte <- subset(Schutte, !is.na(fatigue) & case != 4)


##-----------------------------------------------------------------
## Model 3: random intercepts, fixed trends
##-----------------------------------------------------------------

hlm1 <- lme(fixed = fatigue ~ week + treatment + trt.week, 
            random = ~ 1 | case, 
            correlation = corAR1(0, ~ week | case),
            data = Schutte,
            method = "REML")
summary(hlm1)
Schutte_all$hlm1 <- predict(hlm1, newdata = Schutte_all)

Schutte_g1 <- g_REML(m_fit = hlm1, p_const = c(0,0,1,7), r_const = c(1,0,1))
summary(Schutte_g1)


##-----------------------------------------------------------------
## Model 4: random intercepts, random baseline trends ##
##-----------------------------------------------------------------

hlm2 <- update(hlm1, random = ~ week | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm2)
Schutte_all$hlm2 <- predict(hlm2, newdata = Schutte_all)

Schutte_g2 <- g_REML(m_fit = hlm2, p_const = c(0,0,1,7), r_const = c(1,0,1,0,0))
summary(Schutte_g2)

anova(hlm1, hlm2)
mean(pchisq(2 * (hlm2$logLik - hlm1$logLik), 1:2, lower.tail=FALSE))

# Confidence intervals
round(CI_g(Schutte_g2),2)
round(with(Schutte_g2, g_AB + c(-1, 1) * qt(0.975, df = nu) * sqrt(V_g_AB)),2)

boots <- 5000

if (parallel) {
  cluster <- makeCluster(parallel::detectCores() - 1, type = "SOCK")
  registerDoSNOW(cluster)
  clusterSetupRNGstream(cluster, 20131003)
  print(system.time(g2_boots <- simulate(Schutte_g2, nsim = boots, parallel=TRUE)))
  stopCluster(cluster)  
} else {
  print(system.time(g2_boots <- simulate(Schutte_g2, nsim = boots)))
}

plot(density(g2_boots$g_AB), col = "red", main="", bty="n")
lines(density(g2_boots$delta_AB))

mean(g2_boots$delta_AB)
mean(g2_boots$delta_AB < Schutte_g2$delta_AB)
sd(g2_boots$delta_AB)
boot.ci(boot.out = list(R=boots, call = "", sim = "parametric"), 
        type = "perc", t0 = Schutte_g2$delta_AB, t = g2_boots$delta_AB)
round(quantile(g2_boots$delta_AB, probs = c(0.025, 0.975)), 3)

mean(g2_boots$g_AB)
mean(g2_boots$g_AB < as.double(Schutte_g2$g_AB))
sd(g2_boots$g_AB)
boot.ci(boot.out = list(R=boots, call = "", sim = "parametric"), 
        type = "perc", t0 = Schutte_g2$g_AB, t = g2_boots$g_AB)
round(quantile(g2_boots$g_AB, probs = c(0.025, 0.975)), 3)



##--------------------------------------------------------------------------------
## Model 5: random intercepts, random baseline trends, random treatment trends
##--------------------------------------------------------------------------------


hlm3 <- update(hlm2, random = ~ week + trt.week | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm3)
Schutte_all$hlm3 <- predict(hlm3, newdata=Schutte_all)

Schutte_g3 <- g_REML(m_fit = hlm3, p_const = c(0,0,1,7), r_const = c(1,0,1,0,0,0,0,0))
summary(Schutte_g3)

anova(hlm1, hlm2, hlm3)
Dev <- 2 * (hlm3$logLik - hlm2$logLik)
mean(pchisq(Dev, 2:3, lower.tail=FALSE))

# Confidence intervals

round(CI_g(Schutte_g3), 2)
round(with(Schutte_g3, g_AB + c(-1, 1) * qt(0.975, df = nu) * sqrt(V_g_AB)), 2)

boots <- 5000

if (parallel) {
  cluster <- makeCluster(parallel::detectCores() - 1, type = "SOCK")
  registerDoSNOW(cluster)
  clusterSetupRNGstream(cluster, 20131003)
  print(system.time(g3_boots <- simulate(Schutte_g3, nsim = boots, parallel=TRUE)))
  stopCluster(cluster)  
} else {
  print(system.time(g3_boots <- simulate(Schutte_g3, nsim = boots)))
}
table(is.na(g3_boots$delta_AB))
g3_boots <- subset(g3_boots, !is.na(delta_AB))

plot(density(g3_boots$g_AB), col = "red", main="", bty="n")
lines(density(g3_boots$delta_AB))

mean(g3_boots$delta_AB)
mean(g3_boots$delta_AB < Schutte_g3$delta_AB)
sd(g3_boots$delta_AB)
boot.ci(boot.out = list(R=dim(g3_boots)[1], call = "", sim = "parametric"), 
        type = "perc", t0 = Schutte_g3$delta_AB, t = g3_boots$delta_AB)
round(quantile(g3_boots$delta_AB, probs = c(0.025, 0.975)), 3)

mean(g3_boots$g_AB)
mean(g3_boots$g_AB < as.double(Schutte_g3$g_AB))
sd(g3_boots$g_AB)
boot.ci(boot.out = list(R=dim(g3_boots)[1], call = "", sim = "parametric"), 
        type = "perc", t0 = Schutte_g3$g_AB, t = g3_boots$g_AB)
round(quantile(g3_boots$g_AB, probs = c(0.025, 0.975)), 3)


## look at individual effects at week 9
individual.effects <- fixed.effects(hlm3)["treatmenttreatment"] + 7 * fixed.effects(hlm3)["trt.week"] + 7 * random.effects(hlm3)["trt.week"]
summary(individual.effects)
individual.effects

fixed.effects(hlm3)["trt.week"] + random.effects(hlm3)["trt.week"]

##--------------------------------------------------------------------------------
## plot data with BLUP estimated trends
##--------------------------------------------------------------------------------

Schutte_all$id <- factor(Schutte_all$case, labels = paste("Case",LETTERS[1:12]))
levels(Schutte_all$treatment) <- c("Baseline","Treatment")

change <- data.frame(id=levels(Schutte_all$id),
                     phase.change = with(subset(Schutte_all, treatment== "Treatment"), 
                                         tapply(week, id, min)) - 0.5)


Schutte.pred <- melt.data.frame(
  subset(Schutte_all, trt.week <=7, select=c("id","week","treatment","hlm1","hlm2","hlm3")),
  id.vars=c("id","week","treatment"))
Schutte.pred$variable <- factor(Schutte.pred$variable, labels=paste0("MB",3:5))
Schutte_all$variable <- "MB3"

ggplot(Schutte.pred, aes(week, value, linetype=variable)) + 
  geom_line() + 
  facet_wrap(~ id, ncol = 3) + 
  labs(color="Phase",linetype="Model", shape="Phase", y ="Fatigue", x="Week") + 
  geom_point(data = subset(Schutte_all, !is.na(fatigue)),
             aes(week, fatigue, shape=treatment, color=treatment)) +
  geom_vline(data = change, aes(xintercept=phase.change)) +
  theme_bw() 

# Load data
library(janitor)

dat <- 
  read.table("{user_path}", header = {user_header}, sep = "{user_sep}", 
             quote = '{user_quote}', stringsAsFactors = FALSE) %>% # Modify the path to the full location of your file
  clean_names(case = "parsed")

# Calculate effect size with g_mlm()
A <- {user_A}
B <- {user_B}
p_const <- c(rep(0L, length({user_FE_base})), (B - A)^as.integer({user_FE_trt})) 
p_const <- {user_pconstant}
r_const <- c({user_rconst_base2}, {user_rconst_trt2}, 
             {user_rconst_base}, {user_rconst_trt}, 
             {user_rconst_cor}, {user_rconst_var}, 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors
r_const <- {user_rconstant}

ES_RML <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
summary(ES_RML)

library(shiny)
library(rclipboard)

source("mappings.R", local = TRUE)

numericInput_inline <- function (inputId, label, value, min = NA, max = NA, step = NA, width = NULL) {
  
  labelTag <- tags$label(label, `for` = inputId)
      
  inputTag <- tags$input(id = inputId, type = "number", class = "form-control", 
                         value = format(value, scientific = FALSE, digits = 15))
  
  if (!is.na(min)) inputTag$attribs$min = min
  if (!is.na(max)) inputTag$attribs$max = max
  if (!is.na(step)) inputTag$attribs$step = step
  
  if (!is.null(width)) {
    labelTag$attribs$width = width[1]
    inputTag$attribs$display = "inline-block"
    inputTag$attribs$width = width[2]
  } 
  
  div(class = "form-group shiny-input-container", labelTag, inputTag)
  
}

ui <- 
  shinyUI(fluidPage(
   tags$head(tags$style(HTML("div#inline label { width: 50%; } div#inline input { display: inline-block; width: 25%;}"))),
   titlePanel("Between-case standardized mean difference estimator"),
   tabsetPanel(id = "scdhlm_calculator", type = "tabs",
        
        #--------------------
        # Load the data
        #--------------------
        tabPanel("scdhlm",
                 navlistPanel(widths = c(3,9),
                              tabPanel("About", includeMarkdown("markdown/scdhlm.md")),
                              tabPanel("Accessing scdhlm", includeMarkdown("markdown/Accessing_scdhlm.md")),
                              tabPanel("Demo videos", includeMarkdown("markdown/demos.md")),
                              tabPanel("References", includeMarkdown("markdown/references.md")),
                              tabPanel("Example data", includeMarkdown("markdown/example-data.md"))
                 )
        ),
        tabPanel("Load",
           br(),
             fluidRow(
               column(4,
                      radioButtons('dat_type', 
                                   'What data do you want to use?',
                                   c("Use an example" = "example",
                                     "Upload data from a .csv or .txt file" = "dat",
                                     "Upload data from a .xlsx file" = "xlsx")),
                      conditionalPanel(
                        condition = "input.dat_type == 'example'",
                        selectInput("example", label = "Choose an example", 
                                    choices = exampleChoices)
                      ),
                      conditionalPanel(
                        condition = "input.dat_type == 'dat'",
                        fileInput('dat', 'Upload a .csv or .txt file', accept=c('text/csv', 'text/comma-separated-values,text/plain', '.csv', '.txt')),
                        checkboxInput('header', 'File has a header?', TRUE),
                        radioButtons('sep', 'Data seperator', c(Commas=',', Semicolons=';', Tabs='\t', Spaces=' ')),
                        radioButtons('quote', 'Include quotes?', c('No'='', 'Double Quotes'='"', 'Single Quotes'="'"))
                      ),
                      conditionalPanel(
                        condition = "input.dat_type == 'xlsx'",
                        fileInput('xlsx', 'Upload a .xlsx file', accept = c('.xlsx')),
                        checkboxInput('col_names', 'File has a header?', TRUE),
                        selectInput("inSelect", "Select a sheet", "")
                      ),
                    ),
               column(8,
                      tableOutput("contents"),
                      conditionalPanel(
                        condition = "output.fileUploaded & input.dat_type == 'dat' || output.fileUploaded & input.dat_type == 'xlsx' || input.dat_type == 'loaded'",
                        selectInput("design", label = "1. Please specify the study design.",
                                    choices = design_names),
                        strong("2. Please select the variable containing each type of information."),
                        column(12, br()),
                        uiOutput("variableMapping"),
                        uiOutput("sessionIssues1"),
                        uiOutput("sessionIssues2"),
                        uiOutput("outcomeMapping"),
                        uiOutput("outcomeIssue"),
                        strong("3. Please specify the baseline and treatment levels."),
                        column(12, br()),
                        uiOutput("phaseMapping"), 
                        strong("4. Please select the variables you wish to filter (optional)."),
                        column(12, br()),
                        uiOutput("filtervarMapping")
                      ),
                      
                      uiOutput("filterMapping")
               )
             ),
           fluidRow(br(),br(),br())
        ),
        
        #--------------------
        # Show loaded data
        #--------------------
        tabPanel("Inspect", 
          tabsetPanel(type = "tabs",
            tabPanel("Graph",
              column(12, br()),
              column(12,
                plotOutput("raw_plot", height = "auto")
              )
            ),
            tabPanel("Data",
              column(12, br()),
              tableOutput("datTable")
            )
          )
        ), 
        
        #--------------------
        # Model building
        #--------------------
        tabPanel("Model", 
           br(),
           uiOutput("estMethod"), 
           conditionalPanel(condition = "input.method == 'HPS'",
              plotOutput("HPS_plot", height = "auto")   
           ),
           
           conditionalPanel(condition = "input.method == 'RML'",
              fluidRow(
                column(6,
                       wellPanel(
                         h4("Baseline phase"),
                         uiOutput("modelDegree_baseline"),
                         uiOutput("modelSpec_baseline")
                       )
                ),
                column(6,
                       wellPanel(
                         h4("Treatment phase"),
                         uiOutput("modelDegree_treatment"),
                         uiOutput("modelSpec_treatment")
                       )
                )
              ),
              fluidRow(
                column(12, 
                  htmlOutput("model_spec")
                )
              ),
              fluidRow(
                column(12,
                  textOutput("ES_timing_message")
                )
              ),
              fluidRow(
                column(12, 
                       wellPanel(
                         h4("Session-level error structure assumptions"),
                         fluidRow(
                           column(6,
                                  selectInput("corStruct",
                                               label = "Correlation structure of session-level errors",
                                               choices = c("Auto-regressive (AR1)" = "AR(1)", "Moving average (MA1)" = "MA(1)", "Independent" = "IID"),
                                               selected = "AR(1)")
                           ),
                           column(6,
                                  selectInput("varStruct",
                                               label = "Variance of session-level errors",
                                               choices = c("Constant variance" = "hom",
                                                           "Variance differs by phase" = "het"),
                                               selected = "hom")
                           )                           
                         )
                       ))
              ),
              tabsetPanel(type = "tabs",
                tabPanel("Graph",
                  column(12, br(),
                    plotOutput("RML_plot", height = "auto")
                  )
                ),
                tabPanel("Model estimates",
                         column(12, br()),
                         conditionalPanel(condition = "input.degree_base != 0",
                                          uiOutput("model_centering")),
                         fluidRow(
                           column(12, h4("Model fit"), tableOutput("model_sample_size")),
                           column(12, tableOutput("model_fit_fixed")),
                           column(12, tableOutput("model_fit_random")),
                           column(12, tableOutput("model_fit_corr")),
                           column(12, tableOutput("model_fit_var")),
                           column(12, tableOutput("model_info")),
                           column(12, h4("Convergence"), tableOutput("model_fit_convg"))
                         )
                )
              )
           )
        ), 
        
        #------------------------------
        # Effect size estimation
        #------------------------------
        
        tabPanel("Effect size", 
          br(),
          uiOutput("ES_timing"),
          h4("Effect size estimates and auxilliary information"),
          div(id = "inline", 
              numericInput("coverage","CI coverage level (%)", value = 95, min = 0, max = 100, step = 0.5)
          ),
          tableOutput("effect_size_report"),
          downloadButton('download_ES', 'Download')
        ),
        
        
        tabPanel("Syntax for R",
                 rclipboardSetup(),
                 uiOutput("clip"),
                 verbatimTextOutput("syntax")
        )
   )
))

# Calculate effect size
res <- effect_size_ABk(outcome = {user_outcome}, 
                       treatment = trt, 
                       id = {user_case}, 
                       phase = {user_phase_pair}, 
                       time = {user_session},
                       data = dat)
res

# rm(list=ls())
# source("inst/shiny-examples/scdhlm/mappings.R")
# source("inst/shiny-examples/scdhlm/graphing-functions.R")
# source("inst/shiny-examples/scdhlm/helper-functions.R")

write_formula <- function(powers, var_names) {
  var_name_1 <- if (var_names[1] == "NULL") NULL else var_names[1]
  if (is.null(powers)) {
    var_name_1
  } else {
    paste(c(if (0 %in% powers) var_names[2] else var_name_1, 
            paste0("I(",var_names[3],"^",powers,")")[powers != 0]), 
          collapse = " + ")
  }
}

lme_fit_MB <- function(design, dat, FE_base, RE_base, RE_base_2, FE_trt, RE_trt, RE_trt_2, 
                       corStruct = "AR(1)", varStruct = "hom", center = 0, phi_init = 0.01) {
  
  require(nlme)
  
  # sort the data
  if (design == "RMBB") {
    dat <- dat[order(dat$case, dat$series, dat$session),]
  } else if (design == "CMB") {
    dat <- dat[order(dat$cluster, dat$case, dat$session),]
  } else {
    dat <- dat[order(dat$case, dat$session),]
  }
  
  dat$session <- dat$session - center
  
  session_FE <- write_formula(FE_base, c("0","1","session"))
  trt_FE <- write_formula(FE_trt, c("NULL", "trt", "session_trt"))
  fixed <- as.formula(paste("outcome ~", paste(c(session_FE, trt_FE), collapse = " + ")))
  
  session_RE <- write_formula(RE_base, c("0","1","session"))
  trt_RE <- write_formula(RE_trt, c("NULL","trt","session_trt"))
  if (design %in% c("MBP", "TR")) {
    random <- as.formula(paste(" ~ ",paste(c(session_RE, trt_RE), collapse = " + "), "| case"))
  } else {
    session_RE_2 <- write_formula(RE_base_2, c("0","1","session"))
    trt_RE_2 <- write_formula(RE_trt_2, c("NULL","trt","session_trt"))
    if (design == "RMBB") {
      random <- eval(parse(text = paste0("list(case = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                                         ", series = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")))
    } else {
      random <- eval(parse(text = paste0("list(cluster = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                                         ", case = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")))
    }
    
  }
  
  nesting_str <- switch(design, 
                        MBP = "case",
                        RMBB = "case/series",
                        CMB = "cluster/case")
  
  cor_struct <- switch(corStruct,
                       `MA(1)` = eval(parse(text = paste0("corARMA(0, ~ session | ",nesting_str,", p = 0, q = 1)"))),
                       `AR(1)` = eval(parse(text = paste0("corAR1(", phi_init, ", ~ session | ", nesting_str, ")"))),
                       `IID` = NULL)

  if (varStruct == "het") {
    var_struct <- eval(parse(text = "varIdent(form = ~ 1 | phase)"))
  } else {
    var_struct <- NULL
  }
  
  W <- TRUE
  E <- NULL
  RML_fit <- withCallingHandlers(
    tryCatch(lme(fixed = fixed, random = random,
                 correlation = cor_struct,
                 weights = var_struct,
                 data = dat, 
                 control = lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE)),
             error = function(e) E <<- e),
    warning = function(w) W <<- w)
  
  RML_fit$call$fixed <- fixed
  RML_fit$call$random <- random
  
  list(fixed = fixed,
       random = random,
       fit = RML_fit,
       converged = if (is.null(E)) W else E)
}

lme_fit_TR <- function(dat, FE_base, RE_base, FE_trt, RE_trt, corStruct = "AR(1)", varStruct = "hom", phi_init = 0.01, ...) {
  require(nlme)
  
  dat <- dat[order(dat$case, dat$session),]
  
  session_FE <- if (is.null(FE_base) | !(0 %in% FE_base)) "0" else "1"
  trt_FE <- if (is.null(FE_trt) | !(0 %in% FE_trt)) NULL else "trt"
  fixed <- as.formula(paste("outcome ~", paste(c(session_FE, trt_FE), collapse = " + ")))
  
  session_RE <- if (is.null(RE_base) | !(0 %in% RE_base)) "0" else "1"
  trt_RE <- if (is.null(RE_trt) | !(0 %in% RE_trt)) NULL else "trt"
  random <- as.formula(paste(" ~",paste(c(session_RE, trt_RE), collapse = " + "), "| case"))
  
  cor_struct <- switch(corStruct,
                       `MA(1)` = eval(parse(text = paste0("corARMA(0, ~ session | case, p = 0, q = 1)"))),
                       `AR(1)` = eval(parse(text = paste0("corAR1(", phi_init, ", ~ session | case)"))),
                       `IID` = NULL)
  
  if (varStruct == "het") {
    var_struct <- eval(parse(text = "varIdent(form = ~ 1 | phase)"))
  } else {
    var_struct <- NULL
  }
  
  W <- TRUE
  E <- NULL
  RML_fit <- withCallingHandlers(
    tryCatch(lme(fixed = fixed, random = random,
                 correlation = cor_struct,
                 weights = var_struct,
                 data = dat, 
                 control = lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE)),
             error = function(e) E <<- e),
    warning = function(w) W <<- w)
  
  RML_fit$call$fixed <- fixed
  RML_fit$call$random <- random
  
  list(fixed = fixed,
       random = random,
       fit = RML_fit,
       converged = if (is.null(E)) W else E)
  
}


#---------------------------------------------------------------
# calculate effect sizes
#---------------------------------------------------------------

effect_size_RML <- function(design, dat, FE_base, RE_base, RE_base_2, FE_trt, RE_trt, RE_trt_2, 
                            corStruct, varStruct, A, B, center = 0, phi_init = 0.01) {
  
  fit_function <- switch(design, 
                         MBP = "lme_fit_MB", 
                         RMBB = "lme_fit_MB", 
                         CMB = "lme_fit_MB", 
                         TR = "lme_fit_TR")
  
  m_fit <- do.call(fit_function, 
                   args = list(design = design, dat = dat, 
                               FE_base = FE_base, RE_base = RE_base, RE_base_2 = RE_base_2,
                               FE_trt = FE_trt, RE_trt = RE_trt, RE_trt_2 = RE_trt_2,
                               corStruct = corStruct, varStruct = varStruct,
                               center = center, phi_init = phi_init))
  fixed <- m_fit$fixed
  random <- m_fit$random
  mod <- m_fit$fit
  mod$call$fixed <- fixed
  mod$call$random <- random
  p_const <- c(rep(0L, length(FE_base)), (B - A)^as.integer(FE_trt))
  
  r_dim <- length(RE_base) + length(RE_trt)
  r_const_dim <- r_dim * (r_dim + 1) / 2
  bc_vec <- (B - center)^as.integer(RE_base)
  bc_mat <- 2 * tcrossprod(bc_vec) - diag(bc_vec^2)
  r_const_base <- bc_mat[upper.tri(bc_mat, diag = TRUE)]
  r_const_trt <- rep(0L, r_const_dim - length(r_const_base))
  
  if (design %in% c("MBP", "TR")) {
    r_const <- c(r_const_base,
                 r_const_trt,
                 rep(0L, length(mod$modelStruct$corStruct)),
                 rep(0L, length(mod$modelStruct$varStruct)),
                 1L)
  } else {
    
    r_dim2 <- length(RE_base_2) + length(RE_trt_2)
    r_const_dim2 <- r_dim2 * (r_dim2 + 1) / 2
    bc_vec2 <- (B - center)^as.integer(RE_base_2)
    bc_mat2 <- 2 * tcrossprod(bc_vec2) - diag(bc_vec2^2)
    r_const_base2 <- bc_mat2[upper.tri(bc_mat2, diag = TRUE)]
    r_const_trt2 <- rep(0L, r_const_dim2 - length(r_const_base2))
    
    r_const <- c(r_const_base2,
                 r_const_trt2,
                 r_const_base,
                 r_const_trt,
                 rep(0L, length(mod$modelStruct$corStruct)),
                 rep(0L, length(mod$modelStruct$varStruct)),
                 1L)
  }
  
  g_mlm(mod = mod, p_const = p_const, r_const = r_const, infotype = "expected")
  
}

set.seed(1)
#------------------------------------
# Design parameters
#------------------------------------
iterations <- 20000
beta <- c(0,1,0,0)
phi <- seq(-7L, 7L, 2) / 10L
rho <- seq(0.0, 0.8, 0.2)
tau2_ratio <- c(0.1, 0.5)
tau_corr <- 0
m <- c(3,4,5,6,9,12)
n <- c(8, 16)

print(lengths <- c(length(phi), length(rho), length(tau2_ratio), length(tau_corr), length(m), length(n)))
print(combos <- prod(lengths))
parms <- expand.grid(phi = phi, rho = rho, tau2_ratio = tau2_ratio, tau_corr = tau_corr, m = m, n=n)[sample(combos),]
head(parms)


#--------------------------------------------------------
# run simulations in parallel on Windows via SNOW
#--------------------------------------------------------

library(plyr)
library(snow)
library(foreach)
library(iterators)
library(doSNOW)
library(rlecuyer)

cluster <- makeCluster(parallel::detectCores(), type = "SOCK")
registerDoSNOW(cluster)

# set up parallel random number generator
clusterSetupRNGstream(cluster, 20131003)

# execute simulations
system.time(MB4_array <- maply(parms, .fun = simulate_MB4, 
                                iterations = iterations, beta = beta, p_const = p_const,
                                .drop=FALSE, .parallel=TRUE,
                                .paropts = list(.packages="scdhlm")))
stopCluster(cluster)


##------------------------------------------------
## reshape and save results
##------------------------------------------------

library(reshape)

names(dimnames(MB4_array))[7:8] <- c("stat","moment")
MB4results <- cast(melt(MB4_array), ... ~ moment)
attr(MB4results, "iterations") <- iterations
attr(MB4results, "beta") <- beta
attr(MB4results, "p_const") <- p_const
save(MB4results, file="data/MB4results.RData", compress="xz")

# Calculate effect size with g_mlm()
p_const <- {user_pconstant}
r_const <- c({user_rconst_base}, {user_rconst_trt}, 
             {user_rconst_cor}, {user_rconst_var}, 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors
r_const <- {user_rconstant}

ES_RML <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
summary(ES_RML)

#---------------------------------------------------------------
# parse code chunks with user-specified arguments
#---------------------------------------------------------------

parse_code_chunk <- function(chunk, args) {
  chunk_path <- system.file("shiny-examples/scdhlm/code-chunks", paste0(chunk,".R"), package = "scdhlm")
  raw_code <- readLines(chunk_path)
  code_chunk <- paste(raw_code, collapse = "\n")
  glue::glue_data(.x = args, code_chunk)
}

#---------------------------------------------------------------
# paste an object in server for code chunks
#---------------------------------------------------------------
paste_object <- function(object) {
  paste("c(", paste(object, collapse = ","), ")", sep = "")
}

#---------------------------------------------------------------
# model validation
#---------------------------------------------------------------

validate_specification <- function(design, n_outer_levels, 
                                   FE_base, RE_base, FE_trt, RE_trt, 
                                   RE_base2 = NULL, RE_trt2 = NULL) {
  
  errors <- vector(mode = "character")
  if (!("0" %in% FE_base)) {
    errors <- c(errors, "<font color='red'>Model must include a fixed effect for baseline level.</font>")
  }
  if (!("0" %in% RE_base)) {
    errors <- c(errors, "<font color='red'>Model must include a random effect for baseline level.</font>")
  }
  if (design %in% c("RMBB", "CMB") && !("0" %in% RE_base2)) {
    errors <- c(errors, "<font color='red'>Model must include a random effect for baseline level.</font>")
  }
  if (length(FE_trt)==0) {
    errors <- c(errors, "<font color='red'>Model must include at least one fixed effect for the treatment phase.</font>")
  }

  if (n_outer_levels < 3L) {
    errors <- c(errors, "<font color='red'>Model must include at least three cases. Currently, you have less than three cases.</font>")
  }
  
  if (length(errors)==0) {
    return(NULL)
  } else if (length(errors) == 1) {
    error_string <- paste("<b><font color='red'>Error:</font></b>", errors, "<br/>")
  } else {
    error_string <- paste("<b><font color='red'>Errors:</font></b><br/>", paste(errors, collapse = "<br/>"), "<br/>")
  } 
  
  return(HTML(error_string))
}

#---------------------------------------------------------------
# effect size report table
#---------------------------------------------------------------

summarize_ES <- function(res, filter_vals, 
                         design, method, 
                         FE_base, RE_base, RE_base_2, 
                         FE_trt, RE_trt, RE_trt_2, 
                         corStruct, varStruct,
                         A, B, coverage = 95L) {
  
  if (method=="RML") {
    ES_summary <- data.frame(
      ES = as.numeric(res$g_AB),
      SE = as.numeric(res$SE_g_AB)
    )
    
    if (design %in% c("RMBB", "CMB")) {
      rho_level2 <- round(with(res, (theta$Tau[[1]][1] + theta$Tau[[2]][1]) / 
                           (theta$Tau[[1]][1] + theta$Tau[[2]][1] + theta$sigma_sq)), 3)
      rho_level3 <- round(with(res, theta$Tau[[1]][1] / 
                           (theta$Tau[[1]][1] + theta$Tau[[2]][1] + theta$sigma_sq)), 3)
      res$rho <- paste0("Level2:", rho_level2, "  Level3:", rho_level3)
    } else {
      res$rho <- with(res, theta$Tau[[1]][1] / (theta$Tau[[1]][1] + theta$sigma_sq))
    }
    
    res$phi <- if (corStruct == "IID") NA_real_ else res$theta$cor_params
    res$var_param <- if (varStruct == "hom") NA_real_ else res$theta$var_params
    
  } else {
    
    ES_summary <- data.frame(
      ES = res$delta_hat,
      SE = sqrt(res$V_delta_hat)
    )
    
  }
  
  CI <- CI_g(res, cover = coverage / 100L)
  
  ES_summary$CI_L <- if (CI[1] < 100 & CI[1] > -100) CI[1] else format(CI[1], scientific = TRUE)
  ES_summary$CI_U <- if (CI[2] < 100 & CI[2] > -100) CI[2] else format(CI[2], scientific = TRUE)
  ES_summary$df <- res$nu
  ES_summary$phi <- res$phi
  ES_summary$var_param <- if (method == "RML") res$var_param else NA_real_
  ES_summary$rho <- res$rho
  ES_summary$design <- names(design_names[which(design_names==design)])
  ES_summary$method <- names(estimation_names[which(estimation_names==method)])
  ES_summary$baseline <- paste0("F:", paste(FE_base, collapse = ""), 
                                " R1:", paste(RE_base, collapse = ""),
                                " R2:", paste(RE_base_2, collapse = ""))
  ES_summary$trt <- paste0("F:", paste(FE_trt, collapse = ""), 
                           " R1:", paste(RE_trt, collapse = ""),
                           " R2:", paste(RE_trt_2, collapse = ""))
  
  if (method=="RML" & design %in% c("MBP", "RMBB", "CMB") & !is.null(A) & !is.null(B)) {
    ES_summary$A <- A
    ES_summary$B <- B
  } else {
    ES_summary$A <- NA
    ES_summary$B <- NA
  }
  
  CI_names <- paste0(coverage, "% CI ", c("(lower)", "(upper)"))
  
  row.names(ES_summary) <- NULL
  names(ES_summary) <- c("BC-SMD estimate","Std. Error", CI_names,
                         "Degrees of freedom","Auto-correlation", "Variance parameter", "Intra-class correlation",
                         "Study design","Estimation method",
                         "Baseline specification", "Treatment specification",
                         "Initial treatment time","Follow-up time")
  
  if (!is.null(filter_vals)) {
    ES_summary <- cbind(ES_summary, filter_vals)
  } else {
    ES_summary <- ES_summary
  }
  
  ES_summary  
}

# Clean data

dat <- dat[,c("{user_parms}")]
names(dat) <- c("cluster","case","phase","session","outcome")

dat <- preprocess_SCD(design = "{user_design}", 
                      case = case, 
                      phase = phase, 
                      session = session, 
                      outcome = outcome, 
                      cluster = cluster,
                      center = {user_model_center},
                      data = dat)

# Load data
library(readxl)
library(janitor)

dat <- 
  read_excel(path = "{user_path}", sheet = "{user_sheet}") %>% # Modify the path to the full location of your file
  clean_names(case = "parsed")

# Calculate effect size
res <- effect_size_MB(outcome = {user_outcome}, 
                      treatment = trt, 
                      id = {user_case}, 
                      time = {user_session},
                      data = dat) 
res

output$RCT_plot <- renderPlot({
  if (studyDesign()=="MB" & input$method=="RML" & "lme" %in% class(model_fit()$fit)) {
    range <- time_range()$range
    A <- input$A_time
    B <- input$B_time
    cases <- levels(datClean()$case)
    sessions <- seq(from = range[1], to = range[2])
    dat_RCT <- data.frame(case = rep(cases, each = length(sessions)),
                          session = sessions,
                          trt = as.integer(sessions > A),
                          session_trt = ifelse(sessions > A, sessions - A - 1, 0))
    dat_RCT$phase <- levels(datClean()$phase)[dat_RCT$trt + 1]
    graph_SCD(dat_RCT, design = studyDesign())
  }
}, height = function() 120 * nlevels(datClean()$case),
width = function() 700)

library(shiny)
library(markdown)
library(ggplot2)
library(scdhlm)
library(readxl)
library(janitor)

source("mappings.R", local = TRUE)
source("helper-functions.R", local = TRUE)
source("lme-fit.R", local = TRUE)

server <- 
  shinyServer(function(input, output, session) {
    
    if (exists("dataset", where = environment(server))) if (!is.null(dataset)) {
      updateRadioButtons(
        session, 
        inputId = 'dat_type', 
        label = 'What data do you want to use?',
        choices = c("Use an example" = "example",
                    "Upload data from a .csv or .txt file" = "dat",
                    "Upload data from a .xlsx file" = "xlsx",
                    "Use the dataset specified at initialization" = "loaded"),
        selected = "loaded"
      )
    }
    
    sheetname <- reactive({
      if (input$dat_type == "xlsx") {
        inFile <- input$xlsx
        if (is.null(inFile)) return(NULL)
        sheetnames <- excel_sheets(inFile$datapath)
      } 
    })
    
    observe({
      sheets <- sheetname()
      updateSelectInput(session, "inSelect", label = "Select a sheet",
                        choices = sheets,
                        selected = sheets[1])
    })
    
    # Read in data
    
    datFile <- reactive({ 

      if (input$dat_type == "dat") {
        
        inFile <- input$dat
        
        if (is.null(inFile)) return(NULL)
        
        read.table(inFile$datapath, header=input$header, 
                   sep=input$sep, quote=input$quote,
                   stringsAsFactors = FALSE) %>% 
          clean_names(case = "parsed")
        
      } else if (input$dat_type == "xlsx") {
        
        inFile <- input$xlsx
        
        if (is.null(inFile) || is.null(input$inSelect) || nchar(input$inSelect) == 0) return(NULL)
        
        readxl::read_xlsx(inFile$datapath, col_names = input$col_names,
                          sheet = input$inSelect) %>% 
          clean_names(case = "parsed") %>%
          as.data.frame()
        
      } else if (input$dat_type == "loaded") {
        
        dataset %>% 
          clean_names(case = "parsed")
        
      } 
    })
    
    # Check that file is uploaded
    
    output$fileUploaded <- reactive({
      return(!is.null(datFile()))
    })
    
    outputOptions(output, 'fileUploaded', suspendWhenHidden=FALSE)
    
    # Study design with defaults 
    
    studyDesign <- reactive({
      
      if (input$dat_type == "example") {
        exampleMapping[[input$example]]$design
      } else {
        input$design
      }
      
    })
    
    # Variable mapping interface
    
    output$variableMapping <- renderUI({
      var_names <- names(datFile())
      n_var <- length(var_names)
      if (studyDesign() %in% c("MBP", "TR")) {
        list(
          selectInput("caseID", label = "Case identifier", choices = var_names, selected = var_names[n_var - 3]),
          selectInput("phaseID", label = "Phase identifier", choices = var_names, selected = var_names[n_var - 2]),
          selectInput("session", label = "Session number", choices = var_names, selected = var_names[n_var - 1])
        )
      } else if (studyDesign() == "RMBB") {
        list(
          selectInput("caseID", label = "Case identifier", choices = var_names, selected = var_names[n_var - 4]),
          selectInput("seriesID", label = "Series identifier", choices = var_names, selected = var_names[n_var - 3]),
          selectInput("phaseID", label = "Phase identifier", choices = var_names, selected = var_names[n_var - 2]),
          selectInput("session", label = "Session number", choices = var_names, selected = var_names[n_var - 1])
        )
      } else if (studyDesign() == "CMB") {
        list(
          selectInput("clusterID", label = "Cluster identifier", choices = var_names, selected = var_names[n_var - 4]),
          selectInput("caseID", label = "Case identifier", choices = var_names, selected = var_names[n_var - 3]),
          selectInput("phaseID", label = "Phase identifier", choices = var_names, selected = var_names[n_var - 2]),
          selectInput("session", label = "Session number", choices = var_names, selected = var_names[n_var - 1])
        )
      }
      
    })
    
    variablesLoaded <- reactive({
      if (is.null(input$caseID) | is.null(input$phaseID) | is.null(input$session) | is.null(input$outcome)) {
        FALSE
      } else {
        (nchar(input$caseID) > 0) & (nchar(input$phaseID) > 0) & (nchar(input$session) > 0) & (nchar(input$outcome) > 0)
      }
    }) 
    
    # Check Session variable
    
    output$sessionIssues1 <- renderUI({
      
      if (!variablesLoaded()) return(NULL)
      
      sessions <- datFile()[,input$session]
      
      if (!is.numeric(sessions) && !is.integer(sessions)) {
        list(
          strong(style="color:red", "Session variable contains non-numeric data."),
          br("")
        )
        
      } else if (!isTRUE(all.equal(sessions, round(sessions)))) {
        checkboxInput('round_session', 'Round Session variable to nearest integer?', TRUE)
      }
    })
    
    output$sessionIssues2 <- renderUI({
      
      if (!variablesLoaded()) return(NULL)
      
      sessions <- datFile()[,input$session]
      case_var <- datFile()[,input$caseID,drop=FALSE]
      filter_vars <- datFile()[,input$filters,drop=FALSE]
      split_vars <- cbind(case_var, filter_vars)
      if (studyDesign() == "RMBB") {
        series_var <- datFile()[, input$seriesID, drop=FALSE]
        split_vars <- cbind(case_var, series_var, filter_vars)
      } else if (studyDesign() == "CMB") {
        cluster_var <- datFile()[, input$clusterID, drop=FALSE]
        split_vars <- cbind(cluster_var, case_var, filter_vars)
      }
      
      if (length(split_vars) > 0 & is.numeric(sessions)) {
        
        if (!is.null(input$round_session)) if (input$round_session) sessions <- round(sessions)
        
        unique_sessions <- tapply(sessions, split_vars, 
                                  function(x) isTRUE(all.equal(x, unique(x))))
        
        if (!all(unique_sessions, na.rm = TRUE)) {
          list(
            strong(style="color:red", "Session variable contains repeated values. Please ensure that each data point has a unique value within each case."),
            br("") 
          )
        }
      }
      
    })
    
    output$outcomeMapping <- renderUI({
      var_names <- names(datFile())
      n_var <- length(var_names)
      selectInput("outcome", label = "Outcome variable", choices = var_names, selected = var_names[n_var])
    })
    
    # Check Outcome variable
    
    output$outcomeIssue <- renderUI({
      
      if (!variablesLoaded()) return(NULL)
      
      outcomes <- datFile()[,input$outcome]
      
      if (!is.numeric(outcomes) && !is.integer(outcomes)) {
        list(
          strong(style="color:red", "Outcome variable contains non-numeric data. Please use a numeric outcome variable."),
          br("")
        )
      } 
      
    })
    
    # Treatment label mapping interface
    
    output$phaseMapping <- renderUI({
      
      if (!variablesLoaded()) return(NULL)
      
      phases <- levels(as.factor(datFile()[,input$phaseID]))
      list(
        selectInput("baseline", label = "Baseline level", choices = phases, selected = phases[1]),
        selectInput("treatment", label = "Treatment level", choices = phases, selected = phases[2])
      )
      
    })
    
    # Filtering interface
    
    output$filtervarMapping <- renderUI({
      
      if (input$dat_type == "example") {
      } else {
        var_names <- names(datFile())
        n_var <- length(var_names)
        list(
          selectizeInput("filters", label = "Filtering variables", choices = var_names, selected = NULL, multiple = TRUE)
        )
      }
      
    })
    
    output$filterMapping <- renderUI({
      
      if (input$dat_type == "example") {
        if (is.null(input$example)) return(NULL)
        example_parms <- exampleMapping[[input$example]]
        filter_vars <- example_parms$filters  
        filter_vals <- lapply(filter_vars, function(x) example_parms[[paste0("filter_", x)]])
        names(filter_vals) <- filter_vars
        header <- strong("Please select the variables you wish to filter.")
      } else {
        filter_vars <- input$filters  
        filter_vals <- lapply(filter_vars, function(x) levels(as.factor(datFile()[,x])))
        names(filter_vals) <- filter_vars
        header <- strong("Values for the filtering variables.")
      }
      
      filter_selects <- lapply(filter_vars, function(x) 
        selectizeInput(paste0("filter_",x), label = x, choices = filter_vals[[x]], 
                       selected = filter_vals[[x]][1], multiple = TRUE))
      
      if (length(filter_vars) > 0) {
        filter_selects <- list(header, column(12, br()), filter_selects)
      }
      
      filter_selects
    
    })
    
    # Clean the data
    
    datClean <- reactive({
      
      if (input$dat_type == "example") {
        data(list = input$example)
        dat <- get(input$example)
        example_parms <- exampleMapping[[input$example]]
        filter_vars <- example_parms$filters  
        if (!is.null(filter_vars)) {
          subset_vals <- sapply(filter_vars, function(x) levels(dat[[x]])[dat[[x]]] %in% input[[paste0("filter_",x)]])
          dat <- dat[apply(subset_vals, 1, all),]
        } 
        dat <- dat[,example_parms$vars]
        if (studyDesign() %in% c("MBP", "TR")) {
          names(dat) <- c("case","session","phase","outcome")
          cluster <- series <- NULL
        } else if (studyDesign() == "RMBB"){
          names(dat) <- c("studyID","case", "series", "outcome", "session", "phase", "trt_time", "time_c")
          cluster <- NULL
        } else if (studyDesign() == "CMB") {
          names(dat) <- c("studyID","school", "cluster", "case", "phase", "session", "session_trt", "outcome", "session_c")
          series <- NULL
        }
        
        dat <- preprocess_SCD(design = studyDesign(), 
                              cluster = cluster, 
                              case = case, series = series, 
                              phase = phase, 
                              session = session, outcome = outcome, 
                              data = dat)
        
      } else {
        
        case <- datFile()[,input$caseID]
        phase <- datFile()[,input$phaseID]
        session <- as.numeric(datFile()[,input$session])
        outcome <- as.numeric(datFile()[,input$outcome])
        cluster <- series <- NULL
        dat <- data.frame(case = case, phase = phase, session = session, outcome = outcome)
        
        if (studyDesign() == "RMBB") {
          dat$series <- datFile()[,input$seriesID]
          dat <- dat[order(dat$case, dat$series, dat$session),]
        } else if (studyDesign() == "CMB") {
          dat$cluster <- datFile()[,input$clusterID]
          dat <- dat[order(dat$cluster, dat$case, dat$session),]
        } else {
          dat <- dat[order(dat$case, dat$session),]
        }
        
        if (!is.null(input$filters)) {
          subset_vals <- sapply(input$filters, function(x) datFile()[[x]] %in% input[[paste0("filter_",x)]])
          dat <- dat[apply(subset_vals, 1, all),]
        } 
        
        design <- studyDesign()
        round_session <- if (!is.null(input$round_session)) TRUE else FALSE
        treatment_name <- input$treatment
        
        dat <- preprocess_SCD(design = design, 
                              cluster = cluster, case = case, series = series,
                              phase = phase, session = session, outcome = outcome, 
                              round_session = round_session, treatment_name = treatment_name,
                              data = dat)

      }
      
      # if (studyDesign() == "MBP") {
      #   names(dat)[6] <- "session_trt"
      # } else if (studyDesign() == "TR") {
      #   names(dat)[6] <- "phase_pair"
      # } else if (studyDesign() %in% c("RMBB", "CMB")) {
      #   names(dat)[7] <- "session_trt"
      # } 
      
      return(dat)
      
    })

    output$datTable <- renderTable(datClean())
    
    # Estimation method
    
    output$estMethod <- renderUI({
      estimation_choices <- if (studyDesign() %in% c("MBP", "TR")) {
        estimation_names 
      } else {
        c("Restricted Maximum Likelihood" = "RML")
      }
      
      selectInput("method", label = "Estimation method",
                  choices = estimation_choices, 
                  selected = "RML")
    })
    
    # Centering and timing sliders for RML estimation of MBD
    
    time_range <- reactive({
      
      cluster <- if (studyDesign() == "CMB") substitute(cluster) else NULL
      series <- if (studyDesign() == "RMBB") substitute(series) else NULL
      default_times(
        design = studyDesign(),
        case = case, phase = phase, session = session,
        cluster = cluster, series = series,
        treatment_name = NULL, data = datClean()
      )
      
    })
    
    output$model_centering <- renderUI({
      if (studyDesign() %in% c("MBP", "RMBB", "CMB") & input$method=="RML") {
        session_range <- time_range()$range
        sliderInput("model_center", "Center session at", 
                    min=session_range[1], max=session_range[2], 
                    value=time_range()$B, step = 1)
      }
    })
    
    output$ES_timing_message <- renderText({
      if (studyDesign() %in% c("MBP", "RMBB", "CMB") & input$method=="RML" &
          any(input$degree_trt != 0, (input$degree_base != 0 & input$RE_base > 0))) {
        note_txt <- "Note: Options for selecting an initial treatment time and follow-up time can be modified on the next page." 
        HTML(note_txt)
      }
    })
    
    output$ES_timing <- renderUI({
      if (studyDesign() %in% c("MBP", "RMBB", "CMB") & input$method=="RML" & 
          any(input$degree_trt != 0, (input$degree_base != 0 & input$RE_base > 0))) {
        timings <- time_range()
        wellPanel(
          fluidRow(
            column(12,
                   h4("Hypothetical experimental parameters")
            ),
            column(6,
                   sliderInput("A_time", "Initial treatment time",
                               min = timings$range[1], max = timings$range[2],
                               value = timings$A, step = 1)
            ),
            column(6,
                   sliderInput("B_time", "Follow-up time",
                               min = timings$range[1], max = timings$range[2],
                               value = timings$B, step = 1)
            )
          )
        )
      }
    })

    
    # Model degree
    
    output$modelDegree_baseline <- renderUI({
      if (studyDesign() %in% c("MBP", "RMBB", "CMB")) {
        selectInput("degree_base", label = "Type of time trend", choices = time_trends_baseline, width = "40%")  
      }
    })
    
    output$modelDegree_treatment <- renderUI({
      if (studyDesign() %in% c("MBP", "RMBB", "CMB")) {
        selectInput("degree_trt", label = "Type of time trend", choices = time_trends_treatment, width = "40%")  
      }
    })
    
    # Model specification
    
    output$modelSpec_baseline <- renderUI({
      deg_base <- if (is.null(input$degree_base)) 0 else as.numeric(input$degree_base)
      degree_base_list <- 0:deg_base
      names(degree_base_list) <- degree_names_baseline[1:(deg_base + 1)]
      if (studyDesign() %in% c("MBP", "TR")) {
        fluidRow(
          column(6,
                 checkboxGroupInput("FE_base", labs_MBP$fixed, degree_base_list, selected = degree_base_list)
          ),
          column(6,
                 checkboxGroupInput("RE_base", labs_MBP$random, degree_base_list, selected = 0)
          )
        )
      } else if (studyDesign() == "RMBB") {
        fluidRow(
          column(4,
                 checkboxGroupInput("FE_base", labs_RMBB$fixed, degree_base_list, selected = degree_base_list)
          ),
          column(4,
                 checkboxGroupInput("RE_base", labs_RMBB$random_series, degree_base_list, selected = 0)
          ),
          column(4,
                 checkboxGroupInput("RE_base2", labs_RMBB$random_case, degree_base_list, selected = 0)
          )
        )
      } else if (studyDesign() == "CMB") {
        fluidRow(
          column(4,
                 checkboxGroupInput("FE_base", labs_CMB$fixed, degree_base_list, selected = degree_base_list)
          ),
          column(4,
                 checkboxGroupInput("RE_base", labs_CMB$random_case, degree_base_list, selected = 0)
          ),
          column(4,
                 checkboxGroupInput("RE_base2", labs_CMB$random_cluster, degree_base_list, selected = 0)
          )
        )
      }
      
    })
    
    output$modelSpec_treatment <- renderUI({
      deg_trt <- if (is.null(input$degree_trt)) 0 else as.numeric(input$degree_trt)
      degree_trt_list <- 0:deg_trt
      names(degree_trt_list) <- degree_names_treatment[1:(deg_trt + 1)]
      if (studyDesign() %in% c("MBP", "TR")) {
        fluidRow(
          column(6,
                 checkboxGroupInput("FE_trt", labs_MBP$fixed, degree_trt_list, selected = degree_trt_list)
          ),
          column(6,
                 checkboxGroupInput("RE_trt", labs_MBP$random, degree_trt_list, selected = NULL)
          )
        )
      } else if (studyDesign() == "RMBB") {
        fluidRow(
          column(4,
                 checkboxGroupInput("FE_trt", labs_RMBB$fixed, degree_trt_list, selected = degree_trt_list)
          ),
          column(4,
                 checkboxGroupInput("RE_trt", labs_RMBB$random_series, degree_trt_list, selected = NULL)
          ),
          column(4,
                 checkboxGroupInput("RE_trt2", labs_RMBB$random_case, degree_trt_list, selected = NULL)
          )
        )
      } else if (studyDesign() == "CMB") {
        fluidRow(
          column(4,
                 checkboxGroupInput("FE_trt", labs_CMB$fixed, degree_trt_list, selected = degree_trt_list)
          ),
          column(4,
                 checkboxGroupInput("RE_trt", labs_CMB$random_case, degree_trt_list, selected = NULL)
          ),
          column(4,
                 checkboxGroupInput("RE_trt2", labs_CMB$random_cluster, degree_trt_list, selected = NULL)
          )
        )
      }

    })
    
    # Validate model specification
    
    model_validation <- reactive({
      if (studyDesign() %in% c("MBP", "TR")) {
        validate_specification(design = studyDesign(), n_outer_levels = nlevels(datClean()$case),
                               FE_base = input$FE_base, RE_base = input$RE_base, 
                               FE_trt = input$FE_trt, RE_trt = input$RE_trt)
      } else if (studyDesign() == "RMBB") {
        validate_specification(design = studyDesign(), n_outer_levels = nlevels(datClean()$case),
                               FE_base = input$FE_base, RE_base = input$RE_base,  
                               FE_trt = input$FE_trt, RE_trt = input$RE_trt,
                               RE_base2 = input$RE_base2, RE_trt2 = input$RE_trt2)
      } else {
        validate_specification(design = studyDesign(), n_outer_levels = nlevels(datClean()$cluster),
                               FE_base = input$FE_base, RE_base = input$RE_base,  
                               FE_trt = input$FE_trt, RE_trt = input$RE_trt,
                               RE_base2 = input$RE_base2, RE_trt2 = input$RE_trt2)
      }
      
    })
    
    output$model_spec <- renderUI({model_validation()})
    
    # Fit model 
    
    model_fit <- reactive({
      
      center <- if (input$degree_base == 0 || is.null(input$model_center)) 0L else input$model_center
      
      fit_function <- switch(studyDesign(), 
                             MBP = "lme_fit_MB", 
                             RMBB = "lme_fit_MB", 
                             CMB = "lme_fit_MB", 
                             TR = "lme_fit_TR")
      
      if (studyDesign() %in% c("MBP", "TR")) {
        RE_base2 <- RE_trt2 <- NULL
      } else {
        RE_base2 <- input$RE_base2
        RE_trt2 <- input$RE_trt2
      }
      do.call(fit_function,
              args = list(design = studyDesign(), dat = datClean(), 
                          FE_base = input$FE_base, RE_base = input$RE_base, RE_base_2 = RE_base2,
                          FE_trt = input$FE_trt, RE_trt = input$RE_trt, RE_trt_2 = RE_trt2,
                          varStruct = input$varStruct,
                          corStruct = input$corStruct,
                          center = center))
      
    })
    
    
    # Model spec output
    
    output$model_sample_size <- renderTable({
      if (input$method == "RML") {
        if (studyDesign() %in% c("MBP", "TR")) {
          data.frame("Total number of observations" = nobs(model_fit()$fit), 
                     "Total number of groups" = nlevels(summary(model_fit()$fit)$groups$case),
                     check.names = FALSE) 
        } else if (studyDesign() == "CMB") {
          data.frame("Total number of observations" = nobs(model_fit()$fit), 
                     "Total number of cases" = nlevels(summary(model_fit()$fit)$groups$case),
                     "Total number of clusters" = nlevels(summary(model_fit()$fit)$groups$cluster),
                     check.names = FALSE)
        } else if (studyDesign() == "RMBB") {
          data.frame("Total number of observations" = nobs(model_fit()$fit),
                     "Total number of series" = nlevels(summary(model_fit()$fit)$groups$series),
                     "Total number of cases" = nlevels(summary(model_fit()$fit)$groups$case),
                     check.names = FALSE)
        }
      }
    }, 
    caption = "Sample sizes", 
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, include.rownames = FALSE)
    
    output$model_fit_fixed <- renderTable({
      if (input$method == "RML") {
        summary(model_fit()$fit)$tTable
      }
    }, 
    caption = "Fixed effects", 
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, include.rownames = TRUE)
    
    output$model_fit_random <- renderTable({
      if (input$method == "RML") {
        random_table <- data.frame(VarCorr(model_fit()$fit)[])
        rownames <- rownames(VarCorr(model_fit()$fit)[])
        
        # set the rownames
        if (studyDesign() %in% c("CMB", "RMBB")) {
          row_index_lvl3 <- length(input$RE_base2)+length(input$RE_trt2)+1
          rownames_lvl3 <- c("", rownames[2:row_index_lvl3])
          rownames_lvl2 <- c("", rownames[(row_index_lvl3 + 2):(length(rownames))])
          name_lvl3 <- if (studyDesign() == "CMB") "<strong>Cluster-level</strong>" else "<strong>Case-level</strong>"
          name_lvl2 <- if (studyDesign() == "CMB") "<strong>Case-level</strong>" else "<strong>Series-level</strong>"
          random_levels <- c(name_lvl3, rep("", row_index_lvl3-1), name_lvl2, rep("", (length(rownames)-row_index_lvl3-1)))
          random_table[1, 1] <- NA # get rid of pdLogChol
          random_table[(row_index_lvl3 + 1), 1] <- NA
          random_table <- cbind("Level" = random_levels, "Rowname" = c(rownames_lvl3, rownames_lvl2), random_table)
          random_table[, 3:4] <- lapply(random_table[, 3:4], function(x) as.numeric(x))
          names(random_table)[2] <- ""
        } else if (studyDesign() %in% c("MBP", "TR")) {
          random_table <- cbind("Rowname" = rownames, random_table)
          random_table[, 2:3] <- lapply(random_table[, 2:3], function(x) as.numeric(x))
          names(random_table)[1] <- ""
        }
        
        # set the column names
        columnnames <- names(random_table)
        names(random_table) <- gsub("V[0-9]|Var.[0-9]", "", columnnames)
        random_table
        
      }
    },
    caption = "Random effects",
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, na = "", include.rownames = FALSE, sanitize.text.function = function(x){x})
    
    output$model_fit_corr <- renderTable({
      if (input$method=="RML" & input$corStruct != "IID") {
        data.frame(
          "Correlation parameter" = effect_size()$`Auto-correlation`,
          check.names = FALSE
        )
      }
    }, 
    caption = "Correlation structure", 
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, include.rownames = FALSE)
    
    output$model_fit_var <- renderTable({
      if (input$method == "RML" & input$varStruct == "het") {
        data.frame(
          "Baseline" = 1,
          "Treatment" = effect_size()$`Variance parameter`
        )
      }
    }, 
    caption = "Variance structure", 
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, include.rownames = FALSE)
    
    output$model_info <- renderTable({
      if (input$method == "RML") {
        data.frame("AIC" = AIC(model_fit()$fit), 
                   "BIC" = BIC(model_fit()$fit), 
                   "Log likelihood" = as.numeric(model_fit()$fit$logLik),
                   check.names = FALSE) 
      }
    }, 
    caption = "Information criteria", 
    caption.placement = getOption("xtable.caption.placement", "top"),
    digits = 4, include.rownames = FALSE)
    
    output$model_fit_convg <- renderTable({
      if (input$method=="RML") {
        if (isTRUE(model_fit()$converged)) {
          "The model converged."
        } else {
          "The model did not converge."
        }
      }
    }, colnames = FALSE)
    
    
    # Calculate effect sizes
    
    effect_size <- reactive({
      
      if ("lme" %in% class(model_fit()$fit)) {
        
        if (studyDesign() %in% c("MBP", "TR")) {
          RE_base2 <- RE_trt2 <- NULL
        } else {
          RE_base2 <- input$RE_base2
          RE_trt2 <- input$RE_trt2
        }
        
        if (input$method=="RML") {
          center <- if (input$degree_base == 0 || is.null(input$model_center)) 0L else input$model_center
          A <- if (is.null(input$A_time)) 0L else input$A_time
          B <- if (is.null(input$B_time)) 1L else input$B_time
          res <- effect_size_RML(design = studyDesign(), dat = datClean(), 
                                 FE_base = input$FE_base, RE_base = input$RE_base, RE_base_2 = RE_base2,
                                 FE_trt = input$FE_trt, RE_trt = input$RE_trt, RE_trt_2 = RE_trt2,
                                 varStruct = input$varStruct,
                                 corStruct = input$corStruct,
                                 A = A, B = B, center = center)
        } else {
          if (studyDesign()=="MBP") {
            res <- with(datClean(), effect_size_MB(outcome = outcome, treatment = trt, id = case, time = session))
          } else if (studyDesign()=="TR") {
            res <- with(datClean(), effect_size_ABk(outcome = outcome, treatment = trt, id = case, phase = phase_pair, time = session))
          } 
        } 
        
        example_parms <- exampleMapping[[input$example]]
        if (input$dat_type == "example" & !is.null(example_parms$filters)) {
          filter_vars <- example_parms$filters
          filter_vals <- if(length(filter_vars) > 0) lapply(paste0("filter_", filter_vars), 
                                                            function(x) paste0(input[[x]], collapse = ",")) else NULL
          names(filter_vals) <- filter_vars
          filter_vals <- as.data.frame(filter_vals)
        } else if (input$dat_type %in% c("dat", "xlsx") & !is.null(input$filters)) {
          filter_vars <- input$filters
          filter_vals <- if(length(filter_vars) > 0) lapply(paste0("filter_", filter_vars), 
                                                            function(x) paste0(input[[x]], collapse = ",")) else NULL
          names(filter_vals) <- filter_vars
          filter_vals <- as.data.frame(filter_vals)
        } else {
          filter_vars <- NULL
          filter_vals <- NULL
        } 
        
        summarize_ES(res, 
                     filter_vals = filter_vals, 
                     design = studyDesign(), method = input$method, 
                     FE_base = input$FE_base, RE_base = input$RE_base, RE_base_2 = RE_base2, 
                     FE_trt = input$FE_trt, RE_trt = input$RE_trt, RE_trt_2 = RE_trt2,
                     corStruct = input$corStruct,
                     varStruct = input$varStruct,
                     A = input$A_time, B = input$B_time,
                     coverage = input$coverage)
        
      }
      
    })
  
  # Effect size output
  
  output$effect_size_report <- renderTable({
    effect_size()
  }, digits = 4, include.rownames = FALSE)
  
  output$download_ES <- downloadHandler(
    filename = function() {
      fname <- if (input$dat_type == "example") input$example else input$dat
      paste(fname, '- effect size estimate.csv')
    },
    content = function(file) {
      dat <- effect_size()
      write.csv(dat, file, row.names=FALSE)
    },
    contentType = "text/csv"
  )
  
  # Graphs
  
  raw_graph <- reactive({
    cluster <- if (studyDesign() == "CMB") substitute(cluster) else NULL
    series <- if (studyDesign() == "RMBB") substitute(series) else NULL
    graph_SCD(design = studyDesign(), 
              cluster = cluster, case = case, series = series, 
              phase = phase, session = session, outcome = outcome, 
              treatment_name = NULL, model_fit = NULL, data = datClean())

  })
  
  output$raw_plot <- renderPlot({
    raw_graph()
  }, height = function() 120 * nlevels(datClean()[[1]]),
  width = function() 700)
  
  output$HPS_plot <- renderPlot({
    raw_graph() + 
      geom_smooth(formula = y ~ 1, method = "lm", se = FALSE)
  }, height = function() 120 * nlevels(datClean()[[1]]),
  width = function() 700)
  
  output$RML_plot <- renderPlot({
    
    if ("lme" %in% class(model_fit()$fit)) {
      cluster <- if (studyDesign() == "CMB") substitute(cluster) else NULL
      series <- if (studyDesign() == "RMBB") substitute(series) else NULL
      graph_SCD(design = studyDesign(), 
                cluster = cluster, case = case, series = series, 
                phase = phase, session = session, outcome = outcome, 
                model_fit = model_fit()$fit)
    }
  }, height = function() 120 * nlevels(datClean()[[1]]),
  width = function() 700)
  
  
  #------------------------------
  # Syntax for replication in R
  #------------------------------

  ES_syntax <- reactive({
    
    header_res <- c(
      '# Load packages',
      'library(nlme)',
      'library(scdhlm)',
      ''
    )
    
    # read in file code
    
    if(input$dat_type == "example") {
      read_res <- c(
        parse_code_chunk("load-example", args = list(example_name = input$example)),
        ''
      )
    } else if(input$dat_type == "dat"){
      inFile <- input$dat
      read_res <- c(
        parse_code_chunk("load-dat", 
                         args = list(user_path = inFile$name, user_header = input$header, 
                                     user_sep = input$sep, user_quote = input$quote)),
        ''
      )
    } else if (input$dat_type == "xlsx") {
      inFile <- input$xlsx
      read_res <- c(
        parse_code_chunk("load-excel", args = list(user_path = inFile$name, user_sheet = input$inSelect)),
        ''
      )
    } else {
      read_res <- c(
        parse_code_chunk("load-from-function", args = NULL),
        ''
      )
    }

    # Clean the data
  
    if (input$degree_base == 0 || is.null(input$model_center)) {
      model_center <- 0
    } else {
      model_center <- input$model_center
    }
    
    if (input$dat_type == "example") {
      
      example_parms <- exampleMapping[[input$example]]
      filter_vars <- example_parms$filters
      filter_vals <- if(length(filter_vars) > 0) lapply(paste0("filter_", filter_vars), 
                                                        function(x) paste0('"', input[[x]], '"', collapse = ",")) else NULL
      filter_vals <- paste0("%in% c(", filter_vals, ")")
      filter_string <- paste(example_parms$filters, filter_vals, collapse = " & ")
      
      if (!is.null(example_parms$filters)) {
        clean_dat_A <- c(
          parse_code_chunk("clean-example-filter", 
                           args = list(user_filterString = filter_string))
        )
      } else {
        clean_dat_A <- c()
      }
      
      case <- "case"
      session <- "session"
      phase <- "phase"
      outcome <- "outcome"
      series <- "series"
      cluster <- "cluster"
      
      if (studyDesign() == "TR") {
        clean_dat <- c(clean_dat_A,
                       '',
                       parse_code_chunk("clean-example-nofilter-TR",
                                        args = list(user_parms = paste(example_parms$vars, collapse='", "'),
                                                    user_design = studyDesign()))
        )
      } else if (studyDesign() == "MBP") {
        clean_dat <- c(clean_dat_A,
                       '',
                       parse_code_chunk("clean-example-nofilter",
                                        args = list(user_parms = paste(example_parms$vars, collapse='", "'),
                                                    user_design = studyDesign(),
                                                    user_model_center = model_center))
        )
      } else if (studyDesign() == "RMBB") {
        clean_dat <- c(clean_dat_A,
                       '',
                       parse_code_chunk("clean-example-nofilter-RMBB",
                                        args = list(user_parms = paste(example_parms$vars[2:6], collapse='", "'),
                                                    user_design = studyDesign(),
                                                    user_model_center = model_center))
        )
      } else if (studyDesign() == "CMB") {
        clean_dat <- c(clean_dat_A,
                       '',
                       parse_code_chunk("clean-example-nofilter-CMB",
                                        args = list(user_parms = paste(example_parms$vars[-c(1,2,7,9)], collapse='", "'),
                                                    user_design = studyDesign(),
                                                    user_model_center = model_center))
        )
      }
      
    } else {
      
      case <- input$caseID
      session <- input$session
      phase <- input$phaseID
      outcome <- input$outcome
      cluster <- if (studyDesign() == "CMB") input$clusterID else "NULL"
      series <- if (studyDesign() == "RMBB") input$seriesID else "NULL"
      round_session <- if (!is.null(input$round_session)) TRUE else FALSE
      
      filter_vars <- input$filters
      filter_vals <- if(length(filter_vars) > 0) lapply(paste0("filter_", filter_vars), 
                                                        function(x) paste0('"', input[[x]], '"', collapse = ",")) else NULL
      filter_vals <- paste0("%in% c(", filter_vals, ")")
      filter_string <- paste(input$filters, filter_vals, collapse = " & ")
      
      if (!is.null(input$filters)) {
        clean_dat_B <- c(
          '',
          parse_code_chunk("clean-inputdata-filter", args = list(user_filterString = filter_string))
        )
      } else {
        clean_dat_B <- c()
      }

      if (studyDesign() == "TR") {
          clean_dat <- c(clean_dat_B,
                         '',
                         parse_code_chunk("clean-inputdata-nofilter-TR", 
                                          args = list(user_caseID = case, 
                                                      user_session = session,
                                                      user_phaseID = phase,
                                                      user_outcome = outcome,
                                                      user_design = studyDesign(),
                                                      user_treatment = input$treatment,
                                                      user_round = round_session))
          )
        } else if (studyDesign() %in% c("MBP", "RMBB", "CMB")) {
          clean_dat <- c(clean_dat_B,
                         '',
                         parse_code_chunk("clean-inputdata-nofilter", 
                                          args = list(user_caseID = case,
                                                      user_session = session,
                                                      user_phaseID = phase,
                                                      user_outcome = outcome,
                                                      user_design = studyDesign(),
                                                      user_model_center = model_center,
                                                      user_treatment = input$treatment,
                                                      user_round = round_session,
                                                      user_clusterID = cluster,
                                                      user_seriesID = series))
          )
        }
      }
    
    # Fit the model
    
    if (input$method=="RML") {
      
      if (studyDesign() == "TR") {
        session_FE <- if (is.null(input$FE_base) | !(0 %in% input$FE_base)) "0" else "1"
        trt_FE <- if (is.null(input$FE_trt) | !(0 %in% input$FE_trt)) NULL else "trt"
        session_RE <- if (is.null(input$RE_base) | !(0 %in% input$RE_base)) "0" else "1"
        trt_RE <- if (is.null(input$RE_trt) | !(0 %in% input$RE_trt)) NULL else "trt"
      } else {
        session_FE <- write_formula(input$FE_base, c("0","1", session))
        trt_FE <- write_formula(input$FE_trt, c("NULL", "trt", paste0(session, "_trt")))
        session_RE <- write_formula(input$RE_base, c("0","1", session))
        trt_RE <- write_formula(input$RE_trt, c("NULL","trt", paste0(session, "_trt")))
        if (studyDesign() %in% c("RMBB", "CMB")) {
          session_RE_2 <- write_formula(input$RE_base2, c("0","1", session))
          trt_RE_2 <- write_formula(input$RE_trt2, c("NULL","trt", paste0(session, "_trt")))
        }
      }
      
      fixed <- paste(outcome, "~", paste(c(session_FE, trt_FE), collapse = " + "))
      if (studyDesign() == "RMBB") {
        random <- paste0("list(", case, " = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                         ", ", series, " = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")
      } else if (studyDesign() == "CMB") {
        random <- paste0("list(",cluster, " = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                         ", ", case, " = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")
      } else {
        random <- paste("~", paste(c(session_RE, trt_RE), collapse = " + "), "|", case)
      }
      
      corr_struct <- switch(input$corStruct,
                            "AR(1)" = 
                              if (studyDesign() == "RMBB") {
                                paste0("\n               correlation = corAR1(0.01, ~ ", session, " | ", case," / ", series, "),")
                              } else if (studyDesign() == "CMB") {
                                paste0("\n               correlation = corAR1(0.01, ~ ", session, " | ", cluster, " / ", case, "),")
                              } else {
                                paste0("\n               correlation = corAR1(0.01, ~ ", session, " | ", case, "),")
                              },
                            "MA(1)" = 
                              if (studyDesign() == "RMBB") {
                                paste0("\n               correlation = corARMA(0, ~ ", session, " | ", case," / ", series, ", p = 0, q = 1),")
                              } else if (studyDesign() == "CMB") {
                                paste0("\n               correlation = corARMA(0, ~ ", session, " | ", cluster, " / ", case, ", p = 0, q = 1),")
                              } else {
                                paste0("\n               correlation = corARMA(0, ~ ", session, " | ", case, ", p = 0, q = 1),")
                              },
                            "IID" = "",
                            c())
      
      var_struct <- switch(input$varStruct,
                           "hom" = "",
                           "het" = paste0("\n               weights = varIdent(form = ~ 1 | ", phase, "),"),
                           c())
      
      fit_mod <- parse_code_chunk("fit-RML", args = list(user_fixed = fixed, 
                                                         user_random = random,
                                                         corr_struct = corr_struct,
                                                         var_struct = var_struct))
      
    } else {
      fit_mod <- c()
    }
    
    # Calculate effect size
    
    if (input$method == "RML") {
      center <- if (input$degree_base == 0 || is.null(input$model_center)) 0L else input$model_center
      A <- if (is.null(input$A_time)) 0L else input$A_time
      B <- if (is.null(input$B_time)) 1L else input$B_time
      p_const <- c(rep(0L, length(input$FE_base)), (B - A)^as.integer(input$FE_trt))
      
      # get r_const when centering at an arbitrary time instead of B
      r_dim <- length(input$RE_base) + length(input$RE_trt)
      r_const_dim <- r_dim * (r_dim + 1) / 2
      bc_vec <- (B - center)^as.integer(input$RE_base)
      bc_mat <- 2 * tcrossprod(bc_vec) - diag(bc_vec^2)
      r_const_base <- bc_mat[upper.tri(bc_mat, diag = TRUE)]
      r_const_trt <- rep(0L, (r_const_dim - length(r_const_base)))
      r_const_cor <- rep(0L, length(model_fit()$fit$modelStruct$corStruct))
      r_const_var <- rep(0L, length(model_fit()$fit$modelStruct$varStruct))
      if (studyDesign() %in% c("MBP", "TR")) {
        r_const <- c(r_const_base, r_const_trt, r_const_cor, r_const_var, 1L)
      } else {
        r_dim2 <- length(input$RE_base2) + length(input$RE_trt2)
        r_const_dim2 <- r_dim2 * (r_dim2 + 1) / 2
        bc_vec2 <- (B - center)^as.integer(input$RE_base2)
        bc_mat2 <- 2 * tcrossprod(bc_vec2) - diag(bc_vec2^2)
        r_const_base2 <- bc_mat2[upper.tri(bc_mat2, diag = TRUE)]
        r_const_trt2 <- rep(0L, r_const_dim2 - length(r_const_base2))
        r_const <- c(r_const_base2, r_const_trt2, r_const_base, r_const_trt, r_const_cor, r_const_var, 1L)
      }
      
      if (studyDesign() == "MBP") {
        calc_ES <- parse_code_chunk("es-RML-MB", args = list(user_A = A,
                                                          user_B = B,
                                                          user_FE_base = paste_object(input$FE_base), 
                                                          user_FE_trt = paste_object(input$FE_trt),
                                                          user_rconst_base = paste_object(r_const_base),
                                                          user_rconst_trt = paste_object(r_const_trt),
                                                          user_rconst_cor = paste_object(r_const_cor),
                                                          user_rconst_var = paste_object(r_const_var),
                                                          user_pconstant = paste_object(p_const),
                                                          user_rconstant= paste_object(r_const)))
      } else if (studyDesign() %in% c("RMBB", "CMB")) {
        calc_ES <- parse_code_chunk("es-RML-3level", args = list(user_A = A,
                                                             user_B = B,
                                                             user_FE_base = paste_object(input$FE_base), 
                                                             user_FE_trt = paste_object(input$FE_trt),
                                                             user_rconst_base = paste_object(r_const_base),
                                                             user_rconst_trt = paste_object(r_const_trt),
                                                             user_rconst_base2 = paste_object(r_const_base2),
                                                             user_rconst_trt2 = paste_object(r_const_trt2),
                                                             user_rconst_cor = paste_object(r_const_cor),
                                                             user_rconst_var = paste_object(r_const_var),
                                                             user_pconstant = paste_object(p_const),
                                                             user_rconstant= paste_object(r_const)))
      } else{
        calc_ES <- parse_code_chunk("es-RML-TR", args = list(user_rconst_base = paste_object(r_const_base),
                                                          user_rconst_trt = paste_object(r_const_trt),
                                                          user_rconst_cor = paste_object(r_const_cor),
                                                          user_rconst_var = paste_object(r_const_var),
                                                          user_pconstant = paste_object(p_const),
                                                          user_rconstant= paste_object(r_const)))
      }
      
    } else {
      if (studyDesign() == "MBP") {
        calc_ES <- parse_code_chunk("es-MB", args = list(user_case = case,
                                                         user_session = session,
                                                         user_outcome = outcome))
      } else {
        phase_pair <- paste0(phase, "_pair")
        calc_ES <- parse_code_chunk("es-ABK", args = list(user_case = case,
                                                          user_session = session,
                                                          user_outcome = outcome,
                                                          user_phase_pair = phase_pair))
      }
    }
    
    furtherArg_design <- switch(studyDesign(),
                                "RMBB" = paste0("series = ", series, ","),
                                "CMB" = paste0("cluster = ", cluster, ","),
                                "MBP" = "",
                                "TR" = "",
                                c())
    
    furtherArg_method <- switch(input$method,
                                "RML" = paste0("model_fit = fit_RML,"),
                                "HPS" = "",
                                c())
    
    SCD_graph <- parse_code_chunk("graph-scd", args = list(user_case = case,
                                                           user_phase = phase,
                                                           user_session = session,
                                                           user_outcome = outcome,
                                                           user_design = studyDesign(),
                                                           furtherArg_method = furtherArg_method,
                                                           furtherArg_design = furtherArg_design))

    res <- c(header_res, read_res, clean_dat, '', fit_mod, '', calc_ES, '', SCD_graph, '')
    paste(res, collapse = "\n")
  })
  
  
  output$syntax <- renderPrint({
    cat(ES_syntax(), sep = "\n")
  })
  
  output$clip <- renderUI({
    rclipboard::rclipButton("clipbtn", "Copy", ES_syntax(), modal = FALSE, icon("clipboard"))
  })
  
  session$onSessionEnded(stopApp)
    
})

# Clean data
dat <- preprocess_SCD(design = "{user_design}", 
                      case = {user_caseID}, 
                      phase = {user_phaseID}, 
                      session = {user_session}, 
                      outcome = {user_outcome}, 
                      round_session = {user_round}, 
                      treatment_name = "{user_treatment}", 
                      data = dat)

# Fit the model
fit_RML <- lme(fixed = {user_fixed}, 
               random = {user_random}, {corr_struct} {var_struct}
               data = dat,
               control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
summary(fit_RML)

#------------------------------------------------
# Information about pre-loaded examples
#------------------------------------------------

exampleChoices <- c("Alber-Morgan (multiple baseline across participants)" = "AlberMorgan",
                    "Anglesea (ABAB design)" = "Anglesea", 
                    "Barton-Arwood (multiple baseline across participants)" = "BartonArwood",
                    "Bryant-et-al. (clustered multiple baseline across participants)" = "Bryant2018",
                    "Case-Harris-Graham (multiple probe across participants)" = "CaseHarrisGraham",
                    "Datchuk (multiple probe across participants)" = "Datchuk",
                    "Delemere-Dounavi (multiple baseline across participants)" = "DelemereDounavi",
                    "Gunning-Espie (multiple baseline across participants)" = "GunningEspie",
                    "Lambert (ABAB design)" = "Lambert",
                    "Laski (multiple baseline across participants)" = "Laski",
                    "Peltier (multiple probe across participants)" = "Peltier",
                    "Rodgers (multiple baseline across participants)" = "Rodgers",
                    "Rodriguez (multiple baseline across participants)" = "Rodriguez",
                    "Saddler (multiple probe across participants)" = "Saddler",
                    "Schutte (multiple baseline across participants)" = "Schutte",
                    "Thiemann-Goldstein (replicated multiple baseline across behaviors)" = "Thiemann2001",
                    "Thorne (ABAB design)" = "Thorne")

exampleMapping <- list(
  AlberMorgan = list(design = "MBP",
                     vars = c("case","session","condition","outcome"),
                     phases = c("baseline","treatment")),
  Anglesea = list(design = "TR",
                  vars = c("case","session","condition","outcome"),
                  phases = c("baseline","treatment")),
  BartonArwood = list(design = "MBP",
                      vars = c("case","session","condition","outcome"),
                      phases = c("A","B")),
  Bryant2018 = list(design = "CMB",
                    vars = c("Study_ID", "school", "group", "case", "treatment", "session", "session_trt", "outcome", "session_c"),
                    phases = c("baseline", "treatment")),
  CaseHarrisGraham = list(design = "MBP",
                          vars = c("case","session","condition","outcome"),
                          phases = c("baseline","treatment")),
  Datchuk = list(design = "MBP",
                 vars = c("case","session","condition","outcome"),
                 phases = c("baseline","treatment")),
  DelemereDounavi = list(design = "MBP",
                         vars = c("case","session","condition","outcome"),
                         phases = c("baseline","treatment")),
  GunningEspie = list(design = "MBP",
                      vars = c("case","session","condition","outcome"),
                      phases = c("baseline","treatment", "follow-up")),
  Lambert = list(design = "TR",
                 vars = c("case","time","treatment","outcome"),
                 phases = c("SSR","RC"),
                 filters = "measure",
                 filter_measure = c("academic response", "disruptive behavior")),
  Laski = list(design = "MBP",
               vars = c("case","time","treatment","outcome"),
               phases = c(0,1)),
  Rodgers = list(design = "MBP",
                 vars = c("case","session","condition","outcome"),
                 phases = c("baseline","treatment")),
  Peltier = list(design = "MBP",
                 vars = c("case","session","condition","outcome"),
                 phases = c("baseline","treatment")),
  Rodriguez = list(design = "MBP",
                   vars = c("case","session","condition","outcome"),
                   phases = c("A","B")),
  Saddler = list(design = "MBP",
                 vars = c("case","time","treatment","outcome"),
                 phases = c(0,1),
                 filters = "measure",
                 filter_measure = c("writing quality", "T-unit length", "number of constructions")),
  Schutte = list(design = "MBP",
                 vars = c("case","week","treatment","fatigue"),
                 phases = c("baseline","treatment")),
  Thiemann2001 = list(design = "RMBB",
                      vars = c("Study_ID","case", "series", "outcome", "time", "treatment", "trt_time", "time_c"),
                      phases = c("baseline", "treatment")),
  Thorne = list(design = "TR",
                vars = c("case","session","condition","outcome"),
                phases = c("A","B"),
                filters = "measure",
                filter_measure = c("Academic Engagement","Inappropriate Verbalizations"))
)


#------------------------------------------------
# Model term labels
#------------------------------------------------

labs_MBP <- list(
  fixed =  "Include fixed effect &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;&nbsp;",
  random = "Include case-level random effect"
)

labs_RMBB <- list(
  fixed         = "Include fixed effect &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",
  random_series = "Include series-level random effect",
  random_case   = "Include case-level random effect &nbsp; &nbsp; &nbsp; &nbsp;"
)

labs_CMB <- list(
  fixed          = "Include fixed effect &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",
  random_case    = "Include case-level random effect &nbsp; &nbsp; &nbsp; &nbsp;",
  random_cluster = "Include cluster-level random effect"
)

handle_white_space <- function(x) span(HTML(x), style="white-space: break-spaces;")

labs_MBP <- lapply(labs_MBP, handle_white_space)
labs_RMBB <- lapply(labs_RMBB, handle_white_space)
labs_CMB <- lapply(labs_CMB, handle_white_space)

#------------------------------------------------
# Polynomial degree names
#------------------------------------------------

degree_names_baseline <- c("level", "linear trend", "quadratic trend", "cubic trend","
                           quartic trend", "quintic trend", "sextic trend")
degree_names_treatment <- c("change in level", "change in linear trend", "change in quadratic trend", 
                            "change in cubic trend", "change in quartic trend",
                            "change in quintic trend", "change in sextic trend")

time_trends_baseline <- 0:6
time_trends_treatment <- 0:6
names(time_trends_baseline) <- degree_names_baseline
names(time_trends_treatment) <- degree_names_treatment

#------------------------------------------------
# Design names
#------------------------------------------------

design_names <- c("Treatment Reversal" = "TR", 
                  "Multiple Baseline/Multiple Probe across participants" = "MBP",
                  "Replicated multiple baseline across behaviors" = "RMBB",
                  "Clustered multiple baseline across participants" = "CMB")

#------------------------------------------------
# Estimation names
#------------------------------------------------

estimation_names <- c("Moment estimation" = "HPS", "Restricted Maximum Likelihood" = "RML")

# Graph 
graph_SCD(design = "{user_design}", 
          case = {user_case}, phase = {user_phase}, 
          session = {user_session}, outcome = {user_outcome}, {furtherArg_method} {furtherArg_design} 
          data = dat)

# Clean data

dat <- dat[,c("{user_parms}")]
names(dat) <- c("case","session","phase","outcome")

dat <- preprocess_SCD(design = "{user_design}", 
                      case = case, 
                      phase = phase, 
                      session = session, 
                      outcome = outcome, 
                      data = dat)

## ----message = FALSE----------------------------------------------------------
library(scdhlm)

## ----Lambert------------------------------------------------------------------
data(Lambert)

Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                              id = case, phase = phase, time = time, 
                              data = Lambert)

print(Lambert_ES, 5)

## ----Lambert-summary----------------------------------------------------------
summary(Lambert_ES)

## ----Lambert_sensitivity------------------------------------------------------
Lambert_ES2 <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                id = case, phase = phase, time = time, 
                data = Lambert, phi = 0.10)
print(Lambert_ES2, digits = 5)

Lambert_ES3 <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                id = case, phase = phase, time = time, 
                data = Lambert, phi = 0.35)
print(Lambert_ES3, digits = 5)

## ----Anglesea-----------------------------------------------------------------
data(Anglesea)
Anglesea_ES <- effect_size_ABk(outcome, condition, case, phase, session, data = Anglesea)
Anglesea_ES

## ----Saddler------------------------------------------------------------------
data(Saddler)

quality_ES <- effect_size_MB(outcome, treatment, case, time, 
                             data = subset(Saddler, measure=="writing quality"))
complexity_ES <- effect_size_MB(outcome, treatment, case, time , 
                                data = subset(Saddler, measure=="T-unit length"))
construction_ES <- effect_size_MB(outcome, treatment, case, time, 
                                  data = subset(Saddler, measure=="number of constructions"))

data.frame(
  quality = unlist(quality_ES),
  complexity = unlist(complexity_ES), 
  construction = unlist(construction_ES) 
)[c("delta_hat","V_delta_hat","nu","phi","rho"),]


## ----quality-RML--------------------------------------------------------------
quality_RML <- lme(fixed = outcome ~ treatment, 
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = subset(Saddler, measure=="writing quality"))
summary(quality_RML)

## ----quality-g-mlm, eval=TRUE-------------------------------------------------
quality_ES_RML <- g_mlm(quality_RML, p_const = c(0,1), r_const = c(1,0,1))
summary(quality_ES_RML)

## ----Laski-RML, eval = TRUE---------------------------------------------------
data(Laski)

# Hedges, Pustejovsky, & Shadish (2013)
Laski_ES_HPS <- effect_size_MB(outcome, treatment, case, time, data= Laski)

# Pustejovsky, Hedges, & Shadish (2014)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
summary(Laski_RML)

## ----Laski-ES-RML, eval = TRUE------------------------------------------------
Laski_ES_RML <- g_mlm(Laski_RML, p_const = c(0,1), r_const = c(1,0,1))

# compare the estimates
data.frame(
  HPS = with(Laski_ES_HPS, c(SMD = delta_hat, SE = sqrt(V_delta_hat), 
                             phi = phi, rho = rho, nu = nu)),
  RML = with(Laski_ES_RML, c(g_AB, SE_g_AB, theta$cor_params, 
                             theta$Tau$case / (theta$Tau$case + theta$sigma_sq), nu))
)

## ----Laski-RML2---------------------------------------------------------------
Laski_RML2 <- lme(fixed = outcome ~ treatment,
                 random = ~ treatment | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
summary(Laski_RML2)
anova(Laski_RML, Laski_RML2)

## ----Laski-ES-RML2, eval=TRUE-------------------------------------------------
Laski_ES_RML2 <- g_mlm(Laski_RML2, p_const = c(0,1), r_const = c(1,0,0,0,1))
Laski_ES_RML2

## ----Schutte-graph, warning = FALSE, message = FALSE, fig.width = 7, fig.height = 7----

if (requireNamespace("ggplot2", quietly = TRUE)) {
library(ggplot2)

data(Schutte)

graph_SCD(case = case, phase = treatment,
          session = week, outcome = fatigue,
          design = "MB", data = Schutte) + 
  facet_wrap(~ case, ncol = 3) + 
  theme(legend.position = "bottom")
}


## ----Schutte-center-----------------------------------------------------------
# time-point constants
A <- 2
B <- 9
# center at follow-up time

# create time-by-trt interaction
Schutte <- 
  preprocess_SCD(case = case, phase = treatment,
                 session = week, outcome = fatigue,
                 design = "MB", center = B, data = Schutte)


## ----Schutte-Model3-----------------------------------------------------------
hlm1 <- lme(fixed = fatigue ~ week + treatment + week_trt, 
            random = ~ 1 | case, 
            correlation = corAR1(0, ~ week | case),
            data = Schutte,
            method = "REML")
summary(hlm1)

## ----Schutte-ES-Model3, eval = TRUE-------------------------------------------
Schutte_g1 <- g_mlm(hlm1, p_const = c(0,0,1, B - A), r_const = c(1,0,1))
Schutte_g1

## ----Schutte-Model4-----------------------------------------------------------
hlm2 <- update(hlm1, random = ~ week | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm2)
anova(hlm1, hlm2)

## ----Schutte-ES-Model4, eval = TRUE-------------------------------------------
Schutte_g2 <- g_mlm(hlm2, p_const = c(0,0,1, B - A), r_const = c(1,0,0,0,1))
Schutte_g2

## -----------------------------------------------------------------------------
unlist(extract_varcomp(hlm2))

## ----Schutte-Model5, warning = FALSE------------------------------------------
hlm3 <- update(hlm2, random = ~ week + week_trt | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm3)
anova(hlm2, hlm3)

## ----Schutte-ES-Model5, eval = TRUE-------------------------------------------
Schutte_g3 <- g_mlm(hlm3, p_const = c(0,0,1,B - A), r_const = c(1,0,0,0,0,0,0,1))
Schutte_g3

# compare effect size estimates
data.frame(
  MB3 = round(unlist(Schutte_g1[c("g_AB","SE_g_AB","nu")]), 3), 
  MB4 = round(unlist(Schutte_g2[c("g_AB","SE_g_AB","nu")]), 3),
  MB5 = round(unlist(Schutte_g3[c("g_AB","SE_g_AB","nu")]), 3)
)

# You're using data that you specified when starting the app.
# You'll need to read it in yourself and store it in an object called 'dat'.

# Clean the variable names 
dat <- janitor::clean_names(dat, case = "parsed")

set.seed(1)
#------------------------------------
# Design parameters
#------------------------------------

iterations <- 20000
beta <- c(0,1,0,0)
phi <- seq(-7L, 7L, 2) / 10L
rho <- seq(0.0, 0.8, 0.2)
m <- 3:6
n <- c(8, 16)

print(lengths <- c(length(phi), length(rho), length(m), length(n)))
print(combos <- prod(lengths))
parms <- expand.grid(phi = phi, rho = rho, m = m, n=n)[sample(combos),]
head(parms)

#--------------------------------------------------------
# run simulations in parallel on Windows via SNOW
#--------------------------------------------------------

library(plyr)
library(snow)
library(foreach)
library(iterators)
library(doSNOW)
library(rlecuyer)

cluster <- makeCluster(parallel::detectCores(), type = "SOCK")
registerDoSNOW(cluster)

# set up parallel random number generator
clusterSetupRNGstream(cluster, 20131220)

# execute simulations
system.time(MB1_array <- maply(parms, .fun = compare_RML_HPS, 
                             iterations = iterations, beta = beta, 
                             .drop=FALSE, .parallel=TRUE,
                             .paropts = list(.packages="scdhlm")))
stopCluster(cluster)

##------------------------------------------------
## reshape and save results
##------------------------------------------------
library(reshape)

names(dimnames(MB1_array))[5:6] <- c("stat","moment")
MB1results <- cast(melt(MB1_array), ... ~ moment)
attr(MB1results, "iterations") <- iterations
attr(MB1results, "beta") <- beta
save(MB1results, file="data/MB1results.RData", compress="xz")

## Create correlation matrix ####

HLM_AR1_corr <- function(id_fac, time, rho, phi) {
  C.mat <- matrix(0,length(id_fac), length(id_fac))
  for (i in levels(id_fac)) {
    ind <- (id_fac == i)
    C.mat[ind,ind] <- rho + (1 - rho) * phi^as.matrix(dist(time[ind]))
  }
  return(C.mat)
}

  
## Calculate lagged sums of squares ####

auto_SS <- function(x, n = length(x)) {
  if (n > 1) {
    e <- x - mean(x)
    auto_0 <- sum(e * e)
    auto_1 <- sum(e[1:(n-1)] * e[2:n])
  } else {
    auto_0 <- NA
    auto_1 <- NA
  }
  return(c(auto_0, auto_1))
}





## calculate effect size (with associated estimates) for multiple baseline design ####

#' @title Calculates HPS effect size
#' 
#' @description Calculates the HPS effect size estimator based on data from a multiple baseline design, 
#' as described in Hedges, Pustejovsky, & Shadish (2013). Note that the data must contain one row per 
#' measurement occasion per subject.
#' 
#' @param outcome vector of outcome data or name of variable within \code{data}. May not contain any missing values.
#' @param treatment vector of treatment indicators or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param id factor vector indicating unique cases or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param time vector of measurement occasion times or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param data (Optional) dataset to use for analysis. Must be data.frame. 
#' @param phi (Optional) value of the auto-correlation nuisance parameter, to be used 
#' in calculating the small-sample adjusted effect size
#' @param rho (Optional) value of the intra-class correlation nuisance parameter, to be used 
#' in calculating the small-sample adjusted effect size
#' 
#' @note If phi or rho is left unspecified (or both), estimates for the nuisance
#' parameters will be calculated.
#' 
#' @export 
#' 
#' @return A list with the following components
#' \tabular{ll}{
#' \code{g_dotdot} \tab total number of non-missing observations \cr
#' \code{K} \tab number of time-by-treatment groups containing at least one observation \cr
#' \code{D_bar} \tab numerator of effect size estimate \cr
#' \code{S_sq} \tab sample variance, pooled across time points and treatment groups \cr
#' \code{delta_hat_unadj} \tab unadjusted effect size estimate \cr
#' \code{phi} \tab corrected estimate of first-order auto-correlation \cr
#' \code{sigma_sq_w} \tab corrected estimate of within-case variance \cr
#' \code{rho} \tab estimated intra-class correlation \cr
#' \code{theta} \tab estimated scalar constant \cr
#' \code{nu} \tab estimated degrees of freedom \cr
#' \code{delta_hat} \tab corrected effect size estimate \cr
#' \code{V_delta_hat} \tab estimated variance of \code{delta_hat}
#' }
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). 
#' A standardized mean difference effect size for multiple baseline designs across individuals. 
#' \emph{Research Synthesis Methods, 4}(4), 324-341. \doi{10.1002/jrsm.1086}
#' 
#' @examples
#' data(Saddler)
#' effect_size_MB(outcome = outcome, treatment = treatment, id = case, 
#'                time = time, data = subset(Saddler, measure=="writing quality"))
#' 
#' data(Laski)
#' effect_size_MB(outcome = outcome, treatment = treatment, id = case, 
#'                time = time, data = Laski)
#' 


effect_size_MB <- function(outcome, treatment, id, time, data = NULL, phi = NULL, rho = NULL) {
  
  ###########
  ## setup ##
  ###########
  
  # create factor variables
  
  if (!is.null(data)) {
    outcome_call <- substitute(outcome)
    treatment_call <- substitute(treatment)
    id_call <- substitute(id)
    time_call <- substitute(time)
    
    env <- list2env(data, parent = parent.frame())
    
    outcome <- eval(outcome_call, env)
    treatment <- eval(treatment_call, env)
    id <- eval(id_call, env)
    time <- eval(time_call, env)
  }
  
  
  dat <- data.frame(id_fac = factor(id),
                    treatment_fac = factor(treatment),
                    time_fac = factor(time),
                    outcome, time)
  
  dat <- dat[with(dat, order(id_fac, time)),]
  
  # unique times, unique cases, calculate sample sizes  
  time_points <- seq(min(time), max(time),1)            # unique measurement occasions j = 1,...,N
  N <- length(time_points)                              # number of unique measurement occasions
  h_i_p <- with(dat, table(id_fac, treatment_fac))      # number of non-missing observations for case i in phase p
  cases <- levels(dat$id_fac)                           # unique cases i = 1,...,m
  m <- length(cases)                                    # number of cases
  g_dotdot <- length(outcome)                           # total number of non-missing observations
  
  
  ######################################
  ## calculate unadjusted effect size ##
  ######################################
  
  # fixed effects regression with id-by-treatment interaction
  case_FE <- lm(outcome ~ id_fac + id_fac:treatment_fac + 0, data = dat)
  X_case_FE <- model.matrix(case_FE)                            # design matrix from id-by-treatment fixed-effects regression
  X_trt <- attr(X_case_FE, "assign") == 2                       # indicator for individual treatment effects
  XtX_inv_case_FE <- solve(t(X_case_FE) %*% X_case_FE)          # inverse of (X'X) for this design matrix
  
  # calculate D-bar
  D_bar <- mean(coef(case_FE)[X_trt])           # See p. 11, Eq. (2).
  
  # fixed effects regression with time-by-treatment interaction
  time_FE <- lm(outcome ~ time_fac + time_fac:treatment_fac + 0, data = dat)
  X_time_FE <- model.matrix(time_FE)[,time_FE$qr$pivot[1:time_FE$qr$rank]]    # design matrix for time-by-treatment fixed-effects regression
  
  # calculate pooled variance S-squared
  S_sq <- summary(time_FE)$sigma^2              # See p. 12, Eq. (3) and also footnote 5.
  K <- time_FE$rank                             # number of time-by-treatment groups containing at least one observation. See p. 11.
  
  # calculate unadjusted effect size
  delta_hat_unadj <- D_bar / sqrt(S_sq)         # See p. 13, Eq. (4)
  
  
  ##################################
  ## nuisance parameter estimates ##
  ##################################
  
  if (is.null(phi) | is.null(rho)) {
    # auto-covariances - See first display equation on p. 32.
    acv_SS <- matrix(unlist(tapply(case_FE$residuals, dat$id_fac, auto_SS)), m, 2, byrow=TRUE)
    
    # calculate adjusted autocorrelation
    if (is.null(phi)) {
      phi_YW <- sum(acv_SS[,2], na.rm=T) / sum(acv_SS[,1], na.rm=T)
      phi_correction <- sum((h_i_p - 1) / h_i_p) / (g_dotdot - 2 * m)   # This is the constant C given on p. 33.
      phi <- phi_YW + phi_correction                                # See last display equation on p. 32.      
    }
    
    if (is.null(rho)) {
      # calculate adjusted within-case variance estimate
      sigma_sq_correction <- g_dotdot - product_trace(XtX_inv_case_FE, t(X_case_FE) %*% 
                                                        with(dat, HLM_AR1_corr(id_fac, time, 0, phi)) %*% X_case_FE) 
      
      # This correction is equal to g_dotdot * F, where F is given on p. 33. 
      sigma_sq_w <- sum(acv_SS[,1], na.rm=T) / sigma_sq_correction
      
      # calculate intra-class correlation
      rho <- max(0, 1 - sigma_sq_w / S_sq)                          # See last display equation on p. 33.    
    } else sigma_sq_w <- NA
  } else sigma_sq_w <- NA
  
  
  ############################################
  ## calculate degrees of freedom and theta ##
  ############################################
  
  # create A matrix
  A_mat <- diag(rep(1, g_dotdot)) - X_time_FE %*% solve(t(X_time_FE) %*% X_time_FE) %*% t(X_time_FE)  # S^2 = y'(A_mat)y / (g_dotdot - K). See p. 29. 
  
  # create correlation matrix V_mat, which is the matrix Sigma on p. 28, scaled by tau^2 + sigma^2. 
  V_mat <- with(dat, HLM_AR1_corr(id_fac, time, rho, phi))
  
  # calculate degrees of freedom
  AV <- A_mat %*% V_mat
  nu <- (g_dotdot - K)^2 / product_trace(AV, AV)          # See p. 15, Eq. (11). Also note that product_trace(AV, AV) = tr(A Sigma A Sigma). See p. 30.
  
  # calculate theta
  theta <- sqrt(sum(diag(XtX_inv_case_FE %*% t(X_case_FE) %*% V_mat %*% X_case_FE %*% XtX_inv_case_FE)[X_trt])) / m   # See p. 15, Eq. (10). 
  
  
  #######################################
  ## adjusted effect size and variance ##
  #######################################
  
  # calculate adjusted effect size
  delta_hat <- J(nu) * delta_hat_unadj    # See p. 15, Eq. (13). 
  
  # calculate variance of adjusted effect size
  nu_trunc <- max(nu, 2.001)
  V_delta_hat <- J(nu)^2 * (theta^2 * nu_trunc / (nu_trunc - 2) + delta_hat^2 * (nu_trunc / (nu_trunc - 2) - 1 / J(nu)^2))   # See p. 15, Eq. (14). 
  
  
  ####################
  ## return results ##
  ####################
  
  results <- list(g_dotdot = g_dotdot, K = K, D_bar = D_bar, S_sq = S_sq, delta_hat_unadj = delta_hat_unadj, 
                  phi = phi, sigma_sq_w = sigma_sq_w, rho = rho, 
                  theta = theta, nu = nu, delta_hat = delta_hat, V_delta_hat = V_delta_hat)
  class(results) <- "g_HPS"
  
  return(results)
}

## calculate effect size (with associated estimates) for (AB)^k design ####

#' @title Calculates HPS effect size
#' 
#' @description Calculates the HPS effect size estimator based on data from an (AB)^k design, 
#' as described in Hedges, Pustejovsky, & Shadish (2012). Note that the data must contain one row per 
#' measurement occasion per subject.
#' 
#' @param outcome vector of outcome data or name of variable within \code{data}. May not contain any missing values.
#' @param treatment vector of treatment indicators or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param id factor vector indicating unique cases or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param phase factor vector indicating unique phases (each containing one contiguous control 
#' condition and one contiguous treatment condition) or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param time vector of measurement occasion times or name of variable within \code{data}. Must be the same length as \code{outcome}.
#' @param data (Optional) dataset to use for analysis. Must be data.frame. 
#' @param phi (Optional) value of the auto-correlation nuisance parameter, to be used 
#' in calculating the small-sample adjusted effect size
#' @param rho (Optional) value of the intra-class correlation nuisance parameter, to be used 
#' in calculating the small-sample adjusted effect size
#' 
#' @note If phi or rho is left unspecified (or both), estimates for the nuisance
#' parameters will be calculated.
#' 
#' @export 
#' 
#' @return A list with the following components
#' \tabular{ll}{
#' \code{M_a} \tab Matrix reporting the total number of time points with data for all ids, 
#' by phase and treatment condition \cr
#' \code{M_dot} \tab Total number of time points used to calculate the total variance (the sum of \code{M_a}) \cr
#' \code{D_bar} \tab numerator of effect size estimate \cr
#' \code{S_sq} \tab sample variance, pooled across time points and treatment groups \cr
#' \code{delta_hat_unadj} \tab unadjusted effect size estimate \cr
#' \code{phi} \tab corrected estimate of first-order auto-correlation \cr
#' \code{sigma_sq_w} \tab corrected estimate of within-case variance \cr
#' \code{rho} \tab estimated intra-class correlation \cr
#' \code{theta} \tab estimated scalar constant \cr
#' \code{nu} \tab estimated degrees of freedom \cr
#' \code{delta_hat} \tab corrected effect size estimate \cr
#' \code{V_delta_hat} \tab estimated variance of the effect size
#' }
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
#' A standardized mean difference effect size for single case designs. 
#' \emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
#' 
#' @examples
#' data(Lambert)
#' effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
#'                 phase = phase, time = time, data = Lambert)
#'    
#' data(Anglesea)
#' effect_size_ABk(outcome = outcome, treatment = condition, id = case, 
#'                 phase = phase, time = session, data = Anglesea)
#' 

effect_size_ABk <- function(outcome, treatment, id, phase, time, data = NULL, phi=NULL, rho=NULL) {
  
  ###########
  ## setup ##
  ###########
  
  if (!is.null(data)) {
  outcome_call <- substitute(outcome)
  treatment_call <- substitute(treatment)
  id_call <- substitute(id)
  phase_call <- substitute(phase)
  time_call <- substitute(time)
  
  env <- list2env(data, parent = parent.frame())
  
  outcome <- eval(outcome_call, env)
  treatment <- eval(treatment_call, env)
  id <- eval(id_call, env)
  phase <- eval(phase_call, env)
  time <- eval(time_call, env)
  }
  
  # create factor variables
  dat <- data.frame(id_fac = factor(id),
                    phase_fac = factor(phase), 
                    treatment_fac = factor(treatment),
                    outcome, time)
  dat <- dat[with(dat, order(id_fac, phase_fac, treatment_fac, time)),]
  
  # number of cases
  m <- nlevels(dat$id_fac)                            
  
  # re-number time points per HPS (2012)
  dat$phase_point <- with(dat, unlist(tapply(outcome, 
                                             list(treatment_fac, phase_fac, id_fac), 
                                             function(x) 1:length(x))))
  dat$phase_point_fac <- ordered(dat$phase_point)
  
  # determine M^a values. See p. 231, formulas (16-17)
  M_a <- with(dat, apply(tapply(outcome, list(phase_point, treatment_fac, phase_fac), 
                                function(x) length(x) == m), c(2,3), sum, na.rm = TRUE))
  M_dot <- sum(M_a, na.rm = TRUE)  
  
  dat$include <- mapply(function(phase, treat, point)
    m == sum(phase==dat$phase_fac & treat==dat$treatment_fac & point==dat$phase_point),
    dat$phase_fac, dat$treatment_fac, dat$phase_point)
  
  ######################################
  ## calculate unadjusted effect size ##
  ######################################
  
  if (nlevels(dat$phase_fac) > 1) {
    # fixed effects regression with id-by-phase-by-treatment interaction
    case_FE <- lm(outcome ~ id_fac:phase_fac + id_fac:phase_fac:C(treatment_fac, contr.treatment) + 0, data = dat)
    # fixed effects regression with phase-point-by-treatment-by-phase interaction
    time_FE <- lm(outcome ~ phase_point_fac:treatment_fac:phase_fac + 0,
                  data = dat, subset = dat$include)    
  } else {
    # fixed effects regression with id-by-phase-by-treatment interaction
    case_FE <- lm(outcome ~ id_fac + id_fac:C(treatment_fac, treatment) + 0, data = dat)
    # fixed effects regression with phase-point-by-treatment-by-phase interaction
    time_FE <- lm(outcome ~ phase_point_fac:treatment_fac + 0,
                  data = dat, subset = dat$include)    
    
  }
  
  # design matrix from id-by-phase-by-treatment fixed-effects regression
  X_case_FE <- model.matrix(case_FE)
  X_keep <- !is.na(coef(case_FE))
  X_trt <- (attr(X_case_FE, "assign") == 2)[X_keep]    # indicator for individual treatment effects
  XtX_inv_case_FE <- chol2inv(chol(t(X_case_FE[,X_keep,drop=FALSE]) %*% X_case_FE[,X_keep,drop=FALSE]))          # inverse of (X'X) for this design matrix
  
  # calculate D-bar
  D_bar <- mean(coef(case_FE)[X_keep][X_trt])           # See p. 231, formula (19).
  
  # design matrix for phase-point-by-treatment-by-phase fixed-effects regression
  X_time_FE <- model.matrix(time_FE)[,time_FE$qr$pivot[1:time_FE$qr$rank]]    
  
  # calculate pooled variance S-squared
  S_sq <- summary(time_FE)$sigma^2              # See p. 231, formula (18).
  
  # calculate unadjusted effect size
  delta_hat_unadj <- D_bar / sqrt(S_sq)         # See p. 231, formula (20).
  
  
  ##################################
  ## nuisance parameter estimates ##
  ##################################
  
  if (missing(phi) | missing(rho)) {
    # auto-covariances - See first display equation on p. 32.
    YW <- with(dat, aggregate(outcome, by = list(id_fac, phase_fac, treatment_fac), 
                              function(x) c(auto_SS(x), length(x)))) # calculate auto-covariances by case by phase
    YW <- cbind(YW[,1:3], YW$x)
    names(YW) <- c("id_fac","phase_fac","treatment_fac","g0","g1","n")
    
    # calculate adjusted estimate of pooled auto-correlation. See p. 238.
    if (missing(phi)) phi <- sum(YW$g1, na.rm=T) / sum(YW$g0, na.rm=T) + sum(1 - 1 / YW$n) / sum(YW$n - 1)
    
    if (missing(rho)) {
      # calculate adjusted within-case variance estimate
      sigma_sq_correction <- length(dat$outcome) - 
        product_trace(XtX_inv_case_FE, 
                      t(X_case_FE[,X_keep,drop=FALSE]) %*% 
                        with(dat, HLM_AR1_corr(id_fac, phase_point, 0, phi)) %*% 
                        X_case_FE[,X_keep,drop=FALSE]) 
      sigma_sq_w <- sum(YW$g0, na.rm=T) / sigma_sq_correction
      
      rho <- max(0, 1 - sigma_sq_w / S_sq)      # See last display equation on p. 238.
    } else sigma_sq_w <- NA
  } else sigma_sq_w <- NA
  
  
  ############################################
  ## calculate degrees of freedom and theta ##
  ############################################
  
  # create A matrix. S^2 = y'(A_mat)y / (M_dot * (m - 1)). See p. 236. 
  A_mat <- diag(rep(1, dim(X_time_FE)[1])) - X_time_FE %*% solve(t(X_time_FE) %*% X_time_FE) %*% t(X_time_FE)  
  
  # V_mat is the matrix Sigma_T on p. 236, scaled by tau^2 + sigma^2. 
  V_mat <- with(dat, HLM_AR1_corr(id_fac, time, rho, phi))
  
  # calculate degrees of freedom. See p. 232, formula (28). 
  AV <- A_mat %*% V_mat[dat$include, dat$include]
  nu <- (M_dot * (m - 1))^2 / product_trace(AV, AV)
  
  # calculate theta. See p. 232, formula (27).
  theta <- sqrt(sum((XtX_inv_case_FE %*% t(X_case_FE[,X_keep,drop=FALSE]) %*% 
                       V_mat %*% X_case_FE[,X_keep,drop=FALSE] %*% XtX_inv_case_FE)[X_trt, X_trt])) / sum(X_trt)
  
  
  #######################################
  ## adjusted effect size and variance ##
  #######################################
  
  # calculate adjusted effect size. See p. 232, formula (29).
  delta_hat <- J(nu) * delta_hat_unadj    # See p. 15, Eq. (13). 
  
  # calculate variance of adjusted effect size. See p. 232, formula (30).
  nu_trunc <- max(nu, 2.001)
  V_delta_hat <- J(nu)^2 * (theta^2 * nu_trunc / (nu_trunc - 2) + delta_hat^2 * (nu_trunc / (nu_trunc - 2) - 1 / J(nu)^2))   # See p. 15, Eq. (14). 
  
  
  ####################
  ## return results ##
  ####################
  
  results <- list(M_a = M_a, M_dot = M_dot,
                  D_bar = D_bar, S_sq = S_sq, delta_hat_unadj = delta_hat_unadj, 
                  phi = phi, sigma_sq_w = sigma_sq_w, rho = rho, 
                  theta = theta, nu = nu, 
                  delta_hat = delta_hat, V_delta_hat = V_delta_hat)
  
  class(results) <- "g_HPS"
  
  return(results)
}

#' @export

summary.g_HPS <- function(object, digits = 3, ...) {
  
  varcomp <- with(object, cbind(est = c("within-case variance" = sigma_sq_w,
                                        "sample variance" = S_sq,
                                        "intra-class correlation" = rho,
                                        "auto-correlation" = phi),
                                se = c(NA, NA, NA, NA)))
  
  beta <- with(object, cbind(est = c("numerator of effect size estimate" = D_bar), se = c(NA)))
  
  
  ES <- with(object, cbind(est = c("unadjusted effect size" = delta_hat_unadj, 
                                   "adjusted effect size" = delta_hat,
                                   "degree of freedom" = nu, 
                                   "scalar constant" = theta),
                           se = c(sqrt(V_delta_hat) / J(nu), sqrt(V_delta_hat), NA, NA)))
  
  print(round(rbind(varcomp, beta, ES), digits), na.print = "")

}

#' @export

print.g_HPS <- function(x, digits = 3, ...) {
  
  ES <- with(x, cbind(est = c("unadjusted effect size" = delta_hat_unadj,
                              "adjusted effect size" = delta_hat,
                              "degree of freedom" = nu),
                           se = c(sqrt(V_delta_hat) / J(nu), sqrt(V_delta_hat), NA)))
  
  print(round(ES, digits), na.print = "")
}



##-----------------------------------------------------------------------------
## calculate HPS effect size (with associated estimates)
## Note: this is a vectorized version of the function for use in simulations 
##-----------------------------------------------------------------------------

HPS_effect_size <- function(outcomes, treatment, id, time) {
  
  ## setup ##
  
  # create factor variables
  treatment_fac <- factor(treatment)
  id_fac <- factor(id)
  time_fac <- factor(time)
  time_list <- by(time, id_fac, function(x) x)
  
  # unique times, unique cases, calculate sample sizes  
  time_points <- seq(min(time), max(time),1)    # unique measurement occasions j = 1,...,N
  N <- length(time_points)                      # number of unique measurement occasions
  h_i_p <- table(id_fac, treatment_fac)         # number of non-missing observations for case i in phase p
  cases <- levels(id_fac)                       # unique cases i = 1,...,m
  m <- length(cases)                            # number of cases
  g_dotdot <- length(treatment)                 # total number of non-missing observations
  
  
  ## calculate unadjusted effect size ##
  
  y <- as.matrix(outcomes)[,1]
  
  # fixed effects regression with id-by-treatment interaction
  case_FE <- lm(y ~ id_fac + id_fac:treatment_fac + 0)
  X_case_FE <- model.matrix(case_FE)                            # design matrix from id-by-treatment fixed-effects regression
  X_trt <- attr(X_case_FE, "assign") == 2                       # indicator for individual treatment effects
  XtX_inv_Xt_case_FE <- solve(t(X_case_FE) %*% X_case_FE) %*% t(X_case_FE)          #  (X'X)^-1 X'
  
  # calculate D-bar
  D_bar <- colMeans((XtX_inv_Xt_case_FE[X_trt,] %*% outcomes))
  
  # fixed effects regression with time-by-treatment interaction
  time_FE <- lm(y ~ time_fac + time_fac:treatment_fac + 0)
  X_time_FE <- model.matrix(time_FE)[,time_FE$qr$pivot[1:time_FE$qr$rank]]    # design matrix for time-by-treatment fixed-effects regression
  
  # calculate pooled variance S-squared
  
  A_mat <- diag(rep(1, g_dotdot)) - X_time_FE %*% solve(t(X_time_FE) %*% X_time_FE) %*% t(X_time_FE)  # S^2 = y'(A_mat)y / (g_dotdot - K). See p. 29. 
  K <- time_FE$rank                             # number of time-by-treatment groups containing at least one observation. See p. 11.
  S_sq <- colSums(outcomes * (A_mat %*% outcomes)) / (g_dotdot - K)
  
  # calculate unadjusted effect size
  delta_hat_unadj <- D_bar / sqrt(S_sq)         # See p. 13, Eq. (4)
  
  
  ## nuisance parameter estimates ##
  
  # auto-covariances - See first display equation on p. 32.
  residuals <- (diag(rep(1, g_dotdot)) - X_case_FE %*% XtX_inv_Xt_case_FE) %*% outcomes
  acv_SS <- rowSums(matrix(unlist(by(residuals, id_fac, function(x) colSums(x[1:(dim(x)[1]-1),] * x[2:dim(x)[1],]))), dim(outcomes)[2], m))
  SS_within <- colSums(residuals * residuals)
  
  # calculate adjusted autocorrelation
  phi_correction <- sum((h_i_p - 1) / h_i_p) / (g_dotdot - 2 * m)     # This is the constant C given on p. 33.
  phi_hat <- acv_SS / SS_within + phi_correction                      # See last display equation on p. 32.
  
  # calculate adjusted within-case variance estimate
  sigma_sq_correction <- g_dotdot - sapply(phi_hat, function(phi)
    product_trace(XtX_inv_Xt_case_FE, 
      prod_blockmatrix(AR1_corr_block(phi=phi, block=id_fac, times=time_list), 
                       X_case_FE)))
  
  # This correction is equal to g_dotdot * F, where F is given on p. 33. 
  sigma_sq_w <- SS_within / sigma_sq_correction
  
  # calculate intra-class correlation
  rho_hat <- pmax(0, 1 - sigma_sq_w / S_sq) # See last display equation on p. 33.
  
  
  ## calculate degrees of freedom and theta ##
  
  df_theta <- function(rho, phi) {
    V_mat <- lmeAR1_cov_block(block=id_fac, Z_design=rep(1,g_dotdot), 
                    theta = list(sigma_sq = 1 - rho, phi=phi, Tau = rho), 
                    times = time_list)
    AV <- prod_matrixblock(A_mat, V_mat, block=id_fac)
    nu <- (g_dotdot - K)^2 / product_trace(AV, AV)          # See p. 15, Eq. (11). Also note that product.trace(AV, AV) = tr(A Sigma A Sigma). See p. 30.
    theta <- sqrt(sum(diag(prod_matrixblock(XtX_inv_Xt_case_FE[X_trt,], V_mat, block=id_fac) %*% t(XtX_inv_Xt_case_FE[X_trt,])))) / m   # See p. 15, Eq. (10). 
    return(c(nu, theta))
  }
  
  df_theta <- mapply(df_theta, rho=rho_hat, phi=phi_hat)
  df <- df_theta[1,]
  theta <- df_theta[2,]
  
  ## adjusted effect size and variance ##
  
  # calculate adjusted effect size
  delta_hat <- J(df) * delta_hat_unadj    # See p. 15, Eq. (13). 
  
  # calculate variance of adjusted effect size
  df_trunc <- pmax(df, 2.001)
  V_delta_hat <- J(df)^2 * (theta^2 * df_trunc / (df_trunc - 2) + delta_hat^2 * (df_trunc / (df_trunc - 2) - 1 / J(df)^2))   # See p. 15, Eq. (14). 
  
  
  
  ## return results ##
  
  rbind(D_bar = D_bar, S_sq = S_sq, delta_hat_unadj = delta_hat_unadj, 
        phi_hat = phi_hat, rho_hat = rho_hat, kappa_hat = theta, df = df,
        delta_hat = delta_hat, V_delta_hat = V_delta_hat)
}

library(testthat)

data(Saddler)

Saddler_quality <- subset(Saddler, measure=="writing quality")


test_that("effect_size_MB results are the same as effect_size_MB_new for the Saddler data", {
  
  data1 <- effect_size_MB(Saddler_quality$outcome, 
                          Saddler_quality$treatment, 
                          Saddler_quality$case, 
                          Saddler_quality$time)
  
  data2 <- effect_size_MB(outcome, treatment, case, time, data = Saddler_quality)
  
  expect_equal(data1, data2)
 
})




test_that("effect_size_ABk results are the same as effect_size_ABk_new for the Saddler data", {
  
  
  data(Anglesea)
  
  Anglesea <- as.data.frame(Anglesea)
  
  
  data3 <- effect_size_ABk(outcome = Anglesea$outcome, 
                           treatment = Anglesea$condition, 
                           id = Anglesea$case,
                           phase = Anglesea$phase, 
                           time = Anglesea$session)
  
  data4 <- effect_size_ABk(data = Anglesea, 
                           outcome = outcome, 
                           treatment = condition, 
                           id = case, 
                           phase = phase, 
                           time = session)
  
  expect_equal(data3, data4)
  
})
  
  
  

context("Examples from HPS (2012,2013)")

data(Lambert)
Lambert <- subset(Lambert, measure == "disruptive behavior")

test_that("Example 1 from HPS 2012", {
  
  ES_L <- with(Lambert, effect_size_ABk(outcome, treatment, case, phase, time))
  expect_equal(23, ES_L$M_dot)
  expect_equal(-5.457995, round(ES_L$D_bar,6))
  expect_equal(4.673913, round(ES_L$S_sq, 6))
  expect_equal(-2.524601, round(ES_L$delta_hat_unadj, 6))
  expect_equal(0.224680, round(ES_L$phi, 6))
  expect_equal(4.534312, round(ES_L$sigma_sq_w, 6))
  expect_equal(0.029868, round(ES_L$rho, 6))
  expect_equal(0.144926, round(ES_L$theta, 6))
  expect_equal(164.492271, round(ES_L$nu, 6))
  expect_equal(-2.513073, round(ES_L$delta_hat, 6))
  expect_equal(0.04054616, round(ES_L$V_delta_hat, 8))
  
})

data(Anglesea)

test_that("Example 2 from HPS 2012", {
  
  ES_A <- with(Anglesea, effect_size_ABk(outcome, condition, case, phase, session))
  expect_equal(13, ES_A$M_dot)
  expect_equal(86.870476, round(ES_A$D_bar,6))
  expect_equal(2347.845708, round(ES_A$S_sq, 6))
  expect_equal(1.792823, round(ES_A$delta_hat_unadj, 6))
  expect_equal(0.175859, round(ES_A$phi, 6))
  expect_equal(198.369968, round(ES_A$sigma_sq_w, 6))
  expect_equal(0.915510, round(ES_A$rho, 6))
  expect_equal(0.091347, round(ES_A$theta, 6))
  expect_equal(2.340135, round(ES_A$nu, 6))
  expect_equal(1.149507, round(ES_A$delta_hat, 6))
  expect_equal(2.43956417, round(ES_A$V_delta_hat, 8))
  
})

data(Saddler)

test_that("Example 1 from HPS 2013", {
  
  ES_S1 <- with(subset(Saddler, measure=="writing quality"), 
                effect_size_MB(outcome, treatment, case, time))
  expect_equal(41, ES_S1$g_dotdot)
  expect_equal(13, ES_S1$K)
  expect_equal(2.097222, round(ES_S1$D_bar,6))
  expect_equal(0.952381, round(ES_S1$S_sq, 6))
  expect_equal(2.149013, round(ES_S1$delta_hat_unadj, 6))
  expect_equal(0.099650, round(ES_S1$phi, 6))
  expect_equal(0.349322, round(ES_S1$sigma_sq_w, 6))
  expect_equal(0.633212, round(ES_S1$rho, 6))
  expect_equal(0.200641, round(ES_S1$theta, 6))
  expect_equal(8.918146, round(ES_S1$nu, 6))
  expect_equal(1.963073, round(ES_S1$delta_hat, 6))
  expect_equal(0.33491289, round(ES_S1$V_delta_hat, 8))
  
  ES_S2 <- with(subset(Saddler, measure=="T-unit length"), 
                effect_size_MB(outcome, treatment, case, time))
  expect_equal(0.785400, round(ES_S2$delta_hat, 6))
  expect_equal(0.08023320, round(ES_S2$V_delta_hat, 8))
  
  ES_S3 <- with(subset(Saddler, measure=="number of constructions"), 
                effect_size_MB(outcome, treatment, case, time))
  expect_equal(0.747554, round(ES_S3$delta_hat, 6))
  expect_equal(0.07847359, round(ES_S3$V_delta_hat, 8))
  
})

data(Laski)

test_that("Example 2 from HPS 2013", {
  
  ES_Laski <- with(Laski, effect_size_MB(outcome, treatment, case, time))
  expect_equal(128, ES_Laski$g_dotdot)
  expect_equal(27, ES_Laski$K)
  expect_equal(31.822475, round(ES_Laski$D_bar,6))
  expect_equal(465.815861, round(ES_Laski$S_sq, 6))
  expect_equal(1.474439, round(ES_Laski$delta_hat_unadj, 6))
  expect_equal(0.016526, round(ES_Laski$phi, 6))
  expect_equal(142.555268, round(ES_Laski$sigma_sq_w, 6))
  expect_equal(0.693966, round(ES_Laski$rho, 6))
  expect_equal(0.102497, round(ES_Laski$theta, 6))
  expect_equal(13.100415, round(ES_Laski$nu, 6))
  expect_equal(1.388385, round(ES_Laski$delta_hat, 6))
  expect_equal(0.10050132, round(ES_Laski$V_delta_hat, 8))
  
})

data(Thorne)

test_that("Order doesn't matter for ABk designs", {
  
  Lambert_orig <- with(Lambert, effect_size_ABk(outcome, treatment, case, phase, time))
  Lambert_new <- with(Lambert[sample.int(nrow(Lambert)),], effect_size_ABk(outcome, treatment, case, phase, time))
  expect_identical(Lambert_orig, Lambert_new)
  
  Anglesea_orig <- with(Anglesea, effect_size_ABk(outcome, condition, case, phase, session))
  Anglesea_new <- with(Anglesea[sample.int(nrow(Anglesea)),], effect_size_ABk(outcome, condition, case, phase, session))
  expect_identical(Anglesea_orig, Anglesea_new)
  
  Thorne$phase_pair <- with(Thorne, substr(phase_id, 2, 2))
  Thorne_A <- subset(Thorne, measure == "Academic Engagement")
  Thorne_A_orig <- with(Thorne_A, effect_size_ABk(outcome, phase_indicator, case, phase_pair, session))
  Thorne_A_new <- with(Thorne_A[sample.int(nrow(Thorne_A)),], effect_size_ABk(outcome, phase_indicator, case, phase_pair, session))
  expect_identical(Thorne_A_orig, Thorne_A_new)
  
  Thorne_B <- subset(Thorne, measure == "Inappropriate Verbalizations")
  Thorne_B_orig <- with(Thorne_B, effect_size_ABk(outcome, phase_indicator, case, phase_pair, session))
  Thorne_B_new <- with(Thorne_B[sample.int(nrow(Thorne_B)),], effect_size_ABk(outcome, phase_indicator, case, phase_pair, session))
  expect_identical(Thorne_B_orig, Thorne_B_new)
  
})

test_that("Order doesn't matter for multiple baseline designs", {
  
  Saddler_1 <- subset(Saddler, measure=="writing quality")
  Saddler_orig <- with(Saddler_1, effect_size_MB(outcome, treatment, case, time))
  Saddler_new <- with(Saddler_1[sample.int(nrow(Saddler_1)),], effect_size_MB(outcome, treatment == "treatment", case, time))
  expect_identical(Saddler_orig, Saddler_new)
  
  Laski_orig <- with(Laski, effect_size_MB(outcome, treatment, case, time))
  Laski_new <- with(Laski[sample.int(nrow(Laski)),], effect_size_MB(outcome, treatment == "treatment", case, time))
  expect_identical(Laski_orig, Laski_new)
  
})

test_that("MB and ABk give similar results", {
  
  S1_MB <- with(subset(Saddler, measure=="writing quality"), effect_size_MB(outcome, treatment == "treatment", case, time))
  S1_ABk <- with(subset(Saddler, measure=="writing quality"), effect_size_ABk(outcome, treatment, case, rep(1, length(outcome)), time))
  expect_equal(S1_MB$D_bar, S1_ABk$D_bar)

  Laski_MB <- with(Laski, effect_size_MB(outcome, treatment == "treatment", case, time))
  Laski_ABk <- with(Laski, effect_size_ABk(outcome, treatment, case, rep(1, length(outcome)), time))
  expect_equal(Laski_MB$D_bar, Laski_ABk$D_bar)
  
})

test_that("summary and print work for effect_size_ABK() and effect_size_MB() objects", {
  Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
                                phase = phase, time = time, data= Lambert)
  expect_output(summary(Lambert_ES))
  expect_output(print(Lambert_ES))
  
  quality_ES <- effect_size_MB(outcome, treatment, case, time, 
                               data= subset(Saddler, measure=="writing quality"))
  expect_output(summary(quality_ES))
  expect_output(print(quality_ES))
})

context("Test scdhlm Shiny app")

skip_if_not_installed("shiny")
skip_if_not_installed("shinytest")
skip_if_not_installed("rvest")
skip_if_not_installed("ggplot2")
skip_if_not_installed("markdown")
skip_if_not_installed("readxl")
skip_if_not_installed("glue")
skip_if_not_installed("janitor")
skip_if_not_installed("rclipboard")

suppressWarnings(library(shiny))
suppressWarnings(library(shinytest))
suppressWarnings(library(rvest))
suppressWarnings(library(nlme))

skip_if_not(dependenciesInstalled())

appDir <- system.file("shiny-examples", "scdhlm", package = "scdhlm")

test_that("Title and tabs are correct", {
  
  skip_on_cran()
  
  app <- ShinyDriver$new(appDir, loadTimeout = 6e+05)
  
  # title
  appTitle <- app$getTitle()[[1]]
  expect_equal(appTitle, "Between-case standardized mean difference estimator")
  
  # tabs
  app$waitForValue("scdhlm_calculator")
  # app$findWidget("scdhlm_calculator")$listTabs()
  expect_equal(app$findWidget("scdhlm_calculator")$listTabs(), 
               c("scdhlm", "Load", "Inspect", "Model", "Effect size", "Syntax for R"))
  
})

# test the app output vs. README examples
check_readme <- function(data, estMethod, digits = 3) {
  app <- ShinyDriver$new(appDir, loadTimeout = 6e+05)
  
  app$setInputs(scdhlm_calculator = "Load")
  app$setInputs(example = data)
  app$setInputs(scdhlm_calculator = "Inspect")
  app$setInputs(corStruct = "hom")
  app$setInputs(scdhlm_calculator = "Model")
  app$setInputs(method = estMethod)
  app$setInputs(scdhlm_calculator = "Effect size")
  
  Sys.sleep(0.5)
  output <- app$getValue(name = "effect_size_report")
  app_output <- 
    read_html(output) |> 
    html_table(fill = TRUE) |>
    as.data.frame()
  
  app_output <- lapply(app_output, \(x) if (is.numeric(x)) round(x, digits) else x)

  return(app_output)
}

test_that("App output matches README example output", {
  
  skip_on_cran()
  
  # Laski
  app_output_Laski <- check_readme("Laski", estMethod = "RML")
  
  data("Laski")
  Laski_RML <- lme(fixed = outcome ~ treatment,
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = Laski)
  Laski_ES_RML <- g_mlm(Laski_RML, p_const = c(0, 1), r_const = c(1, 0, 1))
  pkg_output_Laski <- 
    data.frame(
      g_AB = Laski_ES_RML$g_AB,
      SE_g_AB = Laski_ES_RML$SE_g_AB,
      df = Laski_ES_RML$nu
    ) |> 
    round(digits = 3)
    
  # CI_Laski_RML <- CI_g(Laski_ES_RML, symmetric = TRUE)
  
  expect_equal(app_output_Laski$BC.SMD.estimate, pkg_output_Laski$g_AB)
  expect_equal(app_output_Laski$Std..Error, pkg_output_Laski$SE_g_AB)
  expect_equal(app_output_Laski$Degrees.of.freedom, pkg_output_Laski$df)
  
  
  # Lambert
  app_output_Lambert <- check_readme("Lambert", estMethod = "HPS")
  
  data("Lambert")
  Lambert_academic <- subset(Lambert, measure == "academic response")
  Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
                                phase = phase, time = time, data = Lambert_academic)
  pkg_output_Lambert <- 
    data.frame(
      g_AB = Lambert_ES$delta_hat,
      SE_g_AB = sqrt(Lambert_ES$V_delta_hat),
      df = Lambert_ES$nu
    ) |> 
    round(digits = 3)
  
  expect_equal(app_output_Lambert$BC.SMD.estimate, pkg_output_Lambert$g_AB)
  expect_equal(app_output_Lambert$Std..Error, pkg_output_Lambert$SE_g_AB)
  expect_equal(app_output_Lambert$Degrees.of.freedom, pkg_output_Lambert$df)
  
  # Saddler
  app_output_S <- check_readme("Saddler", estMethod = "HPS")
  
  data(Saddler)
  Saddler_quality <- subset(Saddler, measure=="writing quality")
  quality_ES <- effect_size_MB(outcome, treatment, case, time, data = Saddler_quality)
  pkg_output_S <- 
    data.frame(
      g_AB = quality_ES$delta_hat,
      SE_g_AB = sqrt(quality_ES$V_delta_hat),
      df = quality_ES$nu
    ) |> 
    round(digits = 3)
  
  expect_equal(app_output_S$BC.SMD.estimate, pkg_output_S$g_AB)
  expect_equal(app_output_S$Std..Error, pkg_output_S$SE_g_AB)
  expect_equal(app_output_S$Degrees.of.freedom, pkg_output_S$df)
  
})

# test the app output vs syntax output (RML)
check_syntax <- function(data, corStruct = "AR1", varStruct = "hom", digits = 4L) {
  app <- ShinyDriver$new(appDir, loadTimeout = 6e+05)
  
  app$setInputs(scdhlm_calculator = "Load")
  app$setInputs(example = data)
  app$setInputs(scdhlm_calculator = "Inspect")
  app$setInputs(scdhlm_calculator = "Model")
  # app$setInputs(degree_base = degree_base) # consider using if
  # app$setInputs(degree_trt = degree_trt)
  app$setInputs(corStruct = corStruct)
  app$setInputs(varStruct = varStruct)
  app$setInputs(scdhlm_calculator = "Effect size")
  app$setInputs(scdhlm_calculator = "Syntax for R")
  app$setInputs(clipbtn = "click")
  
  Sys.sleep(0.5)
  output <- app$getValue(name = "effect_size_report")
  summary_output <- 
    read_html(output) |>
    html_table(fill = TRUE) |>
    as.data.frame() |> 
    subset(select = c(g_AB = BC.SMD.estimate, SE_g_AB = Std..Error, df = Degrees.of.freedom)) |>
    round(digits = digits)

  names(summary_output) <- c("g_AB","SE_g_AB","df")
  
  raw_syntax <- app$getValue(name = "syntax")
  raw_syntax_cut <- sub("summary\\(ES_RML).*", "", raw_syntax)
  code_file <- tempfile(fileext = ".R")
  cat(raw_syntax_cut, file = code_file)
  source(code_file)
  
  pkg_output <- 
    data.frame(
      g_AB = ES_RML$g_AB,
      SE_g_AB = ES_RML$SE_g_AB,
      df = ES_RML$nu
    ) |> 
    round(digits = digits)
  
  return(identical(summary_output, pkg_output))
}

test_that("The summary table output matches the syntax results", {
  skip_on_cran()
  # AlberMorgan
  expect_true(check_syntax("AlberMorgan"))
  expect_true(check_syntax("AlberMorgan", corStruct = "AR1", varStruct = "het"))
  expect_true(check_syntax("AlberMorgan", corStruct = "MA1", varStruct = "hom"))
  expect_true(check_syntax("AlberMorgan", corStruct = "IID", varStruct = "het"))
  
  # BartonArwood
  expect_true(check_syntax("BartonArwood", corStruct = "MA1", varStruct = "hom"))
  expect_true(check_syntax("BartonArwood", corStruct = "IID", varStruct = "hom"))

  # Anglese
  expect_true(check_syntax("Anglese"))
  expect_true(check_syntax("Anglese", corStruct = "AR1", varStruct = "het"))
  expect_true(check_syntax("Anglese", corStruct = "MA1", varStruct = "hom"))
  expect_true(check_syntax("Anglese", corStruct = "IID", varStruct = "hom"))

  # GunningEspie
  expect_true(check_syntax("GunningEspie", corStruct = "AR1", varStruct = "het"))

  # Thiemann2001
  expect_true(check_syntax("Thiemann2001"))
  expect_true(check_syntax("Thiemann2001", corStruct = "AR1", varStruct = "het"))
  expect_true(check_syntax("Thiemann2001", corStruct = "MA1", varStruct = "hom"))
  expect_true(check_syntax("Thiemann2001", corStruct = "MA1", varStruct = "het"))
  expect_true(check_syntax("Thiemann2001", corStruct = "IID", varStruct = "hom"))
  expect_true(check_syntax("Thiemann2001", corStruct = "IID", varStruct = "het"))
  
  # Bryant2018
  expect_true(check_syntax("Bryant2018", corStruct = "AR1", varStruct = "het"))
  expect_true(check_syntax("Bryant2018", corStruct = "MA1", varStruct = "hom"))

})

context("HPS estimation functions")

test_that("effect_size_ABk works for BAB designs", {
  data(Carson)
  CarsonHPS <- with(Carson, effect_size_ABk(outcome, treatment, case, phase, time))
  expect_true(is.list(CarsonHPS))
})

test_that("effect_size_ABk works with treatment factors", {
  
})

test_that("effect_size_ABk works with id factors", {
  
})

test_that("effect_size_ABk works with phase factors", {
  
})

test_that("effect_size_MB works with treatment factors", {
  
})

test_that("effect_size_MB works with id factors", {
  
})

test_that("Vectorized version of effect_size_ABk returns identical results", {
  
})

test_that("Vectorized version of effect_size_MB returns identical results", {
  
})

#---------------------------------------------------------------
# calculate session-by-treatment interaction
#---------------------------------------------------------------

#' @title Calculate session-by-treatment interactions for a unique case
#'
#' @description Calculate session-by-treatment interactions based on phases and
#'   session numbering.
#'
#' @param phase vector of treatment indicators or a character or factor vector indicating unique treatment phases.
#' @param session numeric vector of measurement occasions.
#' @param trt_phase character string indicating the phase value corresponding to
#'   the treatment condition.
#'   
#' @export
#' 

session_by_treatment <- function(phase, session, trt_phase) {
  pmax(0, session - min(session[phase==trt_phase]) + 1)
}



#---------------------------------------------------------------
# calculate phase-pairs based on phases and session numbering
#---------------------------------------------------------------

#' @title Calculate phase-pairs for a unique case
#' 
#' @description Calculate phase-pairs based on phases and session numbering. 
#' 
#' @param phase vector of treatment indicators or a character or factor vector indicating unique treatment phases.
#' @param session numeric vector of measurement occasions.
#' @export 
#' 
#' @examples 
#' 
#' phases <- rep(c("A","B","A","B"), each = 4)
#' sessions <- 1:length(phases)
#' 
#' phase_pairs(phases, sessions)
#' 
#' phases <- rep(c("A","B","C","A","B","C","D"), each = 4)
#' phase_pairs(phases)
#' 
#' phases <- rep(c("B","A","C","B","A","B","C","A"), each = 4)
#' phase_pairs(phases)
#' 

phase_pairs <- function(phase, session = seq_along(phase)) {
  phase <- as.character(phase[order(session)])
  n <- length(phase)
  y <- rep(1L, n)
  condition_list <- phase[1]
  for (i in 2:n) {
    if (phase[i] == phase[i - 1]) {
      y[i] <- y[i - 1]
    } else if (!(phase[i] %in% condition_list)) {
      y[i] <- y[i - 1]
      condition_list <- c(condition_list, phase[i])
    } else {
      y[i] <- y[i - 1] + 1L
      condition_list <- phase[i]
    }
  }
  y[order(order(session))]
}

#' @title Clean Single Case Design Data
#' 
#' @description Clean single case design data for treatment reversal and multiple baseline designs. 
#' 
#' @param center Numeric value for the centering value for session. Default is 0.
#' @param round_session Logical indicating whether to round \code{session} to the nearest integer. Defaults to \code{TRUE}.
#' @inheritParams graph_SCD
#' 
#' @note If treatment_name is left null it will choose the second level of the phase variable to be the treatment phase.
#' 
#' @export 
#' 
#' @return A cleaned SCD dataset that can be used for model fitting and effect size calculation.
#' 
#' @examples
#' data(Laski)
#' preprocess_SCD(design = "MBP", 
#'                case = case, phase = treatment,
#'                session = time, outcome = outcome, 
#'                center = 4, data = Laski)
#'                
#' data(Anglesea) 
#' preprocess_SCD(design="TR",
#'                case=case, phase=condition,
#'                session=session, outcome=outcome,
#'                treatment_name = "treatment",
#'                data=Anglesea)
#'                
#' data(Thiemann2001)
#' preprocess_SCD(design = "RMBB", 
#'                case = case, series = series, phase = treatment, 
#'                session = time, outcome = outcome, 
#'                data = Thiemann2001)
#'           

preprocess_SCD <- function(design, 
                           case, phase, session, outcome, 
                           cluster = NULL, series = NULL,
                           center = 0, 
                           round_session = TRUE,
                           treatment_name = NULL,
                           data = NULL) {
  
  design <- match.arg(design, choices = c("MBP","TR","RMBB","CMB"))
  
  if (missing(case)) stop("Please specify a case variable.")
  if (missing(phase)) stop("Please specify a phase variable.")
  if (missing(session)) stop("Please specify a session variable.")
  if (missing(outcome)) stop("Please specify an outcome variable.")
  if (missing(design)) stop("Please specify a study design of 'MB' (multiple baseline), 
                            'TR' (treatment reversal),
                            'RMBB' (replicated multiple baseline across behaviors),
                            'CMB' (clustered multiple baseline).")
  if (design == "CMB" & missing(cluster)) stop("Please specify a cluster variable.")
  if (design == "RMBB" & missing(series)) stop("Please specify a series variable.")
  
  case_call <- substitute(case)
  phase_call <- substitute(phase)
  session_call <- substitute(session)
  outcome_call <- substitute(outcome)
  cluster_call <- substitute(cluster)
  series_call <- substitute(series)

  case_name <- deparse(case_call)
  phase_name <- deparse(phase_call)
  session_name <- deparse(session_call)
  outcome_name <- deparse(outcome_call)
  cluster_name <- deparse(cluster_call)
  series_name <- deparse(series_call)
  
  env <- if (!is.null(data)) list2env(data, parent = parent.frame()) else env <- parent.frame()
  case <- eval(case_call, env)
  phase <- eval(phase_call, env)
  session <- eval(session_call, env)
  outcome <- eval(outcome_call, env)
  cluster <- eval(cluster_call, env)
  series <- eval(series_call, env)
  
  
  # get treatment name
  if (is.null(treatment_name)) {
    treatment_name <- levels(as.factor(phase))[2]
  }
  
  # get the data
  if (design == "RMBB") {
    dat <- data.frame(case = as.factor(case),
                      series = as.factor(series),
                      phase = as.factor(phase),
                      session, 
                      outcome)
  } else if (design == "CMB") {
    dat <- data.frame(cluster = as.factor(cluster),
                      case = as.factor(case),
                      phase = as.factor(phase),
                      session, 
                      outcome)
  } else {
    dat <- data.frame(case = as.factor(case),
                      phase = as.factor(phase),
                      session, 
                      outcome)
  }
  
  # round session
  if (round_session == TRUE) {
    dat$session <- as.integer(round(dat$session))
  } else {
    dat$session <- as.integer(dat$session)
  }
  
  dat$trt <- as.numeric(dat$phase == treatment_name) # create trt variable

  by_var <- switch(design,
                   MBP = dat$case,
                   TR = dat$case,
                   RMBB = as.factor(paste(dat$case, dat$series, sep = "-")),
                   CMB = as.factor(paste(dat$cluster, dat$case, sep = "-")))

  session_by <- split(dat$session, by_var)
  phase_by <- split(dat$phase, by_var)
  
  if (design %in% c("MBP","RMBB","CMB")) {
    
    # calculate session-by-treatment interaction
    session_trt_by <- mapply(session_by_treatment, 
                             phase = phase_by, 
                             session = session_by,
                             MoreArgs = list(trt_phase = treatment_name), SIMPLIFY = FALSE)
    dat$session_trt <- unsplit(session_trt_by, by_var)
    dat$session <- dat$session - center
    names(dat)[which(names(dat) == "session_trt")] <- paste0(session_name, "_trt")
    
  } else if (design == "TR") {
    
    # calculate phase pairs for treatment reversal designs
    phase_pairs_by <- mapply(phase_pairs, phase = phase_by, session = session_by, SIMPLIFY = FALSE)
    dat$phase_pair <- unsplit(phase_pairs_by, dat$case)
    names(dat)[which(names(dat) == "phase_pair")] <- paste0(phase_name, "_pair")
    
  }

  dat <- dat[!is.na(dat$outcome), ]
  dat <- droplevels(dat)
  unique_sessions <- sapply(session_by, function(x) isTRUE(all.equal(x, unique(x))))
  
  if (!all(unique_sessions, na.rm = TRUE)) {
    stop("Session variable contains repeated values. Please ensure that each data point has a unique value within each case.")
  }

  if (design %in% c("MBP", "TR")) {
    names(dat)[1:4] <- c(case_name, phase_name, session_name, outcome_name)
  } else if (design == "RMBB") {
    names(dat)[1:5] <- c(case_name, series_name, phase_name, session_name, outcome_name)
  } else if (design == "CMB") {
    names(dat)[1:5] <- c(cluster_name, case_name, phase_name, session_name, outcome_name)
  }
  
  return(dat)
  
}

skip_if_not_installed("nlme")

library(nlme)


test_that("calc_BCSMD() effect size does not depend on centering time for Alber-Morgan data.", {
  data(AlberMorgan)
  
  cent <- 2
  dat <- preprocess_SCD(design = "MBP", 
                        case = case, 
                        phase = condition, 
                        session = session, 
                        outcome = outcome, 
                        center = cent, 
                        data = AlberMorgan)
  
  # Fit the model
  fit_RML <- lme(fixed = outcome ~ 1 + I(session^1) + trt + I(session_trt^1), 
                 random = ~ 1 + I(session^1) | case, 
                 correlation = corAR1(0.01, ~ session | case), 
                 data = dat,
                 control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  
  # Calculate effect size with g_mlm()
  A <- 5
  B <- 31
  p_const <- c(rep(0L, length(c(0,1))), (B - A)^as.integer(c(0,1))) 
  r_const <- c(c(1, 2 * (B - cent), (B - cent)^2), c(), 
               c(0), c(), 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors
  
  ES1 <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
  
  ES2 <- calc_BCSMD(design = "MBP",
                     case = case, 
                     phase = condition, 
                     session = session, 
                     outcome = outcome, 
                     center = cent, 
                     data = AlberMorgan,
                     FE_base = c(0,1),
                     RE_base = c(0,1),
                     FE_trt = c(0,1),
                     RE_trt = NULL,
                     A = 5, B = 31, 
                    summary = FALSE)
  
  expect_equal(ES1$g_AB, ES2$g_AB, tol = 1e-5)
  expect_equal(ES1$SE_g_AB, ES2$SE_g_AB, tol = 1e-5)
  
  ES3 <- calc_BCSMD(design = "MBP",
                    case = case, 
                    phase = condition, 
                    session = session, 
                    outcome = outcome, 
                    center = 20, 
                    data = AlberMorgan,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    FE_trt = c(0,1),
                    RE_trt = NULL,
                    A = 5, B = 31, 
                    summary = FALSE)

  expect_equal(ES2$g_AB, ES3$g_AB, tol = 1e-5)
  expect_equal(ES2$SE_g_AB, ES3$SE_g_AB, tol = 1e-5)
  
})


test_that("calc_BCSMD() effect size does not depend on centering time for Peltier data.", {
  data(Peltier)
  
  cent <- 13
  
  dat <- preprocess_SCD(design = "MBP", 
                        case = case, 
                        phase = condition, 
                        session = session, 
                        outcome = outcome, 
                        center = cent, 
                        data = Peltier)
  
  # Fit the model
  suppressWarnings(
    fit_RML <- lme(fixed = outcome ~ 1 + I(session^1) + trt + I(session_trt^1) + I(session_trt^2), 
                   random = ~ 1 + I(session^1) | case,  
                   data = dat,
                   control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  )

  # Calculate effect size with g_mlm()
  A <- 10
  B <- 16
  p_const <- c(rep(0L, length(c(0,1))), (B - A)^as.integer(c(0,1,2))) 
  r_const <- c(c(1, 2 * (B - cent), (B - cent)^2), c(), 
               c(), c(), 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors
  
  ES1 <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
  
  ES2 <- calc_BCSMD(design = "MBP",
                    case = case, 
                    phase = condition, 
                    session = session, 
                    outcome = outcome, 
                    center = cent, 
                    data = Peltier,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    FE_trt = c(0,1,2),
                    corStruct = "IID",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES1$g_AB, ES2$g_AB, tol = 1e-5)
  expect_equal(ES1$SE_g_AB, ES2$SE_g_AB, tol = 1e-5)
  
  ES3 <- calc_BCSMD(design = "MBP",
                    case = case, 
                    phase = condition, 
                    session = session, 
                    outcome = outcome, 
                    center = 30, 
                    data = Peltier,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    FE_trt = c(0,1,2),
                    corStruct = "IID",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES2$g_AB, ES3$g_AB, tol = 1e-5)
  expect_equal(ES2$SE_g_AB, ES3$SE_g_AB, tol = 1e-5)
  
})


test_that("calc_BCSMD() effect size does not depend on centering time for Bryant data.", {
  data(Bryant2018)
  
  cent <- 13
  
  dat <- preprocess_SCD(design = "CMB", 
                        case = case, 
                        phase = treatment, 
                        session = session, 
                        outcome = outcome, 
                        cluster = group,
                        center = cent,
                        data = Bryant2018)
  
  # Fit the model
  suppressWarnings(
    fit_RML <- lme(fixed = outcome ~ 1 + I(session^1) + trt + I(session_trt^1), 
                   random = list(group = ~ 1 + I(session^1), case = ~ 1 + I(session^1) + I(session_trt^1)), 
                   correlation = corAR1(0.01, ~ session | group / case), 
                   data = dat,
                   control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  )
  
  # Calculate effect size with g_mlm()
  A <- 1
  B <- 59
  p_const <- c(rep(0L, length(c(0,1))), (B - A)^as.integer(c(0,1))) 
  r_const <- c(c(1,2 * (B - cent),(B - cent)^2), c(), 
               c(1,2 * (B - cent),(B - cent)^2), c(0,0,0), 
               c(0), c(), 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors

  ES1 <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
  
  ES2 <- calc_BCSMD(design = "CMB",
                    case = case, 
                    phase = treatment, 
                    session = session, 
                    outcome = outcome, 
                    cluster = group,
                    center = cent, 
                    data = Bryant2018,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0,1),
                    FE_trt = c(0,1),
                    RE_trt = c(1),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES1$g_AB, ES2$g_AB, tol = 1e-5)
  expect_equal(ES1$SE_g_AB, ES2$SE_g_AB, tol = 1e-5)
  
  ES3 <- calc_BCSMD(design = "CMB",
                    case = case, 
                    phase = treatment, 
                    session = session, 
                    outcome = outcome, 
                    cluster = group,
                    center = cent, 
                    data = Bryant2018,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0),
                    FE_trt = c(0,1),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)
  
  ES4 <- calc_BCSMD(design = "CMB",
                    case = case, 
                    phase = treatment, 
                    session = session, 
                    outcome = outcome, 
                    cluster = group,
                    center = 1, 
                    data = Bryant2018,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0),
                    FE_trt = c(0,1),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES3$g_AB, ES4$g_AB, tol = 1e-3)
  expect_equal(ES3$SE_g_AB, ES4$SE_g_AB, tol = 1e-3)
  
  ES5 <- calc_BCSMD(design = "CMB",
                    case = case, 
                    phase = treatment, 
                    session = session, 
                    outcome = outcome, 
                    cluster = group,
                    center = 40, 
                    data = Bryant2018,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0),
                    FE_trt = c(0,1),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)

  expect_equal(ES3$g_AB, ES5$g_AB, tol = 1e-3)
  expect_equal(ES3$SE_g_AB, ES5$SE_g_AB, tol = 1e-3)
  
})

test_that("calc_BCSMD() effect size does not depend on centering time for Thiemann data.", {
  data(Thiemann2001)
  
  cent <- 20
  
  dat <- preprocess_SCD(design = "RMBB", 
                        case = case, 
                        series = series,
                        phase = treatment, 
                        session = time, 
                        outcome = outcome, 
                        center = cent,
                        data = Thiemann2001)
  
  # Fit the model
  suppressWarnings(
    fit_RML <- lme(fixed = outcome ~ 1 + I(time^1) + trt + I(time_trt^1) + I(time_trt^2), 
                   random = list(case = ~ 1 + trt, series = ~ 1 + I(time^1)), 
                   correlation = corAR1(0.01, ~ time | case / series), 
                   data = dat,
                   control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  )
  
  # Calculate effect size with g_mlm()
  A <- 5
  B <- 30
  p_const <- c(rep(0L, length(c(0,1))), (B - A)^as.integer(c(0,1,2))) 
  r_const <- c(c(1), c(0,0), 
               c(1,2 * (B - cent),(B - cent)^2), c(), 
               c(0), c(), 1L) # specify whether using random effects, cor struct, var struct, and level-1 errors
  
  ES1 <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
  
  ES2 <- calc_BCSMD(design = "RMBB", 
                    case = case, 
                    series = series,
                    phase = treatment, 
                    session = time, 
                    outcome = outcome,
                    center = cent, 
                    data = Thiemann2001,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0),
                    FE_trt = c(0,1,2),
                    RE_trt_2 = c(0),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES1$g_AB, ES2$g_AB, tol = 1e-5)
  expect_equal(ES1$SE_g_AB, ES2$SE_g_AB, tol = 1e-5)
  
  ES3 <- calc_BCSMD(design = "RMBB", 
                    case = case, 
                    series = series,
                    phase = treatment, 
                    session = time, 
                    outcome = outcome,
                    center = 10, 
                    data = Thiemann2001,
                    FE_base = c(0,1),
                    RE_base = c(0,1),
                    RE_base_2 = c(0),
                    FE_trt = c(0,1,2),
                    RE_trt_2 = c(0),
                    corStruct = "AR1",
                    A = A, B = B, 
                    summary = FALSE)
  
  expect_equal(ES2$g_AB, ES3$g_AB, tol = 1e-3)
  expect_equal(ES2$SE_g_AB, ES3$SE_g_AB, tol = 1e-3)
  
})

context("Preprocess_SCD function")

# MB design
MB_dat <- data.frame(
  Student = rep(LETTERS[1:4], each = 10),
  Phase = rep(rep(c("Base","Trt"), each = 5), 4),
  Time = rep(1:10, 4),
  Behavior = rnorm(40)
)

dat_dup_session <- MB_dat
dat_dup_session$Time <- rep(1:8, 5)

MB_dat_clean <- preprocess_SCD(case = Student, phase = Phase, 
                               session = Time, outcome = Behavior, 
                               design = "MBP", data = MB_dat)


test_that("The returned dataset is consistent with the input dataset for MBP designs.", {
  
  expect_error(preprocess_SCD(case = Student, phase = Phase, 
                              session = Time, outcome = Behavior, 
                              design = "MB", data = dat_dup_session))
  
  expect_equal(names(MB_dat), names(MB_dat_clean)[1:4])
  
  data("Laski")
  
  dat1_Laski <- preprocess_SCD(case = case, phase = treatment, 
                               session = time, outcome = outcome, 
                               design = "MBP", data = Laski)
  
  dat2_Laski <- preprocess_SCD(case = case, phase = treatment, 
                               session = time, outcome = outcome, 
                               design = "MBP", center = 4, data = Laski)
  
  dat3_Laski <- preprocess_SCD(case = Laski$case, phase = Laski$treatment,
                               session = Laski$time, outcome = Laski$outcome,
                               design = "MB")
  
  expect_error(preprocess_SCD(data = Laski))
  expect_equivalent(dat1_Laski, dat3_Laski)
  expect_equal(Laski$case, dat1_Laski$case)
  expect_equal(Laski$treatment, dat1_Laski$treatment)
  expect_equal(Laski$time, dat1_Laski$time)
  expect_equal(Laski$outcome, dat1_Laski$outcome)
  expect_equal(Laski$time, dat2_Laski$time + 4)
  
})

test_that("The returned dataset is consistent with the input dataset for TR designs.", {
  
  data("Anglesea")
  
  dat1_Ang <- preprocess_SCD(case = case, phase = condition,
                             session = session, outcome = outcome, 
                             design = "TR", data = Anglesea)
  
  dat2_Ang <- preprocess_SCD(case = Anglesea$case, phase = Anglesea$condition,
                             session = Anglesea$session, outcome = Anglesea$outcome, 
                             design = "TR")
  
  expect_equivalent(dat1_Ang, dat2_Ang)
  expect_equal(Anglesea$case, dat1_Ang$case)
  expect_equal(Anglesea$condition, dat1_Ang$condition)
  expect_equal(Anglesea$session, dat1_Ang$session)
  expect_equal(Anglesea$outcome, dat1_Ang$outcome)
  
  Ang_case <- Anglesea$case
  Ang_condition <- Anglesea$condition
  Ang_session <- Anglesea$session
  Ang_outcome <- Anglesea$outcome
  
  dat3_Ang <- preprocess_SCD(case = Ang_case, phase = Ang_condition,
                             session = Ang_session, outcome = Ang_outcome, 
                             design = "TR")
  expect_equivalent(dat1_Ang, dat3_Ang)
  
  
  
})

test_that("The returned dataset is consistent with the input dataset for RMBB designs.", {
  
  data("Thiemann2001")
  
  dat1_Thiemann <- preprocess_SCD(design = "RMBB",
                                  case = case, series = series,
                                  phase = treatment, session = time,
                                  outcome = outcome, data = Thiemann2001)
  
  expect_equal(Thiemann2001$case, dat1_Thiemann$case)
  expect_equal(Thiemann2001$series, dat1_Thiemann$series)
  expect_equal(Thiemann2001$treatment, dat1_Thiemann$treatment)
  expect_equal(Thiemann2001$time, dat1_Thiemann$time)
  expect_equal(Thiemann2001$outcome, dat1_Thiemann$outcome)

  dat2_Thiemann <- preprocess_SCD(design = "RMBB",
                                  case = Thiemann2001$case, series = Thiemann2001$series,
                                  phase = Thiemann2001$treatment, session = Thiemann2001$time,
                                  outcome = Thiemann2001$outcome)
  
  expect_equivalent(dat1_Thiemann, dat2_Thiemann)
  
  Thi_case <- Thiemann2001$case
  Thi_series <- Thiemann2001$series
  Thi_phase <- Thiemann2001$treatment
  Thi_session <- Thiemann2001$time
  Thi_outcome <- Thiemann2001$outcome
  
  dat3_Thiemann <- preprocess_SCD(design = "RMBB",
                                  case = Thi_case, series = Thi_series,
                                  phase = Thi_phase, session = Thi_session,
                                  outcome = Thi_outcome)
  
  expect_equivalent(dat1_Thiemann, dat3_Thiemann)
  
  scramble <- sample(1:nrow(Thiemann2001))
  
  dat4_Thiemann <- preprocess_SCD(design = "RMBB",
                                  case = case, series = series,
                                  phase = treatment, session = time,
                                  outcome = outcome, data = Thiemann2001[scramble,])
  
  expect_equivalent(dat1_Thiemann[scramble,], dat4_Thiemann)
  
})

test_that("The returned dataset is consistent with the input dataset for CMB designs.", {
  
  data("Bryant2018")
  
  dat1_Bry <- preprocess_SCD(design = "CMB",
                             cluster = group, case = case,
                             phase = treatment, session = session,
                             outcome = outcome, data = Bryant2018)
  Bryant <- na.omit(Bryant2018)
  expect_equal(Bryant$group, dat1_Bry$group)
  expect_equal(Bryant$case, dat1_Bry$case)
  expect_equal(Bryant$treatment, dat1_Bry$treatment)
  expect_equal(Bryant$session, dat1_Bry$session)
  expect_equal(Bryant$outcome, dat1_Bry$outcome)

  dat2_Bry <- preprocess_SCD(design = "CMB",
                             cluster = Bryant2018$group, case = Bryant2018$case,
                             phase = Bryant2018$treatment, session = Bryant2018$session,
                             outcome = Bryant2018$outcome)
  
  expect_equivalent(dat1_Bry, dat2_Bry)
  
  Bry_group <- Bryant2018$group
  Bry_case <- Bryant2018$case
  Bry_treatment <- Bryant2018$treatment
  Bry_session <- Bryant2018$session
  Bry_outcome <- Bryant2018$outcome
  
  dat3_Bry <- preprocess_SCD(design = "CMB",
                             cluster = Bry_group, case = Bry_case,
                             phase = Bry_treatment, session = Bry_session,
                             outcome = Bry_outcome)
  
  expect_equivalent(dat1_Bry, dat3_Bry)
  
  scramble <- sample(1:nrow(Bryant))

  dat4_Bry <- preprocess_SCD(design = "CMB",
                             cluster = group, case = case,
                             phase = treatment, session = session,
                             outcome = outcome, data = Bryant[scramble,])

  expect_equivalent(dat1_Bry[, 1:6], dat4_Bry[order(scramble), 1:6])
    
})

context("Graphing functions")

test_that("graph_SCD works for design = 'TR'", {
  
  skip_if_not_installed("ggplot2")

  Spriggs <- read.csv("../testdata/BC-SMD-Calculations.csv")  

  # Clean data
  dat <- preprocess_SCD(design = "TR", 
                        case = Participant, 
                        phase = Phase, 
                        session = Session, 
                        outcome = PercentageCorrect, 
                        round_session = FALSE, 
                        treatment_name = "B", 
                        data = Spriggs)
  
  # Fit the model
  fit_RML <- lme(fixed = PercentageCorrect ~ 1 + trt, 
                 random = ~ 1 | Participant, 
                 correlation = corAR1(0.01, ~ Session | Participant), 
                 data = dat,
                 control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  
  # Calculate effect size with g_mlm()
  p_const <- c(0,1)
  r_const <- c(1,0,1)
  
  ES_RML <- g_mlm(fit_RML, p_const = p_const, r_const = r_const, infotype = "expected")
  
  # Graph 
  p <- graph_SCD(design = "TR", 
                 case = Participant, phase = Phase, 
                 session = Session, outcome = PercentageCorrect, model_fit = fit_RML,  
                 data = dat)
  
  expect_s3_class(p, "ggplot")
  expect_invisible(print(p))
  
})

test_that("graph_SCD works for design = 'TR'", {
  
  skip_if_not_installed("ggplot2")
  
  data("Anglesea")
  
  Ang_graph1 <- graph_SCD(case=case, phase=condition, session=session, outcome=outcome, 
                        design="TR", treatment_name = NULL, model_fit=NULL, data=Anglesea)
  expect_s3_class(Ang_graph1, "ggplot")
  expect_invisible(print(Ang_graph1))
  
  Ang_graph2 <- graph_SCD(case=case, phase=condition, session=session, outcome=outcome, 
                         design="TR", treatment_name = "treatment", model_fit=NULL, data=Anglesea)
  expect_s3_class(Ang_graph2, "ggplot")
  expect_invisible(print(Ang_graph2))
  
  Ang_graph3 <- graph_SCD(case=Anglesea$case, phase=Anglesea$condition, session=Anglesea$session, 
                          outcome=Anglesea$outcome, design="TR", treatment_name = "treatment", model_fit=NULL)
  expect_s3_class(Ang_graph3, "ggplot")
  expect_invisible(print(Ang_graph3))
  
  Ang_graph4 <- graph_SCD(case=Anglesea$case, phase=Anglesea$condition, session=Anglesea$session, 
                         outcome=Anglesea$outcome, design="TR", model_fit=NULL)
  expect_s3_class(Ang_graph4, "ggplot")
  expect_invisible(print(Ang_graph4))
  
  Ang_case <- Anglesea$case
  Ang_condition <- Anglesea$condition
  Ang_session <- Anglesea$session
  Ang_outcome <- Anglesea$outcome
  
  Ang_graph5 <- graph_SCD(case=Ang_case, phase=Ang_condition, session=Ang_session, 
                          outcome=Ang_outcome, design="TR")
  expect_s3_class(Ang_graph5, "ggplot")
  expect_invisible(print(Ang_graph5))
  
  expect_equivalent(Ang_graph1$data, Ang_graph2$data)
  
  keys <- c("scales","theme","coordinates")
  expect_equal(Ang_graph1[keys], Ang_graph3[keys])
  expect_equal(Ang_graph1[keys], Ang_graph4[keys])
  expect_equal(Ang_graph1[keys], Ang_graph5[keys])
  
})

test_that("graph_SCD works for design = 'MBP'", {
  
  skip_if_not_installed("ggplot2")

  data("Laski")
  
  Laski_RML <- lme(fixed = outcome ~ 1 + treatment,
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = Laski)

  Laski_graph1 <- graph_SCD(case=case, phase=treatment, session=time, outcome=outcome, 
                            design="MBP", treatment_name = "treatment", model_fit=Laski_RML, data=Laski)
  expect_s3_class(Laski_graph1, "ggplot")
  expect_invisible(print(Laski_graph1))
  
  Laski_graph2 <- graph_SCD(case=case, phase=treatment, session=time, outcome=outcome, 
                            design="MBP", treatment_name = "treatment", model_fit=Laski_RML)
  expect_s3_class(Laski_graph2, "ggplot")
  expect_invisible(print(Laski_graph2))
  
  keys <- setdiff(names(Laski_graph1), c("data","plot_env", "labels","layers", "layout"))
  expect_equal(Laski_graph1[keys], Laski_graph2[keys])
  

  Laski_clean <- preprocess_SCD(design = "MBP", case=case, phase=treatment, session=time, outcome=outcome, data = Laski)
  Laski_trend <- lme(fixed = outcome ~ 1 + time_trt,
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = Laski_clean)
  
  Laski_graph3 <- graph_SCD(case=case, phase=treatment, session=time, outcome=outcome, 
                            design="MBP", treatment_name = "treatment", model_fit=Laski_trend, data=Laski_clean)
  expect_s3_class(Laski_graph3, "ggplot")
  expect_invisible(print(Laski_graph3))
  
  Laski_graph4 <- graph_SCD(case=case, phase=treatment, session=time, outcome=outcome, 
                            design="MBP", treatment_name = "treatment", model_fit=Laski_trend)
  expect_s3_class(Laski_graph4, "ggplot")
  expect_invisible(print(Laski_graph4))
  
  keys <- setdiff(names(Laski_graph1), c("data","plot_env", "labels","layers", "layout"))
  expect_equal(Laski_graph3[keys], Laski_graph4[keys])
  
})

test_that("graph_SCD works for design = 'RMBB'", {
  
  skip_if_not_installed("ggplot2")

  data("Thiemann2001")
  Thiemann2001_RML <- lme(outcome ~ 1 + time_c + treatment + trt_time,
                          random = ~ 1 | case / series,
                          data = Thiemann2001)
  
  # graph using data = 
  Thiemann_graph1 <- graph_SCD(design = "RMBB",
                               case = case, series = series,
                               phase = treatment, session = time, outcome = outcome,
                               treatment_name = "treatment", 
                               data = Thiemann2001)
  expect_s3_class(Thiemann_graph1, "ggplot")
  expect_invisible(print(Thiemann_graph1))
  
  # graph using vectors only
  Thiemann_graph2 <- graph_SCD(design = "RMBB",
                               case = Thiemann2001$case, series = Thiemann2001$series,
                               phase = Thiemann2001$treatment, session = Thiemann2001$time, 
                               outcome = Thiemann2001$outcome,
                               treatment_name = "treatment", model_fit = Thiemann2001_RML)
  expect_s3_class(Thiemann_graph2, "ggplot")
  expect_invisible(print(Thiemann_graph2))
  keys <- c("scales","theme","coordinates")
  expect_equal(Thiemann_graph1[keys], Thiemann_graph2[keys])
  
  # graph with model_fit = Thiemann2001_RML (with data)
  Thiemann_graph3 <- graph_SCD(design = "RMBB",
                               case = case, series = series,
                               phase = treatment, session = time, outcome = outcome,
                               treatment_name = "treatment", 
                               data = Thiemann2001, model_fit = Thiemann2001_RML)
  expect_s3_class(Thiemann_graph3, "ggplot")
  expect_invisible(print(Thiemann_graph3))
  
  # graph with model_fit = Thiemann2001_RML (without data)
  Thiemann_graph4 <- graph_SCD(design = "RMBB",
                               case = case, series = series,
                               phase = treatment, session = time, outcome = outcome,
                               treatment_name = "treatment", model_fit = Thiemann2001_RML)
  expect_s3_class(Thiemann_graph4, "ggplot")
  expect_invisible(print(Thiemann_graph4))
  
  keys <- setdiff(names(Thiemann_graph3), c("data","plot_env", "labels","layers", "layout"))
  expect_equal(Thiemann_graph3[keys], Thiemann_graph4[keys])
  
})

test_that("graph_SCD works for design = 'CMB'", {
  
  skip_if_not_installed("ggplot2")
  
  data("Bryant2018")
  
  Bryant2018_RML <-lme(fixed = outcome ~ treatment,
                       random = ~ 1 | group / case,
                       correlation = corAR1(0, ~ session | group / case),
                       weights = varIdent(form = ~ 1 | treatment),
                       data = Bryant2018,
                       na.action = na.omit)

  # graph using data = 
  Bry_graph1 <- graph_SCD(design = "CMB",
                          cluster = group, case = case,
                          phase = treatment, session = session, outcome = outcome,
                          treatment_name = "treatment", 
                          data = Bryant2018)
  expect_s3_class(Bry_graph1, "ggplot")
  expect_invisible(print(Bry_graph1))
  
  # graph using vectors only
  Bry_graph2 <- graph_SCD(design = "CMB",
                          cluster = Bryant2018$group, case = Bryant2018$case,
                          phase = Bryant2018$treatment, session = Bryant2018$session, 
                          outcome = Bryant2018$outcome,
                          treatment_name = "treatment", model_fit = Bryant2018_RML)
  expect_s3_class(Bry_graph2, "ggplot")
  expect_invisible(print(Bry_graph2))
  keys <- c("scales","theme","coordinates")
  expect_equal(Bry_graph1[keys], Bry_graph2[keys])
  
  # graph with model_fit (with data)
  Bry_graph3 <- graph_SCD(design = "CMB",
                          cluster = group, case = case,
                          phase = treatment, session = session, outcome = outcome,
                          treatment_name = "treatment", 
                          data = Bryant2018,
                          model_fit = Bryant2018_RML)
  expect_s3_class(Bry_graph3, "ggplot")
  expect_invisible(print(Bry_graph3))
  
  # graph with model_fit (without data)
  Bry_graph4 <- graph_SCD(design = "CMB",
                          cluster = group, case = case,
                          phase = treatment, session = session, outcome = outcome,
                          treatment_name = "treatment", model_fit = Bryant2018_RML)
  expect_s3_class(Bry_graph4, "ggplot")
  expect_invisible(print(Bry_graph4))
  
  keys <- setdiff(names(Bry_graph3), c("data","plot_env", "labels","layers", "layout", "guides"))
  expect_equal(Bry_graph3[keys], Bry_graph4[keys])
  
})


test_that("graph_SCD works with hypothetical newdata", {
  
  data("Anglesea")
  Anglesea_clean <- preprocess_SCD(design = "TR", 
                        case = case, 
                        phase = phase, 
                        session = session, 
                        outcome = outcome, 
                        data = Anglesea)
  
  # Fit the model
  suppressWarnings(
    Ang_RML <- lme(fixed = outcome ~ 1 + trt, 
                   random = ~ 1 + trt | case, 
                   correlation = corAR1(0.01, ~ session | case), 
                   data = Anglesea_clean,
                   control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE))
  )

  Ang_graph1 <- graph_SCD(case=case, phase=phase, session=session, outcome=outcome, 
                          design="TR", treatment_name = NULL, model_fit=Ang_RML)
  expect_s3_class(Ang_graph1, "ggplot")
  expect_invisible(print(Ang_graph1))
  
  Anglesea_mod <- Anglesea_clean
  Anglesea_mod$phase <- factor(1, levels = levels(Anglesea_clean$phase))
  Anglesea_mod$trt <- 0
  
  Ang_graph2 <- graph_SCD(case=case, phase=phase, session=session, outcome=outcome, 
                          design="TR", treatment_name = NULL, newdata = Anglesea_mod, model_fit=Ang_RML)
  expect_s3_class(Ang_graph2, "ggplot")
  expect_invisible(print(Ang_graph2))
  
  Ang_graph3 <- graph_SCD(case=case, phase=phase, session=session, outcome=outcome, 
                          design="TR", treatment_name = NULL, data = Anglesea_clean, newdata = Anglesea_mod, model_fit=Ang_RML)
  expect_s3_class(Ang_graph3, "ggplot")
  expect_invisible(print(Ang_graph3))
  
  data("Laski")
  
  Laski_RML <- lme(fixed = outcome ~ 1 + treatment,
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = Laski)
  Laski_mod <- Laski
  Laski_mod$treatment <- factor("baseline", levels = levels(Laski$treatment))
  
  Laski_graph <- graph_SCD(case=case, phase=treatment, session=time, outcome=outcome, 
                            design="MBP", treatment_name = "treatment", model_fit=Laski_RML, newdata=Laski_mod)
  expect_s3_class(Laski_graph, "ggplot")
  expect_invisible(print(Laski_graph))
  
  
  data("Thiemann2001")
  Thiemann2001_RML <- lme(outcome ~ 1 + time_c + treatment + trt_time,
                          random = ~ 1 | case / series,
                          data = Thiemann2001)
  Thiemann2001_mod <- Thiemann2001
  Thiemann2001_mod$treatment <- factor("baseline", levels = levels(Thiemann2001$treatment))
  Thiemann2001_mod$trt_time <- 0
  
  
  Thiemann_graph <- graph_SCD(design = "RMBB",
                               case = case, series = series,
                               phase = treatment, session = time, outcome = outcome,
                               treatment_name = "treatment", 
                               model_fit = Thiemann2001_RML,
                               newdata = Thiemann2001_mod)
  expect_s3_class(Thiemann_graph, "ggplot")
  expect_invisible(print(Thiemann_graph))
  
  data("Bryant2018")
  
  Bryant2018_RML <-lme(fixed = outcome ~ treatment,
                       random = ~ 1 | group / case,
                       correlation = corAR1(0, ~ session | group / case),
                       weights = varIdent(form = ~ 1 | treatment),
                       data = Bryant2018,
                       na.action = na.omit)
  Bryant2018_mod <- Bryant2018
  Bryant2018_mod$treatment <- factor("baseline", levels = levels(Bryant2018$treatment))
  
  Bry_graph <- graph_SCD(design = "CMB",
                          cluster = group, case = case,
                          phase = treatment, session = session, outcome = outcome,
                          treatment_name = "treatment", 
                          model_fit = Bryant2018_RML, newdata = Bryant2018_mod)
  expect_s3_class(Bry_graph, "ggplot")
  expect_invisible(print(Bry_graph))
  
})

#' @title A shiny interface for the scdhlm package
#'
#' @description An interactive shiny interface for estimating design-comparable
#'   standardized mean difference effect sizes from single-case designs.
#'   Estimation methods for multiple baseline and treatment reversal designs are
#'   available.
#'
#' @param dataset Optionally, a data.frame or path to a file from which to read
#'   data. If specified, the app will open with the data loaded. Default is
#'   NULL. If \code{dataset} is a data.frame, then it will be passed directly. If
#'   a file path with a \code{.xlsx} extension is specified, it will be read using \code{read_excel}.
#'   If a file path with a \code{.csv} extension is specified, it will be read using \code{read.csv}.
#'   If a file path with a different extension is specified, it will be read using \code{read.table}.
#' @param ... Further arguments passed to \code{read_excel}, \code{read.csv}, or \code{read.table}.
#'
#' @examples 
#' \dontrun{
#' shine_scd()
#' data(Laski)
#' shine_scd(dataset = Laski)
#' shine_scd(dataset = "SCD_data.xlsx", sheet = "Laski")
#' shine_scd(dataset = "Laski.csv") 
#' }
#' 
#' @export
#' 
#' @importFrom readxl read_excel
#' 

shine_scd <- function(dataset = NULL, ...) {
  
  req_pkgs <- c("shiny","ggplot2","markdown","glue","rclipboard","readxl","janitor")
  missing_pkgs <- unlist(lapply(req_pkgs, check_for_package))
  
  if (length(missing_pkgs) > 1) {
    missing_pkgs <- paste(missing_pkgs, collapse = ", ")
    stop(paste0("The scdhlm app requires the following packages: ", missing_pkgs,". Please install them."), call. = FALSE)
  } else if (length(missing_pkgs) == 1) {
    stop(paste("The scdhlm app requires the", missing_pkgs,"package. Please install it."), call. = FALSE)
  }
    
  uiDir <- system.file("shiny-examples/scdhlm", "ui.R", package = "scdhlm")
  serveDir <- system.file("shiny-examples/scdhlm", "server.R", package = "scdhlm")
  if (uiDir == "" | serveDir == "") {
    stop("Could not find the application directory. Try re-installing the scdhlm package.", call. = FALSE)
  }
  
  ui <- server <- NULL
  source(uiDir, local = TRUE, chdir = TRUE)
  source(serveDir, local = TRUE, chdir = TRUE)
  
  if (!is.null(dataset)) {
    if (!inherits(dataset, "data.frame")) {
      if (grepl(".xlsx$", dataset)) {
        dataset <- as.data.frame(readxl::read_xlsx(dataset, ...))
      } else if (grepl(".csv$", dataset)) {
        dataset <- utils::read.csv(dataset, ...)
      } else {
        dataset <- utils::read.table(dataset, ...)
      }
    } else if (inherits(dataset, "tbl")) {
      dataset <- as.data.frame(dataset)
    } 
    
    server_env <- environment(server)
    ui_env <- environment(ui)
    server_env$dataset <- dataset
    ui_env$dataset <- dataset
    
  } 
  
  app <- shiny::shinyApp(ui, server)
  shiny::runApp(app, display.mode = "normal", launch.browser = TRUE)
  
}

check_for_package <- function(pkg) {
  req <- requireNamespace(pkg, quietly = TRUE)
  if (!req) pkg else NULL
}

#' @import stats
#' @import nlme
#' 
NULL

#' Example 1 from Hedges, Pustejovsky, & Shadish (2012)
#' 
#' Data from an ABAB design conducted by Lambert, Cartledge, Heward, & Lo (2008). 
#' The variables are as follows:
#' \itemize{
#'   \item \code{measure}. Outcome measure description (disruptive behavior or academic response).
#'   \item \code{case}. Student identifier.  
#'   \item \code{treatment}. Factor indicating treatment or control condition. 
#'         SSR = single-subject responding. RC = response cards.
#'   \item \code{phase}. Study phase (including both control and treatment condition)
#'   \item \code{time}. Measurement occasion.
#'   \item \code{outcome}. Outcome scores.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 461 rows and 6 variables
#' @name Lambert
#' @source Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). 
#' Effects of response cards on disruptive behavior and academic responding 
#' during math lessons by fourth-grade urban students. 
#' \emph{Journal of Positive Behavior Interventions, 8}(2), 88-99. 
#' \doi{10.1177/10983007060080020701}
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
#' A standardized mean difference effect size for single case designs. 
#' \emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
NULL

#' Example 2 from Hedges, Pustejovsky, & Shadish (2012)
#' 
#' Data from an ABAB design conducted by Anglesea, Hoch, & Taylor (2008). 
#' The variables are as follows:
#' \itemize{
#'   \item \code{case} Case identifier.  
#'   \item \code{condition} Factor indicating baseline or treatment condition
#'   \item \code{phase} Study phase (including both control and treatment condition)
#'   \item \code{session} Measurement occasion
#'   \item \code{outcome} Total seconds of eating time
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 55 rows and 5 variables
#' @name Anglesea
#' @source Anglesea, M. M., Hoch, H., & Taylor, B. A. (2008). 
#' Reducing rapid eating in teenagers with autism: Use of a pager prompt. 
#' \emph{Journal of Applied Behavior Analysis, 41}(1), 107-111. 
#' \doi{10.1901/jaba.2008.41-107}
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
#' A standardized mean difference effect size for single case designs. 
#' \emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
NULL

#' Example 1 from Hedges, Pustejovsky, & Shadish (2013)
#' 
#' Data from a multiple baseline design conducted by 
#' Saddler, Behforooz, & Asaro, (2008). The variables are as follows:
#' \itemize{
#'   \item \code{case} Student identifier  
#'   \item \code{measure} Factor indicating the outcome measure (writing quality, T-unit length, number of constructions)
#'   \item \code{outcome} Value of outcome measure.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{treatment}. Factor indicating the treatment phase.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 124 rows and 5 variables
#' @name Saddler
#' @source Saddler, B., Behforooz, B., & Asaro, K. (2008). 
#' The effects of sentence-combining instruction on the writing of 
#' fourth-grade students with writing difficulties. 
#' \emph{The Journal of Special Education, 42}(2), 79-90. 
#' \doi{10.1177/0022466907310371}
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). 
#' A standardized mean difference effect size for multiple baseline designs across individuals. 
#' \emph{Research Synthesis Methods, 4}(4), 324-341. \doi{10.1002/jrsm.1086}

NULL

#' Example 2 from Hedges, Pustejovsky, & Shadish (2013)
#' 
#' Data from a multiple baseline design conducted by 
#' Laski, Charlop, & Schreibman (1988). The variables are as follows:
#' \itemize{
#'   \item \code{case}. Child identifier.  
#'   \item \code{outcome}. Frequency of child vocalization, as measured by 
#'   a partial interval recording procedure with 60 ten-second intervals per session.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{treatment}. Indicator for treatment phase.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 128 rows and 4 variables
#' @name Laski
#' @source Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). 
#' Training parents to use the natural language paradigm to increase 
#' their autistic children's speech. 
#' \emph{Journal of Applied Behavior Analysis, 21}(4), 391-400.
#' \doi{10.1901/jaba.1988.21-391}
#' 
#' @references Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). 
#' A standardized mean difference effect size for multiple baseline designs across individuals. 
#' \emph{Research Synthesis Methods, 4}(4), 324-341. \doi{10.1002/jrsm.1086}

NULL


#' Example from Pustejovsky, Hedges, & Shadish (2014)
#' 
#' Data from a multiple baseline design conducted by 
#' Schutte, Malouff, & Brown (2008). Case 4 is excluded because nearly all 
#' of these measurements are at the upper extreme of the scale. The variables are as follows:
#' \itemize{
#'   \item \code{case}. Participant identifier.  
#'   \item \code{week}. Measurement occasion.
#'   \item \code{treatment}. Factor indicating baseline or treatment phase.
#'   \item \code{fatigue}. Fatigue severity scale scores.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 136 rows and 4 variables
#' @name Schutte
#' @source Schutte, N. S., Malouff, J. M., & Brown, R. F. (2008). 
#' Efficacy of an emotion-focused treatment for prolonged fatigue. 
#' \emph{Behavior Modification, 32}(5), 699-713. \doi{10.1177/0145445508317133}
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
NULL

#' Thorne (2005)
#'
#' Data from an ABAB design conducted by Thorne and Kamps (2008). The variables
#' are as follows: 
#' \itemize{ 
#'   \item \code{case}. Participant identifier. 
#'   \item \code{measure}. Outcome measure description (academic engagement or inappropriate verbalizations). 
#'   \item \code{session}. Measurement occasion.
#'   \item \code{phase_id}. Categorical variable describing the phase of the study design for each case. 
#'   \item \code{condition} Categorical variable describing whether each phase is a baseline (A) phase or intervention (B) phase. 
#'   \item \code{phase_indicator}. Indicator variable equal to 1 during intervention phases. 
#'   \item \code{outcome}. Outcome scores 
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 776 rows and 7 variables
#' @name Thorne
#' @source Thorne, S., & Kamps, D. (2008). The effects of a group contingency
#'   intervention on academic engagement and problem behavior of at-risk
#'   students. \emph{Behavior Analysis in Practice, 1}(2), 12-18.
#'   \doi{10.1007/BF03391723}
NULL

#' Carson (2008)
#' 
#' Data from a BAB design conducted by Carson, Gast, & Ayres (2008). The
#' variables are as follows: 
#' \itemize{ 
#'    \item \code{case} Participant identifier 
#'    \item \code{treatment} Factor describing the treatment condition
#'    \item \code{phase} Numeric describing the phase of the study design for each case 
#'    \item \code{outcome} Outcome scores 
#'    \item \code{time} Measurement occasion 
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 47 rows and 5 variables
#' @name Carson
#' @source Carson, K. D., Gast, D. L., & Ayres, K. M. (2008). Effects of a photo
#'   activity schedule book on independent task changes by students with
#'   intellectual disabilities in community and school job sites. \emph{European
#'   Journal of Special Needs Education, 23}, 269-279. 
#'   
NULL

#' Musser (2001)
#' 
#' Data from a multiple baseline design conducted by 
#' Musser, Bray, Kehle, and Jenson (2001). The variables are as follows:
#' \itemize{
#'   \item \code{student} Participant identifier
#'   \item \code{session} Measurement occasion
#'   \item \code{outcome} Percentage of disruptive intervals
#'   \item \code{treatment} Factor indicating baseline, treatment, or follow-up phase
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 136 rows and 4 variables
#' @name Musser
#' @source Musser, E. H., Bray, M. A., Kehle, T. J., & Jenson, W. R. (2001).
#' Reducing disruptive behaviors in students with serious emotional disturbance.
#' School Psychology Review, 30(2), 294-304.
NULL

#' Barton-Arwood, Wehby, & Falk (2005)
#' 
#' Data from a multiple baseline design conducted by Barton-Arwood, Wehby, and
#' Falk (2005). The variables are as follows: \itemize{ \item \code{case}
#' Participant identifier \item \code{condition} Factor identifying
#' the phase of the design (A or B) \item \code{session} Measurement occasion 
#' \item \code{outcome} Oral reading fluency score (words per minute) }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 143 rows and 4 variables
#' @name BartonArwood
#' @source Barton-Arwood, S. M., Wehby, J. H., & Falk, K. B. (2005). Reading
#'   instruction for elementary-age students with emotional and behavioral
#'   disorders: Academic and behavioral outcomes. \emph{Exceptional Children, 72}(1),
#'   7-27. \doi{10.1177/001440290507200101}
#'   
NULL

#' Rodriguez & Anderson (2014)
#' 
#' Data from a multiple baseline design conducted by Rodriguez and Anderson (2014). 
#' The variables are as follows: \itemize{ 
#'   \item \code{case} Participant identifier 
#'   \item \code{condition} Factor identifying the phase of the design (A or B) 
#'   \item \code{session} Measurement occasion 
#'   \item \code{outcome} Percentage of intervals with problem behavior 
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 148 rows and 4 variables
#' @name Rodriguez
#' @source Rodriguez, B. J., & Anderson, C. M. (2014). Integrating a social
#'   behavior intervention during small group academic instruction using a total
#'   group criterion intervention. \emph{Journal of Positive Behavior Interventions,
#'   16}(4), 234-245. \doi{10.1177/1098300713492858}
#'   
NULL

#' Romaniuk (2002)
#' 
#' Data from a treatment reversal design conducted by Romaniuk and colleagues (2002).
#' The variables are as follows: \itemize{ 
#'   \item \code{case} Participant identifier 
#'   \item \code{phase} Factor identifying the phase of the design
#'   \item \code{condition} Factor identifying the treatment condition 
#'   \item \code{session} Measurement occasion 
#'   \item \code{outcome} Problem behavior
#'   \item \code{measurement} Character string describing how problem behavior was measured
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 148 rows and 4 variables
#' @name Romaniuk
#' @source Romaniuk, C., Miltenberger, R., Conyers, C., Jenner, N., Jurgens, M.,
#'   & Ringenberg, C. (2002). The influence of activity choice on problem
#'   behaviors maintained by escape versus attention. \emph{Journal of Applied
#'   Behavior Analysis, 35}(4), 349-62. \doi{10.1901/jaba.2002.35-349}
#'   
#'   
NULL

#' Alber-Morgan, et al. (2007)
#' 
#' Data from a multiple baseline design conducted by Alber-Morgan, Ramp,
#' Anderson, & Martin (2007). The variables are as follows: \itemize{ \item
#' \code{case} Participant identifier \item \code{condition} Factor identifying
#' the phase of the design (baseline or treatment) \item \code{session}
#' Measurement occasion \item \code{outcome} Number of words read correctly per
#' minute }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 119 rows and 4 variables
#' @name AlberMorgan
#' @source Alber-Morgan, S. R., Ramp, E. M., Anderson, L. L., & Martin, C. M.
#'   (2007). Effects of repeated readings, error correction, and performance
#'   feedback on the fluency and comprehension of middle school students with
#'   behavior problems. Journal of Special Education, 41(1), 17-30.
#'   \doi{10.1177/00224669070410010201}
#'   
#'   
NULL


#' Salazar, et al. (2020)
#' 
#' Data from a multiple baseline design conducted by Salazar, Ruiz,
#' Ramírez1, &  Cardona-Betancourt (2020). The variables are as follows:
#' \itemize{
#'   \item \code{case}. Participant identifier.
#'   \item \code{measure}. Outcome measure description (AFQ-Y, PTQ-C, or GPQ-C).
#'   \item \code{treatment} Factor indicating baseline, treatment, post, or follow-up phase.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{outcome}. Outcome scores.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 324 rows and 5 variables
#' @name Salazar
#' 
#' @source Salazar, D., Ruiz, F., Ramírez, E., & Cardona-Betancourt, V. (2020).
#'   Acceptance and Commitment Therapy Focused on Repetitive Negative Thinking for
#'   Child Depression: A Randomized Multiple-Baseline Evaluation. The Psychological
#'   Record. \doi{10.1007/s40732-019-00362-5}
#'
NULL

#' Bryant et al. (2018)
#'
#' Data from a clustered multiple baseline across participants design conducted by
#' Bryant et al. (2018). The variables are as follows:
#' \itemize{
#'   \item \code{Study_ID}. Study identifier.
#'   \item \code{school}. School identifier.
#'   \item \code{group}. Group identifier.
#'   \item \code{case}. Student identifier.
#'   \item \code{treatment}. Indicator for treatment phase.
#'   \item \code{session}. Measurement occasion.
#'   \item \code{session_trt}. Measurement occasion times treatment phase.
#'   \item \code{outcome}. Texas Early Mathematics Inventory (TEMI-Aim Check) scores.
#'   \item \code{session_c}. Measurement occasion centered at the follow-up time.
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 536 rows and 8 variables
#' @name Bryant2018
#' @source Bryant, D. R., Bryant, B. R., Sorelle-Miner, D. A., Falcomata, T. S.
#' & Nozari, M. (2018).
#' Tier 3 intensified intervention for second grade students with severe
#' mathematics difficulties. \emph{Archives of Psychology, 2}(11), 1-24.
#' \doi{10.31296/aop.v2i11.86}
#'
NULL

#' Thiemann & Goldstein (2001)
#'
#' Data from a multiple baseline across behaviors design conducted by
#' Thiemann & Goldstein (2001). The variables are as follows:
#' \itemize{
#'   \item \code{Study_ID}. Study identifier.
#'   \item \code{case}. Student identifier.
#'   \item \code{series}. Series identifier.
#'   \item \code{outcome}. Frequency of coded social communication skills, as measured by
#'   a direct observation coding system with 15-second intervals recording for the occurrence
#'   of any of the four social measures: contingent responses, securing attention, initiating
#'   comments, and initiating requests.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{treatment}. Indicator for treatment phase.
#'   \item \code{trt_time}. Measurement occasion times treatment phase.
#'   \item \code{time_c}. Measurement occasion centered at the follow-up time.
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 363 rows and 8 variables
#' @name Thiemann2001
#' @source Thiemann, K.S., & Goldstein, H. (2001).
#' Social stories, written text cues, and video feedback: effects on social communication
#' of children with Autism. \emph{Journal of Applied Behavior Analysis, 34}(4), 425-446.
#' \doi{10.1901/jaba.2001.34-425}
#'
NULL

#' Ruiz, et al. (2020)
#' 
#' Data from a multiple baseline design conducted by  Ruiz,
#' Luciano, Florez, Suarez-Falcon, &  Cardona-Betancourt (2020). The variables are as follows:
#' \itemize{
#'   \item \code{case}. Participant identifier.
#'   \item \code{measure}. Outcome measure description (AAQ-II, ANXIETY, CFQ, DASS-TOTAL, DEPRESSION, PSWQ, PTQ, STRESS, VQ-OBSTRUCTION, or VQ-PROGRESS).
#'   \item \code{treatment} Factor indicating baseline, treatment, post, or follow-up phase.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{outcome}. Outcome scores.
#' }
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame with 840 rows and 5 variables
#' @name Ruiz
#' 
#' @source Ruiz, F., Luciano, C., Florez, C., Suarez-Falcon, J., & Cardona-Betancourt, V. (2020).
#'   A Multiple-Baseline Evaluation of Acceptance and Commitment Therapy Focused on Repetitive Negative 
#'   Thinking for Comorbid Generalized Anxiety Disorder and Depression. Frontiers in Psychology, 11. 
#'   \doi{10.3389/fpsyg.2020.00356}
#'
#'
NULL


#' Thiemann & Goldstein (2004)
#'
#' Data from a multiple baseline across behaviors design conducted by
#' Thiemann & Goldstein (2004). The variables are as follows:
#' \itemize{
#'   \item \code{Study_ID}. Study identifier.
#'   \item \code{case}. Student identifier.
#'   \item \code{series}. Series identifier.
#'   \item \code{outcome}. Frequency of unprompted targeted social communication skills,
#'   as measured by a direct observation, paper and pencil coding system during the 10-minute
#'   social activity for each behavior for all sessions.
#'   \item \code{time}. Measurement occasion.
#'   \item \code{treatment}. Indicator for treatment phase.
#'   \item \code{trt_time}. Measurement occasion times treatment phase.
#'   \item \code{time_c}. Measurement occasion centered at the follow-up time.
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 408 rows and 8 variables
#' @name Thiemann2004
#' @source Thiemann, K.S., & Goldstein, H. (2004).
#' Effects of peer training and written text cueing on social communication of
#' school-age children with pervasive developmental disorder. \emph{Journal of Speech Language 
#' and Hearing Research, 47}(1), 126-144. \doi{10.1044/1092-4388(2004/012)}
#'
NULL


#' Case, Harris, and Graham (1992)
#'
#' Data from a multiple baseline design conducted by Case, Harris, and Graham
#' (1992). The variables are as follows: \itemize{ \item \code{case}.
#' Participant identifier. \item \code{session}. Measurement occasion. \item
#' \code{condition}. Factor identifying the phase of the design (baseline or
#' treatment). \item \code{outcome}. Number of subtraction equations and answers
#' correct on each word problem probe. }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 56 rows and 4 variables
#' @name CaseHarrisGraham
#' @source Case, L. P., Harris, K. R., & Graham, S. (1992). Improving the
#'   mathematical problem-solving skills of students with learning disabilities:
#'   Self-regulated strategy development. \emph{The Journal of Special Education,
#'   26}(1), 1-19. \doi{10.1177/002246699202600101}
#'
#'   
NULL

#' Peltier et al. (2020)
#'
#' Data from a multiple baseline design conducted by Peltier, Sinclair, Pulos, &
#' Suk (2020). The variables are as follows: \itemize{ \item \code{case}.
#' Participant identifier. \item \code{session}. Measurement occasion. \item
#' \code{condition}. Factor identifying the phase of the design (baseline or
#' treatment). \item \code{outcome}. Mathematical problem-solving performance
#' (percentage). }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 232 rows and 4 variables
#' @name Peltier
#' @source Peltier, C., Sinclair, T. E., Pulos, J. M., & Suk, A. (2020). Effects
#'   of schema-based instruction on immediate, generalized, and combined
#'   structured word problems. \emph{The Journal of Special Education, 54}(2),
#'   101-112. \doi{10.1177/0022466919883397}
#'
#'   
NULL

#' Delemere & Dounavi (2018)
#'
#' Data from a multiple baseline design conducted by Delemere & Dounavi (2018).
#' The variables are as follows: 
#' \itemize{ 
#'   \item \code{intervention}. Type of intervention received: bedtime fading or positive routines.
#'   \item \code{case}. Participant identifier. 
#'   \item \code{session}. Measurement occasion. 
#'   \item \code{condition}. Factor identifying the phase of the design (baseline or treatment). 
#'   \item \code{outcome}. Total sleep onset latency in minutes across nights. 
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 187 rows and 5 variables
#' @name DelemereDounavi
#' @source Delemere, E., & Dounavi, K. (2018). Parent-implemented bedtime fading
#'   and positive routines for children with autism spectrum disorders.
#'   \emph{Journal of Autism and Developmental Disorders, 48}(4), 1002-1019.
#'   \doi{10.1007/s10803-017-3398-4}
#'
#'   
NULL

#' Gunning & Espie (2003)
#'
#' Data from a multiple baseline design conducted by Gunning & Espie (2003). The
#' variables are as follows: 
#' \itemize{ 
#'   \item \code{case}. Participant identifier.  
#'   \item \code{session}. Measurement occasion. 
#'   \item \code{condition}. Factor identifying the phase of the design (baseline or treatment). 
#'   \item \code{outcome}. Sleep onset latency in minutes. 
#' }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 301 rows and 4 variables
#' @name GunningEspie
#' @source Gunning, M. J., & Espie, C.A. (2003). Psychological treatment of
#'   reported sleep disorder in adults with intellectual disability using a
#'   multiple baseline design. \emph{Journal of Intellectual Disability
#'   Research, 47}(3), 191-202. \doi{10.1046/j.1365-2788.2003.00461.x}
#'
#'   
NULL

#' Datchuk (2016)
#'
#' Data from a multiple baseline design conducted by Datchuk (2016). The
#' variables are as follows: \itemize{ \item \code{case}. Participant
#' identifier. \item \code{session}. Measurement occasion. \item
#' \code{condition}. Factor identifying the phase of the design (baseline or
#' treatment). \item \code{outcome}. Correct word sequences per minute. }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 74 rows and 4 variables
#' @name Datchuk
#' @source Datchuk, S. M. (2016). Writing simple sentences and descriptive
#'   paragraphs: Effects of an intervention on adolescents with writing
#'   difficulties. \emph{Journal of Behavioral Education, 25}(2), 166-188.
#'   \doi{10.1007/s10864-015-9236-x}
#'   
NULL

#' Rodgers et al. (2021)
#'
#' Data from a multiple baseline design conducted by Rodgers, Datchuk, & Rila
#' (2021). The variables are as follows: \itemize{ \item \code{case}.
#' Participant identifier. \item \code{session}. Measurement occasion. \item
#' \code{condition}. Factor identifying the phase of the design (baseline or
#' treatment). \item \code{outcome}. The number of correct writing sequences in
#' 1 minute. }
#'
#' @docType data
#' @keywords datasets
#' @format A data frame with 83 rows and 4 variables
#' @name Rodgers
#' @source Rodgers, D. B., Datchuk, S. M., & Rila, A. L. (2021).  Effects of a
#'   text-writing fluency intervention for postsecondary students with
#'   intellectual and developmental disabilities. \emph{Exceptionality, 29}(4),
#'   310-325. \doi{10.1080/09362835.2020.1850451}
#'   
NULL



#--------------------------
# Simulation results
#--------------------------



#' MB1 simulation results
#' 
#' Simulation results for model MB1 from  Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB1results
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
NULL

#' MB2 simulation results
#' 
#' Simulation results for model MB2 from  Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB2results
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
NULL

#' MB4 simulation results
#' 
#' Simulation results for model MB4 from  Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB4results
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
NULL

#' MB1 simulation time
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB1time
NULL

#' MB2 simulation time
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB2time
NULL

#' MB4 simulation time
#' 
#' @docType data
#' @keywords datasets
#' @format A data frame
#' @name MB4time
NULL

library(shiny)
library(markdown)
library(ggplot2)
library(scdhlm)
rm(list=ls())
source("inst/shiny-examples/scdhlm/mappings.R")
source("inst/shiny-examples/scdhlm/graphing-functions.R")
source("inst/shiny-examples/scdhlm/helper-functions.R")
source("inst/shiny-examples/scdhlm/lme-fit.R")

datFile <- read.table("auxilliary/Alber-Morgan-2007.csv", 
                      header=TRUE, sep=",", quote="",
                      stringsAsFactors = FALSE)

phases <- levels(as.factor(datFile$phase))

case_vec <- datFile$case
caseID <- factor(case_vec, levels = unique(case_vec))
session <- as.numeric(datFile$session)
phaseID <- as.factor(datFile$phase)
outcome <- as.numeric(datFile$outcome)
dat <- data.frame(case = caseID, session = session, phase = phaseID, outcome = outcome)
trt_phase <- "treatment"
dat$trt <- as.numeric(dat$phase==trt_phase)
dat$session_trt <- unlist(by(dat, dat$case, session_by_treatment, trt_phase = trt_phase))


FE_base <- c(0,1)
RE_base <- c(0,1)
FE_trt <- c(0)
RE_trt <- c(0)

model_fit <- lme_fit_MB(dat = dat,
                        FE_base = FE_base,
                        RE_base = RE_base,
                        FE_trt = FE_trt,
                        RE_trt = RE_trt,
                        center = 5)

A <- 5
B <- 15
res <- effect_size_RML(design = "MB", dat = dat, 
                       FE_base = FE_base, RE_base = RE_base,
                       FE_trt = FE_trt, RE_trt = RE_trt, 
                       A = A, B = B)

check_convergence <- function(mod) {
  warns <- warnings()
  if (length(warns) == 0L) return(TRUE)
  
  nlminb_warns <- grepl("nlminb problem", names(warns))
  if (!any(nlminb_warns)) return(TRUE)
  
  warning_calls <- warns[nlminb_warns]
  
  elements <- c("fixed","random","correlation","weights","data","control")
  calls_match <- sapply(warning_calls, function(x) mod$call[elements] == x[elements], USE.NAMES = FALSE)
  !any(calls_match)
}


##------------------------------------------------------------------------
## block-diagonal matrix addition, multiplication, and trace functions 
##------------------------------------------------------------------------

# turn block-diagonal into regular matrix

unblock <- function(A, block=NULL) {
  if (is.null(block)) block <- rep(names(A), times = sapply(A, function(x) dim(x)[1]))
  n <- length(block)
  mat <- matrix(0, n, n)
  for (i in unique(block)) {
    index <- i == block
    mat[index,index] <- A[[i]]
  }
  return(mat)
}


# sum of two conformable block-diagonal matrices

sum_blockblock <- function(A, B)
  mapply(function(a,b) a + b, a = A, b = B, SIMPLIFY = FALSE)


# generic matrix minus block-diagonal

matrix_minus_block <- function(A, B, block=NULL) {
  if (is.null(block)) block <- rep(names(B), times = sapply(B, function(x) dim(x)[1]))
  
  mat <- A
  for (i in unique(block)) {
    index <- i == block
    mat[index,index] <- mat[index, index] - B[[i]]
  }
  return(mat)
}


# block-diagonal minus generic matrix

block_minus_matrix <- function(A, B, block=NULL) {
  if (is.null(block))
    block <- rep(names(A), times = sapply(A, function(x) dim(x)[1]))
  
  mat <- -B
  for (i in unique(block)) {
    index <- i == block
    mat[index,index] <- mat[index, index] + A[[i]]
  }
  return(mat)
}


# product of two conformable block-diagonal matrices

prod_blockblock <- function(A, B)
  mapply(function(a, b) a %*% b, a = A, b = B, SIMPLIFY = FALSE)



# product of a block-diagonal matrix and a generic matrix

prod_blockmatrix <- function(A, B, block = NULL) {
  
  if (is.null(names(A))) names(A) <- 1:length(A)
  A_names <- names(A)
  
  if (is.null(block)) block <- rep(A_names, times = sapply(A, function(x) dim(x)[1]))
  
  C <- matrix(0, length(block), dim(B)[2])
  
  for (b in A_names) {
    ind <- block == b
    C[ind, ] <- A[[b]] %*% B[ind,]
  }
  return(C)
}  


# product of a generic matrix and a block-diagonal matrix

prod_matrixblock <- function(A, B, block = NULL) {
  
  if (is.null(names(B))) names(B) <- 1:length(B)
  B_names <- names(B)
  
  if (is.null(block)) block <- rep(B_names, times = sapply(B, function(x) dim(x)[2]))
  
  C <- matrix(0, dim(A)[1], length(block))
  
  for (b in B_names) {
    ind <- block == b
    C[,ind] <- A[,ind] %*% B[[b]]
  }
  return(C)
}  



# trace of the product of two generic matrices

product_trace <- function(A,B) sum(as.vector(t(A)) * as.vector(B))


# trace of the product of two conformable block-diagonal matrices

product_trace_blockblock <- function(A, B) 
  mapply(function(a, b) product_trace(a,b), a = A, b = B)



# Hedges G correction

J <- function(x) 1 - 3 / (4 * x - 1)

##------------------------------------------------------------------------------
## create design matrix 
##------------------------------------------------------------------------------

MBTreatTimes <- function(m, n, min_B = 3, min_T = 3) {
  if (n / (m + 1) >= max(min_B, min_T)) {
    treat_times <- 1 + round((1:m) * n / (m + 1))
  } else if ((n - min_B) / m >= min_T) {
    treat_times <- min_B + round((0:(m-1)) * (n - min_B) / m)
  } else if ((n - min_B - min_T + 1) / (m - 1) > 1) {
    treat_times <- min_B + 1 + round((0:(m-1)) * (n - min_B - min_T) / (m - 1))
  } else {
    treat_times <- min_B + rep(1:(n - min_B - min_T + 1), length.out = m)
  }
  return(sort(treat_times))
}


#' @title Create a design matrix for a single-case design
#' 
#' @description Create a design matrix containing a linear trend, a treatment effect, and a 
#' trend-by-treatment interaction for a single-case design with \code{m} cases and \code{n} 
#' measurement occasions.
#' 
#' @param m number of cases
#' @param n number of time points
#' @param treat_times (Optional) vector of length \code{m} listing treatment introduction times for each case. 
#' @param center centering point for time trend.
#'   
#' @export 
#' 
#' @return A design matrix 
#' 
#' @examples 
#' design_matrix(3, 16, c(5,9,13))

design_matrix <- function(m, n, treat_times = n / 2 + 1, center = 0) {
  X <- data.frame(matrix(NA, m * n, 5))
  attr(X, "bal") <- (length(unique(treat_times)) == 1)
  if (length(treat_times) < m) treat_times <- rep(treat_times, length.out=m)
  
  names(X) <- c("id","constant","treatment","trend","interaction")
  
  X$id <- factor(rep(1:m, each = n))
  X$constant <- rep(1, m * n)
  X$treatment <- as.vector(sapply(treat_times, function(t) c(rep(0, t - 1), rep(1,n - t + 1))))
  X$trend <- rep(1:n, m) - center
  X$interaction <- as.vector(sapply(treat_times, function(t) c(rep(0, t - 1), 1:(n - t + 1))))
  
  return(X)
}

##------------------------------------------------------------------------
## generate simulation data
##------------------------------------------------------------------------

# iterations <- 10
# design <- design_matrix(m=3, n=6, treat_times=2:4)
# beta <- c(0, 1.2, 0, 0)
# sigma_sq <- 0.5
# phi <- 0.4
# Tau <- (1 - sigma_sq) * diag(c(1,1,0,0))

simulation_data <- function(iterations, design, beta, sigma_sq, phi, Tau) {
  N <- dim(design)[1]
  block <- design[,1]
  V_mat <- lmeAR1_cov_block(block=block, Z_design=design[,-1], 
                theta = list(sigma_sq=sigma_sq, phi=phi, Tau=Tau))
  V_chol <- lapply(V_mat, chol)  
  mean_vector <- as.matrix(design[,-1]) %*% beta
  errors <- t(prod_matrixblock(matrix(rnorm(iterations * N), iterations, N), V_chol, block))
  matrix(mean_vector, N, iterations) + errors
}


##------------------------------------------------------------------------
## estimate model, calculate effect size statistics 
##------------------------------------------------------------------------

random_formula <- function(random_terms)
  as.formula(paste("~ 0 + ",paste(random_terms, collapse=" + "),"| id"))

lme_fit <- function(y, design, fixed_terms, random_terms, method="REML") {
  lme(fixed = as.formula(paste("outcome ~ 0 + ", paste(fixed_terms, collapse=" + "))), 
      random = random_formula(random_terms), 
      correlation = corAR1(0, ~ trend | id),
      data = cbind(design, outcome = y),
      method = method, keep.data = FALSE,
      control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
}


##------------------------------------------------------------------------
## Compare RML to HPS for model MB1
##------------------------------------------------------------------------

#' @title Run simulation comparing REML and HPS estimates
#' 
#' @description Simulates data from a simple linear mixed effects model, then calculates 
#' REML and HPS effect size estimators as described in Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @param iterations number of independent iterations of the simulation
#' @param beta vector of fixed effect parameters
#' @param rho intra-class correlation parameter
#' @param phi autocorrelation parameter 
#' @param design design matrix. If not specified, it will be calculated based on \code{m}, \code{n}, and \code{MB}.
#' @param m number of cases. Not used if \code{design} is specified.
#' @param n number of measurement occasions. Not used if \code{design} is specified.
#' @param MB If true, a multiple baseline design will be used; otherwise, an AB design will be used. Not used if \code{design} is specified.
#'   
#' @export 
#' 
#' @return A matrix reporting the mean and variance of the effect size estimates
#' and various associated statistics.
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
#' 
#' @examples
#' compare_RML_HPS(iterations=10, beta = c(0,1,0,0), rho = 0.3, 
#'                  phi = 0.5, design=design_matrix(m=3,n=8))

compare_RML_HPS <- function(iterations, beta, rho, phi, design, m, n, MB = TRUE) {
  if (missing(design)) {
    treat_times <- if (MB) MBTreatTimes(m=m,n=n) else rep(n / 2 + 1, m)
    design <- design_matrix(m=m, n=n, treat_times = treat_times)
  }
    
  fixed_terms <- c("constant","treatment")
  random_terms <- c("constant")
  p_const <- c(0,1)
  r_const <- c(1,0,1)
  
  y_sims <- simulation_data(iterations, design, beta, sigma_sq = 1 - rho, phi, Tau = diag(c(rho,0,0,0)))
  
  RML <- apply(y_sims, 2, function(y) 
    g_REML(lme_fit(y, design, fixed_terms, random_terms),
           p_const, r_const,
           X_design=as.matrix(design[,fixed_terms]), 
           Z_design=as.matrix(design[,random_terms]),
           block=design$id, times=NULL, returnModel=FALSE)[-14:-12])
  RML_mat <- matrix(unlist(RML), length(unlist(RML[[1]])), iterations, 
                    dimnames = list(names(unlist(RML[[1]]))))
  rho <- RML_mat["Tau.id.var(constant)",] / (RML_mat["Tau.id.var(constant)",] + RML_mat["sigma_sq",])
  HPS <- HPS_effect_size(y_sims, treatment=design$treatment, id=design$id, time=design$trend)
  
  RML_coverage1 <- coverage(delta = beta[2], CI = CI_SMD(delta = RML_mat["delta_AB",], kappa = RML_mat["kappa",], nu = RML_mat["nu",]))
  RML_coverage2 <- abs(RML_mat["g_AB",] - beta[2]) / sqrt(RML_mat["V_g_AB",]) < qt(0.975, df = RML_mat["nu",])
  HPS_coverage1 <- coverage(delta = beta[2], CI = CI_SMD(delta = HPS["delta_hat_unadj",], kappa = HPS["kappa_hat",], nu = HPS["df",]))
  HPS_coverage2 <- abs(HPS["delta_hat",] - beta[2]) / sqrt(HPS["V_delta_hat",]) < qt(0.975, df = HPS["df",])
  estimates <- rbind(RML_mat, rho, RML_coverage1, RML_coverage2, HPS, HPS_coverage1, HPS_coverage2)
  rbind(cbind(mean = rowMeans(estimates), var = apply(estimates, 1, var)), 
       cov = c(NA, cov(estimates["g_AB",], estimates["delta_hat",])))
}



##------------------------------------------------------------------------
## Simulate model MB2
##------------------------------------------------------------------------

convergence_handler_MB2 <- function(design, y, method="REML") {
  fixed_terms <- c("constant","treatment")
  p_const <- c(0,1)
  W <- NULL
  m_full <- withCallingHandlers(
    tryCatch(
      lme_fit(y, design, fixed_terms=fixed_terms, 
              random_terms=c("constant","treatment"), method=method),
      error = function(e) e),
    warning = function(w) W <<- w)
  attr(m_full, "warning") <- W
  
  if (!inherits(m_full,"error")) {
    g <- g_REML(m_full, p_const=p_const, r_const=c(1,0,1,0,0),
                X_design=as.matrix(design[,fixed_terms]), 
                Z_design=as.matrix(design[,c("constant","treatment")]),
                block=design$id, times=NULL, returnModel=FALSE)[-14:-12]
  } else {
    m_reduced <- lme_fit(y, design, fixed_terms=fixed_terms, 
                         random_terms="constant", method=method)
    attr(m_reduced,"warning") <- m_full
    g <- g_REML(m_reduced, p_const=p_const, r_const=c(1,0,1),
                  X_design=as.matrix(design[,fixed_terms]), 
                  Z_design=as.matrix(design[,"constant"]),
                  block=design$id, times=NULL, returnModel=FALSE)[-14:-12]
    g$Tau <- c(g$Tau, "id.cov(treatment,constant)" = 0, "id.var(treatment)" = 0)
  }
  g
}

#' @title Simulate Model MB2 from Pustejovsky, Hedges, & Shadish (2014)
#' 
#' @description Simulates data from a linear mixed effects model, then calculates 
#' REML effect size estimator as described in Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @param iterations number of independent iterations of the simulation
#' @param beta vector of fixed effect parameters
#' @param rho intra-class correlation parameter
#' @param phi autocorrelation parameter 
#' @param tau1_ratio ratio of treatment effect variance to intercept variance 
#' @param tau_corr correlation between case-specific treatment effects and intercepts  
#' @param design design matrix. If not specified, it will be calculated based on \code{m}, \code{n}, and \code{MB}.
#' @param m number of cases. Not used if \code{design} is specified.
#' @param n number of measurement occasions. Not used if \code{design} is specified.
#' @param MB If true, a multiple baseline design will be used; otherwise, an AB design will be used. Not used if \code{design} is specified.
#'   
#' @export 
#' 
#' @return A matrix reporting the mean and variance of the effect size estimates
#' and various associated statistics.
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
#' 
#' @examples
#' 
#' set.seed(8)
#' simulate_MB2(iterations = 5, beta = c(0,1,0,0), rho = 0.4, phi = 0.5, 
#'              tau1_ratio = 0.5, tau_corr = -0.4, design = design_matrix(m=3, n=8))
#'              
#' set.seed(8)
#' simulate_MB2(iterations = 5, beta = c(0,1,0,0), rho = 0.4, phi = 0.5, 
#'              tau1_ratio = 0.5, tau_corr = -0.4, m = 3, n = 8, MB = FALSE)
#'              


simulate_MB2 <- function(iterations, beta, rho, phi, tau1_ratio, tau_corr, design, m, n, MB = TRUE) {

  if (missing(design)) {
    treat_times <- if (MB) MBTreatTimes(m=m,n=n) else rep(n / 2 + 1, m)
    design <- design_matrix(m=m, n=n, treat_times = treat_times)
  }
  
  Tau <- rbind(cbind(
    rho * matrix(c(1,rep(tau_corr * sqrt(tau1_ratio),2),tau1_ratio), 2, 2), 
    matrix(0,2,2)), matrix(0,2,4))
  
  y_sims <- simulation_data(iterations, design, beta, sigma_sq = 1 - rho, phi, Tau = Tau)
  RML <- apply(y_sims, 2, convergence_handler_MB2, design=design)
  RML_mat <- matrix(unlist(RML), length(unlist(RML[[1]])), iterations, 
                    dimnames = list(names(unlist(RML[[1]]))))
  RML_coverage1 <- coverage(delta = beta[2], CI = CI_SMD(delta = RML_mat["delta_AB",], kappa = RML_mat["kappa",], nu = RML_mat["nu",]))
  RML_coverage2 <- abs(RML_mat["g_AB",] - beta[2]) / sqrt(RML_mat["V_g_AB",]) < qt(0.975, df = RML_mat["nu",])
  estimates <- rbind(RML_mat, RML_coverage1, RML_coverage2)
  
  cbind(mean = rowMeans(estimates), var = apply(estimates, 1, var))
}


##------------------------------------------------------------------------
## Simulate model MB4 
##------------------------------------------------------------------------

convergence_handler_MB4 <- function(design, y, p_const, r_const) {
  fixed_terms <- c("constant","treatment","trend","interaction")
  W <- NULL
  m_full <- withCallingHandlers(
    tryCatch(
      lme_fit(y, design, fixed_terms=fixed_terms, random_terms=c("constant","trend")),
      error = function(e) e),
    warning = function(w) W <<- w)
  attr(m_full, "warning") <- W
  
  if (!inherits(m_full,"error")) {
    g <- g_REML(m_full, p_const=p_const, r_const=r_const,
                X_design=as.matrix(design[,fixed_terms]), 
                Z_design=as.matrix(design[,c("constant","trend")]),
                block=design$id, times=NULL, returnModel=FALSE)[-14:-12]
  } else {
    m_reduced <- lme_fit(y, design, fixed_terms=fixed_terms, random_terms="constant")
    attr(m_reduced,"warning") <- m_full
    g <- g_REML(m_reduced, p_const=p_const, r_const=c(1,0,1),
                X_design=as.matrix(design[,fixed_terms]), 
                Z_design=as.matrix(design[,"constant"]),
                block=design$id, times=NULL, returnModel=FALSE)[-14:-12]
    g$Tau <- c(g$Tau, "id.cov(trend,constant)" = 0, "id.var(trend)" = 0)
  }
  g
}

#' @title Simulate Model MB4 from Pustejovsky, Hedges, & Shadish (2014)
#' 
#' @description Simulates data from a linear mixed effects model, then calculates 
#' REML effect size estimator as described in Pustejovsky, Hedges, & Shadish (2014).
#' 
#' @param iterations number of independent iterations of the simulation
#' @param beta vector of fixed effect parameters
#' @param rho intra-class correlation parameter
#' @param phi autocorrelation parameter 
#' @param tau2_ratio ratio of trend variance to intercept variance 
#' @param tau_corr correlation between case-specific trends and intercepts
#' @param p_const vector of constants for calculating numerator of effect size
#' @param r_const vector of constants for calculating denominator of effect size
#' @param design design matrix. If not specified, it will be calculated based on \code{m}, \code{n}, and \code{MB}.
#' @param m number of cases. Not used if \code{design} is specified.
#' @param n number of measurement occasions. Not used if \code{design} is specified.
#' @param MB If true, a multiple baseline design will be used; otherwise, an AB design will be used. Not used if \code{design} is specified.
#'   
#' @export 
#' 
#' @return A matrix reporting the mean and variance of the effect size estimates
#' and various associated statistics.
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
#' 
#' @examples
#' 
#' simulate_MB4(iterations = 5, beta = c(0,1,0,0), rho = 0.8, phi = 0.5, 
#'              tau2_ratio = 0.5, tau_corr = 0, 
#'              p_const = c(0,1,0,7), r_const = c(1,0,1,0,0), 
#'              design = design_matrix(3, 16, treat_times=c(5,9,13), center = 12))
#'              
#' simulate_MB4(iterations = 5, beta = c(0,1,0,0), rho = 0.8, phi = 0.5, 
#'              tau2_ratio = 0.5, tau_corr = 0, m = 6, n = 8)
#'              

simulate_MB4 <- function(iterations, beta, rho, phi, tau2_ratio, tau_corr, 
                         p_const, r_const, design, m, n, MB = TRUE) {

  if (missing(design)) {
    if (missing(p_const)) p_const <- c(0,1,0, n/4)
    if (missing(r_const)) r_const <- c(1,0,1,0,0)
    treat_times <- if (MB) MBTreatTimes(m=m,n=n) else rep(n / 2 + 1, m)
    design <- design_matrix(m=m, n=n, treat_times = treat_times, center = 3*n/4)
  }
  
  Tau <- rbind(cbind(
    rho * matrix(c(1,rep(tau_corr * sqrt(tau2_ratio),2),tau2_ratio), 2, 2), 
    matrix(0,2,2)), matrix(0,2,4))[c(1,3,2,4),c(1,3,2,4)]
  
  y_sims <- simulation_data(iterations, design, beta, sigma_sq = 1 - rho, phi, Tau = Tau)
  RML <- apply(y_sims, 2, convergence_handler_MB4, 
               design=design, p_const=p_const, r_const=r_const)
  RML_mat <- matrix(unlist(RML), length(unlist(RML[[1]])), iterations, 
                    dimnames = list(names(unlist(RML[[1]]))))
  RML_coverage1 <- coverage(delta = sum(beta * p_const), CI = CI_SMD(delta = RML_mat["delta_AB",], kappa = RML_mat["kappa",], nu = RML_mat["nu",]))
  RML_coverage2 <- abs(RML_mat["g_AB",] - sum(beta * p_const)) / sqrt(RML_mat["V_g_AB",]) < qt(0.975, df = RML_mat["nu",])
  estimates <- rbind(RML_mat, RML_coverage1, RML_coverage2)
  
  cbind(mean = rowMeans(estimates), var = apply(estimates, 1, var))
}

#------------------------------------------
# Simulate from fitted g_REML model 
#------------------------------------------

fit_g <- function(y, object) {
  W <- NULL
  m_fit <- withCallingHandlers(
    tryCatch(
      m_fit <- update(object, fixed = update(as.formula(object$call$fixed), y ~ .), 
                      data = data.frame(y=y, object$data)),
      error = function(e) e),
    warning = function(w) W <<- w)
  attr(m_fit, "warning") <- W
  
  if (!inherits(m_fit,"error")) {
    unlist(g_REML(m_fit, p_const=object$p_const, object$r_const, 
                  X_design = model.matrix(object, data = object$data), 
                  Z_design = model.matrix(object$modelStruct$reStruct, data = object$data), 
                  block = object$groups[[1]], 
                  times = attr(object$modelStruct$corStruct, "covariate"), 
                  returnModel=FALSE)[-14:-12])    
  } else {
    rep(NA, length(unlist(object[1:11])))
  }
}

#' @title Simulate data from a fitted \code{g_REML} object
#' 
#' @description Simulates data from the linear mixed effects model used to estimate the
#' specified standardized mean difference effect size. Suitable for parametric bootstrapping.
#' 
#' @param object a \code{g_REML} object
#' @param nsim number of models to simulate
#' @param seed seed value. See documentation for \code{\link{simulate}}
#' @param parallel if \code{TRUE}, run in parallel using \code{foreach} backend.
#' @param ... additional optional arguments
#' 
#' @export
#' 
#' @return A matrix with one row per simulation, with columns corresponding to the output
#' of \code{g_REML}.
#' 
#' @examples
#' data(Laski)
#' Laski_RML <- lme(fixed = outcome ~ treatment,
#'                  random = ~ 1 | case,
#'                  correlation = corAR1(0, ~ time | case), 
#'                  data = Laski)
#' 
#' suppressWarnings(
#'   Laski_g <- g_REML(Laski_RML, p_const = c(0,1), r_const = c(1,0,1))
#' )
#' 
#' if (requireNamespace("plyr", quietly = TRUE)) {
#'   simulate(Laski_g, nsim = 5)
#' }


simulate.g_REML <- function(object, nsim = 1, seed = NULL, parallel = FALSE, ...) {
  if (!requireNamespace("plyr", quietly = TRUE)) {
    stop("The plyr package is needed for this function to work. Please install it.",
         call. = FALSE)
  }
  set.seed(seed)
  
  X_design <- model.matrix(object, data = object$data)
  Z_design <- model.matrix(object$modelStruct$reStruct, data = object$data)
  block <- object$groups[[1]]
  
  nXterms <- dim(X_design)[2]
  Zterms <- match(colnames(Z_design), colnames(X_design))
  Tau <- matrix(0, nXterms, nXterms)
  Tau[Zterms,Zterms] <- Tau_matrix(object$Tau)
  
  sim_data <- simulation_data(iterations = nsim, 
                              design = cbind(block,X_design), 
                              beta = object$coefficients$fixed, 
                              sigma_sq = object$sigma_sq, phi = object$phi, Tau = Tau)

  if (parallel) paropts <- list(.packages = c("nlme","scdhlm")) else paropts = NULL

  as.data.frame(plyr::aaply(sim_data, 2, fit_g, object = object, .parallel = parallel, .paropts = paropts))

}

library(readr)

Krupp_reg <- read_csv("auxilliary/Krupp (2011) Regular Education Setting wo Maintenance.csv")
Krupp_SH <- read_csv("auxilliary/Krupp (2011) Study Hall Setting wo Maintenance.csv")

Krupp_reg_fit <- 
  lme(Outcome ~ Condition,
      random = ~ 1 | Case,
      correlation = corAR1(0, ~ Session | Case),
      data = Krupp_reg)

summary(Krupp_reg_fit)

reg_ES <- 
  g_REML(Krupp_reg_fit, 
         p_const = c(0,1), 
         r_const = c(1,0,1), 
         returnModel = FALSE)
reg_ES$g_AB


Krupp_SH_fit <- 
  lme(Outcome ~ Condition,
      random = ~ Condition | Case,
      correlation = corAR1(0, ~ Session | Case),
      data = Krupp_SH,
      control = lmeControl())

summary(Krupp_SH_fit)

SH_ES <- 
  g_REML(Krupp_SH_fit, 
         p_const = c(0,1), 
         r_const = c(1,0,1), 
         returnModel = FALSE)
SH_ES$g_AB

# Calculate timing defaults

#' @title Calculate default initial treatment time and follow-up time.
#'
#' @description Calculate the default initial treatment time and follow-up time
#'   that are used to define and estimate the between-case standardized mean
#'   differences for multiple baseline designs and variations.
#'
#' @inheritParams graph_SCD
#'
#' @note If treatment_name is left null, it will choose the second level of the
#'   phase variable to be the treatment phase.
#'
#' @export
#'
#' @return A list of time range, default initial treatment time, and default
#'   follow-up time.
#'
#'
#' @examples
#' data(Laski)
#' default_times(design = "MBP", 
#'               case = case, phase = treatment, session = time, 
#'               data = Laski)
#'
#' data(Thiemann2001)
#' default_times(design = "RMBB", 
#'               case = case, series = series, 
#'               phase = treatment, session = time, 
#'               data = Thiemann2001)
#'
#' data(Bryant2018)
#' default_times(design = "CMB", 
#'               cluster = group, case = case, 
#'               phase = treatment, session = session, 
#'               data = Bryant2018)
#'               

default_times <- function(design,
                          case, phase, session, cluster = NULL, series = NULL, 
                          treatment_name = NULL, data = NULL) {
  
  design <- match.arg(design, choices = c("MBP","TR","RMBB","CMB"))
  
  if (missing(case)) stop("Please specify a case variable.")
  if (missing(phase)) stop("Please specify a phase variable.")
  if (missing(session)) stop("Please specify a session variable.")
  if (missing(design)) stop("Please specify a study design of 'MB' (multiple baseline), 
                            'RMBB' (replicated multiple baseline across behaviors),
                            'CMB' (clustered multiple baseline).")
  if (design == "CMB" & missing(cluster)) stop("Please specify a cluster variable.")
  if (design == "RMBB" & missing(series)) stop("Please specify a series variable.")
  
  case_call <- substitute(case)
  phase_call <- substitute(phase)
  session_call <- substitute(session)
  cluster_call <- substitute(cluster)
  series_call <- substitute(series)
  
  env <- if (!is.null(data)) list2env(data, parent = parent.frame()) else env <- parent.frame()
  case <- eval(case_call, env)
  phase <- eval(phase_call, env)
  session <- eval(session_call, env)
  cluster <- eval(cluster_call, env)
  series <- eval(series_call, env)
  
  if (is.null(treatment_name)) {
    treatment_name <- levels(as.factor(phase))[2]
  }
  
  # set the index for each design type
  if (design == "RMBB") {
    dat <- data.frame(case = as.factor(case),
                      series = as.factor(series),
                      phase = as.factor(phase),
                      session,
                      index = paste(case, series, sep = "-"))
  } else if (design == "CMB") {
    dat <- data.frame(cluster = as.factor(cluster),
                      case = as.factor(case),
                      phase = as.factor(phase),
                      session,
                      index = paste(cluster, case, sep = "-")) 
  } else {
    dat <- data.frame(case = as.factor(case),
                      phase = as.factor(phase),
                      session,
                      index = case)
  }
  
  dat <- droplevels(dat)
  
  # calculate the default times
  if (design != "TR") {
    range <- range(dat$session)
    case_base_last <- with(dat, tapply(session[phase != treatment_name], index[phase != treatment_name], max))
    case_trt_range <- with(dat, tapply(session[phase == treatment_name], index[phase == treatment_name], function(x) diff(range(x)) + 1))
    A <- min(case_base_last)
    B <- A + min(case_trt_range[which(case_base_last == min(case_base_last))])
  } else {
    range <- A <- B <- NA
  }
    
  return(list(range = range, A = A, B = B))
  
}


write_formula <- function(powers, var_names) {
  var_name_1 <- if (var_names[1] == "NULL") NULL else var_names[1]
  if (is.null(powers)) {
    var_name_1
  } else {
    paste(c(if (0 %in% powers) var_names[2] else var_name_1, 
            paste0("I(",var_names[3],"^",powers,")")[powers != 0]), 
          collapse = " + ")
  }
}


#' @title A convenience function for calculating design comparable effect sizes
#'
#' @description In one call, 1) clean single-case design data for treatment
#'   reversal and multiple baseline designs, 2) fit a multi-level model using
#'   restricted maximum likelihood estimation, and 3) estimate a standardized
#'   mean difference effect size.
#'
#' @inheritParams preprocess_SCD
#' @param FE_base Vector of integers specifying which fixed effect terms to
#'   include in the baseline phase. Setting \code{FE_base = 0} includes only a
#'   level. Setting \code{FE_base = c(0,1)} includes a level and a linear time
#'   trend.
#' @param RE_base Vector of integers specifying which random effect terms to
#'   include in the baseline phase. Setting \code{RE_base = 0} includes only
#'   levels (i.e., random intercepts). Setting \code{RE_base = c(0,1)} includes
#'   random levels and random linear trends.
#' @param RE_base_2 Vector of integers specifying which random effect terms to
#'   include in the baseline phase for the cluster level in clustered multiple
#'   baseline design across participants or for the case level in replicated
#'   multiple baseline across behaviors. Setting \code{RE_base_2 = 0} includes
#'   only levels (i.e., random intercepts). Setting \code{RE_base_2 = c(0,1)}
#'   includes random levels and random linear trends.
#' @param FE_trt Vector of integers specifying which fixed effect terms to
#'   include in the treatment phase. Setting \code{FE_trt = 0} includes only a
#'   change in level. Setting \code{FE_trt = c(0,1)} includes a change in level
#'   and a treatment-by-linear time trend.
#' @param RE_trt Vector of integers specifying which random effect terms to
#'   include in the treatment phase. Setting \code{RE_trt = 0} includes only
#'   random changes in level. Setting \code{RE_trt = c(0,1)} includes random
#'   changes in level and random treatment-by-linear time trends.
#' @param RE_trt_2 Vector of integers specifying which random effect terms to
#'   include in the treatment phase for the cluster level in clustered multiple
#'   baseline design across participants or for the case level in replicated
#'   multiple baseline across behaviors. Setting \code{RE_trt_2 = 0} includes
#'   only random changes in level. Setting \code{RE_trt_2 = c(0,1)} includes
#'   random changes in level and random treatment-by-linear time trends.
#' @param corStruct (Optional) character string indicating the correlation
#'   structure of session-level errors. Options are \code{"AR1"} (default),
#'   \code{"MA1"}, or \code{"IID"}.
#' @param varStruct (Optional) character string indicating the
#'   heteroscedasticity structure of session-level errors. Options are
#'   \code{"hom"} (default) or \code{"het"}, which allows for the session-level
#'   error variances to differ by phase.
#' @param A The time point immediately before the start of treatment in the
#'   hypothetical between-group design.
#' @param B The time point at which outcomes are measured in the hypothetical
#'   between-group design.
#' @param D Numerical indicating the treatment duration across cases. Note that
#'   \code{B = A + D} and it is not allowed to specify both \code{B} and
#'   \code{D}.
#' @param cover Confidence level.
#' @param bound Numerical tolerance for non-centrality parameter in
#'   \code{\link[stats]{qt}}.
#' @param symmetric If \code{TRUE} (the default), use a symmetric confidence
#'   interval. If \code{FALSE}, use a non-central t approximation to obtain an
#'   asymmetric confidence interval.
#' @param summary Logical indicating whether to return a data frame with effect
#'   size estimates and other information. If \code{TRUE} (default), return a
#'   \code{data.frame} containing the effect size estimate, standard error,
#'   confidence interval, and other information. If \code{FALSE}, return a list
#'   with effect size estimate, degrees of freedom, and other information.
#' @param ... further arguments.
#'
#' @export
#'
#' @return If \code{summary == TRUE}, a data frame containing the
#'   design-comparable effect size estimate, standard error, confidence
#'   interval, and other information. If \code{summary == FALSE}, a list
#'   containing all elements of a `g_mlm()` object, plus the fitted `lme()`
#'   model.
#'
#' @examples
#' data(Laski)
#' 
#' # Change-in-levels model with fixed treatment effect
#' calc_BCSMD(design = "MBP",
#'            case = case, phase = treatment,
#'            session = time, outcome = outcome,
#'            FE_base = 0, RE_base = 0, FE_trt = 0,
#'            data = Laski)
#'
#' # Model with linear time trends in baseline and treatment phases,
#' # random baseline slopes, fixed treatment effects
#' calc_BCSMD(design = "MBP",
#'            case = case, phase = treatment,
#'            session = time, outcome = outcome, center = 4,
#'            FE_base = c(0,1), RE_base = c(0,1), 
#'            FE_trt = c(0,1),
#'            data = Laski)
#'
#'
#' data(Anglesea)
#' calc_BCSMD(design = "TR",
#'            case = case, phase = condition,
#'            session = session, outcome = outcome,
#'            treatment_name = "treatment",
#'            FE_base = 0, RE_base = 0, 
#'            FE_trt = 0,
#'            data = Anglesea)
#'
#'
#' data(Thiemann2001)
#' calc_BCSMD(design = "RMBB",
#'            case = case, series = series, phase = treatment,
#'            session = time, outcome = outcome,
#'            FE_base = 0, RE_base = 0, RE_base_2 = 0, 
#'            FE_trt = 0,
#'            data = Thiemann2001)
#'
#'
#' data(Bryant2018)
#' calc_BCSMD(design = "CMB",
#'            cluster = group, case = case, phase = treatment,
#'            session = session, outcome = outcome, center = 49,
#'            treatment_name = "treatment",
#'            FE_base = c(0,1), RE_base = 0, RE_base_2 = 0,
#'            FE_trt = c(0,1), RE_trt = NULL, RE_trt_2 = NULL,
#'            data = Bryant2018)
#' 
#' 

calc_BCSMD <- function(design, 
                       case, phase, session, outcome, 
                       cluster = NULL, series = NULL,
                       center = 0, 
                       round_session = TRUE,
                       treatment_name = NULL,
                       FE_base = 0, RE_base = 0, RE_base_2 = NULL, FE_trt = 0, RE_trt = NULL, RE_trt_2 = NULL,
                       corStruct = "AR1", varStruct = "hom",
                       A = NULL, B = NULL, D = NULL,
                       cover = 95, bound = 35, symmetric = TRUE,
                       summary = TRUE, 
                       data = NULL, ...) {
  
  # clean data
  
  design <- match.arg(design, choices = c("MBP","TR","RMBB","CMB"))
  
  if (missing(case)) stop("Please specify a case variable.")
  if (missing(phase)) stop("Please specify a phase variable.")
  if (missing(session)) stop("Please specify a session variable.")
  if (missing(outcome)) stop("Please specify an outcome variable.")
  if (missing(design)) stop("Please specify a study design of 'MB' (multiple baseline), 
                            'TR' (treatment reversal),
                            'RMBB' (replicated multiple baseline across behaviors),
                            'CMB' (clustered multiple baseline).")
  if (design == "CMB" & missing(cluster)) stop("Please specify a cluster variable.")
  if (design == "RMBB" & missing(series)) stop("Please specify a series variable.")
  if (design == "TR" & ( any(c(FE_base, RE_base, FE_trt) != 0) | (!is.null(RE_trt) && any(RE_trt != 0)) | any(!is.null(c(RE_base_2, RE_trt_2))) )) 
    stop("Please use the default specifications for fixed and random effects in treatment reversal design.")
  
  case_call <- substitute(case)
  phase_call <- substitute(phase)
  session_call <- substitute(session)
  outcome_call <- substitute(outcome)
  cluster_call <- substitute(cluster)
  series_call <- substitute(series)
  
  env <- if (!is.null(data)) list2env(data, parent = parent.frame()) else env <- parent.frame()
  case <- eval(case_call, env)
  phase <- eval(phase_call, env)
  session <- eval(session_call, env)
  outcome <- eval(outcome_call, env)
  cluster <- eval(cluster_call, env)
  series <- eval(series_call, env)
  
  dat <- preprocess_SCD(design = design, cluster = cluster, series = series, center = center, 
                        case = case, phase = phase, session = session, outcome = outcome, 
                        round_session = round_session, treatment_name = treatment_name)
  
  # fit the model
  
  if (design %in% c("MBP", "RMBB", "CMB")) {
    session_FE <- write_formula(FE_base, c("0","1","session"))
    trt_FE <- write_formula(FE_trt, c("NULL", "trt", "session_trt"))
    fixed <- as.formula(paste("outcome ~", paste(c(session_FE, trt_FE), collapse = " + ")))
    
    session_RE <- write_formula(RE_base, c("0","1","session"))
    trt_RE <- write_formula(RE_trt, c("NULL","trt","session_trt"))
    if (design == "MBP") {
      random <- as.formula(paste(" ~ ",paste(c(session_RE, trt_RE), collapse = " + "), "| case"))
    } else {
      session_RE_2 <- write_formula(RE_base_2, c("0","1","session"))
      trt_RE_2 <- write_formula(RE_trt_2, c("NULL","trt","session_trt"))
      if (design == "RMBB") {
        random <- eval(parse(text = paste0("list(case = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                                           ", series = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")))
      } else {
        random <- eval(parse(text = paste0("list(cluster = ~ ", paste(c(session_RE_2, trt_RE_2), collapse = " + "),
                                           ", case = ~ ", paste(c(session_RE, trt_RE), collapse = " + "),")")))
      }
    }
  } else if (design == "TR") {
    session_FE <- if (is.null(FE_base) | !(0 %in% FE_base)) "0" else "1"
    trt_FE <- if (is.null(FE_trt) | !(0 %in% FE_trt)) NULL else "trt"
    fixed <- as.formula(paste("outcome ~", paste(c(session_FE, trt_FE), collapse = " + ")))
    
    session_RE <- if (is.null(RE_base) | !(0 %in% RE_base)) "0" else "1"
    trt_RE <- if (is.null(RE_trt) | !(0 %in% RE_trt)) NULL else "trt"
    random <- as.formula(paste(" ~",paste(c(session_RE, trt_RE), collapse = " + "), "| case"))
  }

  nesting_str <- switch(design, 
                        MBP = "case",
                        TR = "case",
                        RMBB = "case/series",
                        CMB = "cluster/case")
  
  cor_struct <- switch(corStruct,
                       `MA1` = eval(parse(text = paste0("corARMA(0, ~ session | ", nesting_str, ", p = 0, q = 1)"))),
                       `AR1` = eval(parse(text = paste0("corAR1(", 0.01, ", ~ session | ", nesting_str, ")"))),
                       `IID` = NULL)
  
  if (varStruct == "het") {
    var_struct <- eval(parse(text = "varIdent(form = ~ 1 | phase)"))
  } else if (varStruct == "hom") {
    var_struct <- NULL
  }

  W <- TRUE
  E <- NULL
  m_fit <- withCallingHandlers(
    tryCatch(lme(fixed = fixed, random = random,
                 correlation = cor_struct,
                 weights = var_struct,
                 data = dat,
                 na.action = na.omit,
                 control = lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE)),
             error = function(e) E <<- e),
    warning = function(w) W <<- w)
  m_fit$call$fixed <- fixed
  m_fit$call$random <- random
  converged <- if (is.null(E)) W else E
  
  # calculate effect size
  
  if (!is.null(B) & !is.null(D)) {
    stop("Please specify either the 'B' or 'D' argument, or use the default treatment duration by setting both 'B' and 'D' to NULL.")
    
  } else {
    
    default_AB <- default_times(design = design, 
                                case = case, phase = phase, session = session, 
                                cluster = cluster, series = series,
                                treatment_name = treatment_name)
    
    if (is.null(A)) {
      A <- default_AB$A
    } 
    
    if (is.null(B) & is.null(D)) {
      B <- default_AB$B
    } else if (!is.null(D)) {
      B <- A + D
    }
  }
  
  p_const <- c(rep(0L, length(FE_base)), (B - A)^as.integer(FE_trt))

  r_dim <- length(RE_base) + length(RE_trt)
  r_const_dim <- r_dim * (r_dim + 1) / 2
  bc_vec <- (B - center)^as.integer(RE_base)
  bc_mat <- 2 * tcrossprod(bc_vec) - diag(bc_vec^2)
  r_const_base <- bc_mat[upper.tri(bc_mat, diag = TRUE)]
  r_const_trt <- rep(0L, r_const_dim - length(r_const_base))

  if (design %in% c("MBP", "TR")) {
    r_const <- c(r_const_base,
                 r_const_trt,
                 rep(0L, length(m_fit$modelStruct$corStruct)),
                 rep(0L, length(m_fit$modelStruct$varStruct)),
                 1L)
  } else {

    r_dim2 <- length(RE_base_2) + length(RE_trt_2)
    r_const_dim2 <- r_dim2 * (r_dim2 + 1) / 2
    bc_vec2 <- (B - center)^as.integer(RE_base_2)
    bc_mat2 <- 2 * tcrossprod(bc_vec2) - diag(bc_vec2^2)
    r_const_base2 <- bc_mat2[upper.tri(bc_mat2, diag = TRUE)]
    r_const_trt2 <- rep(0L, r_const_dim2 - length(r_const_base2))

    r_const <- c(r_const_base2,
                 r_const_trt2,
                 r_const_base,
                 r_const_trt,
                 rep(0L, length(m_fit$modelStruct$corStruct)),
                 rep(0L, length(m_fit$modelStruct$varStruct)),
                 1L)
  }

  g_RML <- g_mlm(m_fit, p_const = p_const, r_const = r_const, infotype = "expected")

  if (summary) {

    ES_summary <- data.frame(
      ES = as.numeric(g_RML$g_AB),
      SE = as.numeric(g_RML$SE_g_AB)
    )

    if (design %in% c("RMBB", "CMB")) {
      rho_level2 <- round(with(g_RML, (theta$Tau[[1]][1] + theta$Tau[[2]][1]) /
                                 (theta$Tau[[1]][1] + theta$Tau[[2]][1] + theta$sigma_sq)), 3)
      rho_level3 <- round(with(g_RML, theta$Tau[[1]][1] /
                                 (theta$Tau[[1]][1] + theta$Tau[[2]][1] + theta$sigma_sq)), 3)
      g_RML$rho <- paste0("Level2:", rho_level2, "  Level3:", rho_level3)
    } else {
      g_RML$rho <- with(g_RML, theta$Tau[[1]][1] / (theta$Tau[[1]][1] + theta$sigma_sq))
    }

    g_RML$phi <- g_RML$theta$cor_params
    g_RML$var_param <- g_RML$theta$var_params

    CI <- CI_g(g_RML, cover = cover / 100L, bound = bound, symmetric = symmetric)

    ES_summary$CI_L <- if (CI[1] < 100 & CI[1] > -100) CI[1] else format(CI[1], scientific = TRUE)
    ES_summary$CI_U <- if (CI[2] < 100 & CI[2] > -100) CI[2] else format(CI[2], scientific = TRUE)
    ES_summary$df <- g_RML$nu
    ES_summary$phi <- g_RML$phi
    ES_summary$var_param <- if (varStruct == "het") g_RML$var_param else NA_real_
    ES_summary$rho <- g_RML$rho

    if (design %in% c("MBP", "RMBB", "CMB")) {
      ES_summary$A <- A
      ES_summary$B <- B
    } else {
      ES_summary$A <- NA
      ES_summary$B <- NA
    }

    ES_summary$converged <- if (isTRUE(converged)) "Yes" else "No"

    CI_names <- paste0(cover, "% CI ", c("(lower)", "(upper)"))
    row.names(ES_summary) <- NULL
    names(ES_summary) <- c("BC-SMD estimate","Std. Error", CI_names,
                           "Degrees of freedom","Auto-correlation", "Variance parameter", "Intra-class correlation",
                           "Initial treatment time","Follow-up time","Converged")

    return(ES_summary)

  } else {
    g_RML$model <- m_fit
    class(g_RML) <- c("enhanced_g_mlm",class(g_RML))
    return(g_RML)

  }
  
}


#' @title A convenience function for calculating multiple design-comparable
#'   effect sizes from a dataset that compiles data from multiple single-case
#'   design studies
#'
#' @description Calculates standardized mean difference effect sizes for a data
#'   set including one or multiple single-case design studies using the same
#'   design (treatment reversal, multiple baseline/probe across participants,
#'   replicated multiple baseline across behaviors, or clustered multiple
#'   baseline across participants).
#'
#' @param data A data frame containing SCD data for which design-comparable
#'   effect sizes will be calculated.
#' @param grouping A variable name or list of (unquoted) variable names that
#'   uniquely identify each study.
#' @param case A variable name (unquoted) that identifies unique cases within
#'   each \code{grouping} variable.
#' @param phase A variable name (unquoted) that identifies unique treatment
#'   phases.
#' @param session A variable name (unquoted) that contains the measurement times
#'   for each data series.
#' @param outcome A variable name (unquoted) that contains the outcome
#'   measurements for each data series.
#' @param cluster (Optional) variable name (unquoted) that identifies the unique
#'   clusters of cases for \code{CMB} designs.
#' @param series (Optional) variable name (unquoted) that identifies the unique
#'   data series for \code{RMBB} designs.
#'
#' @inheritParams calc_BCSMD
#'
#' @export
#'
#' @return A data frame containing the design-comparable effect size estimate,
#'   standard error, confidence interval, and other information, for each unique
#'   category of \code{grouping} variable(s).
#'
#'
#' @importFrom rlang !!!
#' @importFrom rlang !!
#' @importFrom dplyr .data
#' @importFrom magrittr %>%
#'
#' @examples
#' data(Thiemann2001)
#' data(Thiemann2004)
#' datThiemann <- rbind(Thiemann2001, Thiemann2004)
#'
#' # Change-in-levels model with a fixed treatment effect
#' batch_calc_BCSMD(data = datThiemann,
#'                  grouping = Study_ID,
#'                  design = "RMBB",
#'                  case = case, series = series, phase = treatment,
#'                  session = time, outcome = outcome,
#'                  FE_base = 0, RE_base = 0,
#'                  RE_base_2 = 0, FE_trt = 0)
#'
#' # Models with linear time trends in baseline and treatment phase,
#' # random baseline slope at series level, fixed treatment effects
#' batch_calc_BCSMD(data = datThiemann,
#'                  grouping = Study_ID,
#'                  design = "RMBB",
#'                  case = case, series = series, phase = treatment,
#'                  session = time, outcome = outcome,
#'                  FE_base = c(0,1), RE_base = c(0,1),
#'                  RE_base_2 = 0, FE_trt = c(0,1))
#'
     
batch_calc_BCSMD <- function(data,
                             design, 
                             grouping,
                             case, phase, session, outcome, 
                             cluster = NULL, series = NULL,
                             center = 0, 
                             round_session = TRUE,
                             treatment_name = NULL,
                             FE_base = 0, RE_base = 0, RE_base_2 = NULL, 
                             FE_trt = 0, RE_trt = NULL, RE_trt_2 = NULL,
                             corStruct = "AR1", varStruct = "hom",
                             A = NULL, B = NULL, D = NULL,
                             cover = 95, bound = 35, symmetric = TRUE,
                             ...) {
  
  design <- tryCatch(match.arg(design, c("TR","MBP","RMBB","CMB")),
                     error = function(e) stop("The `design` argument must be a character string specifying 'TR', 'MBP', 'RMBB', or 'CMB'."))
  
  grouping <- rlang::enquos(grouping)
  grouping <- tryCatch(tidyselect::vars_select(names(data), !!!grouping),
                       error = function(e) stop("Grouping variables are not in the dataset."))
  
  case <- tryCatch(tidyselect::vars_pull(names(data), !! rlang::enquo(case)), 
                   error = function(e) stop("The `case` variable is not in the dataset."))
  
  phase <- tryCatch(tidyselect::vars_pull(names(data), !! rlang::enquo(phase)), 
                    error = function(e) stop("The `phase` variable is not in the dataset."))
  
  session <- tryCatch(tidyselect::vars_pull(names(data), !! rlang::enquo(session)), 
                      error = function(e) stop("The `session` variable is not in the dataset."))
  
  outcome <- tryCatch(tidyselect::vars_pull(names(data), !! rlang::enquo(outcome)), 
                      error = function(e) stop("The `outcome` variable is not in the dataset."))
  
  if (tryCatch(!is.null(cluster), error = function(e) TRUE)) {
    cluster <- tryCatch(tidyselect::vars_select(names(data), !!!rlang::enquos(cluster)), 
                        error = function(e) stop("The `cluster` variables are not in the dataset."))
  }
  
  if (tryCatch(!is.null(series), error = function(e) TRUE)) {
    series <- tryCatch(tidyselect::vars_select(names(data), !!!rlang::enquos(series)), 
                       error = function(e) stop("The `series` variables are not in the dataset."))
  }
  
  res <- 
    data %>% 
    dplyr::group_by(!!!rlang::syms(c(grouping))) %>%
    dplyr::summarise(
      calc_BCSMD(
        design = design,
        case = .data[[case]],
        phase = .data[[phase]],
        session = .data[[session]],
        outcome = .data[[outcome]],
        cluster = if (design == "CMB") .data[[cluster]] else NULL,
        series = if (design == "RMBB") .data[[series]] else NULL,
        center = center,
        round_session = round_session,
        treatment_name = treatment_name,
        FE_base = FE_base,
        RE_base = RE_base,
        RE_base_2 = RE_base_2,
        FE_trt = FE_trt,
        RE_trt = RE_trt,
        RE_trt_2 = RE_trt_2,
        corStruct = corStruct,
        varStruct = varStruct,
        A = A,
        B = B,
        D = D,
        cover = cover,
        bound = bound,
        symmetric = symmetric,
        ...
      ),
      .groups = 'drop'
    ) %>% 
    
  
  return(res)
  
}

skip_if_not_installed("nlme")

# check if calc_BCSMD() works

library(nlme)

test_that("The logic for the A, B, and D argument in calc_BCSMD() is correct.", {
  
  data(Laski)
  
  # use the default_times function to calculate default A and B
  default_AB <- default_times(design = "MBP", case = case, phase = treatment, session = time, data = Laski)
  default_AB_2 <- default_times(design = "MBP", case = Laski$case, phase = Laski$treatment, session = Laski$time)
  expect_equal(default_AB, default_AB_2)
  default_A <- default_AB$A
  default_B <- default_AB$B
  
  # !is.null(B) & !is.null(D), stop and show error
  expect_error(
    calc_BCSMD(design = "MBP", case = case, phase = treatment, session = time, outcome = outcome,
               A = NULL, B = 13, D = 9, data = Laski)
  )
  
  expect_error(
    calc_BCSMD(design = "MBP", case = case, phase = treatment, session = time, outcome = outcome,
               A = 4, B = 13, D = 9, data = Laski)
  )
  
  # is.null(A) & is.null(B) & is.null(D), get default A and B
  Las1 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, session = time, outcome = outcome, data = Laski)
  expect_equal(Las1$`Initial treatment time`, default_A)
  expect_equal(Las1$`Follow-up time`, default_B)
  
  # is.null(A) & !is.null(B) & is.null(D), get default A and specified B
  Las2 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, 
                     session = time, outcome = outcome, 
                     B = 12, data = Laski)
  expect_equal(Las2$`Initial treatment time`, default_A)
  expect_equal(Las2$`Follow-up time`, 12)
  
  # is.null(A) & is.null(B) & !is.null(D), get default A, B = A + D
  Las3 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, 
                     session = time, outcome = outcome, 
                     D = 8, data = Laski)
  expect_equal(Las3$`Initial treatment time`, default_A)
  expect_equal(Las3$`Follow-up time`, default_A + 8)
  
  # !is.null(A) & is.null(B) & !is.null(D), get specified A, B = A + D
  Las4 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, 
                     session = time, outcome = outcome, 
                     A = 3, D = 8, data = Laski)
  expect_equal(Las4$`Initial treatment time`, 3)
  expect_equal(Las4$`Follow-up time`, 3 + 8)
  
  # !is.null(A) & !is.null(B) & is.null(D), get specified A and specified B
  Las5 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, 
                     session = time, outcome = outcome, 
                     A = 3, B = 11, data = Laski)
  expect_equal(Las5$`Initial treatment time`, 3)
  expect_equal(Las5$`Follow-up time`, 11)
  
  # !is.null(A) & is.null(B) & is.null(D), get specified A and default B
  Las6 <- calc_BCSMD(design = "MBP", case = case, phase = treatment, 
                     session = time, outcome = outcome, 
                     A = 3, data = Laski)
  expect_equal(Las6$`Initial treatment time`, 3)
  expect_equal(Las6$`Follow-up time`, default_B)
  
  # if centered
  Las_c <- suppressWarnings(
    calc_BCSMD(design = "MBP", case = case, phase = treatment, 
               session = time, outcome = outcome, 
               center = 4, 
               FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1),
               A = 3, data = Laski)
   )
  expect_equal(Las_c$`Initial treatment time`, 3)
  expect_equal(Las_c$`Follow-up time`, default_B)
  
})


test_that("The default_times() calculates A and B correctly.", {
  
  # MBP
  data(Laski)
  default_AB <- default_times(design = "MBP", case = case, phase = treatment, session = time, data = Laski)
  default_A <- default_AB$A
  default_B <- default_AB$B
  
  case_base_last <- with(Laski, tapply(time[treatment == "baseline"], case[treatment == "baseline"], max))
  case_trt_range <- with(Laski, tapply(time[treatment == "treatment"], case[treatment == "treatment"], function(x) diff(range(x)) + 1))
  A <- min(case_base_last)
  B <- A + min(case_trt_range[which(case_base_last == min(case_base_last))])
  
  expect_equal(default_A, A)
  expect_equal(default_B, B)
  
  # TR
  data(Anglesea)
  default_AB_Ang <- default_times(design = "TR", case = case, phase = condition, session = session, data = Anglesea)
  expect_true(is.na(default_AB_Ang$A))
  expect_true(is.na(default_AB_Ang$B))
  
  # three-level
  data("Thiemann2001")
  Thiemann2001$caseSeries <- with(Thiemann2001, paste(case, series, sep = "-"))
  default_AB_Thi <- default_times(design = "RMBB", case = case, series = series, phase = treatment, session = time, data = Thiemann2001)
  default_A_Thi <- default_AB_Thi$A
  default_B_Thi <- default_AB_Thi$B
  
  series_base_last <- with(Thiemann2001, tapply(time[treatment == "baseline"], caseSeries[treatment == "baseline"], max))
  series_trt_range <- with(Thiemann2001, tapply(time[treatment == "treatment"], caseSeries[treatment == "treatment"], function(x) diff(range(x)) + 1))
  A_Thi <- min(series_base_last)
  B_Thi <- A_Thi + min(series_trt_range[which(series_base_last == min(series_base_last))])
  
  expect_equal(default_A_Thi, A_Thi)
  expect_equal(default_B_Thi, B_Thi)
  
  data("Bryant2018")
  Bryant2018$clusterCase <- with(Bryant2018, paste(group, case, sep = "-"))
  default_AB_Bry <- default_times(design = "CMB", cluster = group, case = case, phase = treatment, session = session, data = Bryant2018)
  default_A_Bry <- default_AB_Bry$A
  default_B_Bry <- default_AB_Bry$B
  
  cluster_base_last <- with(Bryant2018, tapply(session[treatment == "baseline"], clusterCase[treatment == "baseline"], max))
  cluster_trt_range <- with(Bryant2018, tapply(session[treatment == "treatment"], clusterCase[treatment == "treatment"], function(x) diff(range(x)) + 1))
  A_Bry <- min(cluster_base_last)
  B_Bry <- A_Bry + min(cluster_trt_range[which(cluster_base_last == min(cluster_base_last))])
  
  expect_equal(default_A_Bry, A_Bry)
  expect_equal(default_B_Bry, B_Bry)
  
})


test_that("The calc_BCSMD() works appropriately for treatment reversal designs.", {
  
  data(Anglesea)
  Ang1_RML <- lme(fixed = outcome ~ 1 + condition,
                  random = ~ 1 | case,
                  correlation = corAR1(0.01, ~ session | case),
                  data = Anglesea)
  Ang1_g <- g_mlm(Ang1_RML, p_const = c(0,1), r_const = c(1,0,1))
    
  Ang1_BCSMD <- calc_BCSMD(design = "TR",
                           case = case, phase = condition,
                           session = session, outcome = outcome,
                           FE_base = 0, RE_base = 0, FE_trt = 0,
                           summary = FALSE,
                           data = Anglesea)
  Ang1_BCSMD$model <- NULL
  
  expect_equal(Ang1_g, Ang1_BCSMD, check.attributes = FALSE)

  Ang2_RML <- lme(fixed = outcome ~ 1 + condition,
                  random = ~ condition | case,
                  correlation = corAR1(0.01, ~ session | case),
                  data = Anglesea)
  Ang2_g <- g_mlm(Ang2_RML, p_const = c(0,1), r_const = c(1,0,0,0,1))
  
  Ang2_BCSMD <- calc_BCSMD(design = "TR",
                           case = case, phase = condition,
                           session = session, outcome = outcome,
                           FE_base = 0, RE_base = 0, FE_trt = 0, RE_trt = 0,
                           summary = FALSE,
                           data = Anglesea)
  Ang2_BCSMD$model <- NULL
  
  expect_equal(Ang2_g, Ang2_BCSMD, check.attributes = FALSE)
  
  expect_error(calc_BCSMD(design = "TR",
                          case = case, phase = condition,
                          session = session, outcome = outcome,
                          FE_base = c(0,1), RE_base = 0, FE_trt = 0,
                          summary = FALSE,
                          data = Anglesea))
  
  expect_error(calc_BCSMD(design = "TR",
                          case = case, phase = condition,
                          session = session, outcome = outcome,
                          FE_base = 0, RE_base = 0, FE_trt = 0, RE_trt = 1,
                          summary = FALSE,
                          data = Anglesea))
  
})


test_that("calc_BCSMD() returns same result as g_mlm() for Laski data with a simple model.", {
  
  data(Laski)
  
  # simple model
  Laski_RML1 <- lme(fixed = outcome ~ treatment, 
                    random = ~ 1 | case, 
                    correlation = corAR1(0.01, ~ time | case), 
                    data = Laski)
  Laski_g1_mlm <- g_mlm(Laski_RML1, p_const = c(0,1), r_const = c(1,0,1), returnModel = TRUE)
  
  ## g_mlm VS calc_BCSMD
  Laski_calc_BCSMD <- calc_BCSMD(design = "MBP",
                                 case = case, phase = treatment,
                                 session = time, outcome = outcome,
                                 FE_base = 0, RE_base = 0, FE_trt = 0,
                                 summary = FALSE,
                                 data = Laski)
  Laski_calc_BCSMD$model <- NULL
  expect_equal(Laski_g1_mlm, Laski_calc_BCSMD, check.attributes = FALSE)
  
  Laski_calc_BCSMD_summary <- calc_BCSMD(design = "MBP",
                                         case = case, phase = treatment,
                                         session = time, outcome = outcome,
                                         FE_base = 0, RE_base = 0, FE_trt = 0,
                                         data = Laski)
  expect_equal(Laski_g1_mlm$g_AB, Laski_calc_BCSMD_summary$`BC-SMD estimate`)
  expect_equal(Laski_g1_mlm$SE_g_AB, Laski_calc_BCSMD_summary$`Std. Error`)
  expect_equal(Laski_g1_mlm$nu, Laski_calc_BCSMD_summary$`Degrees of freedom`)
  
})


test_that("calc_BCSMD() returns same result as g_mlm() for Laski data with a complex model.", {

  skip_on_cran()
  
  data(Laski)
  
  # complex model
  default_AB <- default_times(design = "MBP",
                              case = case, phase = treatment, session = time, 
                              data = Laski)
  D <- default_AB$B - default_AB$A
  
  dat <- preprocess_SCD(design = "MBP", 
                        case = case, phase = treatment, session = time, outcome = outcome, 
                        center = 4, data = Laski)
  Laski_RML2 <- suppressWarnings(lme(fixed = outcome ~ time + treatment + time_trt,
                                     random = ~ time | case, 
                                     correlation = corAR1(0.01, ~ time | case), 
                                     data = dat,
                                     control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE)))
  Laski_g2_mlm <- suppressWarnings(g_mlm(Laski_RML2, p_const = c(0,0,1,D), r_const = c(1,18,81,0,1), returnModel = TRUE))
  
  
  ## g_mlm VS calc_BCSMD
  Laski_BCSMD2 <- suppressWarnings(
    calc_BCSMD(design = "MBP", case = case, phase = treatment,
               session = time, outcome = outcome, center = 4,
               FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1),
               summary = FALSE,
               data = Laski)
  )
  
  Laski_BCSMD2$model <- NULL
  expect_equal(Laski_g2_mlm, Laski_BCSMD2, check.attributes = FALSE)
  
  Laski_BCSMD2_summary <- suppressWarnings(
    calc_BCSMD(design = "MBP", case = case, phase = treatment,
               session = time, outcome = outcome, center = 4,
               FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1),
               data = Laski)
  )
  
  expect_equal(Laski_g2_mlm$g_AB, Laski_BCSMD2_summary$`BC-SMD estimate`)
  expect_equal(Laski_g2_mlm$SE_g_AB, Laski_BCSMD2_summary$`Std. Error`)
  expect_equal(Laski_g2_mlm$nu, Laski_BCSMD2_summary$`Degrees of freedom`)
  
})

test_that("calc_BCSMD() returns the same result as g_mlm() for Bryant 2018 data with a simple model.", {
  
  data(Bryant2018)
  Bryant_RML1 <- lme(fixed = outcome ~ treatment,
                     random = ~ 1 | group/case,
                     correlation = corAR1(0.01, ~ session | group/case),
                     data = Bryant2018,
                     na.action = na.omit)
  
  # simple model
  suppressWarnings(expect_error(g_REML(Bryant_RML1, p_const = c(0, 1), r_const = c(1, 0, 1, 1)))) # g_REML not available for 3-level data
  Bry_g1_mlm <- g_mlm(Bryant_RML1, p_const = c(0, 1), r_const = c(1, 1, 0, 1), infotype = "expected", returnModel = TRUE)
  
  Bry_BCSMD <- calc_BCSMD(design = "CMB",
                          cluster = group, case = case, phase = treatment,
                          session = session, outcome = outcome,
                          treatment_name = "treatment",
                          FE_base = 0, RE_base = 0, RE_base_2 = 0, FE_trt = 0,
                          summary = FALSE,
                          data = Bryant2018)
  Bry_BCSMD$model <- NULL
  expect_equal(Bry_g1_mlm, Bry_BCSMD, check.attributes = FALSE)
  
  Bry_BCSMD_summary <- calc_BCSMD(design = "CMB",
                                  cluster = group, case = case, phase = treatment,
                                  session = session, outcome = outcome,
                                  treatment_name = "treatment",
                                  FE_base = 0, RE_base = 0, RE_base_2 = 0, FE_trt = 0,
                                  data = Bryant2018)
  expect_equal(Bry_g1_mlm$g_AB, Bry_BCSMD_summary$`BC-SMD estimate`)
  expect_equal(Bry_g1_mlm$SE_g_AB, Bry_BCSMD_summary$`Std. Error`)
  expect_equal(Bry_g1_mlm$nu, Bry_BCSMD_summary$`Degrees of freedom`)
  
  expect_equal(CI_g(Bry_g1_mlm)[1], Bry_BCSMD_summary$`95% CI (lower)`)
  expect_equal(CI_g(Bry_g1_mlm)[2], Bry_BCSMD_summary$`95% CI (upper)`)

})

test_that("calc_BCSMD() returns the same result as g_mlm() for Bryant 2018 data with a complex model.", {

  skip_on_cran()
  
  data(Bryant2018)
  Bryant_RML1 <- lme(fixed = outcome ~ treatment,
                     random = ~ 1 | group/case,
                     correlation = corAR1(0.01, ~ session | group/case),
                     data = Bryant2018,
                     na.action = na.omit)
  
  # complex model
  default_AB <- default_times(design = "CMB",
                              cluster = group, case = case, phase = treatment, session = session, 
                              data = Bryant2018)
  D <- default_AB$B - default_AB$A
  dat <- preprocess_SCD(design = "CMB", 
                        cluster = group, case = case, phase = treatment,
                        session = session, outcome = outcome, center = default_AB$B,
                        data = Bryant2018)
  Bry_RML2 <- suppressWarnings(lme(fixed = outcome ~ session + treatment + session_trt,
                                   random = list(group = ~ 1, case = ~ session + session_trt), 
                                   correlation = corAR1(0.01, ~ session | group/case),
                                   data = dat,
                                   control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE)))
  Bry_g2_mlm <- g_mlm(Bry_RML2, p_const = c(0,0,1,D), r_const = c(1,1,0,0,0,0,0,0,1), 
                      infotype = "expected", returnModel = TRUE)
  
  Bry_BCSMD2 <- suppressWarnings(
    calc_BCSMD(design = "CMB",
               cluster = group, case = case, phase = treatment,
               session = session, outcome = outcome, center = default_AB$B,
               treatment_name = "treatment",
               FE_base = c(0,1), RE_base = c(0,1), RE_base_2 = 0, 
               FE_trt = c(0,1), RE_trt = 1, RE_trt_2 = NULL,
               summary = FALSE,
               data = Bryant2018)
  )
  
  Bry_BCSMD2$model <- NULL
  expect_equal(Bry_g2_mlm, Bry_BCSMD2, check.attributes = FALSE)
  
  Bry_BCSMD2_summary <- suppressWarnings(
    calc_BCSMD(design = "CMB",
               cluster = group, case = case, phase = treatment,
               session = session, outcome = outcome, center = default_AB$B,
               treatment_name = "treatment",
               FE_base = c(0,1), RE_base = c(0,1), RE_base_2 = 0, 
               FE_trt = c(0,1), RE_trt = 1, RE_trt_2 = NULL,
               data = Bryant2018)
  )
  
  expect_equal(Bry_g2_mlm$g_AB, Bry_BCSMD2_summary$`BC-SMD estimate`)
  expect_equal(Bry_g2_mlm$SE_g_AB, Bry_BCSMD2_summary$`Std. Error`)
  expect_equal(Bry_g2_mlm$nu, Bry_BCSMD2_summary$`Degrees of freedom`)
  expect_equal(CI_g(Bry_g2_mlm)[1], Bry_BCSMD2_summary$`95% CI (lower)`)
  expect_equal(CI_g(Bry_g2_mlm)[2], Bry_BCSMD2_summary$`95% CI (upper)`)
  
})

test_that("The batch_calc_BCSMD() works for MBP design.", {
  
  # single calculator
  data("Laski")
  skip_if_not_installed("dplyr", "1.1.0")
  
  # model 1: basic model without time trends
  Laski1_single <- 
    calc_BCSMD(
      design = "MBP",
      case = case, phase = treatment,
      session = time, outcome = outcome,
      FE_base = 0, RE_base = 0, FE_trt = 0,
      data = Laski
    )
  
  # model 2: varying intercepts, fixed treatment effects, varying trends
  Laski2_single <- 
    suppressWarnings(
      calc_BCSMD(
        design = "MBP",
        case = case, phase = treatment,
        session = time, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1),
        data = Laski)
    )
  
  data("AlberMorgan")
  
  Alber1_single <- 
    calc_BCSMD(
      design = "MBP",
      case = case, phase = condition,
      session = session, outcome = outcome,
      FE_base = 0, RE_base = 0, FE_trt = 0,
      data = AlberMorgan
    )
  
  Alber2_single <- 
    suppressWarnings(
      calc_BCSMD(
        design = "MBP",
        case = case, phase = condition,
        session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1),
        data = AlberMorgan)
    )

  # batch calculator
  Laski$studyID <- "Laski"
  names(Laski) <- c("case", "outcome", "session", "condition", "studyID")
  AlberMorgan$studyID <- "AlberMorgan"
  AlberMorgan <- AlberMorgan[, c(1, 4, 3, 2, 5)]
  dat_MBP <- rbind(Laski, AlberMorgan)

  res_AB <- 
    dat_MBP %>% 
    dplyr::group_by(studyID) %>%
    dplyr::reframe(
      default_times(
        design = "MBP",
        case = case,
        phase = condition,
        session = session,
        cluster = NULL,
        series = NULL
      )
    ) %>% 
    dplyr::mutate(variable = rep(c("range", "A", "B"), length(unique(dat_MBP$studyID)))) %>% 
    dplyr::rename(value = `default_times(...)`)
  
  res_A <- res_AB %>% dplyr::filter(variable == "A") %>% dplyr::pull(value) %>% unlist() %>% as.vector()
  res_B <- res_AB %>% dplyr::filter(variable == "B") %>% dplyr::pull(value) %>% unlist() %>% as.vector()
  
  res1_batch <- 
    suppressWarnings(
      batch_calc_BCSMD(
        data = dat_MBP, grouping = studyID, design = "MBP",
        case = case, phase = condition, session = session, outcome = outcome,
        FE_base = 0, RE_base = 0, FE_trt = 0
      ) 
    )
  
  expect_equal(res1_batch$`Initial treatment time`, res_A)
  expect_equal(res1_batch$`Follow-up time`, res_B)

  res2_batch <- 
    suppressWarnings(
      batch_calc_BCSMD(
        data = dat_MBP, grouping = studyID, design = "MBP",
        case = case, phase = condition, session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1)
      )
    )
  
  # model 1
  expect_equal(Laski1_single, as.data.frame(res1_batch[2, 2:12], ))
  expect_equal(Alber1_single, as.data.frame(res1_batch[1, 2:12], ))
  
  # model 2
  expect_equal(Laski2_single, as.data.frame(res2_batch[2, 2:12], ))
  expect_equal(Alber2_single, as.data.frame(res2_batch[1, 2:12], ))
  
  
  # with specified A, B
  Alber3_single <- 
    suppressWarnings(
      calc_BCSMD(
        design = "MBP",
        case = case, phase = condition,
        session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1), A = 5, B = 12,
        data = AlberMorgan)
    )
  
  res3_batch <- 
    suppressWarnings(
      batch_calc_BCSMD(
        data = dat_MBP, grouping = studyID, design = "MBP",
        case = case, phase = condition, session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1), A = 5, B = 12
      )
    )
  
  expect_equal(Alber3_single, as.data.frame(res3_batch[1, 2:12], ))
  expect_equal(res3_batch$`Initial treatment time`[1], 5)
  expect_equal(res3_batch$`Follow-up time`[1], 12)

  # with specified D
  Alber4_single <- 
    suppressWarnings(
      calc_BCSMD(
        design = "MBP",
        case = case, phase = condition,
        session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1), A = 5, B = 14,
        data = AlberMorgan)
    )
  
  res4_batch <- 
    suppressWarnings(
      batch_calc_BCSMD(
        data = dat_MBP, grouping = studyID, design = "MBP",
        case = case, phase = condition, session = session, outcome = outcome,
        FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1), D = 9
      )
    )
  
  expect_equal(Laski2_single, as.data.frame(res4_batch[2, 2:12], ))
  expect_equal(Alber4_single, as.data.frame(res4_batch[1, 2:12], ))
  
  expect_equal(res4_batch$`Initial treatment time`, res_A)
  expect_equal(res4_batch$`Follow-up time`, res_A + 9)
  
})
  

test_that("The batch_calc_BCSMD() works for RMBB design.", {
  
  data(Thiemann2001)
  data(Thiemann2004)
  dat_RMBB <- rbind(Thiemann2001, Thiemann2004)
  
  # basic model without trends
  
  Thi1_single_2001 <- 
    calc_BCSMD(
      design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = 0, RE_base = 0, RE_base_2 = 0, FE_trt = 0,
      data = Thiemann2001
    )
  
  Thi1_single_2004 <- 
    calc_BCSMD(
      design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = 0, RE_base = 0, RE_base_2 = 0, FE_trt = 0,
      data = Thiemann2004
    )
  
  Thi1_batch <- 
    batch_calc_BCSMD(
      data = dat_RMBB, grouping = Study_ID, design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = 0, RE_base = 0, RE_base_2 = 0, FE_trt = 0
    )
  
  expect_equal(Thi1_single_2001, as.data.frame(Thi1_batch[1, 2:12], ))
  expect_equal(Thi1_single_2004, as.data.frame(Thi1_batch[2, 2:12], ))
  
  # complex model: 
  # varying intercepts at series and case level, varying trends at series level, fixed treatment effects
  
  Thi2_single_2001 <- 
    calc_BCSMD(
      design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = c(0,1), RE_base = c(0,1), RE_base_2 = 0, FE_trt = c(0,1), 
      data = Thiemann2001
    )
  
  Thi2_single_2004 <- 
    calc_BCSMD(
      design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = c(0,1), RE_base = c(0,1), RE_base_2 = 0, FE_trt = c(0,1), 
      data = Thiemann2004
    )
  
  Thi2_batch <- 
    batch_calc_BCSMD(
      data = dat_RMBB, grouping = Study_ID, design = "RMBB",
      case = case, series = series, phase = treatment,
      session = time, outcome = outcome,
      FE_base = c(0,1), RE_base = c(0,1), RE_base_2 = 0, FE_trt = c(0,1)
    )
  
  expect_equal(Thi2_single_2001, as.data.frame(Thi2_batch[1, 2:12], ))
  expect_equal(Thi2_single_2004, as.data.frame(Thi2_batch[2, 2:12], ))
  
})

skip_if_not_installed("nlme")

# check if g_mlm() works

library(nlme)

test_that("g_mlm() is imported appropriately for Laski data.", {
  
  data(Laski)
  
  # simple model
  Laski_RML1 <- lme(fixed = outcome ~ treatment, 
                    random = ~ 1 | case, 
                    correlation = corAR1(0.01, ~ time | case), 
                    data = Laski)
  
  ## g_REML VS g_mlm
  Laski_g1_REML <- suppressWarnings(g_REML(Laski_RML1, p_const = c(0,1), r_const = c(1,0,1), returnModel = TRUE))
  Laski_g1_mlm <- g_mlm(Laski_RML1, p_const = c(0,1), r_const = c(1,0,1), returnModel = TRUE)
  
  expect_warning(g_REML(Laski_RML1, p_const = c(0,1), r_const = c(1,0,1), returnModel = TRUE))
  expect_equal(varcomp_vcov(Laski_RML1)[1, 1], 20214.7841577, tol = 1e-7)
  
  varcomp_ex <- extract_varcomp_lmeAR1(Laski_RML1) # extract varcomp using old function in scdhlm
  varcomp <- extract_varcomp(Laski_RML1) # extract varcomp using new function in lmeInfo
  expect_equal(varcomp_ex$sigma_sq, varcomp$sigma_sq)
  expect_equal(varcomp_ex$phi, varcomp$cor_params)
  expect_equal(varcomp_ex$Tau, varcomp$Tau$case)
  
  expect_output(summary(Laski_g1_REML))
  expect_output(print(Laski_g1_REML))
  expect_output(summary(Laski_g1_mlm))
  expect_output(print(Laski_g1_mlm))
  
  expect_equal(Laski_g1_mlm$p_beta, Laski_g1_REML$p_beta) # numerator of effect size
  expect_equal(Laski_g1_mlm$r_theta, Laski_g1_REML$r_theta) # squared denominator of effect size
  expect_equal(Laski_g1_mlm$delta_AB, Laski_g1_REML$delta_AB) # unadjusted (REML) effect size estimate
  expect_equal(as.numeric(Laski_g1_mlm$nu), Laski_g1_REML$nu) # degrees of freedom
  expect_equal(as.numeric(Laski_g1_mlm$kappa), Laski_g1_REML$kappa) # constant kappa
  expect_equal(as.numeric(Laski_g1_mlm$g_AB), Laski_g1_REML$g_AB) # corrected effect size estimate
  expect_equal(as.numeric(Laski_g1_mlm$SE_g_AB^2), Laski_g1_REML$V_g_AB) # Approximate variance estimate
  expect_equal(Laski_g1_mlm$theta$sigma_sq, Laski_g1_REML$sigma_sq) # Estimated level-1 variance
  expect_equal(Laski_g1_mlm$theta$cor_params, Laski_g1_REML$phi) # Estimated autocorrelation
  expect_equal(Laski_g1_mlm$theta$Tau$case, Laski_g1_REML$Tau) # Vector of level-2 variance components
  expect_equal(det(Laski_g1_mlm$info_inv), det(Laski_g1_REML$I_E_inv)) # Expected information matrix
  
  ## confidence interval
  expect_equal(CI_g(Laski_g1_REML), CI_g(Laski_g1_mlm))
  expect_equal(CI_g(Laski_g1_REML), lmeInfo:::CI_g.g_mlm(Laski_g1_mlm))
  expect_equal(CI_g(Laski_g1_REML, symmetric = FALSE), CI_g(Laski_g1_mlm, symmetric = FALSE))
  

  
  # complex model
  default_AB <- default_times(design = "MBP",
                              case = case, phase = treatment, session = time, 
                              data = Laski)
  D <- default_AB$B - default_AB$A
  
  dat <- preprocess_SCD(design = "MBP", 
                        case = case, phase = treatment, session = time, outcome = outcome, 
                        center = 4, data = Laski)
  Laski_RML2 <- suppressWarnings(lme(fixed = outcome ~ time + treatment + time_trt,
                                     random = ~ time | case, 
                                     correlation = corAR1(0.01, ~ time | case), 
                                     data = dat,
                                     control = lmeControl(msMaxIter = 50, apVar = FALSE, returnObject = TRUE)))
  
  ## g_REML VS g_mlm
  Laski_g2_REML <- suppressWarnings(g_REML(Laski_RML2, p_const = c(0,0,1,D), r_const = c(1,0,1,18,81), returnModel = TRUE))
  Laski_g2_mlm <- suppressWarnings(g_mlm(Laski_RML2, p_const = c(0,0,1,D), r_const = c(1,18,81,0,1), returnModel = TRUE))
  
  expect_equal(Laski_g2_mlm$p_beta, Laski_g2_REML$p_beta) # numerator of effect size
  expect_equal(Laski_g2_mlm$r_theta, Laski_g2_REML$r_theta) # squared denominator of effect size
  expect_equal(Laski_g2_mlm$delta_AB, Laski_g2_REML$delta_AB) # unadjusted (REML) effect size estimate
  expect_equal(as.numeric(Laski_g2_mlm$nu), Laski_g2_REML$nu) # degrees of freedom
  expect_equal(as.numeric(Laski_g2_mlm$kappa), Laski_g2_REML$kappa) # constant kappa
  expect_equal(as.numeric(Laski_g2_mlm$g_AB), Laski_g2_REML$g_AB) # corrected effect size estimate
  expect_equal(as.numeric(Laski_g2_mlm$SE_g_AB^2), Laski_g2_REML$V_g_AB) # Approximate variance estimate
  expect_equal(Laski_g2_mlm$theta$sigma_sq, Laski_g2_REML$sigma_sq) # Estimated level-1 variance
  expect_equal(Laski_g2_mlm$theta$cor_params, Laski_g2_REML$phi) # Estimated autocorrelation
  expect_equal(Laski_g2_mlm$theta$Tau$case, Laski_g2_REML$Tau) # Vector of level-2 variance components
  expect_equal(det(Laski_g2_mlm$info_inv), det(Laski_g2_REML$I_E_inv)) # Expected information matrix
  
  ## confidence interval
  expect_equal(CI_g(Laski_g2_REML), CI_g(Laski_g2_mlm))
  expect_equal(CI_g(Laski_g2_REML), lmeInfo:::CI_g.g_mlm(Laski_g2_mlm))
  expect_equal(CI_g(Laski_g2_REML, symmetric = FALSE), CI_g(Laski_g2_mlm, symmetric = FALSE))
  
})

test_that("g_mlm() is imported appropriately for Bryant 2018 data.", {
  
  data(Bryant2018)
  Bryant_RML1 <- lme(fixed = outcome ~ treatment,
                     random = ~ 1 | group/case,
                     correlation = corAR1(0.01, ~ session | group/case),
                     data = Bryant2018, 
                     na.action = na.omit)
  
  suppressWarnings(expect_error(g_REML(Bryant_RML1, p_const = c(0, 1), r_const = c(1, 0, 1, 1)))) # g_REML not available for 3-level data
  Bry_g1_mlm <- g_mlm(Bryant_RML1, p_const = c(0, 1), r_const = c(1, 1, 0, 1), infotype = "expected", returnModel = TRUE)
  
  expect_output(summary(Bry_g1_mlm))
  expect_output(print(Bry_g1_mlm))
  
})

test_that("Convergence indicator works correctly for g_mlm(), g_REML() and calc_BCSMD().", {
  
  data(Laski)
  
  # simple model
  Laski_BCSMD1 <- suppressWarnings(
    calc_BCSMD(design = "MBP", case = case, phase = treatment,
               session = time, outcome = outcome, center = 4,
               FE_base = 0, RE_base = 0, FE_trt = 0,
               data = Laski)
  )
  
  expect_identical(Laski_BCSMD1$Converged,"Yes")
  
  # complex model
  Laski_BCSMD2 <- suppressWarnings(
    calc_BCSMD(design = "MBP", case = case, phase = treatment,
               session = time, outcome = outcome, center = 4,
               FE_base = c(0,1), RE_base = c(0,1), FE_trt = c(0,1), RE_trt = c(0,1),
               data = Laski)
  )
  
  expect_identical(Laski_BCSMD2$Converged, "No")

})

#---------------------------------------------------------------
# calculate phase border times
#---------------------------------------------------------------

phase_lines <- function(phase, session) {
  phase <- phase[order(session)]
  n <- length(phase)
  switches <- which(phase[2:n] != phase[1:(n-1)])
  (session[switches] + session[switches + 1]) / 2
}

phase_lines_by_case <- function(case, phase, session) {
  
  phases_by <- split(phase, case)
  sessions_by <- split(session, case)
  
  phase_line <- mapply(phase_lines, phase = phases_by, session = sessions_by, SIMPLIFY = FALSE)
  
  case_name <- rep(names(phase_line), times = lengths(phase_line))
  case_name <- factor(case_name, levels = levels(case))
  
  data.frame(case = case_name, phase_time = as.vector(unlist(phase_line)))
}

#' @title Graph Single Case Design Data
#'
#' @description Graphs single case design data for treatment reversal and
#'   multiple baseline designs.
#'
#' @param design Character string to specify whether data comes from a treatment
#'   reversal (\code{"TR"}), multiple baseline across participants
#'   (\code{"MBP"}), replicated multiple baseline across behaviors
#'   (\code{"RMBB"}), or clustered multiple baseline across participants
#'   (\code{"CMB"}).
#' @param case vector of case indicators or name of a character or factor vector
#'   within \code{data} indicating unique cases.
#' @param phase vector of treatment indicators or name of a character or factor
#'   vector within \code{data} indicating unique treatment phases.
#' @param session vector of measurement occasions or name of numeric vector
#'   within \code{data} of measurement times.
#' @param outcome vector of outcome data or name of numeric vector of outcome
#'   data within \code{data}.
#' @param cluster (Optional) vector of cluster indicators or name of a character
#'   or factor vector within \code{data} indicating clusters.
#' @param series (Optional) vector of series indicators or name of a character
#'   or factor vector within \code{data} indicating series.
#' @param treatment_name (Optional) character string corresponding to the name
#'   of the treatment phase.
#' @param model_fit (Optional) lme fitted model that adds predicted values to
#'   graph
#' @param data (Optional) dataset to use for analysis. Must be a
#'   \code{data.frame}.
#' @param newdata (Optional) dataset to use for calculating predicted values
#'   based on \code{model_fit}. Must be a \code{data.frame}.
#'
#' @note If treatment_name is left null it will choose the second level of the
#'   phase variable to be the treatment phase.
#'
#' @export
#'
#' @return A ggplot graph
#'
#'
#' @examples
#'
#' if (requireNamespace("ggplot2", quietly = TRUE)) {
#'
#' data(Anglesea)
#' graph_SCD(design="TR",
#'           case=case, phase=condition,
#'           session=session, outcome=outcome,
#'           treatment_name = "treatment",
#'           data=Anglesea)
#'
#' data(BartonArwood)
#' graph_SCD(design="MBP",
#'           case=case, phase=condition,
#'           session=session, outcome=outcome,
#'           treatment_name = "B",
#'           data=BartonArwood)
#'
#' data(Thiemann2001)
#' graph_SCD(design="RMBB",
#'           case=case, series = series, phase=treatment,
#'           session=time, outcome=outcome,
#'           treatment_name = "treatment",
#'           data=Thiemann2001)
#'
#' data(Bryant2018)
#' graph_SCD(design="CMB",
#'           cluster=group, case=case, phase=treatment,
#'           session=session, outcome=outcome,
#'           treatment_name = "treatment",
#'           data=Bryant2018)
#'
#' }


graph_SCD <- function(design, case, phase, session, outcome, 
                      cluster = NULL, series = NULL, 
                      treatment_name = NULL, model_fit = NULL, 
                      data = NULL, newdata = NULL) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("This function requires the ggplot2 package. Please install it.", call. = FALSE)
  }

  
  case_name <- deparse(substitute(case))
  phase_name <- deparse(substitute(phase))
  session_name <- deparse(substitute(session))
  outcome_name <- deparse(substitute(outcome))
  
  if (!is.null(model_fit) && is.null(data)) {
    model_dat <- nlme::getData(model_fit)
    use_model_data <- all(c(case_name, phase_name, session_name, outcome_name) %in% names(model_dat))
  } else {
    use_model_data <- FALSE
  }

  if (use_model_data) {
    dat <- model_dat
  } else {
    # preprocess_SCD() using the passed arguments
    mf <- match.call()
    m <- match(c("design", "case", "phase","session","outcome","cluster","series","treatment_name","data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf[[1L]] <- quote(preprocess_SCD)
    dat <- eval(mf, envir = parent.frame())
  }

  design <- match.arg(design, choices = c("MBP","TR","RMBB","CMB"))
  
  phase_pair <- phase_time <- NULL
  

  if (design %in% c("MBP", "TR")) {
    phase_line_dat <- phase_lines_by_case(dat[[case_name]], dat[[phase_name]], dat[[session_name]])
    names(phase_line_dat)[1] <- case_name
  } else if (design == "RMBB") {
    series_name <- deparse(substitute(series))
    dat$caseSeries <- as.factor(paste(dat[[case_name]], dat[[series_name]], sep = "-"))
    phase_line_dat <- phase_lines_by_case(dat$caseSeries, dat[[phase_name]], dat[[session_name]])
    names(phase_line_dat)[1] <- "caseSeries"
  } else if (design == "CMB") {
    cluster_name <- deparse(substitute(cluster))
    dat$clusterCase <- as.factor(paste(dat[[cluster_name]], dat[[case_name]], sep = "-:-"))
    phase_line_dat <- phase_lines_by_case(dat$clusterCase, dat[[phase_name]], dat[[session_name]])
    phase_line_dat[[cluster_name]] <- sub("\\-:\\-.*", "", phase_line_dat$case)
    phase_line_dat <- do.call(rbind, by(phase_line_dat, phase_line_dat[[cluster_name]], function(x) x[which.min(x$phase_time), ] ))
    phase_line_dat <- phase_line_dat[,-1]
  }

  if (design=="MBP") {
    p <- 
      ggplot2::ggplot(dat, ggplot2::aes(.data[[session_name]], .data[[outcome_name]], 
                                        color = .data[[phase_name]], shape = .data[[phase_name]])) +
      ggplot2::facet_wrap(ggplot2::sym(case_name), ncol = 1)
  } else if (design == "TR") {
    names(dat)[6] <- "phase_pair"
    p <- 
      ggplot2::ggplot(dat, ggplot2::aes(.data[[session_name]], .data[[outcome_name]], 
                                        color = .data[[phase_name]], shape = .data[[phase_name]], 
                                        group = interaction(.data[[phase_name]], phase_pair))) +
      ggplot2::facet_wrap(ggplot2::sym(case_name), ncol = 1)
  } else if (design == "RMBB") {
    p <- 
      ggplot2::ggplot(dat, ggplot2::aes(.data[[session_name]], .data[[outcome_name]], 
                                        color = .data[[case_name]], group = .data[[phase_name]])) +
      ggplot2::facet_wrap(~ caseSeries, dir = "v", ncol = length(unique(dat[[case_name]])), 
                          labeller = ggplot2::labeller(caseSeries = ggplot2::label_wrap_gen(25)))
  } else if (design == "CMB") {
    p <- 
      ggplot2::ggplot(dat, ggplot2::aes(.data[[session_name]], .data[[outcome_name]], 
                                        shape = .data[[phase_name]], color = .data[[case_name]])) +
      ggplot2::facet_wrap(ggplot2::sym(cluster_name), ncol = 1) + 
      ggplot2::guides(color = "none")
  } else {
    stop("The `design` argument must be one of 'TR', 'MBP', 'RMBB', or 'CMB'.")
  }

  p <- 
    p +
    ggplot2::geom_point() +
    ggplot2::geom_line() +
    ggplot2::theme_bw() +
    ggplot2::labs(color = "", shape = "", x = session_name) + 
    ggplot2::geom_vline(data = phase_line_dat, ggplot2::aes(xintercept = phase_time, color = NULL, group = NULL), linetype = "dashed")

  # If using vector variable inputs, skip model fit lines
  if (is.null(model_fit) || (is.null(data) & !use_model_data)) return(p)

  # Otherwise add model fit lines
  if (is.null(newdata)) {
    if (!is.null(data)) {
      newdata <- data
    } else {
      newdata <- model_dat
    } 
    
  }
  
  if (design == "TR") {
    newdata$phase_pair <- dat$phase_pair
  } else if (design == "RMBB") {
    newdata$caseSeries <- as.factor(paste(newdata[[case_name]], newdata[[series_name]], sep = "-"))
  } else if (design == "CMB") {
    newdata$clusterCase <- as.factor(paste(newdata[[cluster_name]], newdata[[case_name]], sep = "-:-"))
  }
  
  newdata$fitted <- if (design == "CMB") {
    predict(model_fit, newdata = newdata, level = 1) 
  } else {
    predict(model_fit, newdata = newdata)
  }
  
  if (utils::packageVersion("ggplot2") < '3.4.0') {
    p <- p + ggplot2::geom_line(data = newdata, ggplot2::aes(y = fitted), size = 0.8)
  } else {
    p <- p + ggplot2::geom_line(data = newdata, ggplot2::aes(y = fitted), linewidth = 0.8)
  }
  
  return(p)
  
}

#--------------------
# Alber-Morgan
#--------------------

AlberMorgan <- read.csv("auxilliary/Alber-Morgan-2007.csv", stringsAsFactors = FALSE)
str(AlberMorgan)
AlberMorgan <- within(AlberMorgan, {
  case <- factor(case, levels = c("Theo","Kelly","Brian","Andrew"))
  condition <- factor(phase, levels = c("baseline","treatment"))
  phase <- NULL
})
AlberMorgan <- AlberMorgan[c("case","condition","session","outcome")]
str(AlberMorgan)
levels(AlberMorgan$case)

save(AlberMorgan, file = "data/AlberMorgan.RData", compress = TRUE, version = 2)

#--------------------
# Anglesea
#--------------------

Anglesea <- read.csv("auxilliary/Anglesea.csv", stringsAsFactors = FALSE)
Anglesea <- within(Anglesea, {
  case <- factor(paste("Case",case))
  condition <- factor(condition, levels = 0:1, labels = c("baseline","treatment"))
})
str(Anglesea)

save(Anglesea, file = "data/Anglesea.RData", compress = TRUE, version = 2)

#--------------------
# Barton-Arwood
#--------------------

BartonArwood <- read.csv("auxilliary/Barton-Arwood-2005.csv", stringsAsFactors = FALSE)
BartonArwood$phase <- NULL
str(BartonArwood)
BartonArwood <- within(BartonArwood, {
  case <- factor(case, levels = c("Sam","Gerald","Rich","Jack","Emma","Kim"))
  condition <- factor(condition, levels = c("A","B"))
})
str(BartonArwood)

save(BartonArwood, file = "data/BartonArwood.RData", compress = TRUE, version = 2)

#--------------------
# Carson
#---------------------

Carson <- read.csv("auxilliary/Carson.csv", stringsAsFactors = FALSE)
str(Carson)

save(Carson, file = "data/Carson.RData", compress = TRUE, version = 2)


#--------------------
# Lambert
#--------------------

Lambert <- read.csv("auxilliary/Lambert.csv", stringsAsFactors = FALSE)
Lambert <- subset(Lambert, !is.na(outcome))
Lambert <- within(Lambert, {
  case <- factor(paste("case",case))
  measure <- factor(measure, labels = c("academic response", "disruptive behavior"))
  treatment <- factor(treatment, levels = c("SSR","RC"))
  outcome <- round(outcome, 6)
})
str(Lambert)

save(Lambert, file = "data/Lambert.RData", compress = TRUE, version = 2)


#--------------------
# Laski
#--------------------

Laski <- read.csv("auxilliary/Laski.csv", stringsAsFactors = FALSE)
str(Laski)
Laski <- within(Laski, {
  case <- factor(paste("Case",case))
  treatment <- factor(treatment, levels = 0:1, labels = c("baseline","treatment"))
})

save(Laski, file = "data/Laski.RData", compress = TRUE, version = 2)


#--------------------
# Musser
#--------------------

Musser <- read.csv("auxilliary/Musser.csv", stringsAsFactors = FALSE)
str(Musser)

Musser <- within(Musser, {
  student <- factor(student, levels = 1:5, labels = c("Student 1","Student 2","Student 3","Female Control","Male Control"))
  treatment <- factor(treatment, levels = 0:2, labels = c("baseline","treatment","follow-up"))
})

save(Musser, file = "data/Musser.RData", compress = TRUE, version = 2)

#--------------------
# Rodriguez
#--------------------

Rodriguez <- read.csv("auxilliary/Rodriguez-2014.csv", stringsAsFactors = FALSE)
Rodriguez$phase <- NULL
str(Rodriguez)
Rodriguez <- within(Rodriguez, {
  case <- factor(case, levels = unique(case))
  condition <- factor(condition, levels = c("A","B"))
})
str(Rodriguez)

save(Rodriguez, file = "data/Rodriguez.RData", compress = TRUE, version = 2)

#--------------------
# Romaniuk
#--------------------

Romaniuk <- read.csv("auxilliary/Romaniuk-2002.csv", stringsAsFactors = FALSE)
Romaniuk <- within(Romaniuk, {
  case <- factor(case, levels = unique(case))
  condition <- factor(condition, levels = c("No Choice","Choice","Differential reinforcement of alternative behavior"))
  measurement <- ifelse(case=="Riley", "Responses per minute", "Percentage of session time")
})
str(Romaniuk)

save(Romaniuk, file = "data/Romaniuk.RData", compress = TRUE, version = 2)

#--------------------
# Ruiz
#--------------------
library(readxl)
library(tidyverse)

Ruiz <- read_excel('auxilliary//Data_Sheet_1_A Multiple-Baseline Evaluation.xlsx', sheet = "Overall")


Ruiz <- Ruiz %>% 
  pivot_longer(cols= `PSWQ`:`VQ-OBSTRUCTION`, names_to = "measure", values_to = "outcome")


Ruiz <- Ruiz %>%
  select(case, time=session, measure, outcome) %>%
  mutate(
    treatment = case_when(
      time < 6 ~ "baseline",
      time == 6 ~ "treatment",
      time == 7 ~ "treatment",
      time == 8 ~ "post",
      time > 8 ~ "follow-up"
    ),
    treatment = as.factor(treatment),
    case = as.factor(case),
    measure = as.factor(measure)
  )

save(Ruiz, file = "data/Ruiz.RData", compress = TRUE, version = 2, version = 2)


#--------------------
# Saddler
#--------------------

Saddler <- read.csv("auxilliary/Saddler.csv", stringsAsFactors = FALSE)
Saddler <- within(Saddler, {
  case <- factor(paste("Case",case))
  measure <- factor(measure, levels = 1:3, labels = c("writing quality", "T-unit length", "number of constructions"))
  treatment <- factor(treatment, levels = 0:1, labels = c("baseline","treatment"))
})
str(Saddler)
save(Saddler, file = "data/Saddler.RData", compress = TRUE, version = 2)


#--------------------
# Schutte
#--------------------

Schutte <- read.csv("auxilliary/Schutte.csv", stringsAsFactors = FALSE)
str(Schutte)

Schutte <- within(Schutte, {
  case <- factor(case, levels = 1:13, labels = paste("Case",1:13))
  treatment <- factor(treatment, levels = c("baseline","treatment"))
})
Schutte <- droplevels(subset(Schutte, case != "Case 4"))

save(Schutte, file = "data/Schutte.RData", compress = TRUE, version = 2)


#--------------------
# Salazar
#--------------------

library(readxl)
library(tidyverse)

sheet <- excel_sheets('auxilliary//Raw Data.xlsx')
datFile <- lapply(setNames(sheet, sheet), function(x) read_excel("auxilliary//Raw Data.xlsx", sheet=x))
datFile <-  bind_rows(datFile, .id="Sheet")

datFile <- datFile %>% 
  select(Sheet:`GPQ-C`) %>%
  pivot_longer(cols= `AFQ-Y`:`GPQ-C`, names_to = "measure", values_to = "outcome")

datFile <- datFile %>%
  select(case = Sheet, measure, condition= `...1`, outcome) %>%
  mutate(
    treatment = case_when(
      str_detect(condition, "LB") ~ "baseline",
      str_detect(condition, "Int") ~ "treatment",
      str_detect(condition, "Post") ~ "post",
      str_detect(condition, "FU") ~ "follow-up",
    ),
    condition = as.factor(condition),
    measure = as.factor(measure),
    case = as.factor(case)
  ) %>%
  group_by(case, measure) %>%
  mutate(time = as.numeric(row_number())) %>%
  select(case, measure, treatment, time, outcome)

Salazar <- as.data.frame(datFile)

save(Salazar, file = "data/Salazar.RData", compress = TRUE, version = 2)


#--------------------
# Thorne
#--------------------

library(readr)
library(dplyr)

phase_pairs <- function(phase) {
  conditions <- levels(phase)
  n <- length(phase)
  y <- rep(1,n)
  for (i in 2:n) {
    (which_lev <- match(phase[i-1], conditions))
    (which_conditions <- conditions[c(which_lev, which_lev + 1)])
    !(phase[i] %in% which_conditions)
    (y[i] <- y[i - 1] + !(phase[i] %in% which_conditions))
  }
  y
}

Thorne <- 
  read_csv("auxilliary/Thorne.csv") %>%
  select(case = Case_number, measure = Measure, session = Session_number, 
         condition = Condition, phase_indicator = Trt, outcome = Outcome) %>%
  mutate(
    case = factor(case, levels = 1:12, labels = paste("Participant", 1:12)),
    measure = factor(measure),
    phase_id = factor(condition, levels = c("A","B"))
  ) %>%
  arrange(measure, case, session) %>%
  group_by(measure, case) %>% 
  mutate(
    phase_pair = phase_pairs(phase_id),
    phase_id = paste0(phase_id, phase_pair)
  ) %>%
  select(case, measure, session, phase_id, condition, phase_indicator, outcome) %>%
  as.data.frame()

str(Thorne)

save(Thorne, file = "data/Thorne.RData", compress = TRUE, version = 2)

#--------------------
# Bryant2018
#--------------------

Bryant2018 <- read.csv("auxilliary/Bryant2018.csv", stringsAsFactors = FALSE)
str(Bryant2018)

Bryant2018 <- within(Bryant2018, {
  Study_ID <- rep("Bryant2018", nrow(Bryant2018))
  school <- factor(school)
  group <- factor(group)
  case <- factor(case)
  treatment <- factor(treatment)
})

Bryant2018 <- Bryant2018[c("Study_ID", "school", "group","case","treatment","session","session_trt","outcome")]

# time-point constants
A <- 5
B <- 22

# center at follow-up time
Center <- B
Bryant2018$session_c <-Bryant2018$session - Center

save(Bryant2018, file = "data/Bryant2018.RData", compress = TRUE, version = 2)

#--------------------
# Thiemann2001
#--------------------

library(tidyverse)

Thiemann2001 <- read.csv("auxilliary/Thiemann2001.csv", stringsAsFactors = FALSE)
str(Thiemann2001)

Thiemann2001 <- Thiemann2001 %>%
  arrange(case, series, time) %>%
  group_by(case, series) %>%
  mutate(trt_time = pmax(0, time - max(time[treatment == "A"]))) %>%
  as.data.frame()

Thiemann2001 <- within(Thiemann2001, {
  case <- factor(case)
  series <- factor(series)
  treatment <- ifelse(treatment == "A", "baseline", "treatment")
  treatment <- factor(treatment)
})

# time-point constants
A <- 8
B <- 30

# center at follow-up time
Center <- B
Thiemann2001$time_c <- Thiemann2001$time - Center

save(Thiemann2001, file = "data/Thiemann2001.RData", compress = TRUE, version = 2)


#--------------------
# Thiemann2004
#--------------------

Thiemann2004 <- read.csv("auxilliary/Thiemann2004.csv", stringsAsFactors = FALSE)
str(Thiemann2004)

Thiemann2004 <- Thiemann2004 %>%
  arrange(case, series, time) %>%
  group_by(case, series) %>%
  mutate(
    trt_time = pmax(0, time - max(time[treatment == "A"]))
  ) %>%
  as.data.frame()

Thiemann2004 <- within(Thiemann2004, {
  case <- factor(case)
  series <- factor(series)
  treatment <- ifelse(treatment == "A", "baseline", "treatment")
  treatment <- factor(treatment)
})

# time-point constants
A <- 10
B <- 33

# center at follow-up time
Center <- B
Thiemann2004$time_c <- Thiemann2004$time - Center

save(Thiemann2004, file = "data/Thiemann2004.RData", compress = TRUE, version = 2)


#-------------------------
# methods guide datasets
#-------------------------

CaseHarrisGraham <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 1-2.xlsx", sheet = "Case Harris for App rev2")
str(CaseHarrisGraham)
CaseHarrisGraham <- 
  CaseHarrisGraham %>% 
  mutate(
    case = factor(`Case identifier`, levels = c("Ben","Abernathy","Willow","Paladin")),
    condition = factor(`Phase identifier`, levels = c("b", "i"), labels = c("baseline", "treatment"))
  ) %>% 
  dplyr::filter(!is.na(`Outcome variable`)) %>% 
  rename(session = `Session number`, outcome = `Outcome variable`) %>% 
  select(case, session, condition, outcome) %>%
  as.data.frame()

save(CaseHarrisGraham, file = "data/CaseHarrisGraham.RData", compress = TRUE, version = 2)


Peltier <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 1-2.xlsx", sheet = 3)
str(Peltier)
Peltier <- 
  Peltier %>% 
  mutate(
    case = factor(case, levels = c("Asher","Drake","Gary","Joe","Jack","Kloe","Mike","Andy","Kyrie","Ellie","Gabe","Javon")),
    condition = factor(phase, levels = c("b", "i"), labels = c("baseline", "treatment"))
  ) %>% 
  dplyr::filter(!is.na(outcome)) %>% 
  arrange(case, session) %>%
  select(case, session, condition, outcome) %>%
  as.data.frame()

save(Peltier, file = "data/Peltier.RData", compress = TRUE, version = 2)


GunningEspie <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 3-4.xlsx", sheet = 3)

str(GunningEspie)

GunningEspie <- 
  GunningEspie %>% 
  mutate(
    case = factor(Case, levels = c("A", "C", "D", "E", "F", "H", "I")),
    condition = factor(Phase, levels = c("b","i","f"), labels = c("baseline","treatment","follow-up"))
  ) %>% 
  rename(session = Session, outcome = `DV value`) %>% 
  dplyr::filter(condition != "follow-up") %>% 
  arrange(case, session) %>%
  select(case, session, condition, outcome) %>%
  as.data.frame()

save(GunningEspie, file = "data/GunningEspie.RData", compress = TRUE, version = 2)


DelemereDounavi <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 3-4.xlsx", sheet = 2)
str(DelemereDounavi)
DelemereDounavi <- 
  DelemereDounavi %>% 
  mutate(
    case = factor(Case, levels = c("Niamh", "Mary", "Thomas", "Martin", "Alan", "John")),
    intervention = if_else(case %in% c("Martin","Alan","John"), "Positive routines", "Bedtime fading"),
    condition = factor(Phase, levels = c("b", "i"), labels = c("baseline", "treatment"))
  ) %>% 
  rename(session = Session, outcome = `DV value`) %>% 
  arrange(intervention, case, session) %>%
  select(intervention, case, session, condition, outcome) %>%
  as.data.frame()

save(DelemereDounavi, file = "data/DelemereDounavi.RData", compress = TRUE, version = 2)


Datchuk <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 7-8.xlsx", sheet = 2)
str(Datchuk)
Datchuk <- 
  Datchuk %>% 
  mutate(
    case = factor(case, levels = c("Nathan", "Richard", "Danyell", "Kendrick")),
    condition = as.factor(phase),
    session = round(session)
  ) %>%
  select(case, session, condition, outcome) %>%
  as.data.frame()

save(Datchuk, file = "data/Datchuk.RData", compress = TRUE, version = 2)


Rodgers <- readxl::read_xlsx("auxilliary/Methods Guide - DCES Models 7-8.xlsx", sheet = 1)
str(Rodgers)
Rodgers <- 
  Rodgers %>% 
  mutate(
    case = factor(case, levels = c("Brett", "Steve", "Barb", "Denny")),
    condition = as.factor(phase),
    session = round(session)
  ) %>%
  select(case, session, condition, outcome) %>%
  as.data.frame()

save(Rodgers, file = "data/Rodgers.RData", compress = TRUE, version = 2)

##------------------------------------------------------------------------------
## Create AR(1) correlation and inverse correlation matrices
##------------------------------------------------------------------------------

AR1_corr <- function(phi, times) phi^as.matrix(dist(times))


AR1_corr_block <- function(phi, block, times=NULL) {
  if (is.null(times)) times <- lapply(table(block), seq, from=1)
  lapply(times, AR1_corr, phi=phi)
}


AR1_corr_inv <- function(phi, n) 
  diag((rep(1,n) + c(0,rep(phi^2,n-2),0))/(1 - phi^2)) + 
      rbind(cbind(rep(0,n-1), diag(rep(-phi/(1 - phi^2),n-1))), rep(0,n)) + 
      rbind(rep(0,n), cbind(diag(rep(-phi/(1 - phi^2),n-1)),rep(0,n-1)))



##------------------------------------------------------------------------------
## Create lme covariance and inverse covariance matrices
##------------------------------------------------------------------------------

# create covariance matrix from Tau parameters 

Tau_matrix <- function(Tau_coef) {
  q <- (sqrt(1 + 8 * length(Tau_coef)) - 1) / 2
  Tau_mat <- matrix(NA,q,q)
  Tau_mat[upper.tri(Tau_mat, diag=TRUE)] <- Tau_coef
  Tau_mat[lower.tri(Tau_mat)] <- t(Tau_mat)[lower.tri(Tau_mat)]
  return(Tau_mat)
}


# create block-diagonal covariance matrix with AR(1) level-1 error

lmeAR1_cov_block <- function(block, Z_design, theta, times = NULL) {
  if(is.matrix(theta$Tau)) {
    Tau_mat <- theta$Tau 
    } else {
      Tau_mat <- Tau_matrix(theta$Tau)
    }
  ZTauZ <- by(Z_design, block, function(z) {
            z_mat <- as.matrix(z)
            z_mat %*% Tau_mat %*% t(z_mat)})
  AR1_mat <- AR1_corr_block(phi=theta$phi, block=block, times=times)
  mapply(function(a,b) a + theta$sigma_sq * b, ZTauZ, AR1_mat, SIMPLIFY = FALSE)
}


# create inverse covariance matrix for a single block

# Note: This function assumes that either 
# a) the Tau argument is passed as the eigen-decomposition of Tau
# including an indicator vector for which dimensions to use or 
# b) Tau is an invertible matrix.
# Option (a) must be used if Tau is of less than full rank.
# what about 1-dimensional Tau with tau_0^2 = 0?

lmeAR1_cov_inv  <- function(Z_design, sigma_sq, phi, Tau, times=NULL) {
  if (is.null(times)) {
    n <- ifelse(is.vector(Z_design), length(Z_design), dim(Z_design)[1])
    A_inv <- AR1_corr_inv(phi, n) / sigma_sq
  } else {
    A_inv <- solve(AR1_corr(phi, times)) / sigma_sq
  }

  if (is.matrix(Tau)) {
    # otherwise assume that Tau is invertible
    Z_mat <- as.matrix(Z_design)
    Tau_inv <- chol2inv(chol(Tau))
  } else {
    if (sum(Tau$use) == 0) return(A_inv) else {
      # use eigen-decomposition of Tau if available
      Z_mat <- as.matrix(Z_design) %*% Tau$vectors[,Tau$use,drop=FALSE]
      Tau_inv <- diag(1/Tau$values[Tau$use], nrow=sum(Tau$use))
    }
  }
  A_inv_Z <- A_inv %*% Z_mat
  A_inv - A_inv_Z %*% chol2inv(chol(Tau_inv + t(Z_mat) %*% A_inv_Z)) %*% t(A_inv_Z)  
}


# create block-diagonal inverse covariance matrix

lmeAR1_cov_block_inv <- function(block, Z_design, theta, times=NULL) {
  Tau_eigen <- eigen(Tau_matrix(theta$Tau), symmetric=TRUE)
  Tau_eigen$use <- round(Tau_eigen$values,14) > 0
  if (is.null(times)) {
    by(Z_design, block, lmeAR1_cov_inv, sigma_sq=theta$sigma_sq, phi=theta$phi, Tau=Tau_eigen)
  } else {
    Z_list <- by(Z_design, block, identity)
    mapply(lmeAR1_cov_inv, Z_design = Z_list, times = times, 
           MoreArgs = list(sigma_sq=theta$sigma_sq, phi=theta$phi, Tau=Tau_eigen), 
           SIMPLIFY = FALSE)
  }
}


##------------------------------------------------------------------------------
## create Q matrix
##------------------------------------------------------------------------------

Q_matrix <- function(block, X_design, Z_design, theta, times=NULL) {
  V_inv <- lmeAR1_cov_block_inv(block=block, Z_design=Z_design, theta=theta, times=times)
  Vinv_X <- prod_blockmatrix(V_inv, X_design, block = block)
  VinvX_XVXinv_XVinv <- Vinv_X %*% chol2inv(chol(t(X_design) %*% Vinv_X)) %*% t(Vinv_X)
  block_minus_matrix(V_inv, VinvX_XVXinv_XVinv, block)
}


##------------------------------------------------------------------------------
## Create first derivative matrices
##------------------------------------------------------------------------------

dV_dsigmasq <- function(block, times, phi)
  AR1_corr_block(phi=phi, block=block, times=times)
#dV_dsigmasq(block=rep(1:3, each = 4), times=NULL, phi=0.5)

dV_dphi <- function(block, times, phi, sigma_sq) {
  if (is.null(times)) times <- lapply(table(block), seq, from=1)
  time_diff <- lapply(times, function(x) as.matrix(dist(x)))
  lapply(time_diff, function(x) sigma_sq * x * phi^pmax(0, x - 1))
}

dV_dTau_index <- function(block, Z_design, tau_index)
  by(Z_design, block, function(Z) 
    as.matrix(Z)[,tau_index, drop=FALSE] %*% t(Z)[rev(tau_index),,drop=FALSE])

dV_dTau_unstruct <- function(block, Z_design) {
  Tau_q <- dim(Z_design)[2]
  tau_index <- cbind(unlist(sapply(1:Tau_q, function(x) seq(1,x))), 
                     unlist(sapply(1:Tau_q, function(x) rep(x,x))))
  apply(tau_index, 1, function(t) dV_dTau_index(block, Z_design, tau_index=unique(t)))
}

##------------------------------------------------------------------------------
## extract variance components
##------------------------------------------------------------------------------

extract_varcomp_lmeAR1 <- function(m_fit) {

  sigma_sq <- m_fit$sigma^2                                         # sigma^2
  phi <- as.double(coef(m_fit$modelStruct$corStruct, FALSE))        # phi
  Tau_coef <- coef(m_fit$modelStruct$reStruct, FALSE) * sigma_sq    # unique coefficients in Tau

  varcomp <- list(sigma_sq=sigma_sq, phi=phi, Tau = Tau_coef)
  class(varcomp) <- "varcomp"
  return(varcomp)
}


##------------------------------------------------------------------------------
## Expected Information Matrix 
##------------------------------------------------------------------------------

Info_Expected <- function(theta, X_design, Z_design, block, times=NULL) {
  
  Q_mat <- Q_matrix(block, X_design, Z_design, theta, times=times) 
  
  # create N * N * r array with QdV entries
  r <- length(unlist(theta))
  QdV <- array(NA, dim = c(dim(Q_mat),r))
  QdV[,,1] <- prod_matrixblock(Q_mat, dV_dsigmasq(block=block, times=times, phi=theta$phi), block=block)
  QdV[,,2] <- prod_matrixblock(Q_mat, dV_dphi(block=block, times=times, phi=theta$phi, sigma_sq=theta$sigma_sq), block=block)
  QdV[,,-2:-1] <- unlist(lapply(dV_dTau_unstruct(block, Z_design), prod_matrixblock, A=Q_mat, block=block))
  
  # calculate I_E
  I_E <- matrix(NA, r, r)
  for (i in 1:r)
    for (j in 1:i)
      I_E[i,j] <- product_trace(QdV[,,i], QdV[,,j]) / 2
  I_E[upper.tri(I_E)] <- t(I_E)[upper.tri(I_E)]
  
  return(I_E)
}

#' @title Calculate expected information matrix
#' 
#' @description Calculates the expected information matrix from a fitted linear mixed effects
#' model with AR(1) correlation structure in the level-1 errors.
#' 
#' @param m_fit Fitted model of class lme, with AR(1) correlation structure at level 1.
#' 
#' @export 
#' 
#' @return Expected Information matrix corresponding to variance components of \code{m_fit}.
#' 
#' @examples
#' data(Laski)
#' Laski_RML <- lme(fixed = outcome ~ treatment, 
#'                  random = ~ 1 | case, 
#'                  correlation = corAR1(0, ~ time | case), 
#'                  data = Laski)
#' Info_Expected_lmeAR1(Laski_RML)


Info_Expected_lmeAR1 <- function(m_fit) {
  
  theta <- extract_varcomp_lmeAR1(m_fit)
  X_design <- model.matrix(m_fit, data = m_fit$data)
  Z_design <- model.matrix(m_fit$modelStruct$reStruct, data = m_fit$data)
  block <- nlme::getGroups(m_fit)
  times <- attr(m_fit$modelStruct$corStruct, "covariate")
  Info_Expected(theta=theta, X_design=X_design, Z_design=Z_design, block=block, times=times)
}


## estimate adjusted REML effect size (with associated estimates) for multiple baseline design ####

#' @title Calculates adjusted REML effect size
#' 
#' @description Estimates a design-comparable standardized mean difference effect size based on data 
#' from a multiple baseline design, using adjusted REML method as described in Pustejovsky, Hedges, 
#' & Shadish (2014). Note that the data must contain one row per measurement occasion per case.
#' 
#' @param m_fit Fitted model of class lme, with AR(1) correlation structure at level 1.
#' @param p_const Vector of constants for calculating numerator of effect size. 
#' Must be the same length as fixed effects in \code{m_fit}.
#' @param r_const Vector of constants for calculating denominator of effect size. 
#' Must be the same length as the number of variance component parameters in \code{m_fit}.
#' @param X_design (Optional) Design matrix for fixed effects. Will be extracted from \code{m_fit} if not specified.
#' @param Z_design (Optional) Design matrix for random effects. Will be extracted from \code{m_fit} if not specified.
#' @param block (Optional) Factor variable describing the blocking structure. Will be extracted from \code{m_fit} if not specified.
#' @param times (Optional) list of times used to describe AR(1) structure. Will be extracted from \code{m_fit} if not specified.
#' @param returnModel (Optional) If true, the fitted input model is included in the return.
#' 
#' @export 
#' 
#' @return A list with the following components
#' \tabular{ll}{
#' \code{p_beta} \tab Numerator of effect size \cr
#' \code{r_theta} \tab Squared denominator of effect size \cr
#' \code{delta_AB} \tab Unadjusted (REML) effect size estimate \cr
#' \code{nu} \tab Estimated denominator degrees of freedom \cr
#' \code{kappa} \tab Scaled standard error of numerator \cr
#' \code{g_AB} \tab Corrected effect size estimate \cr
#' \code{V_g_AB} \tab Approximate variance estimate \cr
#' \code{cnvg_warn} \tab Indicator that model did not converge \cr
#' \code{sigma_sq} \tab Estimated level-1 variance \cr
#' \code{phi} \tab Estimated autocorrelation \cr
#' \code{Tau} \tab Vector of level-2 variance components \cr
#' \code{I_E_inv} \tab Expected information matrix \cr
#' }
#' 
#' @references Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
#' Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
#' \emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
#' 
#' @examples
#' data(Laski)
#' Laski_RML <- lme(fixed = outcome ~ treatment, 
#'                  random = ~ 1 | case, 
#'                  correlation = corAR1(0, ~ time | case), 
#'                  data = Laski)
#' summary(Laski_RML)
#' g_REML(Laski_RML, p_const = c(0,1), r_const = c(1,0,1), returnModel=FALSE)
#' 
#' data(Schutte)
#' Schutte$trt.week <- with(Schutte, unlist(tapply((treatment=="treatment") * week, 
#'          list(treatment,case), function(x) x - min(x))) + (treatment=="treatment"))
#' Schutte$week <- Schutte$week - 9
#' Schutte_RML <- lme(fixed = fatigue ~ week + treatment + trt.week, 
#'                    random = ~ week | case, 
#'                    correlation = corAR1(0, ~ week | case), 
#'                    data = subset(Schutte, case != 4))
#' summary(Schutte_RML)
#' Schutte_g <- g_REML(Schutte_RML, p_const = c(0,0,1,7), r_const = c(1,0,1,0,0))
#' summary(Schutte_g)

g_REML <- function(m_fit, p_const, r_const, 
                   X_design = model.matrix(m_fit, data = m_fit$data), 
                   Z_design = model.matrix(m_fit$modelStruct$reStruct, data = m_fit$data), 
                   block = nlme::getGroups(m_fit),
                   times = attr(m_fit$modelStruct$corStruct, "covariate"), returnModel=TRUE) {
  
  .Deprecated("g_mlm", msg = "'g_REML()' is deprecated and may be removed in a later version of the package. Please use 'g_mlm()' instead.")

  # basic model estimates
  p_beta <- sum(nlme::fixed.effects(m_fit) * p_const)               # p'Beta
  theta <- extract_varcomp_lmeAR1(m_fit)                                # full theta vector
  r_theta <- sum(unlist(theta) * r_const)                           # r'theta
  delta_AB <- p_beta / sqrt(r_theta)                                # delta_AB              
  kappa_sq <- as.numeric(t(p_const) %*% vcov(m_fit) %*% p_const) / r_theta    # kappa^2
  cnvg_warn <- !check_convergence(m_fit)                       # indicator that RML estimation has not converged
      
  # calculate inverse expected information
  I_E <- Info_Expected(theta=theta, X_design=X_design, Z_design=Z_design, block=block, times=times)
  I_E_inv <- chol2inv(chol(I_E))
  

  nu <- 2 * r_theta^2 / as.numeric(t(r_const) %*% I_E_inv %*% r_const)
  g_AB <- J(nu) * delta_AB
  nu_trunc <- max(nu, 2.001)
  V_g_AB <- J(nu)^2 * (nu_trunc * kappa_sq / (nu_trunc - 2) + g_AB^2 * (nu_trunc / (nu_trunc - 2) - 1 / J(nu_trunc)^2))
  
  res <- c(list(p_beta=p_beta, r_theta=r_theta, delta_AB=delta_AB, nu=nu, kappa = sqrt(kappa_sq),
                   g_AB=g_AB, V_g_AB=V_g_AB, cnvg_warn=cnvg_warn), theta, list(I_E_inv=I_E_inv,
                   p_const=p_const, r_const=r_const))
  
  if (returnModel) {
    res <- c(res, m_fit)
  }    
  
  class(res) <- "g_REML"
  
  return(res)
}

#' @export 

summary.g_REML <- function(object, digits = 3, ...) {
  
  if (is.null(object$modelStruct)) {
    
    stop("'summary()' method only available when setting 'returnModel = TRUE` in `g_REML()`.")
    
  }   
  
  varcomp <- with(object, cbind(est = c(sigma_sq = sigma_sq, phi = phi, Tau, "total variance" = r_theta),
                                se = c(sqrt(diag(I_E_inv)), r_theta * sqrt(2 / nu))))
  betas <- with(object, cbind(est = c(coefficients$fixed, "treatment effect at a specified time" = p_beta),
                              se = c(sqrt(diag(varFix)), kappa * sqrt(r_theta))))
  ES <- with(object, cbind(est = c("unadjusted effect size" = delta_AB, "adjusted effect size" = g_AB,
                                   "degree of freedom" = nu, kappa = kappa, logLik=logLik),
                           se = c(sqrt(V_g_AB) / J(nu), sqrt(V_g_AB), NA, NA, NA)))

  print(round(rbind(varcomp, betas, ES), digits), na.print = "")

}

#' @export 

print.g_REML <- function(x, digits = 3, ...) {
  ES <- with(x, cbind(est = c("unadjusted effect size" = delta_AB,
                              "adjusted effect size" = g_AB,
                              "degree of freedom" = nu),
                      se = c(sqrt(V_g_AB) / J(nu), sqrt(V_g_AB), NA)))
  print(round(ES, digits), na.print = "")
}

filter_vars <- c("A", "B", "C")
filter_vals <- list(A = c("apples","aardvarks"), B = "bananas", C = c("corn on the cob", "crackers"))

# subset(dat, A %in% c("apples","aardvarks") & B %in% c("bananas") & C %in% c("corn on the cob","crackers"))

filter_in <- lapply(filter_vals, function(x) paste0('"', x, '"', collapse = ","))
filter_in <- paste0("%in% c(", filter_in, ")")
filter_string <- paste(filter_vars, filter_vals, collapse = " & ")
filter_string

CI_SMD_single <- function(delta, kappa, nu, V_delta, cover, bound) {
  start_val <- suppressWarnings(J(nu) * (delta + c(-1, 1) * qt(1 - (1 - cover) / 2, df = nu) * sqrt(V_delta)) / kappa)
  L <- kappa * nlminb(start = start_val[1],
                      objective = function(ncp) suppressWarnings((qt((1 - cover) / 2, df = nu, ncp=-ncp) + delta / kappa)^2),
                      lower = -bound, upper = bound)$par
  U <- kappa * nlminb(start = start_val[2],
                      objective = function(ncp) suppressWarnings((qt((1 - cover)  / 2, df = nu, ncp=ncp) - delta / kappa)^2), 
                      lower = -bound, upper = bound)$par
  if (L == -bound * kappa) L <- start_val[1] * kappa
  if (U == bound * kappa) U <- start_val[2] * kappa
  c(L,U)
}

CI_SMD <- function(delta, kappa, nu, cover = 0.95, bound = 35) {
  nu <- pmax(nu, 2.001)
  V_delta = kappa^2 * nu / (nu - 2) + delta^2 * (nu / (nu - 2) - 1 / J(nu)^2)
  mapply(CI_SMD_single, delta = delta, kappa = kappa, nu = nu, V_delta = V_delta,
         MoreArgs = list(cover = cover, bound = bound))
}

coverage <- function(delta, CI) CI[1,] < delta & CI[2,] > delta


#' @method CI_g g_REML
#' @export

CI_g.g_REML <- function(g, cover = 0.95, bound = 35, symmetric = TRUE) {
  delta <- g$delta_AB
  g_AB <- g$g_AB
  V_g_AB <- g$V_g_AB
  kappa <- g$kappa
  nu <- g$nu
  V_delta <- kappa^2 * nu / (nu - 2) + delta^2 * (nu / (nu - 2) - 1 / J(nu)^2)
  
  if (symmetric) {
    
    suppressWarnings(g_AB + c(-1, 1) * stats::qt(1 - (1 - cover) / 2, df = nu) * sqrt(V_g_AB))
    
  } else{
    
    CI_SMD_single(delta = delta, kappa = kappa, nu = nu, V_delta = V_delta,
                  cover = cover, bound = bound)
    
  }
  
}

#' @method CI_g g_HPS
#' @export

CI_g.g_HPS <- function(g, cover = 0.95, bound = 35, symmetric = TRUE) {
  delta <- g$delta_hat_unadj
  delta_adj <- g$delta_hat
  V_delta_adj <- g$V_delta_hat
  kappa <- g$theta
  nu <- g$nu
  V_delta <- kappa^2 * nu / (nu - 2) + delta^2 * (nu / (nu - 2) - 1 / J(nu)^2)
  
  if (symmetric) {
    
    suppressWarnings(delta_adj + c(-1, 1) * stats::qt(1 - (1 - cover) / 2, df = nu) * sqrt(V_delta_adj))
    
  } else {
    
    CI_SMD_single(delta=delta, kappa=kappa, nu=nu, V_delta=V_delta,
                  cover=cover, bound=bound)
    
  }
  
}

#' @importFrom lmeInfo extract_varcomp
#' @importFrom lmeInfo varcomp_vcov
#' @importFrom lmeInfo g_mlm
#' @importFrom lmeInfo CI_g

#' @title lmeInfo
#' @description 
#' Functions imported from the lmeInfo package.
#' 
#' \itemize{
#'   \item \code{\link[lmeInfo]{extract_varcomp}}
#'   \item \code{\link[lmeInfo]{g_mlm}}
#'   \item \code{\link[lmeInfo]{varcomp_vcov}}
#' }
#' 
#' @name lmeInfo
#' @aliases extract_varcomp varcomp_vcov g_mlm
#' @export extract_varcomp g_mlm varcomp_vcov
#' @md
#' 
NULL

## symmetric and approximate non-central t confidence interval ####

#' @title Calculates a confidence interval for a standardized mean difference
#'   effect size
#'
#' @description Calculates a confidence interval given a \code{g_REML}, a
#'   \code{g_HPS}, or a \code{g_mlm} object using either a central t
#'   distribution (for a symmetric interval) or a non-central t distribution
#'   (for an asymmetric interval).
#'
#' @param g an estimated effect size object of class \code{g_REML}, class
#'   \code{g_HPS}, or class \code{g_mlm}.
#' @param cover confidence level
#' @param bound numerical tolerance for non-centrality parameter in
#'   \code{\link[stats]{qt}}.
#' @param symmetric If \code{TRUE} (the default), use a symmetric confidence
#'   interval. If \code{FALSE}, use a non-central t approximation to obtain an
#'   asymmetric confidence interval.
#'   
#' @name CI_g
#' @export CI_g
#'
#' @return A vector of upper and lower confidence bounds.
#'
#' @examples
#' data(Laski)
#' Laski_RML <- lme(fixed = outcome ~ treatment,
#'                  random = ~ 1 | case,
#'                  correlation = corAR1(0, ~ time | case),
#'                  data = Laski)
#' Laski_g_REML <- suppressWarnings(
#'   g_REML(Laski_RML, p_const = c(0,1), 
#'          r_const = c(1,0,1), returnModel = FALSE)
#' )
#' CI_g(Laski_g_REML, symmetric = TRUE)
#' CI_g(Laski_g_REML, symmetric = FALSE)
#'
#' Laski_HPS <- with(Laski, effect_size_MB(outcome, treatment, case, time))
#' CI_g(Laski_HPS, symmetric = FALSE)
#'
#' Laski_g_mlm <- g_mlm(mod = Laski_RML, p_const = c(0,1), r_const = c(1,0,1))
#' CI_g(Laski_g_mlm, symmetric = FALSE)
#' 
NULL

library(nlme)

test_that("scdhlm Works with Narozanic and Blair.", {
  
  skip(message = "Auxiliary dataset not included in package.")
  
  Naro <- read.csv("auxilliary/Narozanic and Blair_datasets_long_academic.csv", stringsAsFactors = FALSE)
  Naro$Session_int <- round(Naro$Session)
  
  Naro_dbl <- lme(Outcome ~ Phase,
                  random = ~ 1 | Participant, 
                  correlation = corAR1(0.1, ~ Session | Participant),
                  data = Naro)
  
  # g_mlm(Naro_dbl, p_const = c(0,1), r_const = c(1,0,1))
  # lmeInfo::Fisher_info(Naro_dbl)
  expect_error(g_mlm(Naro_dbl, p_const = c(0,1), r_const = c(1,0,1)))
  # This throw an error because of some wacky behavior in nlme::corMatrix()
  # where corAR1() gets converted to corARMA(), and Session spacing gets truncated
  # so that there are repeated values of Session within the same Participant
  
  Naro_int <- lme(Outcome ~ Phase,
                  random = ~ 1 | Participant, 
                  correlation = corAR1(0.1, ~ Session_int | Participant),
                  data = Naro)
  
  expect_s3_class(g_mlm(Naro_int, p_const = c(0,1), r_const = c(1,0,1)), "g_mlm")
  
})

### Example data

* Alber-Morgan, S. R., Ramp, E. M., Anderson, L. L., & Martin, C. M. (2007). Effects of repeated readings, error correction, and performance feedback on the fluency and comprehension of middle school students with behavior problems. *Journal of Special Education, 41*(1), 17-30.
DOI: [10.1177/00224669070410010201](http://dx.doi.org/10.1177/00224669070410010201)

* Anglesea, M. M., Hoch, H., & Taylor, B. A. (2008). Reducing rapid eating in teenagers with autism: Use of a pager prompt. *Journal of Applied Behavior Analysis, 41*(1), 107-111. DOI: [10.1901/jaba.2008.41-107](http://dx.doi.org/10.1901/jaba.2008.41-107)

* Barton-Arwood, S. M., Wehby, J. H., & Falk, K. B. (2005). Reading instruction for elementary-age students with emotional and behavioral disorders: Academic and behavioral outcomes. *Exceptional Children, 72*(1), 7-27. DOI: [10.1177/001440290507200101](https://doi.org/10.1177/001440290507200101)

* Bryant, D. R., Bryant, B. R., Sorelle-Miner, D. A., Falcomata, T. S. & Nozari, M. (2018). Tier 3 intensified intervention for second grade students with severe mathematics difficulties. *Archives of Psychology, 2*(11), 1-24. DOI:[10.31296/aop.v2i11.86](https://archivesofpsychology.org/index.php/aop/article/view/86)

* Case, L. P., Harris, K. R., & Graham, S. (1992). Improving the mathematical problem-solving skills of students with learning disabilities: Self-regulated strategy development. *The Journal of Special Education, 26*(1), 1-19. DOI: [10.1177/002246699202600101](https://doi.org/10.1177/002246699202600101)

* Datchuk, S. M. (2016). Writing simple sentences and descriptive paragraphs: Effects of an intervention on adolescents with writing difficulties. *Journal of Behavioral Education, 25*(2), 166-188. DOI: [10.1007/s10864-015-9236-x](https://doi.org/10.1007/s10864-015-9236-x)

* Delemere, E., & Dounavi, K. (2018). Parent-implemented bedtime fading and positive routines for children with autism spectrum disorders. *Journal of Autism and Developmental Disorders, 48*(4), 1002-1019. DOI: [10.1007/s10803-017-3398-4](https://doi.org/10.1007/s10803-017-3398-4)

* Gunning, M. J., & Espie, C.A. (2003). Psychological treatment of reported sleep disorder in adults with intellectual disability using a multiple baseline design. *Journal of Intellectual Disability Research, 47*(3), 191-202. DOI: [10.1046/j.1365-2788.2003.00461.x](https://doi.org/10.1046/j.1365-2788.2003.00461.x)

* Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). Effects of response cards on disruptive behavior and academic responding during math lessons by fourth-grade urban students. *Journal of Positive Behavior Interventions, 8*(2), 88-99. DOI: [10.1177/10983007060080020701](https://doi.org/10.1177/10983007060080020701)

* Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). Training parents to use the natural language paradigm to increase their autistic children's speech. *Journal of Applied Behavior Analysis, 21*(4), 391-400. DOI: [10.1901/jaba.1988.21-391](https://doi.org/10.1901/jaba.1988.21-391)

* Peltier, C., Sinclair, T. E., Pulos, J. M., & Suk, A. (2020). Effects of schema-based instruction on immediate, generalized, and combined structured word problems. *The Journal of Special Education, 54*(2), 101-112. DOI:  [10.1177/0022466919883397](https://doi.org/10.1177/0022466919883397)

* Rodgers, D. B., Datchuk, S. M., & Rila, A. L. (2021). Effects of a text-writing fluency intervention for postsecondary students with intellectual and developmental disabilities. *Exceptionality, 29*(4), 310-325. DOI: [10.1080/09362835.2020.1850451](https://doi.org/10.1080/09362835.2020.1850451)

* Rodriguez, B. J., & Anderson, C. M. (2014). Integrating a social behavior intervention during small group academic instruction using a total group criterion intervention. *Journal of Positive Behavior Interventions, 16*(4), 234-245. DOI: [10.1177/1098300713492858](http://dx.doi.org/10.1177/1098300713492858)

* Saddler, B., Behforooz, B., & Asaro, K. (2008). The effects of sentence-combining instruction on the writing of fourth-grade students with writing difficulties. *The Journal of Special Education, 42*(2), 79-90. DOI: [10.1177/0022466907310371](http://dx.doi.org/10.1177/0022466907310371)

* Schutte, N. S., Malouff, J. M., & Brown, R. F. (2008). Efficacy of an emotion-focused treatment for prolonged fatigue. *Behavior Modification, 32*(5), 699-713. DOI: [10.1177/0145445508317133](http://dx.doi.org/10.1177/0145445508317133)

* Thiemann, K.S., & Goldstein, H. (2001). Social stories, written text cues, and video feedback: effects on social communication of children with Autism. *Journal of Applied Behavior Analysis, 34*(4), 425-446. DOI: [10.1901/jaba.2001.34-425](https://onlinelibrary.wiley.com/doi/abs/10.1901/jaba.2001.34-425)

* Thorne, S., & Kamps, D. (2008). The effects of a group contingency intervention on academic engagement and problem behavior of at-risk students. *Behavior Analysis in Practice, 1*(2), 12-18. DOI: [10.1007/BF03391723](http://dx.doi.org/10.1007/BF03391723)

---
output: md_document
---

### Accessing scdhlm

There are two ways to access this app: on your own computer, using the RStudio software, or via a website. Since you are reading this now, you have presumably already figured out at least one of these methods. 

#### Access through the shinyapps.io website

The simplest way to access the app is via the web, at https://jepusto.shinyapps.io/scdhlm/. This version is hosted by a service called [shinyapps.io](https://www.shinyapps.io/), which imposes limitations on the number of concurrent users of the site and total hours of active use of the site. Consequently, you might find that the site is not always available, especially towards the end of each month. Running the app on your own computer through RStudio requires a bit more effort to get set up, but will allow for extensive use, run faster, and maintain privacy of your data because everything is run locally from your computer. If you intend to use the tool extensively, I would encourage you to follow the steps below in order to install it on your own computer. 

#### Access through RStudio

To access the app on your own computer, you will need to install two pieces of open-source and freely available software (R and RStudio) and follow further steps to configure the software. Details for installation and RStudio access are described below. A video walk-through is available at https://www.youtube.com/watch?v=R0n9eo4oB7k (Mac) or https://www.youtube.com/watch?v=DhrIfIXjF4c (Windows).

1.	Install R via http://cran.r-project.org/
    1.1. Navigate to the “Download and Install R” section and select the relevant download for your computer.
    1.2. Download the latest version of R and install it following the instructions given.

2.	If you are a Windows user, install Rtools via http://cran.r-project.org/bin/windows/Rtools/

3.	Install RStudio from http://www.rstudio.com/products/rstudio/download/
    3.1. Navigate to the “All Installers” section of the home page and select the relevant download for your computer.
    3.2. Follow the installation directions provided.
    
4.	Once you have all of these programs installed, open RStudio. You will need to install several R packages that are required to run the calculator. Navigate to the console in RStudio (usually in the lower left pane) and type the following commands:

    ```{r, eval=FALSE}
    install.packages("ggplot2")
    install.packages("shiny")
    install.packages("markdown")
    install.packages("glue")
    install.packages("rclipboard")
    install.packages("readxl")
    install.packages("janitor")
    install.packages("scdhlm")
    install.packages("rclipboard")
    ```

5. After all of these packages are installed, type the following commands at the console prompt to start the app (which should then open in a new window):

    ```{r, eval = FALSE}
    library(scdhlm)
    shine_scd()
    ```

6. To exit the app, simply close the window in which it appears or click the red "Stop" icon in the upper right-hand corner of the RStudio console window.

### Methodological references

* Chen, M., Pustejovsky, J. E., Klingbeil, D. A., & Van Norman, E. (2023). Between-case standardized mean differences: Flexible methods for single-case designs. _Journal of School Psychology, 98_, 16-38. https://doi.org/10.1016/j.jsp.2023.02.002

* Hedges, L. V, Pustejovsky, J. E., & Shadish, W. R. (2012). A standardized mean difference effect size for single case designs. _Research Synthesis Methods, 3_, 224–239. https://doi.org/10.1002/jrsm.1052

* Hedges, L. V, Pustejovsky, J. E., & Shadish, W. R. (2013). A standardized mean difference effect size for multiple baseline designs across individuals. )Research Synthesis Methods, 4_)(4), 324–341. https://doi.org/10.1002/jrsm.1086

* Pustejovsky, J. E., Hedges, L. V, & Shadish, W. R. (2014). Design-comparable effect sizes in multiple baseline designs: A general modeling framework. Journal of Educational and Behavioral Statistics, 39_(5), 368–393. https://doi.org/10.3102/1076998614547577

* Shadish, W. R., Hedges, L. V, Pustejovsky, J. E., Boyajian, J. G., Sullivan, K. J., Andrade, A., & Barrientos, J. L. (2014). A d-statistic for single-case designs that is equivalent to the usual between-groups d-statistic. _Neuropsychological Rehabilitation: An International Journal, 24_(3-4), 528-553. https://doi.org/10.1080/09602011.2013.819021

* Shadish, W. R., Hedges, L. V, & Pustejovsky, J. E. (2014). Analysis and meta-analysis of single-case designs with a standardized mean difference statistic: A primer and applications. _Journal of School Psychology, 52_, 123–147. https://doi.org/10.1016/j.jsp.2013.11.005

### Accessing scdhlm

There are two ways to access this app: on your own computer, using the
RStudio software, or via a website. Since you are reading this now, you
have presumably already figured out at least one of these methods.

#### Access through the shinyapps.io website

The simplest way to access the app is via the web, at
<https://jepusto.shinyapps.io/scdhlm/>. This version is hosted by a
service called [shinyapps.io](https://www.shinyapps.io/), which imposes
limitations on the number of concurrent users of the site and total
hours of active use of the site. Consequently, you might find that the
site is not always available, especially towards the end of each month.
Running the app on your own computer through RStudio requires a bit more
effort to get set up, but will allow for extensive use, run faster, and
maintain privacy of your data because everything is run locally from
your computer. If you intend to use the tool extensively, I would
encourage you to follow the steps below in order to install it on your
own computer.

#### Access through RStudio

To access the app on your own computer, you will need to install two
pieces of open-source and freely available software (R and RStudio) and
follow further steps to configure the software. Details for installation
and RStudio access are described below. A video walk-through is
available at <https://www.youtube.com/watch?v=R0n9eo4oB7k> (Mac) or
<https://www.youtube.com/watch?v=DhrIfIXjF4c> (Windows).

1.  Install R via <http://cran.r-project.org/> 1.1. Navigate to the
    “Download and Install R” section and select the relevant download
    for your computer. 1.2. Download the latest version of R and install
    it following the instructions given.

2.  If you are a Windows user, install Rtools via
    <http://cran.r-project.org/bin/windows/Rtools/>

3.  Install RStudio from
    <http://www.rstudio.com/products/rstudio/download/> 3.1. Navigate to
    the “All Installers” section of the home page and select the
    relevant download for your computer. 3.2. Follow the installation
    directions provided.

4.  Once you have all of these programs installed, open RStudio. You
    will need to install several R packages that are required to run the
    calculator. Navigate to the console in RStudio (usually in the lower
    left pane) and type the following commands:

        install.packages("ggplot2")
        install.packages("shiny")
        install.packages("markdown")
        install.packages("glue")
        install.packages("rclipboard")
        install.packages("readxl")
        install.packages("janitor")
        install.packages("scdhlm")
        install.packages("rclipboard")

5.  After all of these packages are installed, type the following
    commands at the console prompt to start the app (which should then
    open in a new window):

        library(scdhlm)
        shine_scd()

6.  To exit the app, simply close the window in which it appears or
    click the red “Stop” icon in the upper right-hand corner of the
    RStudio console window.

### scdhlm

Version 0.7.2

Designed by James E. Pustejovsky and Man Chen, with contributions from
Bethany Hamilton and Paulina Grekov.

-   <pustejovsky@wisc.edu>
-   <https://jepusto.com>

[Source code available on
Github](https://github.com/jepusto/scdhlm/tree/master/inst/shiny-examples/scdhlm)

*Your comments, suggestions, and feedback are welcome.*

### Acknowledgement

<img src="https://raw.githubusercontent.com/jepusto/scdhlm/master/images/IES_InstituteOfEducationSciences_RGB.svg" width="30%" align = "right" style = "margin: 0px 10px" alt = "Institute of Education Sciences logo"/>

The development of this application was supported in part by the
Institute of Education Sciences, U.S. Department of Education, through
[Grant
R324U190002](https://ies.ed.gov/funding/grantsearch/details.asp?ID=3358)
to the University of Oregon. The contents of the application do not
necessarily represent the views of the Institute or the U.S. Department
of Education.

### Suggested citation

Pustejovsky, J. E., Chen, M., Hamilton, B., & Grekov, P. (2023). scdhlm:
A web-based calculator for between-case standardized mean differences
(Version 0.7.2) \[Web application\].
<https://jepusto.shinyapps.io/scdhlm>

### Tutorial paper

Valentine, J. C., Tanner-Smith, E. E., & Pustejovsky, J. E. (2016).
[Between-case standardized mean difference effect sizes for single-case
designs: A primer and tutorial using the scdhlm web
application](https://campbellcollaboration.org/library/effect-sizes-single-case-designs-campbell-discussion-paper-1.html).
Oslo, Norway: The Campbell Collaboration. DOI: 10.4073/cmpn.2016.3

---
title: Estimating design-comparable standardized mean differences with scdhlm
author: James E. Pustejovsky
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{"Estimating design-comparable standardized mean differences with scdhlm"}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

The scdhlm package (Pustejovsky, Chen, & Hamilton, 2020) provides several methods for estimating design-comparable standardized mean differences (SMDs) based on data from a single-case design. In principle, a design-comparable SMD is in the same metric as the SMD from a simple, between-groups randomized experiment performed on a comparable sample and with comparable outcome measures. Hedges, Pustejovsky, and Shadish (2012) proposed methods for estimating design-comparable SMDs based on data from an ABAB design (and, more generally, treatment reversal designs with an arbitrary number of phases). Hedges, Pustejovsky, and Shadish (2013) extended the methods to handle data from multiple baseline designs. In both cases, the proposed estimation methods are premised on a simple model for the data, which assumed that the outcome process is stable over time (lacking time trends) and that the treatment effect is constant across cases. Pustejovsky, Hedges, and Shadish (2014) proposed an approach to defining and estimating design-comparable SMDs under a more general model, which can allow for time trends and between-case variation in treatment effects and time trends.

In the scdhlm package, the original estimation methods proposed for the ABAB design and multiple baseline design are implemented in the `effect_size_ABk` and `effect_size_MB` functions, respectively. Both of these functions take the raw data as input and produce as output an effect size estimate, along with accompanying standard error and some other auxiliary information. Thus, there is no distinction between estimating the model and estimating the effect size. In contrast, the more general methods proposed in Pustejovsky, Hedges, and Shadish (2014) entail two steps: first, estimating a hierarchical model for the data; and second, estimating a design-comparable effect size based on the fitted model. The first step is accomplished using the function `lme` from the package `nlme` by Pinheiro, Bates, DebRoy, and Sarkar (2015). The second step is accomplished using the function `g_mlm` from the `scdhlm` package. This vignette demonstrates how to use all of these functions to estimate design-comparable standardized mean difference effect sizes. The R code presented below can be used to replicate the examples found in the papers that proposed the methods. To begin, the user must load the package:

```{r, message = FALSE}
library(scdhlm)
```

## Lambert, Cartledge, Heward, & Lo (2006)

Lambert, Cartledge, Heward, and Lo (2006) tested the effect of using response cards (compared to single-student responding) during math lessons in two fourth-grade classrooms. The investigators collected data on rates of disruptive behavior for nine focal students, using an ABAB design. This example is discussed in Hedges, Pustejovsky, and Shadish (2012), who selected it because the design was close to balanced and used a relatively large number of cases. Their calculations can be replicated using the `effect_size_ABk` function. To use this function, the user must provide five pieces of data: 

* `outcome`: the outcome variable, 
* `treatment`: a variable indicating the treatment condition, 
* `id`: a variable listing the case on which the outcome was measured, 
* `phase`: a variable indicating the phase of treatment (i.e., each replication of a baseline and treatment condition), and 
* `time`: a variable listing the session number.

In the `Lambert` dataset, these variables are called respectively `outcome`, `treatment`, `case`, `phase`, and `time`. Given these inputs, the design-comparable SMD is calculated as follows:

```{r Lambert}
data(Lambert)

Lambert_ES <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                              id = case, phase = phase, time = time, 
                              data = Lambert)

print(Lambert_ES, 5)
```

The adjusted effect size estimate `delta_hat` is equal to `r round(Lambert_ES$delta_hat, 3)`; its variance `V_delta_hat` is equal to `r round(Lambert_ES$V_delta_hat,3)`. A standard error for `delta_hat` can be calculated by taking the square root of `V_delta_hat`: `sqrt(Lambert_ES$V_delta_hat)` = `r round(sqrt(Lambert_ES$V_delta_hat),3)`. The effect size estimate is bias-corrected in a manner analogous to Hedges' g correction for SMDs from a between-subjects design. The degrees of freedom `nu` are estimated based on a Satterthwaite-type approximation, which is equal to `r round(Lambert_ES$nu,1)` in this example.

A summary() method is included to return more detail about the model parameter estimates and effect size estimates:
```{r Lambert-summary}
summary(Lambert_ES)
```

By default, the `effect_size_ABk` function calculates an estimate of the first-order autocorrelation in the outcome series (stored in the entry `phi`) and an estimate of the intra-class correlation, i.e., the ratio of the between-case variance in the outcome to the total cross-sectional variance in the outcome (the intra-class correlation estimate is stored in the entry `rho`). Optionally, the user can specify their own estimates of these parameters as inputs to the function. In this example, the auto-correlation estimate was `r round(Lambert_ES$phi, 3)`. The following code examines the sensitivity of the results to values of the auto-correlation that are larger and smaller than the default estimate of `r round(Lambert_ES$phi, 3)`. 
```{r Lambert_sensitivity}
Lambert_ES2 <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                id = case, phase = phase, time = time, 
                data = Lambert, phi = 0.10)
print(Lambert_ES2, digits = 5)

Lambert_ES3 <- effect_size_ABk(outcome = outcome, treatment = treatment, 
                id = case, phase = phase, time = time, 
                data = Lambert, phi = 0.35)
print(Lambert_ES3, digits = 5)
```
The estimated auto-correlation has only a trivial effect on the effect size estimate and a minor effect on its estimated variance. 

## Anglesea, Hoch, & Taylor (2008)

Anglesea, Hoch, and Taylor (2008) used an ABAB design to test the effect of using a pager prompt to reduce the rapid eating of three teenage boys with autism. The primary outcome was a measure of how quickly each participant consumed one serving of a familiar food. This example is discussed in Hedges, Pustejovsky, and Shadish (2012), who used it to illustrate the calculation of the design-comparable SMD when based on the minimum required number of cases. Their calculations can be replicated using the following code:
```{r Anglesea}
data(Anglesea)
Anglesea_ES <- effect_size_ABk(outcome, condition, case, phase, session, data = Anglesea)
Anglesea_ES
```
Note that the standard error of the effect size estimate is quite large and the degrees of freedom corresponding to the denominator of the SMD estimate are very low. Both quantities are extreme due to the small number of cases used in this example. 

## Saddler, Behforooz, & Asaro (2008)

Saddler, Behforooz, and Asaro (2008) used a multiple baseline design to investigate the effect of an instructional technique on the writing of fourth grade students. The investigators assessed the intervention's effect on measures of writing quality, sentence complexity, and use of target constructions. 

Design-comparable SMDs can be estimated based on these data using the `effect_size_MB` function. The syntax for this function is similar to that of the `effect_size_ABk` function, but does not require the user to input information about the phase of the design (because in the multiple baseline design, phase exactly corresponds to treatment condition). The following code replicates the calculations reported in Hedges, Pustejovsky, and Shadish (2013):
```{r Saddler}
data(Saddler)

quality_ES <- effect_size_MB(outcome, treatment, case, time, 
                             data = subset(Saddler, measure=="writing quality"))
complexity_ES <- effect_size_MB(outcome, treatment, case, time , 
                                data = subset(Saddler, measure=="T-unit length"))
construction_ES <- effect_size_MB(outcome, treatment, case, time, 
                                  data = subset(Saddler, measure=="number of constructions"))

data.frame(
  quality = unlist(quality_ES),
  complexity = unlist(complexity_ES), 
  construction = unlist(construction_ES) 
)[c("delta_hat","V_delta_hat","nu","phi","rho"),]

```

For multiple baseline designs, an alternative to using the `effect_size_MB` function is to estimate a hierarchical linear model for the data and then use the `g_mlm` function. The two alternative approaches differ in how the model parameters and effect size are estimated. Pustejovsky, Hedges, and Shadish (2014) found that the latter approach (based on a hierarchical linear model) has comparable mean-squared error to the former approach, while producing better estimates of the variance of the effect size. The latter approach is implemented in two steps, which we will demonstrate using the writing quality measure. First, we estimate the hierarchical model with an AR(1) within-case error structure using the `lme` function:
```{r quality-RML}
quality_RML <- lme(fixed = outcome ~ treatment, 
                   random = ~ 1 | case, 
                   correlation = corAR1(0, ~ time | case), 
                   data = subset(Saddler, measure=="writing quality"))
summary(quality_RML)
```
The summary of the fitted model displays estimates of the component parameters, including the within-case and between-case standard deviations, auto-correlation, and (unstandardized) treatment effect estimate. The next step is to combine these estimated components into an effect size estimate using the `g_mlm` function. This function takes the fitted `lme` model object as input, followed by the vectors `p_const` and `r_const`, which specify the components of the fixed effects and variance estimates that are to be used in constructing the design-comparable SMD. For details on how to choose these constants, see Pustejovsky, Hedges, and Shadish (2014). In this example:
```{r quality-g-mlm, eval=TRUE}
quality_ES_RML <- g_mlm(quality_RML, p_const = c(0,1), r_const = c(1,0,1))
summary(quality_ES_RML)
```
The adjusted SMD effect size is estimated as `g_AB` = `r round(quality_ES_RML$g_AB, 3)`; its standard error is `SE_g_AB` = `r round(quality_ES_RML$SE_g_AB, 3)`; the estimated auto-correlation is `phi` = `r round(quality_ES_RML$theta$cor_params, 3)`; and the estimated degrees of freedom is `nu` = `r round(quality_ES_RML$nu, 1)`. In this example, the RML effect size estimate is about 10% larger than the estimate from `effect_size_MB`, with a slightly smaller variance estimate. The RML estimate of the auto-correlation is substantially higher than before, but `effect_size_MB` uses a moment estimator that is known to be biased towards zero and that does not perform well when outcomes are intermittently missing for some sessions (as is the case here). 

## Laski, Charlop, & Schreibman (1988)

Laski, Charlop, and Schreibman (1988) used a multiple baseline across individuals to evaluate the effect of a training program for parents on the speech production of their autistic children, as measured using a partial interval recording procedure. The design included eight children; one child was measured separately with each parent; for purposes of simplicity, and following Hedges, Pustejovsky, and Shadish (2013), only the measurements taken with the mother are included in the analysis.

The following code compares the estimates of the design-comparable SMD effect size based on the Hedges, Pustejovsky, and Shadish (2013) approach (using the `effect_size_MB` function) to the estimates based on the hierarchical linear modeling approach described in Pustejovsky, Hedges, and Shadish (2014) (using the `g_mlm` function). 

```{r Laski-RML, eval = TRUE}
data(Laski)

# Hedges, Pustejovsky, & Shadish (2013)
Laski_ES_HPS <- effect_size_MB(outcome, treatment, case, time, data= Laski)

# Pustejovsky, Hedges, & Shadish (2014)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
summary(Laski_RML)
```

```{r Laski-ES-RML, eval = TRUE}
Laski_ES_RML <- g_mlm(Laski_RML, p_const = c(0,1), r_const = c(1,0,1))

# compare the estimates
data.frame(
  HPS = with(Laski_ES_HPS, c(SMD = delta_hat, SE = sqrt(V_delta_hat), 
                             phi = phi, rho = rho, nu = nu)),
  RML = with(Laski_ES_RML, c(g_AB, SE_g_AB, theta$cor_params, 
                             theta$Tau$case / (theta$Tau$case + theta$sigma_sq), nu))
)
```

As in the Saddler example, both methods produce very similar SMD estimates and variance estimates. The RML estimate of auto-correlation is substantially higher than the HPS estimate, while the intra-class correlation estimate is somewhat lower; in combination, these differences lead to larger degrees of freedom.

An advantage of the RML approach is that it is readily extended to more complex models. The above analysis was based on the assumption that the treatment effect is constant across cases. This assumption can be removed by fitting a model that includes a random treatment effect for each case:
```{r Laski-RML2}
Laski_RML2 <- lme(fixed = outcome ~ treatment,
                 random = ~ treatment | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
summary(Laski_RML2)
anova(Laski_RML, Laski_RML2)
```
The fit of the two models can be compared using a likelihood ratio test, which rejects the model with a constant treatment effect. The second model, which allows the treatment effect to vary, is to be preferred. The following code estimates a design-comparable SMD based on the better-fitting model.
```{r Laski-ES-RML2, eval=TRUE}
Laski_ES_RML2 <- g_mlm(Laski_RML2, p_const = c(0,1), r_const = c(1,0,0,0,1))
Laski_ES_RML2
```

The adjusted effect size estimate that is `r round(100 * (1 - Laski_ES_RML2$g_AB / Laski_ES_RML$g_AB))`% smaller than the estimate from the simpler model; with a standard error that is `r round(100 * (Laski_ES_RML2$SE_g_AB / Laski_ES_RML$SE_g_AB - 1))`% larger. The difference between the two models is due to a difference in between-case variance across phases not captured by the assumptions of the simpler model. The between-case variation in the outcome appears to be substantially larger in the baseline phase than in the treatment phase. Maintaining the constant treatment effect assumption constrains the between-case variance to be constant across phases, and so the between-case variance is estimated by pooling across both phases. The constant treatment effect assumption therefore leads to a smaller variance estimate than the estimate based on allowing the between-case variance to differ by phase. 

## Schutte, Malouff, & Brown (2008)

Schutte, Malouff, and Brown (2008) evaluated the effect of an emotion-focused therapy program for adults with prolonged fatigue using a multiple baseline across individuals. The design included 13 adults who met clinical criteria for prolonged fatigue. Fatigue severity was measured weekly using a self-reported scale that ranged from 1 to 63. Following Pustejovsky, Hedges, and Shadish (2014), the data for participant 4 are excluded from the analysis because nearly all of the measurements for this case are at the upper extreme of the scale. The data for the remaining participants are plotted below.

```{r Schutte-graph, warning = FALSE, message = FALSE, fig.width = 7, fig.height = 7}

if (requireNamespace("ggplot2", quietly = TRUE)) {
library(ggplot2)

data(Schutte)

graph_SCD(case = case, phase = treatment,
          session = week, outcome = fatigue,
          design = "MB", data = Schutte) + 
  facet_wrap(~ case, ncol = 3) + 
  theme(legend.position = "bottom")
}

```

Time trends are apparent in the outcome series, as are changes in slope in the treatment condition. In order to operationally define a design-comparable SMD effect sizes in a model that includes time trends and treatment-by-time interactions, one will need to choose a time point A at which the treatment would begin and a time point B at which outcomes would be measured, both in a hypothetical between-subjects design based on the same population of participants. Here, we take A = 2 and B = 9; centering time at week 9 simplifies the effect size calculations. 

```{r Schutte-center}
# time-point constants
A <- 2
B <- 9
# center at follow-up time

# create time-by-trt interaction
Schutte <- 
  preprocess_SCD(case = case, phase = treatment,
                 session = week, outcome = fatigue,
                 design = "MB", center = B, data = Schutte)

```
Having completed the data-cleaning process, three different models will be considered, again following the example from Pustejovsky, Hedges, and Shadish (2014).

### Varying intercepts, fixed treatment effect, fixed trends

The initial model allows for a baseline time trend, treatment effect, and treatment-by-time interaction, all of which are assumed to be constant across the 12 cases; only the baseline intercept is assumed to vary across cases. This specification corresponds to Model MB3 from Pustejovsky, Hedges, and Shadish (2014)
```{r Schutte-Model3}
hlm1 <- lme(fixed = fatigue ~ week + treatment + week_trt, 
            random = ~ 1 | case, 
            correlation = corAR1(0, ~ week | case),
            data = Schutte,
            method = "REML")
summary(hlm1)
```
The design-comparable standardized mean difference corresponds to the treatment effect at week $B = 9$, after $B - A = 7$ weeks of treatment. The corresponding values of `p_const` and `r_const` are specified below.
```{r Schutte-ES-Model3, eval = TRUE}
Schutte_g1 <- g_mlm(hlm1, p_const = c(0,0,1, B - A), r_const = c(1,0,1))
Schutte_g1
```
It will be seen below that this initial model provides a poor fit to the data; thus, the effect size estimate based on it should not be trusted.

### Varying intercepts, fixed treatment effect, varying trends

The next model (Model MB4) allows the baseline time trend to vary across cases. This can be done by modifying the previous model with the `update` function.
```{r Schutte-Model4}
hlm2 <- update(hlm1, random = ~ week | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm2)
anova(hlm1, hlm2)
```
A likelihood ratio test rejects the initial model in favor of this more flexible model. An effect size estimate is calculated from the fitted model as follows[^g-mlm-mb4]:
```{r Schutte-ES-Model4, eval = TRUE}
Schutte_g2 <- g_mlm(hlm2, p_const = c(0,0,1, B - A), r_const = c(1,0,0,0,1))
Schutte_g2
```
The design-comparable SMD is estimated to be `r round(Schutte_g2$g_AB, 3)`, with a standard error of `r round(Schutte_g2$SE_g_AB, 3)`.

[^g-mlm-mb4]: Note that the specification of the `r_const` argument here differs from the specification reported in Pustejovsky, Hedges, and Shadish (2014). The argument appears in a different order than reported in the article because of how the variance components are arranged in the `g_mlm()` function. To see the order of the variance components used by `g_mlm()`, use the `extract_varcomp()` function: 
    ```{r}
    unlist(extract_varcomp(hlm2))
    ```
    The order of parameter estimates appearing in the result is the same as assumed in `g_mlm()`. Thus, entries in `r_const` should correspond to the order of the parameter estimates appearing here. 

### Varying intercepts, varying trends, varying treatment-by-time interactions

The final model (Model MB5) is yet more flexible, in that it allows the treatment-by-time interactions to vary across cases. Given that the data contain only twelve cases, fitting a model with three random effects would be questionable if the goal were to make inferences about the full structure of the model. However, simulation evidence from Pustejovsky, Hedges, and Shadish (2014) suggests that using such a flexible parameterization may nonetheless be reasonable for the more limited purpose of effect size estimation. 
```{r Schutte-Model5, warning = FALSE}
hlm3 <- update(hlm2, random = ~ week + week_trt | case, 
               control=lmeControl(msMaxIter = 50, apVar=FALSE, returnObject=TRUE))
summary(hlm3)
anova(hlm2, hlm3)
```
The REML estimation algorithm fails to converge because the estimate of the correlation between the random slopes and the random treatment-by-time interactions approaches 1. A likelihood ratio test rejects the previous model (which constrained the treatment-by-time interactions to be constant). However, this test cannot be trusted with such small sample of cases, and given variance component estimates that lie on an edge of the parameter space. An effect size estimate based on this fitted model is obtained as follows.[^g-mlm-mb5]
```{r Schutte-ES-Model5, eval = TRUE}
Schutte_g3 <- g_mlm(hlm3, p_const = c(0,0,1,B - A), r_const = c(1,0,0,0,0,0,0,1))
Schutte_g3

# compare effect size estimates
data.frame(
  MB3 = round(unlist(Schutte_g1[c("g_AB","SE_g_AB","nu")]), 3), 
  MB4 = round(unlist(Schutte_g2[c("g_AB","SE_g_AB","nu")]), 3),
  MB5 = round(unlist(Schutte_g3[c("g_AB","SE_g_AB","nu")]), 3)
)
```
The effect size estimate has magnitude comparable to the estimate from Model MB3. However, the variance of the estimate is extremely large, due to the fact that the variance components that go into the denominator of the SMD estimate are estimated with low precision. This final model may represent an outer limit on the complexity of models that can feasibility be estimated, given that most single-case designs have fewer cases than the study examined in this example.

[^g-mlm-mb5]: Again, the specification of the `r_const` argument here differs from the specification reported in Pustejovsky, Hedges, and Shadish (2014). The entries in `r_const` should correspond to the order of the parameter estimates appearing in `unlist(extract_varcomp(hlm3))`.

## References

Anglesea, M. M., Hoch, H., & Taylor, B. A. (2008). Reducing rapid eating in teenagers with autism: Use of a pager prompt. *Journal of Applied Behavior Analysis, 41*(1), 107–111. doi: [10.1901/jaba.2008.41-107](https://doi.org/10.1901/jaba.2008.41-107)

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012). A standardized mean difference effect size for single case designs. *Research Synthesis Methods, 3*, 224-239. doi: [10.1002/jrsm.1052](https://doi.org/10.1002/jrsm.1052)

Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). A standardized mean difference effect size for multiple baseline designs across individuals. *Research Synthesis Methods, 4*(4), 324-341. doi: [10.1002/jrsm.1086](https://doi.org/10.1002/jrsm.1086)

Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). Effects of response cards on disruptive behavior and academic responding during math lessons by fourth-grade urban students. *Journal of Positive Behavior Interventions, 8*(2), 88-99.

Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). Training parents to use the natural language paradigm to increase their autistic children’s speech. *Journal of Applied Behavior Analysis, 21*(4), 391–400.

Pinheiro J., Bates D., DebRoy S., Sarkar D. and R Core Team (2015). _nlme: Linear and Nonlinear Mixed Effects Models_. R package version
3.1-119. https://CRAN.R-project.org/package=nlme

Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). Design-comparable effect sizes in multiple baseline designs: A general modeling framework. *Journal of Educational and Behavioral Statistics, 39*(4), 211-227. doi: [10.3102/1076998614547577](https://doi.org/10.3102/1076998614547577)

Pustejovsky, J. E., Hamilton, B. J., & Chen, M. (2020). _scdhlm: Estimating hierarchical linear models for single-case
designs_. R package version 0.3.2, https://jepusto.github.io/scdhlm/.

Saddler, B., Behforooz, B., & Asaro, K. (2008). The effects of sentence-combining instruction on the writing of fourth-grade students with writing difficulties. *The Journal of Special Education, 42*(2), 79–90. doi: [10.1177/0022466907310371](https://doi.org/10.1177/0022466907310371)

### Demonstration videos

The following videos provide instructions for how to install R, RStudio, and the scdhlm app on your computer:

* Mac: https://www.youtube.com/watch?v=R0n9eo4oB7k 
* Windows: https://www.youtube.com/watch?v=DhrIfIXjF4c

The following videos provide detailed demonstrations of how to use the scdhlm app to estimate a between-case standardized mean difference effect size:

* A model without time trends: https://www.youtube.com/watch?v=arDE6uhLLAc
* A model that includes time trends: https://www.youtube.com/watch?v=DsFJO2MBkjM

## Resubmission

This is a re-submission and a maintenance release. It corrects small bugs created in the latest release of ggplot2. 

## Test environments

* local Windows 10 Education, R 4.2.2
* ubuntu 20.04.3 LTS (on Github), R devel, release, oldrelease
* macOS-latest (on Github), R release
* windows-latest (on Github), R devel, release
* win-builder (devel, release, oldrelease)
* mac-builder (release)

## R CMD check results

There were no ERRORs and no WARNINGs. 

There were several NOTEs:

Possibly mis-spelled words in DESCRIPTION:
  Klingbeil (16:28)
  Pustejovsky (13:29, 14:13, 15:5, 16:15)
  Shadish (13:46, 14:30, 15:30)
  
  The flagged words are spelled correctly.

Found the following (possibly) invalid URLs:
  URL: https://doi.org/10.1002/jrsm.1052
    From: inst/doc/Estimating-effect-sizes.html
    Status: 403
    Message: Forbidden
  URL: https://doi.org/10.1002/jrsm.1086
    From: inst/doc/Estimating-effect-sizes.html
    Status: 403
    Message: Forbidden
  URL: https://doi.org/10.1177/0022466907310371
    From: inst/doc/Estimating-effect-sizes.html
    Status: 403
    Message: Forbidden
  URL: https://doi.org/10.1901/jaba.2008.41-107
    From: inst/doc/Estimating-effect-sizes.html
    Status: 403
    Message: Forbidden
  URL: https://doi.org/10.3102/1076998614547577
    From: inst/doc/Estimating-effect-sizes.html
    Status: 403
    Message: Forbidden    
    
    The flagged URLs are all correct.

Found the following (possibly) invalid DOIs:
  DOI: 10.1002/jrsm.1052
    From: DESCRIPTION
    Status: Forbidden
    Message: 403
  DOI: 10.1002/jrsm.1086
    From: DESCRIPTION
    Status: Forbidden
    Message: 403
  DOI: 10.3102/1076998614547577
    From: DESCRIPTION
    Status: Forbidden
    Message: 403
    
    The flagged DOIs are all correct.
    
## revdepcheck results

We checked 1 reverse dependencies, comparing R CMD check results across CRAN and dev versions of this package.

 * We saw 0 new problems
 * We failed to check 0 packages

---
output: md_document
---

### scdhlm

Version `r packageVersion("scdhlm")`

Designed by James E. Pustejovsky and Man Chen, with contributions from Bethany Hamilton and Paulina Grekov.

* pustejovsky@wisc.edu
* https://jepusto.com

[Source code available on Github](https://github.com/jepusto/scdhlm/tree/master/inst/shiny-examples/scdhlm)

_Your comments, suggestions, and feedback are welcome._


### Acknowledgement

<img src="https://raw.githubusercontent.com/jepusto/scdhlm/master/images/IES_InstituteOfEducationSciences_RGB.svg" width="30%" align = "right" style = "margin: 0px 10px" alt = "Institute of Education Sciences logo"/>

The development of this application was supported in part by the Institute of Education Sciences, U.S. Department of Education, through [Grant R324U190002](https://ies.ed.gov/funding/grantsearch/details.asp?ID=3358) to the University of Oregon. The contents of the application do not necessarily represent the views of the Institute or the U.S. Department of Education.

### Suggested citation

Pustejovsky, J. E., Chen, M., Hamilton, B., & Grekov, P. (`r substr(packageDate("scdhlm"),1,4)`). scdhlm: A web-based calculator for between-case standardized mean differences (Version `r packageVersion("scdhlm")`) [Web application]. https://jepusto.shinyapps.io/scdhlm

### Tutorial paper

Valentine, J. C., Tanner-Smith, E. E., & Pustejovsky, J. E. (2016). [Between-case standardized mean difference effect sizes for single-case designs: A primer and tutorial using the scdhlm web application](https://campbellcollaboration.org/library/effect-sizes-single-case-designs-campbell-discussion-paper-1.html). Oslo, Norway: The Campbell Collaboration. DOI: 10.4073/cmpn.2016.3

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{MB2results}
\alias{MB2results}
\title{MB2 simulation results}
\format{
A data frame
}
\description{
Simulation results for model MB2 from  Pustejovsky, Hedges, & Shadish (2014).
}
\references{
Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
\emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Thiemann2001}
\alias{Thiemann2001}
\title{Thiemann & Goldstein (2001)}
\format{
A data frame with 363 rows and 8 variables
}
\source{
Thiemann, K.S., & Goldstein, H. (2001).
Social stories, written text cues, and video feedback: effects on social communication
of children with Autism. \emph{Journal of Applied Behavior Analysis, 34}(4), 425-446.
\doi{10.1901/jaba.2001.34-425}
}
\description{
Data from a multiple baseline across behaviors design conducted by
Thiemann & Goldstein (2001). The variables are as follows:
\itemize{
  \item \code{Study_ID}. Study identifier.
  \item \code{case}. Student identifier.
  \item \code{series}. Series identifier.
  \item \code{outcome}. Frequency of coded social communication skills, as measured by
  a direct observation coding system with 15-second intervals recording for the occurrence
  of any of the four social measures: contingent responses, securing attention, initiating
  comments, and initiating requests.
  \item \code{time}. Measurement occasion.
  \item \code{treatment}. Indicator for treatment phase.
  \item \code{trt_time}. Measurement occasion times treatment phase.
  \item \code{time_c}. Measurement occasion centered at the follow-up time.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Rodriguez}
\alias{Rodriguez}
\title{Rodriguez & Anderson (2014)}
\format{
A data frame with 148 rows and 4 variables
}
\source{
Rodriguez, B. J., & Anderson, C. M. (2014). Integrating a social
  behavior intervention during small group academic instruction using a total
  group criterion intervention. \emph{Journal of Positive Behavior Interventions,
  16}(4), 234-245. \doi{10.1177/1098300713492858}
}
\description{
Data from a multiple baseline design conducted by Rodriguez and Anderson (2014). 
The variables are as follows: \itemize{ 
  \item \code{case} Participant identifier 
  \item \code{condition} Factor identifying the phase of the design (A or B) 
  \item \code{session} Measurement occasion 
  \item \code{outcome} Percentage of intervals with problem behavior 
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmeInfo-imports.R
\name{CI_g}
\alias{CI_g}
\title{Calculates a confidence interval for a standardized mean difference
  effect size}
\arguments{
\item{g}{an estimated effect size object of class \code{g_REML}, class
\code{g_HPS}, or class \code{g_mlm}.}

\item{cover}{confidence level}

\item{bound}{numerical tolerance for non-centrality parameter in
\code{\link[stats]{qt}}.}

\item{symmetric}{If \code{TRUE} (the default), use a symmetric confidence
interval. If \code{FALSE}, use a non-central t approximation to obtain an
asymmetric confidence interval.}
}
\value{
A vector of upper and lower confidence bounds.
}
\description{
Calculates a confidence interval given a \code{g_REML}, a
  \code{g_HPS}, or a \code{g_mlm} object using either a central t
  distribution (for a symmetric interval) or a non-central t distribution
  (for an asymmetric interval).
}
\examples{
data(Laski)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case,
                 correlation = corAR1(0, ~ time | case),
                 data = Laski)
Laski_g_REML <- suppressWarnings(
  g_REML(Laski_RML, p_const = c(0,1), 
         r_const = c(1,0,1), returnModel = FALSE)
)
CI_g(Laski_g_REML, symmetric = TRUE)
CI_g(Laski_g_REML, symmetric = FALSE)

Laski_HPS <- with(Laski, effect_size_MB(outcome, treatment, case, time))
CI_g(Laski_HPS, symmetric = FALSE)

Laski_g_mlm <- g_mlm(mod = Laski_RML, p_const = c(0,1), r_const = c(1,0,1))
CI_g(Laski_g_mlm, symmetric = FALSE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Romaniuk}
\alias{Romaniuk}
\title{Romaniuk (2002)}
\format{
A data frame with 148 rows and 4 variables
}
\source{
Romaniuk, C., Miltenberger, R., Conyers, C., Jenner, N., Jurgens, M.,
  & Ringenberg, C. (2002). The influence of activity choice on problem
  behaviors maintained by escape versus attention. \emph{Journal of Applied
  Behavior Analysis, 35}(4), 349-62. \doi{10.1901/jaba.2002.35-349}
}
\description{
Data from a treatment reversal design conducted by Romaniuk and colleagues (2002).
The variables are as follows: \itemize{ 
  \item \code{case} Participant identifier 
  \item \code{phase} Factor identifying the phase of the design
  \item \code{condition} Factor identifying the treatment condition 
  \item \code{session} Measurement occasion 
  \item \code{outcome} Problem behavior
  \item \code{measurement} Character string describing how problem behavior was measured
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/lmeInfo-imports.R
\name{lmeInfo}
\alias{lmeInfo}
\alias{extract_varcomp}
\alias{varcomp_vcov}
\alias{g_mlm}
\title{lmeInfo}
\description{
Functions imported from the lmeInfo package.

\itemize{
\item \code{\link[lmeInfo]{extract_varcomp}}
\item \code{\link[lmeInfo]{g_mlm}}
\item \code{\link[lmeInfo]{varcomp_vcov}}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Lambert}
\alias{Lambert}
\title{Example 1 from Hedges, Pustejovsky, & Shadish (2012)}
\format{
A data frame with 461 rows and 6 variables
}
\source{
Lambert, M. C., Cartledge, G., Heward, W. L., & Lo, Y. (2006). 
Effects of response cards on disruptive behavior and academic responding 
during math lessons by fourth-grade urban students. 
\emph{Journal of Positive Behavior Interventions, 8}(2), 88-99. 
\doi{10.1177/10983007060080020701}
}
\description{
Data from an ABAB design conducted by Lambert, Cartledge, Heward, & Lo (2008). 
The variables are as follows:
\itemize{
  \item \code{measure}. Outcome measure description (disruptive behavior or academic response).
  \item \code{case}. Student identifier.  
  \item \code{treatment}. Factor indicating treatment or control condition. 
        SSR = single-subject responding. RC = response cards.
  \item \code{phase}. Study phase (including both control and treatment condition)
  \item \code{time}. Measurement occasion.
  \item \code{outcome}. Outcome scores.
}
}
\references{
Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
A standardized mean difference effect size for single case designs. 
\emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Thiemann2004}
\alias{Thiemann2004}
\title{Thiemann & Goldstein (2004)}
\format{
A data frame with 408 rows and 8 variables
}
\source{
Thiemann, K.S., & Goldstein, H. (2004).
Effects of peer training and written text cueing on social communication of
school-age children with pervasive developmental disorder. \emph{Journal of Speech Language 
and Hearing Research, 47}(1), 126-144. \doi{10.1044/1092-4388(2004/012)}
}
\description{
Data from a multiple baseline across behaviors design conducted by
Thiemann & Goldstein (2004). The variables are as follows:
\itemize{
  \item \code{Study_ID}. Study identifier.
  \item \code{case}. Student identifier.
  \item \code{series}. Series identifier.
  \item \code{outcome}. Frequency of unprompted targeted social communication skills,
  as measured by a direct observation, paper and pencil coding system during the 10-minute
  social activity for each behavior for all sessions.
  \item \code{time}. Measurement occasion.
  \item \code{treatment}. Indicator for treatment phase.
  \item \code{trt_time}. Measurement occasion times treatment phase.
  \item \code{time_c}. Measurement occasion centered at the follow-up time.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Datchuk}
\alias{Datchuk}
\title{Datchuk (2016)}
\format{
A data frame with 74 rows and 4 variables
}
\source{
Datchuk, S. M. (2016). Writing simple sentences and descriptive
  paragraphs: Effects of an intervention on adolescents with writing
  difficulties. \emph{Journal of Behavioral Education, 25}(2), 166-188.
  \doi{10.1007/s10864-015-9236-x}
}
\description{
Data from a multiple baseline design conducted by Datchuk (2016). The
variables are as follows: \itemize{ \item \code{case}. Participant
identifier. \item \code{session}. Measurement occasion. \item
\code{condition}. Factor identifying the phase of the design (baseline or
treatment). \item \code{outcome}. Correct word sequences per minute. }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Thorne}
\alias{Thorne}
\title{Thorne (2005)}
\format{
A data frame with 776 rows and 7 variables
}
\source{
Thorne, S., & Kamps, D. (2008). The effects of a group contingency
  intervention on academic engagement and problem behavior of at-risk
  students. \emph{Behavior Analysis in Practice, 1}(2), 12-18.
  \doi{10.1007/BF03391723}
}
\description{
Data from an ABAB design conducted by Thorne and Kamps (2008). The variables
are as follows: 
\itemize{ 
  \item \code{case}. Participant identifier. 
  \item \code{measure}. Outcome measure description (academic engagement or inappropriate verbalizations). 
  \item \code{session}. Measurement occasion.
  \item \code{phase_id}. Categorical variable describing the phase of the study design for each case. 
  \item \code{condition} Categorical variable describing whether each phase is a baseline (A) phase or intervention (B) phase. 
  \item \code{phase_indicator}. Indicator variable equal to 1 during intervention phases. 
  \item \code{outcome}. Outcome scores 
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{CaseHarrisGraham}
\alias{CaseHarrisGraham}
\title{Case, Harris, and Graham (1992)}
\format{
A data frame with 56 rows and 4 variables
}
\source{
Case, L. P., Harris, K. R., & Graham, S. (1992). Improving the
  mathematical problem-solving skills of students with learning disabilities:
  Self-regulated strategy development. \emph{The Journal of Special Education,
  26}(1), 1-19. \doi{10.1177/002246699202600101}
}
\description{
Data from a multiple baseline design conducted by Case, Harris, and Graham
(1992). The variables are as follows: \itemize{ \item \code{case}.
Participant identifier. \item \code{session}. Measurement occasion. \item
\code{condition}. Factor identifying the phase of the design (baseline or
treatment). \item \code{outcome}. Number of subtraction equations and answers
correct on each word problem probe. }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Laski}
\alias{Laski}
\title{Example 2 from Hedges, Pustejovsky, & Shadish (2013)}
\format{
A data frame with 128 rows and 4 variables
}
\source{
Laski, K. E., Charlop, M. H., & Schreibman, L. (1988). 
Training parents to use the natural language paradigm to increase 
their autistic children's speech. 
\emph{Journal of Applied Behavior Analysis, 21}(4), 391-400.
\doi{10.1901/jaba.1988.21-391}
}
\description{
Data from a multiple baseline design conducted by 
Laski, Charlop, & Schreibman (1988). The variables are as follows:
\itemize{
  \item \code{case}. Child identifier.  
  \item \code{outcome}. Frequency of child vocalization, as measured by 
  a partial interval recording procedure with 60 ten-second intervals per session.
  \item \code{time}. Measurement occasion.
  \item \code{treatment}. Indicator for treatment phase.
}
}
\references{
Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). 
A standardized mean difference effect size for multiple baseline designs across individuals. 
\emph{Research Synthesis Methods, 4}(4), 324-341. \doi{10.1002/jrsm.1086}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Carson}
\alias{Carson}
\title{Carson (2008)}
\format{
A data frame with 47 rows and 5 variables
}
\source{
Carson, K. D., Gast, D. L., & Ayres, K. M. (2008). Effects of a photo
  activity schedule book on independent task changes by students with
  intellectual disabilities in community and school job sites. \emph{European
  Journal of Special Needs Education, 23}, 269-279.
}
\description{
Data from a BAB design conducted by Carson, Gast, & Ayres (2008). The
variables are as follows: 
\itemize{ 
   \item \code{case} Participant identifier 
   \item \code{treatment} Factor describing the treatment condition
   \item \code{phase} Numeric describing the phase of the study design for each case 
   \item \code{outcome} Outcome scores 
   \item \code{time} Measurement occasion 
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_BCSMD.R
\name{default_times}
\alias{default_times}
\title{Calculate default initial treatment time and follow-up time.}
\usage{
default_times(
  design,
  case,
  phase,
  session,
  cluster = NULL,
  series = NULL,
  treatment_name = NULL,
  data = NULL
)
}
\arguments{
\item{design}{Character string to specify whether data comes from a treatment
reversal (\code{"TR"}), multiple baseline across participants
(\code{"MBP"}), replicated multiple baseline across behaviors
(\code{"RMBB"}), or clustered multiple baseline across participants
(\code{"CMB"}).}

\item{case}{vector of case indicators or name of a character or factor vector
within \code{data} indicating unique cases.}

\item{phase}{vector of treatment indicators or name of a character or factor
vector within \code{data} indicating unique treatment phases.}

\item{session}{vector of measurement occasions or name of numeric vector
within \code{data} of measurement times.}

\item{cluster}{(Optional) vector of cluster indicators or name of a character
or factor vector within \code{data} indicating clusters.}

\item{series}{(Optional) vector of series indicators or name of a character
or factor vector within \code{data} indicating series.}

\item{treatment_name}{(Optional) character string corresponding to the name
of the treatment phase.}

\item{data}{(Optional) dataset to use for analysis. Must be a
\code{data.frame}.}
}
\value{
A list of time range, default initial treatment time, and default
  follow-up time.
}
\description{
Calculate the default initial treatment time and follow-up time
  that are used to define and estimate the between-case standardized mean
  differences for multiple baseline designs and variations.
}
\note{
If treatment_name is left null, it will choose the second level of the
  phase variable to be the treatment phase.
}
\examples{
data(Laski)
default_times(design = "MBP", 
              case = case, phase = treatment, session = time, 
              data = Laski)

data(Thiemann2001)
default_times(design = "RMBB", 
              case = case, series = series, 
              phase = treatment, session = time, 
              data = Thiemann2001)

data(Bryant2018)
default_times(design = "CMB", 
              cluster = group, case = case, 
              phase = treatment, session = session, 
              data = Bryant2018)
              
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess-function.R
\name{phase_pairs}
\alias{phase_pairs}
\title{Calculate phase-pairs for a unique case}
\usage{
phase_pairs(phase, session = seq_along(phase))
}
\arguments{
\item{phase}{vector of treatment indicators or a character or factor vector indicating unique treatment phases.}

\item{session}{numeric vector of measurement occasions.}
}
\description{
Calculate phase-pairs based on phases and session numbering.
}
\examples{

phases <- rep(c("A","B","A","B"), each = 4)
sessions <- 1:length(phases)

phase_pairs(phases, sessions)

phases <- rep(c("A","B","C","A","B","C","D"), each = 4)
phase_pairs(phases)

phases <- rep(c("B","A","C","B","A","B","C","A"), each = 4)
phase_pairs(phases)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Salazar}
\alias{Salazar}
\title{Salazar, et al. (2020)}
\format{
A data frame with 324 rows and 5 variables
}
\source{
Salazar, D., Ruiz, F., Ramírez, E., & Cardona-Betancourt, V. (2020).
  Acceptance and Commitment Therapy Focused on Repetitive Negative Thinking for
  Child Depression: A Randomized Multiple-Baseline Evaluation. The Psychological
  Record. \doi{10.1007/s40732-019-00362-5}
}
\description{
Data from a multiple baseline design conducted by Salazar, Ruiz,
Ramírez1, &  Cardona-Betancourt (2020). The variables are as follows:
\itemize{
  \item \code{case}. Participant identifier.
  \item \code{measure}. Outcome measure description (AFQ-Y, PTQ-C, or GPQ-C).
  \item \code{treatment} Factor indicating baseline, treatment, post, or follow-up phase.
  \item \code{time}. Measurement occasion.
  \item \code{outcome}. Outcome scores.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation-functions.R
\name{simulate_MB4}
\alias{simulate_MB4}
\title{Simulate Model MB4 from Pustejovsky, Hedges, & Shadish (2014)}
\usage{
simulate_MB4(
  iterations,
  beta,
  rho,
  phi,
  tau2_ratio,
  tau_corr,
  p_const,
  r_const,
  design,
  m,
  n,
  MB = TRUE
)
}
\arguments{
\item{iterations}{number of independent iterations of the simulation}

\item{beta}{vector of fixed effect parameters}

\item{rho}{intra-class correlation parameter}

\item{phi}{autocorrelation parameter}

\item{tau2_ratio}{ratio of trend variance to intercept variance}

\item{tau_corr}{correlation between case-specific trends and intercepts}

\item{p_const}{vector of constants for calculating numerator of effect size}

\item{r_const}{vector of constants for calculating denominator of effect size}

\item{design}{design matrix. If not specified, it will be calculated based on \code{m}, \code{n}, and \code{MB}.}

\item{m}{number of cases. Not used if \code{design} is specified.}

\item{n}{number of measurement occasions. Not used if \code{design} is specified.}

\item{MB}{If true, a multiple baseline design will be used; otherwise, an AB design will be used. Not used if \code{design} is specified.}
}
\value{
A matrix reporting the mean and variance of the effect size estimates
and various associated statistics.
}
\description{
Simulates data from a linear mixed effects model, then calculates 
REML effect size estimator as described in Pustejovsky, Hedges, & Shadish (2014).
}
\examples{

simulate_MB4(iterations = 5, beta = c(0,1,0,0), rho = 0.8, phi = 0.5, 
             tau2_ratio = 0.5, tau_corr = 0, 
             p_const = c(0,1,0,7), r_const = c(1,0,1,0,0), 
             design = design_matrix(3, 16, treat_times=c(5,9,13), center = 12))
             
simulate_MB4(iterations = 5, beta = c(0,1,0,0), rho = 0.8, phi = 0.5, 
             tau2_ratio = 0.5, tau_corr = 0, m = 6, n = 8)
             
}
\references{
Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
\emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm.R
\name{shine_scd}
\alias{shine_scd}
\title{A shiny interface for the scdhlm package}
\usage{
shine_scd(dataset = NULL, ...)
}
\arguments{
\item{dataset}{Optionally, a data.frame or path to a file from which to read
data. If specified, the app will open with the data loaded. Default is
NULL. If \code{dataset} is a data.frame, then it will be passed directly. If
a file path with a \code{.xlsx} extension is specified, it will be read using \code{read_excel}.
If a file path with a \code{.csv} extension is specified, it will be read using \code{read.csv}.
If a file path with a different extension is specified, it will be read using \code{read.table}.}

\item{...}{Further arguments passed to \code{read_excel}, \code{read.csv}, or \code{read.table}.}
}
\description{
An interactive shiny interface for estimating design-comparable
  standardized mean difference effect sizes from single-case designs.
  Estimation methods for multiple baseline and treatment reversal designs are
  available.
}
\examples{
\dontrun{
shine_scd()
data(Laski)
shine_scd(dataset = Laski)
shine_scd(dataset = "SCD_data.xlsx", sheet = "Laski")
shine_scd(dataset = "Laski.csv") 
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation-functions.R
\name{simulate.g_REML}
\alias{simulate.g_REML}
\title{Simulate data from a fitted \code{g_REML} object}
\usage{
\method{simulate}{g_REML}(object, nsim = 1, seed = NULL, parallel = FALSE, ...)
}
\arguments{
\item{object}{a \code{g_REML} object}

\item{nsim}{number of models to simulate}

\item{seed}{seed value. See documentation for \code{\link{simulate}}}

\item{parallel}{if \code{TRUE}, run in parallel using \code{foreach} backend.}

\item{...}{additional optional arguments}
}
\value{
A matrix with one row per simulation, with columns corresponding to the output
of \code{g_REML}.
}
\description{
Simulates data from the linear mixed effects model used to estimate the
specified standardized mean difference effect size. Suitable for parametric bootstrapping.
}
\examples{
data(Laski)
Laski_RML <- lme(fixed = outcome ~ treatment,
                 random = ~ 1 | case,
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)

suppressWarnings(
  Laski_g <- g_REML(Laski_RML, p_const = c(0,1), r_const = c(1,0,1))
)

if (requireNamespace("plyr", quietly = TRUE)) {
  simulate(Laski_g, nsim = 5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{BartonArwood}
\alias{BartonArwood}
\title{Barton-Arwood, Wehby, & Falk (2005)}
\format{
A data frame with 143 rows and 4 variables
}
\source{
Barton-Arwood, S. M., Wehby, J. H., & Falk, K. B. (2005). Reading
  instruction for elementary-age students with emotional and behavioral
  disorders: Academic and behavioral outcomes. \emph{Exceptional Children, 72}(1),
  7-27. \doi{10.1177/001440290507200101}
}
\description{
Data from a multiple baseline design conducted by Barton-Arwood, Wehby, and
Falk (2005). The variables are as follows: \itemize{ \item \code{case}
Participant identifier \item \code{condition} Factor identifying
the phase of the design (A or B) \item \code{session} Measurement occasion 
\item \code{outcome} Oral reading fluency score (words per minute) }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_BCSMD.R
\name{calc_BCSMD}
\alias{calc_BCSMD}
\title{A convenience function for calculating design comparable effect sizes}
\usage{
calc_BCSMD(
  design,
  case,
  phase,
  session,
  outcome,
  cluster = NULL,
  series = NULL,
  center = 0,
  round_session = TRUE,
  treatment_name = NULL,
  FE_base = 0,
  RE_base = 0,
  RE_base_2 = NULL,
  FE_trt = 0,
  RE_trt = NULL,
  RE_trt_2 = NULL,
  corStruct = "AR1",
  varStruct = "hom",
  A = NULL,
  B = NULL,
  D = NULL,
  cover = 95,
  bound = 35,
  symmetric = TRUE,
  summary = TRUE,
  data = NULL,
  ...
)
}
\arguments{
\item{design}{Character string to specify whether data comes from a treatment
reversal (\code{"TR"}), multiple baseline across participants
(\code{"MBP"}), replicated multiple baseline across behaviors
(\code{"RMBB"}), or clustered multiple baseline across participants
(\code{"CMB"}).}

\item{case}{vector of case indicators or name of a character or factor vector
within \code{data} indicating unique cases.}

\item{phase}{vector of treatment indicators or name of a character or factor
vector within \code{data} indicating unique treatment phases.}

\item{session}{vector of measurement occasions or name of numeric vector
within \code{data} of measurement times.}

\item{outcome}{vector of outcome data or name of numeric vector of outcome
data within \code{data}.}

\item{cluster}{(Optional) vector of cluster indicators or name of a character
or factor vector within \code{data} indicating clusters.}

\item{series}{(Optional) vector of series indicators or name of a character
or factor vector within \code{data} indicating series.}

\item{center}{Numeric value for the centering value for session. Default is 0.}

\item{round_session}{Logical indicating whether to round \code{session} to the nearest integer. Defaults to \code{TRUE}.}

\item{treatment_name}{(Optional) character string corresponding to the name
of the treatment phase.}

\item{FE_base}{Vector of integers specifying which fixed effect terms to
include in the baseline phase. Setting \code{FE_base = 0} includes only a
level. Setting \code{FE_base = c(0,1)} includes a level and a linear time
trend.}

\item{RE_base}{Vector of integers specifying which random effect terms to
include in the baseline phase. Setting \code{RE_base = 0} includes only
levels (i.e., random intercepts). Setting \code{RE_base = c(0,1)} includes
random levels and random linear trends.}

\item{RE_base_2}{Vector of integers specifying which random effect terms to
include in the baseline phase for the cluster level in clustered multiple
baseline design across participants or for the case level in replicated
multiple baseline across behaviors. Setting \code{RE_base_2 = 0} includes
only levels (i.e., random intercepts). Setting \code{RE_base_2 = c(0,1)}
includes random levels and random linear trends.}

\item{FE_trt}{Vector of integers specifying which fixed effect terms to
include in the treatment phase. Setting \code{FE_trt = 0} includes only a
change in level. Setting \code{FE_trt = c(0,1)} includes a change in level
and a treatment-by-linear time trend.}

\item{RE_trt}{Vector of integers specifying which random effect terms to
include in the treatment phase. Setting \code{RE_trt = 0} includes only
random changes in level. Setting \code{RE_trt = c(0,1)} includes random
changes in level and random treatment-by-linear time trends.}

\item{RE_trt_2}{Vector of integers specifying which random effect terms to
include in the treatment phase for the cluster level in clustered multiple
baseline design across participants or for the case level in replicated
multiple baseline across behaviors. Setting \code{RE_trt_2 = 0} includes
only random changes in level. Setting \code{RE_trt_2 = c(0,1)} includes
random changes in level and random treatment-by-linear time trends.}

\item{corStruct}{(Optional) character string indicating the correlation
structure of session-level errors. Options are \code{"AR1"} (default),
\code{"MA1"}, or \code{"IID"}.}

\item{varStruct}{(Optional) character string indicating the
heteroscedasticity structure of session-level errors. Options are
\code{"hom"} (default) or \code{"het"}, which allows for the session-level
error variances to differ by phase.}

\item{A}{The time point immediately before the start of treatment in the
hypothetical between-group design.}

\item{B}{The time point at which outcomes are measured in the hypothetical
between-group design.}

\item{D}{Numerical indicating the treatment duration across cases. Note that
\code{B = A + D} and it is not allowed to specify both \code{B} and
\code{D}.}

\item{cover}{Confidence level.}

\item{bound}{Numerical tolerance for non-centrality parameter in
\code{\link[stats]{qt}}.}

\item{symmetric}{If \code{TRUE} (the default), use a symmetric confidence
interval. If \code{FALSE}, use a non-central t approximation to obtain an
asymmetric confidence interval.}

\item{summary}{Logical indicating whether to return a data frame with effect
size estimates and other information. If \code{TRUE} (default), return a
\code{data.frame} containing the effect size estimate, standard error,
confidence interval, and other information. If \code{FALSE}, return a list
with effect size estimate, degrees of freedom, and other information.}

\item{data}{(Optional) dataset to use for analysis. Must be a
\code{data.frame}.}

\item{...}{further arguments.}
}
\value{
If \code{summary == TRUE}, a data frame containing the
  design-comparable effect size estimate, standard error, confidence
  interval, and other information. If \code{summary == FALSE}, a list
  containing all elements of a `g_mlm()` object, plus the fitted `lme()`
  model.
}
\description{
In one call, 1) clean single-case design data for treatment
  reversal and multiple baseline designs, 2) fit a multi-level model using
  restricted maximum likelihood estimation, and 3) estimate a standardized
  mean difference effect size.
}
\examples{
data(Laski)

# Change-in-levels model with fixed treatment effect
calc_BCSMD(design = "MBP",
           case = case, phase = treatment,
           session = time, outcome = outcome,
           FE_base = 0, RE_base = 0, FE_trt = 0,
           data = Laski)

# Model with linear time trends in baseline and treatment phases,
# random baseline slopes, fixed treatment effects
calc_BCSMD(design = "MBP",
           case = case, phase = treatment,
           session = time, outcome = outcome, center = 4,
           FE_base = c(0,1), RE_base = c(0,1), 
           FE_trt = c(0,1),
           data = Laski)


data(Anglesea)
calc_BCSMD(design = "TR",
           case = case, phase = condition,
           session = session, outcome = outcome,
           treatment_name = "treatment",
           FE_base = 0, RE_base = 0, 
           FE_trt = 0,
           data = Anglesea)


data(Thiemann2001)
calc_BCSMD(design = "RMBB",
           case = case, series = series, phase = treatment,
           session = time, outcome = outcome,
           FE_base = 0, RE_base = 0, RE_base_2 = 0, 
           FE_trt = 0,
           data = Thiemann2001)


data(Bryant2018)
calc_BCSMD(design = "CMB",
           cluster = group, case = case, phase = treatment,
           session = session, outcome = outcome, center = 49,
           treatment_name = "treatment",
           FE_base = c(0,1), RE_base = 0, RE_base_2 = 0,
           FE_trt = c(0,1), RE_trt = NULL, RE_trt_2 = NULL,
           data = Bryant2018)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Bryant2018}
\alias{Bryant2018}
\title{Bryant et al. (2018)}
\format{
A data frame with 536 rows and 8 variables
}
\source{
Bryant, D. R., Bryant, B. R., Sorelle-Miner, D. A., Falcomata, T. S.
& Nozari, M. (2018).
Tier 3 intensified intervention for second grade students with severe
mathematics difficulties. \emph{Archives of Psychology, 2}(11), 1-24.
\doi{10.31296/aop.v2i11.86}
}
\description{
Data from a clustered multiple baseline across participants design conducted by
Bryant et al. (2018). The variables are as follows:
\itemize{
  \item \code{Study_ID}. Study identifier.
  \item \code{school}. School identifier.
  \item \code{group}. Group identifier.
  \item \code{case}. Student identifier.
  \item \code{treatment}. Indicator for treatment phase.
  \item \code{session}. Measurement occasion.
  \item \code{session_trt}. Measurement occasion times treatment phase.
  \item \code{outcome}. Texas Early Mathematics Inventory (TEMI-Aim Check) scores.
  \item \code{session_c}. Measurement occasion centered at the follow-up time.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Saddler}
\alias{Saddler}
\title{Example 1 from Hedges, Pustejovsky, & Shadish (2013)}
\format{
A data frame with 124 rows and 5 variables
}
\source{
Saddler, B., Behforooz, B., & Asaro, K. (2008). 
The effects of sentence-combining instruction on the writing of 
fourth-grade students with writing difficulties. 
\emph{The Journal of Special Education, 42}(2), 79-90. 
\doi{10.1177/0022466907310371}
}
\description{
Data from a multiple baseline design conducted by 
Saddler, Behforooz, & Asaro, (2008). The variables are as follows:
\itemize{
  \item \code{case} Student identifier  
  \item \code{measure} Factor indicating the outcome measure (writing quality, T-unit length, number of constructions)
  \item \code{outcome} Value of outcome measure.
  \item \code{time}. Measurement occasion.
  \item \code{treatment}. Factor indicating the treatment phase.
}
}
\references{
Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2013). 
A standardized mean difference effect size for multiple baseline designs across individuals. 
\emph{Research Synthesis Methods, 4}(4), 324-341. \doi{10.1002/jrsm.1086}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/calc_BCSMD.R
\name{batch_calc_BCSMD}
\alias{batch_calc_BCSMD}
\title{A convenience function for calculating multiple design-comparable
  effect sizes from a dataset that compiles data from multiple single-case
  design studies}
\usage{
batch_calc_BCSMD(
  data,
  design,
  grouping,
  case,
  phase,
  session,
  outcome,
  cluster = NULL,
  series = NULL,
  center = 0,
  round_session = TRUE,
  treatment_name = NULL,
  FE_base = 0,
  RE_base = 0,
  RE_base_2 = NULL,
  FE_trt = 0,
  RE_trt = NULL,
  RE_trt_2 = NULL,
  corStruct = "AR1",
  varStruct = "hom",
  A = NULL,
  B = NULL,
  D = NULL,
  cover = 95,
  bound = 35,
  symmetric = TRUE,
  ...
)
}
\arguments{
\item{data}{A data frame containing SCD data for which design-comparable
effect sizes will be calculated.}

\item{design}{Character string to specify whether data comes from a treatment
reversal (\code{"TR"}), multiple baseline across participants
(\code{"MBP"}), replicated multiple baseline across behaviors
(\code{"RMBB"}), or clustered multiple baseline across participants
(\code{"CMB"}).}

\item{grouping}{A variable name or list of (unquoted) variable names that
uniquely identify each study.}

\item{case}{A variable name (unquoted) that identifies unique cases within
each \code{grouping} variable.}

\item{phase}{A variable name (unquoted) that identifies unique treatment
phases.}

\item{session}{A variable name (unquoted) that contains the measurement times
for each data series.}

\item{outcome}{A variable name (unquoted) that contains the outcome
measurements for each data series.}

\item{cluster}{(Optional) variable name (unquoted) that identifies the unique
clusters of cases for \code{CMB} designs.}

\item{series}{(Optional) variable name (unquoted) that identifies the unique
data series for \code{RMBB} designs.}

\item{center}{Numeric value for the centering value for session. Default is 0.}

\item{round_session}{Logical indicating whether to round \code{session} to the nearest integer. Defaults to \code{TRUE}.}

\item{treatment_name}{(Optional) character string corresponding to the name
of the treatment phase.}

\item{FE_base}{Vector of integers specifying which fixed effect terms to
include in the baseline phase. Setting \code{FE_base = 0} includes only a
level. Setting \code{FE_base = c(0,1)} includes a level and a linear time
trend.}

\item{RE_base}{Vector of integers specifying which random effect terms to
include in the baseline phase. Setting \code{RE_base = 0} includes only
levels (i.e., random intercepts). Setting \code{RE_base = c(0,1)} includes
random levels and random linear trends.}

\item{RE_base_2}{Vector of integers specifying which random effect terms to
include in the baseline phase for the cluster level in clustered multiple
baseline design across participants or for the case level in replicated
multiple baseline across behaviors. Setting \code{RE_base_2 = 0} includes
only levels (i.e., random intercepts). Setting \code{RE_base_2 = c(0,1)}
includes random levels and random linear trends.}

\item{FE_trt}{Vector of integers specifying which fixed effect terms to
include in the treatment phase. Setting \code{FE_trt = 0} includes only a
change in level. Setting \code{FE_trt = c(0,1)} includes a change in level
and a treatment-by-linear time trend.}

\item{RE_trt}{Vector of integers specifying which random effect terms to
include in the treatment phase. Setting \code{RE_trt = 0} includes only
random changes in level. Setting \code{RE_trt = c(0,1)} includes random
changes in level and random treatment-by-linear time trends.}

\item{RE_trt_2}{Vector of integers specifying which random effect terms to
include in the treatment phase for the cluster level in clustered multiple
baseline design across participants or for the case level in replicated
multiple baseline across behaviors. Setting \code{RE_trt_2 = 0} includes
only random changes in level. Setting \code{RE_trt_2 = c(0,1)} includes
random changes in level and random treatment-by-linear time trends.}

\item{corStruct}{(Optional) character string indicating the correlation
structure of session-level errors. Options are \code{"AR1"} (default),
\code{"MA1"}, or \code{"IID"}.}

\item{varStruct}{(Optional) character string indicating the
heteroscedasticity structure of session-level errors. Options are
\code{"hom"} (default) or \code{"het"}, which allows for the session-level
error variances to differ by phase.}

\item{A}{The time point immediately before the start of treatment in the
hypothetical between-group design.}

\item{B}{The time point at which outcomes are measured in the hypothetical
between-group design.}

\item{D}{Numerical indicating the treatment duration across cases. Note that
\code{B = A + D} and it is not allowed to specify both \code{B} and
\code{D}.}

\item{cover}{Confidence level.}

\item{bound}{Numerical tolerance for non-centrality parameter in
\code{\link[stats]{qt}}.}

\item{symmetric}{If \code{TRUE} (the default), use a symmetric confidence
interval. If \code{FALSE}, use a non-central t approximation to obtain an
asymmetric confidence interval.}

\item{...}{further arguments.}
}
\value{
A data frame containing the design-comparable effect size estimate,
  standard error, confidence interval, and other information, for each unique
  category of \code{grouping} variable(s).
}
\description{
Calculates standardized mean difference effect sizes for a data
  set including one or multiple single-case design studies using the same
  design (treatment reversal, multiple baseline/probe across participants,
  replicated multiple baseline across behaviors, or clustered multiple
  baseline across participants).
}
\examples{
data(Thiemann2001)
data(Thiemann2004)
datThiemann <- rbind(Thiemann2001, Thiemann2004)

# Change-in-levels model with a fixed treatment effect
batch_calc_BCSMD(data = datThiemann,
                 grouping = Study_ID,
                 design = "RMBB",
                 case = case, series = series, phase = treatment,
                 session = time, outcome = outcome,
                 FE_base = 0, RE_base = 0,
                 RE_base_2 = 0, FE_trt = 0)

# Models with linear time trends in baseline and treatment phase,
# random baseline slope at series level, fixed treatment effects
batch_calc_BCSMD(data = datThiemann,
                 grouping = Study_ID,
                 design = "RMBB",
                 case = case, series = series, phase = treatment,
                 session = time, outcome = outcome,
                 FE_base = c(0,1), RE_base = c(0,1),
                 RE_base_2 = 0, FE_trt = c(0,1))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Peltier}
\alias{Peltier}
\title{Peltier et al. (2020)}
\format{
A data frame with 232 rows and 4 variables
}
\source{
Peltier, C., Sinclair, T. E., Pulos, J. M., & Suk, A. (2020). Effects
  of schema-based instruction on immediate, generalized, and combined
  structured word problems. \emph{The Journal of Special Education, 54}(2),
  101-112. \doi{10.1177/0022466919883397}
}
\description{
Data from a multiple baseline design conducted by Peltier, Sinclair, Pulos, &
Suk (2020). The variables are as follows: \itemize{ \item \code{case}.
Participant identifier. \item \code{session}. Measurement occasion. \item
\code{condition}. Factor identifying the phase of the design (baseline or
treatment). \item \code{outcome}. Mathematical problem-solving performance
(percentage). }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation-functions.R
\name{design_matrix}
\alias{design_matrix}
\title{Create a design matrix for a single-case design}
\usage{
design_matrix(m, n, treat_times = n/2 + 1, center = 0)
}
\arguments{
\item{m}{number of cases}

\item{n}{number of time points}

\item{treat_times}{(Optional) vector of length \code{m} listing treatment introduction times for each case.}

\item{center}{centering point for time trend.}
}
\value{
A design matrix
}
\description{
Create a design matrix containing a linear trend, a treatment effect, and a 
trend-by-treatment interaction for a single-case design with \code{m} cases and \code{n} 
measurement occasions.
}
\examples{
design_matrix(3, 16, c(5,9,13))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{AlberMorgan}
\alias{AlberMorgan}
\title{Alber-Morgan, et al. (2007)}
\format{
A data frame with 119 rows and 4 variables
}
\source{
Alber-Morgan, S. R., Ramp, E. M., Anderson, L. L., & Martin, C. M.
  (2007). Effects of repeated readings, error correction, and performance
  feedback on the fluency and comprehension of middle school students with
  behavior problems. Journal of Special Education, 41(1), 17-30.
  \doi{10.1177/00224669070410010201}
}
\description{
Data from a multiple baseline design conducted by Alber-Morgan, Ramp,
Anderson, & Martin (2007). The variables are as follows: \itemize{ \item
\code{case} Participant identifier \item \code{condition} Factor identifying
the phase of the design (baseline or treatment) \item \code{session}
Measurement occasion \item \code{outcome} Number of words read correctly per
minute }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulation-functions.R
\name{compare_RML_HPS}
\alias{compare_RML_HPS}
\title{Run simulation comparing REML and HPS estimates}
\usage{
compare_RML_HPS(iterations, beta, rho, phi, design, m, n, MB = TRUE)
}
\arguments{
\item{iterations}{number of independent iterations of the simulation}

\item{beta}{vector of fixed effect parameters}

\item{rho}{intra-class correlation parameter}

\item{phi}{autocorrelation parameter}

\item{design}{design matrix. If not specified, it will be calculated based on \code{m}, \code{n}, and \code{MB}.}

\item{m}{number of cases. Not used if \code{design} is specified.}

\item{n}{number of measurement occasions. Not used if \code{design} is specified.}

\item{MB}{If true, a multiple baseline design will be used; otherwise, an AB design will be used. Not used if \code{design} is specified.}
}
\value{
A matrix reporting the mean and variance of the effect size estimates
and various associated statistics.
}
\description{
Simulates data from a simple linear mixed effects model, then calculates 
REML and HPS effect size estimators as described in Pustejovsky, Hedges, & Shadish (2014).
}
\examples{
compare_RML_HPS(iterations=10, beta = c(0,1,0,0), rho = 0.3, 
                 phi = 0.5, design=design_matrix(m=3,n=8))
}
\references{
Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
\emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Schutte}
\alias{Schutte}
\title{Example from Pustejovsky, Hedges, & Shadish (2014)}
\format{
A data frame with 136 rows and 4 variables
}
\source{
Schutte, N. S., Malouff, J. M., & Brown, R. F. (2008). 
Efficacy of an emotion-focused treatment for prolonged fatigue. 
\emph{Behavior Modification, 32}(5), 699-713. \doi{10.1177/0145445508317133}
}
\description{
Data from a multiple baseline design conducted by 
Schutte, Malouff, & Brown (2008). Case 4 is excluded because nearly all 
of these measurements are at the upper extreme of the scale. The variables are as follows:
\itemize{
  \item \code{case}. Participant identifier.  
  \item \code{week}. Measurement occasion.
  \item \code{treatment}. Factor indicating baseline or treatment phase.
  \item \code{fatigue}. Fatigue severity scale scores.
}
}
\references{
Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
\emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{MB4time}
\alias{MB4time}
\title{MB4 simulation time}
\format{
A data frame
}
\description{
MB4 simulation time
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess-function.R
\name{preprocess_SCD}
\alias{preprocess_SCD}
\title{Clean Single Case Design Data}
\usage{
preprocess_SCD(
  design,
  case,
  phase,
  session,
  outcome,
  cluster = NULL,
  series = NULL,
  center = 0,
  round_session = TRUE,
  treatment_name = NULL,
  data = NULL
)
}
\arguments{
\item{design}{Character string to specify whether data comes from a treatment
reversal (\code{"TR"}), multiple baseline across participants
(\code{"MBP"}), replicated multiple baseline across behaviors
(\code{"RMBB"}), or clustered multiple baseline across participants
(\code{"CMB"}).}

\item{case}{vector of case indicators or name of a character or factor vector
within \code{data} indicating unique cases.}

\item{phase}{vector of treatment indicators or name of a character or factor
vector within \code{data} indicating unique treatment phases.}

\item{session}{vector of measurement occasions or name of numeric vector
within \code{data} of measurement times.}

\item{outcome}{vector of outcome data or name of numeric vector of outcome
data within \code{data}.}

\item{cluster}{(Optional) vector of cluster indicators or name of a character
or factor vector within \code{data} indicating clusters.}

\item{series}{(Optional) vector of series indicators or name of a character
or factor vector within \code{data} indicating series.}

\item{center}{Numeric value for the centering value for session. Default is 0.}

\item{round_session}{Logical indicating whether to round \code{session} to the nearest integer. Defaults to \code{TRUE}.}

\item{treatment_name}{(Optional) character string corresponding to the name
of the treatment phase.}

\item{data}{(Optional) dataset to use for analysis. Must be a
\code{data.frame}.}
}
\value{
A cleaned SCD dataset that can be used for model fitting and effect size calculation.
}
\description{
Clean single case design data for treatment reversal and multiple baseline designs.
}
\note{
If treatment_name is left null it will choose the second level of the phase variable to be the treatment phase.
}
\examples{
data(Laski)
preprocess_SCD(design = "MBP", 
               case = case, phase = treatment,
               session = time, outcome = outcome, 
               center = 4, data = Laski)
               
data(Anglesea) 
preprocess_SCD(design="TR",
               case=case, phase=condition,
               session=session, outcome=outcome,
               treatment_name = "treatment",
               data=Anglesea)
               
data(Thiemann2001)
preprocess_SCD(design = "RMBB", 
               case = case, series = series, phase = treatment, 
               session = time, outcome = outcome, 
               data = Thiemann2001)
          
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/preprocess-function.R
\name{session_by_treatment}
\alias{session_by_treatment}
\title{Calculate session-by-treatment interactions for a unique case}
\usage{
session_by_treatment(phase, session, trt_phase)
}
\arguments{
\item{phase}{vector of treatment indicators or a character or factor vector indicating unique treatment phases.}

\item{session}{numeric vector of measurement occasions.}

\item{trt_phase}{character string indicating the phase value corresponding to
the treatment condition.}
}
\description{
Calculate session-by-treatment interactions based on phases and
  session numbering.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HPS-ES-functions.R
\name{effect_size_ABk}
\alias{effect_size_ABk}
\title{Calculates HPS effect size}
\usage{
effect_size_ABk(
  outcome,
  treatment,
  id,
  phase,
  time,
  data = NULL,
  phi = NULL,
  rho = NULL
)
}
\arguments{
\item{outcome}{vector of outcome data or name of variable within \code{data}. May not contain any missing values.}

\item{treatment}{vector of treatment indicators or name of variable within \code{data}. Must be the same length as \code{outcome}.}

\item{id}{factor vector indicating unique cases or name of variable within \code{data}. Must be the same length as \code{outcome}.}

\item{phase}{factor vector indicating unique phases (each containing one contiguous control 
condition and one contiguous treatment condition) or name of variable within \code{data}. Must be the same length as \code{outcome}.}

\item{time}{vector of measurement occasion times or name of variable within \code{data}. Must be the same length as \code{outcome}.}

\item{data}{(Optional) dataset to use for analysis. Must be data.frame.}

\item{phi}{(Optional) value of the auto-correlation nuisance parameter, to be used 
in calculating the small-sample adjusted effect size}

\item{rho}{(Optional) value of the intra-class correlation nuisance parameter, to be used 
in calculating the small-sample adjusted effect size}
}
\value{
A list with the following components
\tabular{ll}{
\code{M_a} \tab Matrix reporting the total number of time points with data for all ids, 
by phase and treatment condition \cr
\code{M_dot} \tab Total number of time points used to calculate the total variance (the sum of \code{M_a}) \cr
\code{D_bar} \tab numerator of effect size estimate \cr
\code{S_sq} \tab sample variance, pooled across time points and treatment groups \cr
\code{delta_hat_unadj} \tab unadjusted effect size estimate \cr
\code{phi} \tab corrected estimate of first-order auto-correlation \cr
\code{sigma_sq_w} \tab corrected estimate of within-case variance \cr
\code{rho} \tab estimated intra-class correlation \cr
\code{theta} \tab estimated scalar constant \cr
\code{nu} \tab estimated degrees of freedom \cr
\code{delta_hat} \tab corrected effect size estimate \cr
\code{V_delta_hat} \tab estimated variance of the effect size
}
}
\description{
Calculates the HPS effect size estimator based on data from an (AB)^k design, 
as described in Hedges, Pustejovsky, & Shadish (2012). Note that the data must contain one row per 
measurement occasion per subject.
}
\note{
If phi or rho is left unspecified (or both), estimates for the nuisance
parameters will be calculated.
}
\examples{
data(Lambert)
effect_size_ABk(outcome = outcome, treatment = treatment, id = case, 
                phase = phase, time = time, data = Lambert)
   
data(Anglesea)
effect_size_ABk(outcome = outcome, treatment = condition, id = case, 
                phase = phase, time = session, data = Anglesea)

}
\references{
Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
A standardized mean difference effect size for single case designs. 
\emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{GunningEspie}
\alias{GunningEspie}
\title{Gunning & Espie (2003)}
\format{
A data frame with 301 rows and 4 variables
}
\source{
Gunning, M. J., & Espie, C.A. (2003). Psychological treatment of
  reported sleep disorder in adults with intellectual disability using a
  multiple baseline design. \emph{Journal of Intellectual Disability
  Research, 47}(3), 191-202. \doi{10.1046/j.1365-2788.2003.00461.x}
}
\description{
Data from a multiple baseline design conducted by Gunning & Espie (2003). The
variables are as follows: 
\itemize{ 
  \item \code{case}. Participant identifier.  
  \item \code{session}. Measurement occasion. 
  \item \code{condition}. Factor identifying the phase of the design (baseline or treatment). 
  \item \code{outcome}. Sleep onset latency in minutes. 
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Anglesea}
\alias{Anglesea}
\title{Example 2 from Hedges, Pustejovsky, & Shadish (2012)}
\format{
A data frame with 55 rows and 5 variables
}
\source{
Anglesea, M. M., Hoch, H., & Taylor, B. A. (2008). 
Reducing rapid eating in teenagers with autism: Use of a pager prompt. 
\emph{Journal of Applied Behavior Analysis, 41}(1), 107-111. 
\doi{10.1901/jaba.2008.41-107}
}
\description{
Data from an ABAB design conducted by Anglesea, Hoch, & Taylor (2008). 
The variables are as follows:
\itemize{
  \item \code{case} Case identifier.  
  \item \code{condition} Factor indicating baseline or treatment condition
  \item \code{phase} Study phase (including both control and treatment condition)
  \item \code{session} Measurement occasion
  \item \code{outcome} Total seconds of eating time
}
}
\references{
Hedges, L. V., Pustejovsky, J. E., & Shadish, W. R. (2012).
A standardized mean difference effect size for single case designs. 
\emph{Research Synthesis Methods, 3}, 224-239. \doi{10.1002/jrsm.1052}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Musser}
\alias{Musser}
\title{Musser (2001)}
\format{
A data frame with 136 rows and 4 variables
}
\source{
Musser, E. H., Bray, M. A., Kehle, T. J., & Jenson, W. R. (2001).
Reducing disruptive behaviors in students with serious emotional disturbance.
School Psychology Review, 30(2), 294-304.
}
\description{
Data from a multiple baseline design conducted by 
Musser, Bray, Kehle, and Jenson (2001). The variables are as follows:
\itemize{
  \item \code{student} Participant identifier
  \item \code{session} Measurement occasion
  \item \code{outcome} Percentage of disruptive intervals
  \item \code{treatment} Factor indicating baseline, treatment, or follow-up phase
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/REML-ES-functions.R
\name{Info_Expected_lmeAR1}
\alias{Info_Expected_lmeAR1}
\title{Calculate expected information matrix}
\usage{
Info_Expected_lmeAR1(m_fit)
}
\arguments{
\item{m_fit}{Fitted model of class lme, with AR(1) correlation structure at level 1.}
}
\value{
Expected Information matrix corresponding to variance components of \code{m_fit}.
}
\description{
Calculates the expected information matrix from a fitted linear mixed effects
model with AR(1) correlation structure in the level-1 errors.
}
\examples{
data(Laski)
Laski_RML <- lme(fixed = outcome ~ treatment, 
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
Info_Expected_lmeAR1(Laski_RML)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{MB2time}
\alias{MB2time}
\title{MB2 simulation time}
\format{
A data frame
}
\description{
MB2 simulation time
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/REML-ES-functions.R
\name{g_REML}
\alias{g_REML}
\title{Calculates adjusted REML effect size}
\usage{
g_REML(
  m_fit,
  p_const,
  r_const,
  X_design = model.matrix(m_fit, data = m_fit$data),
  Z_design = model.matrix(m_fit$modelStruct$reStruct, data = m_fit$data),
  block = nlme::getGroups(m_fit),
  times = attr(m_fit$modelStruct$corStruct, "covariate"),
  returnModel = TRUE
)
}
\arguments{
\item{m_fit}{Fitted model of class lme, with AR(1) correlation structure at level 1.}

\item{p_const}{Vector of constants for calculating numerator of effect size. 
Must be the same length as fixed effects in \code{m_fit}.}

\item{r_const}{Vector of constants for calculating denominator of effect size. 
Must be the same length as the number of variance component parameters in \code{m_fit}.}

\item{X_design}{(Optional) Design matrix for fixed effects. Will be extracted from \code{m_fit} if not specified.}

\item{Z_design}{(Optional) Design matrix for random effects. Will be extracted from \code{m_fit} if not specified.}

\item{block}{(Optional) Factor variable describing the blocking structure. Will be extracted from \code{m_fit} if not specified.}

\item{times}{(Optional) list of times used to describe AR(1) structure. Will be extracted from \code{m_fit} if not specified.}

\item{returnModel}{(Optional) If true, the fitted input model is included in the return.}
}
\value{
A list with the following components
\tabular{ll}{
\code{p_beta} \tab Numerator of effect size \cr
\code{r_theta} \tab Squared denominator of effect size \cr
\code{delta_AB} \tab Unadjusted (REML) effect size estimate \cr
\code{nu} \tab Estimated denominator degrees of freedom \cr
\code{kappa} \tab Scaled standard error of numerator \cr
\code{g_AB} \tab Corrected effect size estimate \cr
\code{V_g_AB} \tab Approximate variance estimate \cr
\code{cnvg_warn} \tab Indicator that model did not converge \cr
\code{sigma_sq} \tab Estimated level-1 variance \cr
\code{phi} \tab Estimated autocorrelation \cr
\code{Tau} \tab Vector of level-2 variance components \cr
\code{I_E_inv} \tab Expected information matrix \cr
}
}
\description{
Estimates a design-comparable standardized mean difference effect size based on data 
from a multiple baseline design, using adjusted REML method as described in Pustejovsky, Hedges, 
& Shadish (2014). Note that the data must contain one row per measurement occasion per case.
}
\examples{
data(Laski)
Laski_RML <- lme(fixed = outcome ~ treatment, 
                 random = ~ 1 | case, 
                 correlation = corAR1(0, ~ time | case), 
                 data = Laski)
summary(Laski_RML)
g_REML(Laski_RML, p_const = c(0,1), r_const = c(1,0,1), returnModel=FALSE)

data(Schutte)
Schutte$trt.week <- with(Schutte, unlist(tapply((treatment=="treatment") * week, 
         list(treatment,case), function(x) x - min(x))) + (treatment=="treatment"))
Schutte$week <- Schutte$week - 9
Schutte_RML <- lme(fixed = fatigue ~ week + treatment + trt.week, 
                   random = ~ week | case, 
                   correlation = corAR1(0, ~ week | case), 
                   data = subset(Schutte, case != 4))
summary(Schutte_RML)
Schutte_g <- g_REML(Schutte_RML, p_const = c(0,0,1,7), r_const = c(1,0,1,0,0))
summary(Schutte_g)
}
\references{
Pustejovsky, J. E., Hedges, L. V., & Shadish, W. R. (2014). 
Design-comparable effect sizes in multiple baseline designs: A general modeling framework.
\emph{Journal of Educational and Behavioral Statistics, 39}(4), 211-227. \doi{10.3102/1076998614547577}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Ruiz}
\alias{Ruiz}
\title{Ruiz, et al. (2020)}
\format{
A data frame with 840 rows and 5 variables
}
\source{
Ruiz, F., Luciano, C., Florez, C., Suarez-Falcon, J., & Cardona-Betancourt, V. (2020).
  A Multiple-Baseline Evaluation of Acceptance and Commitment Therapy Focused on Repetitive Negative 
  Thinking for Comorbid Generalized Anxiety Disorder and Depression. Frontiers in Psychology, 11. 
  \doi{10.3389/fpsyg.2020.00356}
}
\description{
Data from a multiple baseline design conducted by  Ruiz,
Luciano, Florez, Suarez-Falcon, &  Cardona-Betancourt (2020). The variables are as follows:
\itemize{
  \item \code{case}. Participant identifier.
  \item \code{measure}. Outcome measure description (AAQ-II, ANXIETY, CFQ, DASS-TOTAL, DEPRESSION, PSWQ, PTQ, STRESS, VQ-OBSTRUCTION, or VQ-PROGRESS).
  \item \code{treatment} Factor indicating baseline, treatment, post, or follow-up phase.
  \item \code{time}. Measurement occasion.
  \item \code{outcome}. Outcome scores.
}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graphing-functions.R
\name{graph_SCD}
\alias{graph_SCD}
\title{Graph Single Case Design Data}
\usage{
graph_SCD(
  design,
  case,
  phase,
  session,
  outcome,
  cluster = NULL,
  series = NULL,
  treatment_name = NULL,
  model_fit = NULL,
  data = NULL,
  newdata = NULL
)
}
\arguments{
\item{design}{Character string to specify whether data comes from a treatment
reversal (\code{"TR"}), multiple baseline across participants
(\code{"MBP"}), replicated multiple baseline across behaviors
(\code{"RMBB"}), or clustered multiple baseline across participants
(\code{"CMB"}).}

\item{case}{vector of case indicators or name of a character or factor vector
within \code{data} indicating unique cases.}

\item{phase}{vector of treatment indicators or name of a character or factor
vector within \code{data} indicating unique treatment phases.}

\item{session}{vector of measurement occasions or name of numeric vector
within \code{data} of measurement times.}

\item{outcome}{vector of outcome data or name of numeric vector of outcome
data within \code{data}.}

\item{cluster}{(Optional) vector of cluster indicators or name of a character
or factor vector within \code{data} indicating clusters.}

\item{series}{(Optional) vector of series indicators or name of a character
or factor vector within \code{data} indicating series.}

\item{treatment_name}{(Optional) character string corresponding to the name
of the treatment phase.}

\item{model_fit}{(Optional) lme fitted model that adds predicted values to
graph}

\item{data}{(Optional) dataset to use for analysis. Must be a
\code{data.frame}.}

\item{newdata}{(Optional) dataset to use for calculating predicted values
based on \code{model_fit}. Must be a \code{data.frame}.}
}
\value{
A ggplot graph
}
\description{
Graphs single case design data for treatment reversal and
  multiple baseline designs.
}
\note{
If treatment_name is left null it will choose the second level of the
  phase variable to be the treatment phase.
}
\examples{

if (requireNamespace("ggplot2", quietly = TRUE)) {

data(Anglesea)
graph_SCD(design="TR",
          case=case, phase=condition,
          session=session, outcome=outcome,
          treatment_name = "treatment",
          data=Anglesea)

data(BartonArwood)
graph_SCD(design="MBP",
          case=case, phase=condition,
          session=session, outcome=outcome,
          treatment_name = "B",
          data=BartonArwood)

data(Thiemann2001)
graph_SCD(design="RMBB",
          case=case, series = series, phase=treatment,
          session=time, outcome=outcome,
          treatment_name = "treatment",
          data=Thiemann2001)

data(Bryant2018)
graph_SCD(design="CMB",
          cluster=group, case=case, phase=treatment,
          session=session, outcome=outcome,
          treatment_name = "treatment",
          data=Bryant2018)

}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{Rodgers}
\alias{Rodgers}
\title{Rodgers et al. (2021)}
\format{
A data frame with 83 rows and 4 variables
}
\source{
Rodgers, D. B., Datchuk, S. M., & Rila, A. L. (2021).  Effects of a
  text-writing fluency intervention for postsecondary students with
  intellectual and developmental disabilities. \emph{Exceptionality, 29}(4),
  310-325. \doi{10.1080/09362835.2020.1850451}
}
\description{
Data from a multiple baseline design conducted by Rodgers, Datchuk, & Rila
(2021). The variables are as follows: \itemize{ \item \code{case}.
Participant identifier. \item \code{session}. Measurement occasion. \item
\code{condition}. Factor identifying the phase of the design (baseline or
treatment). \item \code{outcome}. The number of correct writing sequences in
1 minute. }
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scdhlm-package.r
\docType{data}
\name{DelemereDounavi}
\alias{DelemereDounavi}
\title{Delemere & Dounavi (2018)}
\format{
A data frame with 187 rows and 5 variables
}
\source{
Delemere, E., & Dounavi, K. (2018). Parent-implemented bedtime fading
  and positive routines for children with autism spectrum disorders.
  \emph{Journal of Autism and Developmental Disorders, 48}(4), 1002-1019.
  \doi{10.1007/s10803-017-3398-4}
}
\description{
Data from a multiple baseline design conducted by Delemere & Dounavi (2018).
The variables are as follows: 
\itemize{ 
  \item \code{intervention}. Type of intervention received: bedtime fading or positive routines.
  \item \code{case}. Participant identifier. 
  \item \code{session}. Measurement occasion. 
  \item \code{condition}. Factor identifying the phase of the design (baseline or treatment). 
  \item \code{outcome}. Total sleep onset latency in minutes across nights. 
}
}
\keyword{datasets}
